=== modified file 'Makefile'
--- Makefile	2017-11-21 01:01:11 +0000
+++ Makefile	2018-03-24 11:30:25 +0000
@@ -90,6 +90,12 @@
 	# Check that there were no errors reported.
 	subunit-stats < selftest.log
 
+check-ci: docs extensions
+	# FIXME: Remove -Wignore::FutureWarning once
+	# https://github.com/paramiko/paramiko/issues/713 is not a concern
+	# anymore -- vila 2017-05-24
+	python -Werror -Wignore::FutureWarning  -Wignore::ImportWarning -O ./brz selftest -v --parallel=fork -Oselftest.timeout=120 --subunit2
+
 # Run Python style checker (apt-get install pyflakes)
 #
 # Note that at present this gives many false warnings, because it doesn't

=== modified file 'TODO'
--- TODO	2010-03-09 21:42:11 +0000
+++ TODO	2018-02-18 21:42:57 +0000
@@ -1,4 +1,3 @@
-For things to do in Bazaar development, see
+For things to do in Breezy development, see
 
-https://bugs.launchpad.net/bzr/
-https://blueprints.launchpad.net/bzr/
+https://bugs.launchpad.net/brz/

=== modified file 'breezy/__init__.py'
--- breezy/__init__.py	2017-08-26 19:06:32 +0000
+++ breezy/__init__.py	2018-03-04 19:11:21 +0000
@@ -42,9 +42,6 @@
 import sys
 
 
-IGNORE_FILENAME = ".bzrignore"
-
-
 __copyright__ = "Copyright 2005-2012 Canonical Ltd."
 
 # same format as sys.version_info: "A tuple containing the five components of

=== modified file 'breezy/_annotator_py.py'
--- breezy/_annotator_py.py	2017-06-05 20:48:31 +0000
+++ breezy/_annotator_py.py	2018-02-24 15:50:23 +0000
@@ -261,12 +261,9 @@
                         each key is a possible source for the given line.
             lines the text of "key" as a list of lines
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             for text_key, text, num_lines in self._get_needed_texts(key, pb=pb):
                 self._annotate_one(text_key, text, num_lines)
-        finally:
-            pb.finished()
         try:
             annotations = self._annotations_cache[key]
         except KeyError:

=== modified file 'breezy/add.py'
--- breezy/add.py	2017-05-22 00:56:52 +0000
+++ breezy/add.py	2018-03-10 13:18:10 +0000
@@ -22,6 +22,7 @@
 import os
 
 from . import (
+    errors,
     osutils,
     ui,
     )
@@ -127,10 +128,12 @@
         we look for a file with the same name in that directory.
         Else, we look for an entry in the base tree with the same path.
         """
-        if self.base_tree.has_id(parent_ie.file_id):
-            base_path = osutils.pathjoin(
-                self.base_tree.id2path(parent_ie.file_id),
-                osutils.basename(path))
+        try:
+            parent_path = self.base_tree.id2path(parent_ie.file_id)
+        except errors.NoSuchId:
+            pass
+        else:
+            base_path = osutils.pathjoin(parent_path, osutils.basename(path))
             base_id = self.base_tree.path2id(base_path)
             if base_id is not None:
                 return (base_id, base_path)

=== modified file 'breezy/branch.py'
--- breezy/branch.py	2018-02-18 19:18:40 +0000
+++ breezy/branch.py	2018-03-25 00:39:16 +0000
@@ -26,6 +26,7 @@
     config as _mod_config,
     debug,
     fetch,
+    memorytree,
     repository,
     revision as _mod_revision,
     tag as _mod_tag,
@@ -97,7 +98,6 @@
         self._revision_id_to_revno_cache = None
         self._partial_revision_id_to_revno_cache = {}
         self._partial_revision_history_cache = []
-        self._tags_bytes = None
         self._last_revision_info_cache = None
         self._master_branch_cache = None
         self._merge_sorted_revisions_cache = None
@@ -259,23 +259,6 @@
         a_branch = Branch.open(url, possible_transports=possible_transports)
         return a_branch.repository
 
-    def _get_tags_bytes(self):
-        """Get the bytes of a serialised tags dict.
-
-        Note that not all branches support tags, nor do all use the same tags
-        logic: this method is specific to BasicTags. Other tag implementations
-        may use the same method name and behave differently, safely, because
-        of the double-dispatch via
-        format.make_tags->tags_instance->get_tags_dict.
-
-        :return: The bytes of the tags file.
-        :seealso: Branch._set_tags_bytes.
-        """
-        with self.lock_read():
-            if self._tags_bytes is None:
-                self._tags_bytes = self._transport.get_bytes('tags')
-            return self._tags_bytes
-
     def _get_nick(self, local=False, possible_transports=None):
         config = self.get_config()
         # explicit overrides master, but don't look for master if local is True
@@ -834,8 +817,7 @@
 
         Don't call this directly, use set_stacked_on_url(None).
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             pb.update(gettext("Unstacking"))
             # The basic approach here is to fetch the tip of the branch,
             # including all available ghosts, from the existing stacked
@@ -909,17 +891,6 @@
                     old_repository, required_ids=[self.last_revision()],
                     if_present_ids=tags_to_fetch, find_ghosts=True).execute()
                 self.repository.fetch(old_repository, fetch_spec=fetch_spec)
-        finally:
-            pb.finished()
-
-    def _set_tags_bytes(self, bytes):
-        """Mirror method for _get_tags_bytes.
-
-        :seealso: Branch._get_tags_bytes.
-        """
-        with self.lock_write():
-            self._tags_bytes = bytes
-            return self._transport.put_bytes('tags', bytes)
 
     def _cache_revision_history(self, rev_history):
         """Set the cached revision history to rev_history.
@@ -956,7 +927,6 @@
         self._merge_sorted_revisions_cache = None
         self._partial_revision_history_cache = []
         self._partial_revision_id_to_revno_cache = {}
-        self._tags_bytes = None
 
     def _gen_revision_history(self):
         """Return sequence of revision hashes on to this branch.
@@ -1249,7 +1219,7 @@
             self.copy_content_into(result, revision_id=revision_id)
             master_url = self.get_bound_location()
             if master_url is None:
-                result.set_parent(self.controldir.root_transport.base)
+                result.set_parent(self.user_url)
             else:
                 result.set_parent(master_url)
         return result
@@ -1517,6 +1487,13 @@
         if_present_fetch.discard(_mod_revision.NULL_REVISION)
         return must_fetch, if_present_fetch
 
+    def create_memorytree(self):
+        """Create a memory tree for this branch.
+
+        :return: An in-memory MutableTree instance
+        """
+        return memorytree.MemoryTree.create_on_branch(self)
+
 
 class BranchFormat(controldir.ControlComponentFormat):
     """An encapsulation of the initialization and open routines for a format.

=== modified file 'breezy/branchbuilder.py'
--- breezy/branchbuilder.py	2018-02-18 15:21:06 +0000
+++ breezy/branchbuilder.py	2018-03-25 00:39:16 +0000
@@ -112,7 +112,7 @@
             if base_id != self._branch.last_revision():
                 self._move_branch_pointer(base_id,
                     allow_leftmost_as_ghost=allow_leftmost_as_ghost)
-        tree = memorytree.MemoryTree.create_on_branch(self._branch)
+        tree = self._branch.create_memorytree()
         tree.lock_write()
         try:
             if parent_ids is not None:
@@ -157,7 +157,7 @@
             # We are cheating a little bit here, and locking the new tree
             # before the old tree is unlocked. But that way the branch stays
             # locked throughout.
-            new_tree = memorytree.MemoryTree.create_on_branch(self._branch)
+            new_tree = self._branch.create_memorytree()
             new_tree.lock_write()
             self._tree.unlock()
             self._tree = new_tree
@@ -172,7 +172,7 @@
         if self._tree is not None:
             raise AssertionError('You cannot start a new series while a'
                                  ' series is already going.')
-        self._tree = memorytree.MemoryTree.create_on_branch(self._branch)
+        self._tree = self._branch.create_memorytree()
         self._tree.lock_write()
 
     def finish_series(self):
@@ -223,7 +223,7 @@
         if self._tree is not None:
             tree = self._tree
         else:
-            tree = memorytree.MemoryTree.create_on_branch(self._branch)
+            tree = self._branch.create_memorytree()
         tree.lock_write()
         try:
             if parent_ids is not None:

=== modified file 'breezy/builtins.py'
--- breezy/builtins.py	2018-02-18 19:18:40 +0000
+++ breezy/builtins.py	2018-03-25 00:39:16 +0000
@@ -30,6 +30,7 @@
 
 import breezy
 from breezy import (
+    branch as _mod_branch,
     bugtracker,
     bundle,
     cache_utf8,
@@ -52,6 +53,7 @@
     symbol_versioning,
     timestamp,
     transport,
+    tree as _mod_tree,
     ui,
     urlutils,
     views,
@@ -60,7 +62,7 @@
 from breezy.bzr import (
     btree_index,
     )
-from breezy.branch import Branch, UnstackableBranchFormat
+from breezy.branch import Branch
 from breezy.conflicts import ConflictList
 from breezy.transport import memory
 from breezy.smtp_connection import SMTPConnection
@@ -959,11 +961,11 @@
 
         self.add_cleanup(tree.lock_read().unlock)
         if file_list is not None:
-            file_ids = tree.paths2ids(file_list, trees=extra_trees,
-                                      require_versioned=True)
+            paths = tree.find_related_paths_across_trees(
+                    file_list, extra_trees, require_versioned=True)
             # find_ids_across_trees may include some paths that don't
             # exist in 'tree'.
-            entries = tree.iter_entries_by_dir(specific_file_ids=file_ids)
+            entries = tree.iter_entries_by_dir(specific_files=paths)
         else:
             entries = tree.iter_entries_by_dir()
 
@@ -1111,7 +1113,8 @@
         work_tree, file_list = WorkingTree.open_containing_paths(
             names_list, default_directory='.')
         self.add_cleanup(work_tree.lock_tree_write().unlock)
-        rename_map.RenameMap.guess_renames(work_tree, dry_run)
+        rename_map.RenameMap.guess_renames(
+                work_tree.basis_tree(), work_tree, dry_run)
 
     def _run(self, tree, names_list, rel_names, after):
         into_existing = osutils.isdir(names_list[-1])
@@ -1127,10 +1130,9 @@
             else:
                 # 'fix' the case of a potential 'from'
                 from_path = tree.get_canonical_inventory_path(rel_names[0])
-                from_id = tree.path2id(from_path)
                 if (not osutils.lexists(names_list[0]) and
-                    from_id and
-                    tree.stored_kind(from_path, from_id) == "directory"):
+                    tree.is_versioned(from_path) and
+                    tree.stored_kind(from_path) == "directory"):
                     into_existing = False
         # move/rename
         if into_existing:
@@ -1601,7 +1603,7 @@
         try:
             note(gettext('Created new stacked branch referring to %s.') %
                 branch.get_stacked_on_url())
-        except (errors.NotStacked, UnstackableBranchFormat,
+        except (errors.NotStacked, _mod_branch.UnstackableBranchFormat,
             errors.UnstackableRepositoryFormat) as e:
             note(ngettext('Branched %d revision.', 'Branched %d revisions.', branch.revno()) % branch.revno())
         if bind:
@@ -1651,7 +1653,7 @@
                 if name == "":
                     continue
                 active = (active_branch is not None and
-                          active_branch.base == branch.base)
+                          active_branch.user_url == branch.user_url)
                 names[name] = active
             # Only mention the current branch explicitly if it's not
             # one of the colocated branches
@@ -3023,12 +3025,11 @@
     @display_command
     def run(self, filename):
         tree, relpath = WorkingTree.open_containing(filename)
-        file_id = tree.path2id(relpath)
-        b = tree.branch
-        self.add_cleanup(b.lock_read().unlock)
-        touching_revs = log.find_touching_revisions(b, file_id)
-        for revno, revision_id, what in touching_revs:
-            self.outf.write("%6d %s\n" % (revno, what))
+        with tree.lock_read():
+            touching_revs = log.find_touching_revisions(
+                    tree.branch.repository, tree.branch.last_revision(), tree, relpath)
+            for revno, revision_id, what in reversed(list(touching_revs)):
+                self.outf.write("%6d %s\n" % (revno, what))
 
 
 class cmd_ls(Command):
@@ -3476,6 +3477,7 @@
                 raise errors.BzrCommandError(gettext(
                     "{0!r} is not present in revision {1}").format(
                         filename, rev_tree.get_revision_id()))
+        relpath = rev_tree.id2path(actual_file_id)
         if filtered:
             from .filter_tree import ContentFilterTree
             filter_tree = ContentFilterTree(rev_tree,
@@ -4194,8 +4196,8 @@
         # error reporting for past duplicate imports won't have useful
         # backtraces.
         if sys.version_info[0] < 3:
-            # TODO(jelmer): Disable proxying on Python 3, until it's fixed.
-            # pad.lv/1696545
+            # TODO(pad.lv/1696545): Allow proxying on Python 3, since
+            # disallowing it currently leads to failures in many places.
             lazy_import.disallow_proxying()
 
         from . import tests
@@ -4377,7 +4379,7 @@
     committed to record the result of the merge.
 
     merge refuses to run if there are any uncommitted changes, unless
-    --force is given.  If --force is given, then the changes from the source 
+    --force is given.  If --force is given, then the changes from the source
     will be merged with the current working tree, including any uncommitted
     changes in the tree.  The --force option can also be used to create a
     merge revision which has more than two parents.
@@ -4606,6 +4608,15 @@
             raise errors.BzrCommandError(gettext("Cannot do conflict reduction and"
                                          " show base."))
 
+        if (merger.merge_type.requires_file_merge_plan and
+            (not getattr(merger.this_tree, 'plan_file_merge', None) or
+             not getattr(merger.other_tree, 'plan_file_merge', None) or
+             (merger.base_tree is not None and
+                 not getattr(merger.base_tree, 'plan_file_merge', None)))):
+            raise errors.BzrCommandError(
+                 gettext('Plan file merge unsupported: '
+                         'Merge type incompatible with tree formats.'))
+
     def _get_merger_from_branch(self, tree, location, revision, remember,
                                 possible_transports, pb):
         """Produce a merger from a location, assuming it refers to a branch."""
@@ -4763,29 +4774,27 @@
                                          " merges.  Not cherrypicking or"
                                          " multi-merges."))
         repository = tree.branch.repository
-        interesting_ids = None
+        interesting_files = None
         new_conflicts = []
         conflicts = tree.conflicts()
         if file_list is not None:
-            interesting_ids = set()
+            interesting_files = set()
             for filename in file_list:
-                file_id = tree.path2id(filename)
-                if file_id is None:
+                if not tree.is_versioned(filename):
                     raise errors.NotVersionedError(filename)
-                interesting_ids.add(file_id)
-                if tree.kind(filename, file_id) != "directory":
+                interesting_files.add(filename)
+                if tree.kind(filename) != "directory":
                     continue
 
-                # FIXME: Support nested trees
-                for name, ie in tree.root_inventory.iter_entries(file_id):
-                    interesting_ids.add(ie.file_id)
+                for path, ie in tree.iter_entries_by_dir(specific_files=[filename]):
+                    interesting_files.add(path)
             new_conflicts = conflicts.select_conflicts(tree, file_list)[0]
         else:
             # Remerge only supports resolving contents conflicts
             allowed_conflicts = ('text conflict', 'contents conflict')
             restore_files = [c.path for c in conflicts
                              if c.typestring in allowed_conflicts]
-        _mod_merge.transform_tree(tree, tree.basis_tree(), interesting_ids)
+        _mod_merge.transform_tree(tree, tree.basis_tree(), interesting_files)
         tree.set_conflicts(ConflictList(new_conflicts))
         if file_list is not None:
             restore_files = file_list
@@ -4802,7 +4811,7 @@
         tree.set_parent_ids(parents[:1])
         try:
             merger = _mod_merge.Merger.from_revision_ids(tree, parents[1])
-            merger.interesting_ids = interesting_ids
+            merger.interesting_files = interesting_files
             merger.merge_type = merge_type
             merger.show_base = show_base
             merger.reprocess = reprocess
@@ -6762,7 +6771,7 @@
     __doc__ = """Export command helps and error messages in po format."""
 
     hidden = True
-    takes_options = [Option('plugin', 
+    takes_options = [Option('plugin',
                             help='Export help text from named command '\
                                  '(defaults to all built in commands).',
                             type=text_type),

=== modified file 'breezy/bundle/apply_bundle.py'
--- breezy/bundle/apply_bundle.py	2017-11-12 13:53:51 +0000
+++ breezy/bundle/apply_bundle.py	2018-03-01 01:41:45 +0000
@@ -32,9 +32,7 @@
     custom_install = getattr(bundle_reader, 'install', None)
     if custom_install is not None:
         return custom_install(repository)
-    pb = ui.ui_factory.nested_progress_bar()
-    repository.lock_write()
-    try:
+    with repository.lock_write(), ui.ui_factory.nested_progress_bar() as pb:
         real_revisions = bundle_reader.real_revisions
         for i, revision in enumerate(reversed(real_revisions)):
             pb.update(gettext("Install revisions"), i, len(real_revisions))
@@ -43,9 +41,6 @@
             cset_tree = bundle_reader.revision_tree(repository,
                                                        revision.revision_id)
             install_revision(repository, revision, cset_tree)
-    finally:
-        repository.unlock()
-        pb.finished()
 
 
 def merge_bundle(reader, tree, check_clean, merge_type,

=== modified file 'breezy/bundle/serializer/v08.py'
--- breezy/bundle/serializer/v08.py	2017-11-13 22:07:48 +0000
+++ breezy/bundle/serializer/v08.py	2018-02-24 15:50:23 +0000
@@ -117,11 +117,8 @@
         self.check_compatible()
         with source.lock_read():
             self._write_main_header()
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
+            with ui.ui_factory.nested_progress_bar() as pb:
                 self._write_revisions(pb)
-            finally:
-                pb.finished()
 
     def write_bundle(self, repository, target, base, fileobj):
         return self._write_bundle(repository, target, base, fileobj)

=== modified file 'breezy/bundle/serializer/v4.py'
--- breezy/bundle/serializer/v4.py	2017-08-26 13:58:53 +0000
+++ breezy/bundle/serializer/v4.py	2018-02-24 15:50:23 +0000
@@ -675,8 +675,7 @@
         # inventory deltas to apply rather than calling add_inventory from
         # scratch each time.
         inventory_cache = lru_cache.LRUCache(10)
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             num_records = len(records)
             for idx, (key, metadata, bytes) in enumerate(records):
                 pb.update('installing inventory', idx, num_records)
@@ -717,8 +716,6 @@
                 except errors.UnsupportedInventoryKind:
                     raise errors.IncompatibleRevision(repr(self._repository))
                 inventory_cache[revision_id] = target_inv
-        finally:
-            pb.finished()
 
     def _handle_root(self, target_inv, parent_ids):
         revision_id = target_inv.revision_id

=== modified file 'breezy/bzr/branch.py'
--- breezy/bzr/branch.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/branch.py	2018-03-25 00:39:16 +0000
@@ -97,6 +97,7 @@
         self.repository = _repository
         self.conf_store = None
         Branch.__init__(self, possible_transports)
+        self._tags_bytes = None
 
     def __str__(self):
         return '%s(%s)' % (self.__class__.__name__, self.user_url)
@@ -402,6 +403,36 @@
             self._format._update_feature_flags(updated_flags)
             self.control_transport.put_bytes('format', self._format.as_string())
 
+    def _get_tags_bytes(self):
+        """Get the bytes of a serialised tags dict.
+
+        Note that not all branches support tags, nor do all use the same tags
+        logic: this method is specific to BasicTags. Other tag implementations
+        may use the same method name and behave differently, safely, because
+        of the double-dispatch via
+        format.make_tags->tags_instance->get_tags_dict.
+
+        :return: The bytes of the tags file.
+        :seealso: Branch._set_tags_bytes.
+        """
+        with self.lock_read():
+            if self._tags_bytes is None:
+                self._tags_bytes = self._transport.get_bytes('tags')
+            return self._tags_bytes
+
+    def _set_tags_bytes(self, bytes):
+        """Mirror method for _get_tags_bytes.
+
+        :seealso: Branch._get_tags_bytes.
+        """
+        with self.lock_write():
+            self._tags_bytes = bytes
+            return self._transport.put_bytes('tags', bytes)
+
+    def _clear_cached_state(self):
+        super(BzrBranch, self)._clear_cached_state()
+        self._tags_bytes = None
+
 
 class BzrBranch8(BzrBranch):
     """A branch that stores tree-reference locations."""

=== modified file 'breezy/bzr/bzrdir.py'
--- breezy/bzr/bzrdir.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/bzrdir.py	2018-03-25 00:39:16 +0000
@@ -495,8 +495,7 @@
         :return: Tuple with old path name and new path name
         """
 
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             old_path = self.root_transport.abspath('.bzr')
             backup_dir = self._available_backup_name('backup.bzr')
             new_path = self.root_transport.abspath(backup_dir)
@@ -505,8 +504,6 @@
                 urlutils.unescape_for_display(new_path, 'utf-8')))
             self.root_transport.copy_tree('.bzr', backup_dir)
             return (old_path, new_path)
-        finally:
-            pb.finished()
 
     def retire_bzrdir(self, limit=10000):
         """Permanently disable the bzrdir.

=== modified file 'breezy/bzr/groupcompress_repo.py'
--- breezy/bzr/groupcompress_repo.py	2018-02-17 00:06:35 +0000
+++ breezy/bzr/groupcompress_repo.py	2018-03-24 17:48:04 +0000
@@ -402,15 +402,12 @@
                      pb_offset):
         trace.mutter('repacking %d %s', len(keys), message)
         self.pb.update('repacking %s' % (message,), pb_offset)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             stream = vf_to_stream(source_vf, keys, message, child_pb)
             for _ in target_vf._insert_record_stream(stream,
                                                      random_id=True,
                                                      reuse_blocks=False):
                 pass
-        finally:
-            child_pb.finished()
 
     def _copy_revision_texts(self):
         source_vf, target_vf = self._build_vfs('revision', True, False)
@@ -458,16 +455,13 @@
                      len(self._chk_id_roots), len(self._chk_p_id_roots),
                      len(total_keys))
         self.pb.update('repacking chk', 3)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             for stream in self._get_chk_streams(source_vf, total_keys,
                                                 pb=child_pb):
                 for _ in target_vf._insert_record_stream(stream,
                                                          random_id=True,
                                                          reuse_blocks=False):
                     pass
-        finally:
-            child_pb.finished()
 
     def _copy_text_texts(self):
         source_vf, target_vf = self._build_vfs('text', True, True)
@@ -619,11 +613,8 @@
         This is useful to get the side-effects of generating a stream.
         """
         self.pb.update('scanning %s' % (message,), pb_offset)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             list(vf_to_stream(source_vf, keys, message, child_pb))
-        finally:
-            child_pb.finished()
 
     def _copy_inventory_texts(self):
         source_vf, target_vf = self._build_vfs('inventory', True, True)
@@ -1025,8 +1016,7 @@
         rich_root = self.supports_rich_root()
         bytes_to_info = inventory.CHKInventory._bytes_to_utf8name_key
         file_id_revisions = {}
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             revision_keys = [(r,) for r in revision_ids]
             parent_keys = self._find_parent_keys_of_revisions(revision_keys)
             # TODO: instead of using _find_present_inventory_keys, change the
@@ -1058,8 +1048,6 @@
                         file_id_revisions[file_id].add(revision_id)
                     except KeyError:
                         file_id_revisions[file_id] = {revision_id}
-        finally:
-            pb.finished()
         return file_id_revisions
 
     def find_text_key_references(self):
@@ -1077,8 +1065,7 @@
         revision_keys = self.revisions.keys()
         result = {}
         rich_roots = self.supports_rich_root()
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             all_revs = self.all_revision_ids()
             total = len(all_revs)
             for pos, inv in enumerate(self.iter_inventories(all_revs)):
@@ -1091,8 +1078,6 @@
                     if entry.revision == inv.revision_id:
                         result[key] = True
             return result
-        finally:
-            pb.finished()
 
     def reconcile_canonicalize_chks(self):
         """Reconcile this repository to make sure all CHKs are in canonical

=== modified file 'breezy/bzr/inventory.py'
--- breezy/bzr/inventory.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/inventory.py	2018-03-25 00:39:16 +0000
@@ -178,8 +178,11 @@
         candidates = {}
         # identify candidate head revision ids.
         for inv in previous_inventories:
-            if inv.has_id(self.file_id):
+            try:
                 ie = inv[self.file_id]
+            except errors.NoSuchId:
+                pass
+            else:
                 if ie.revision in candidates:
                     # same revision value in two different inventories:
                     # correct possible inconsistencies:
@@ -717,12 +720,12 @@
 
     def _preload_cache(self):
         """Populate any caches, we are about to access all items.
-        
+
         The default implementation does nothing, because CommonInventory doesn't
         have a cache.
         """
         pass
-    
+
     def iter_entries_by_dir(self, from_dir=None, specific_file_ids=None,
         yield_parents=False):
         """Iterate over the entries in a directory first order.

=== modified file 'breezy/bzr/inventorytree.py'
--- breezy/bzr/inventorytree.py	2018-02-17 02:02:38 +0000
+++ breezy/bzr/inventorytree.py	2018-03-24 23:49:53 +0000
@@ -77,7 +77,8 @@
         adjusted to account for existing elements that match case
         insensitively.
         """
-        return list(self._yield_canonical_inventory_paths(paths))
+        with self.lock_read():
+            return list(self._yield_canonical_inventory_paths(paths))
 
     def get_canonical_inventory_path(self, path):
         """Returns the first inventory item that case-insensitively matches path.
@@ -97,7 +98,8 @@
         :return: The input path adjusted to account for existing elements
         that match case insensitively.
         """
-        return next(self._yield_canonical_inventory_paths([path]))
+        with self.lock_read():
+            return next(self._yield_canonical_inventory_paths([path]))
 
     def _yield_canonical_inventory_paths(self, paths):
         for path in paths:
@@ -112,31 +114,34 @@
             for elt in bit_iter:
                 lelt = elt.lower()
                 new_path = None
-                for child in self.iter_children(cur_id):
-                    try:
-                        # XXX: it seem like if the child is known to be in the
-                        # tree, we shouldn't need to go from its id back to
-                        # its path -- mbp 2010-02-11
-                        #
-                        # XXX: it seems like we could be more efficient
-                        # by just directly looking up the original name and
-                        # only then searching all children; also by not
-                        # chopping paths so much. -- mbp 2010-02-11
-                        child_base = os.path.basename(self.id2path(child))
-                        if (child_base == elt):
-                            # if we found an exact match, we can stop now; if
-                            # we found an approximate match we need to keep
-                            # searching because there might be an exact match
-                            # later.  
-                            cur_id = child
-                            new_path = osutils.pathjoin(cur_path, child_base)
-                            break
-                        elif child_base.lower() == lelt:
-                            cur_id = child
-                            new_path = osutils.pathjoin(cur_path, child_base)
-                    except errors.NoSuchId:
-                        # before a change is committed we can see this error...
-                        continue
+                try:
+                    for child in self.iter_child_entries(self.id2path(cur_id), cur_id):
+                        try:
+                            # XXX: it seem like if the child is known to be in the
+                            # tree, we shouldn't need to go from its id back to
+                            # its path -- mbp 2010-02-11
+                            #
+                            # XXX: it seems like we could be more efficient
+                            # by just directly looking up the original name and
+                            # only then searching all children; also by not
+                            # chopping paths so much. -- mbp 2010-02-11
+                            child_base = os.path.basename(self.id2path(child.file_id))
+                            if (child_base == elt):
+                                # if we found an exact match, we can stop now; if
+                                # we found an approximate match we need to keep
+                                # searching because there might be an exact match
+                                # later.  
+                                cur_id = child.file_id
+                                new_path = osutils.pathjoin(cur_path, child_base)
+                                break
+                            elif child_base.lower() == lelt:
+                                cur_id = child.file_id
+                                new_path = osutils.pathjoin(cur_path, child_base)
+                        except errors.NoSuchId:
+                            # before a change is committed we can see this error...
+                            continue
+                except errors.NotADirectory:
+                    pass
                 if new_path:
                     cur_path = new_path
                 else:
@@ -165,6 +170,53 @@
             file_id = file_id[0]
         return self.root_inventory, file_id
 
+    def find_related_paths_across_trees(self, paths, trees=[],
+            require_versioned=True):
+        """Find related paths in tree corresponding to specified filenames in any
+        of `lookup_trees`.
+
+        All matches in all trees will be used, and all children of matched
+        directories will be used.
+
+        :param paths: The filenames to find related paths for (if None, returns
+            None)
+        :param trees: The trees to find file_ids within
+        :param require_versioned: if true, all specified filenames must occur in
+            at least one tree.
+        :return: a set of paths for the specified filenames and their children
+            in `tree`
+        """
+        if paths is None:
+            return None;
+        file_ids = self.paths2ids(
+                paths, trees, require_versioned=require_versioned)
+        ret = set()
+        for file_id in file_ids:
+            try:
+                ret.add(self.id2path(file_id))
+            except errors.NoSuchId:
+                pass
+        return ret
+
+    def paths2ids(self, paths, trees=[], require_versioned=True):
+        """Return all the ids that can be reached by walking from paths.
+
+        Each path is looked up in this tree and any extras provided in
+        trees, and this is repeated recursively: the children in an extra tree
+        of a directory that has been renamed under a provided path in this tree
+        are all returned, even if none exist under a provided path in this
+        tree, and vice versa.
+
+        :param paths: An iterable of paths to start converting to ids from.
+            Alternatively, if paths is None, no ids should be calculated and None
+            will be returned. This is offered to make calling the api unconditional
+            for code that *might* take a list of files.
+        :param trees: Additional trees to consider.
+        :param require_versioned: If False, do not raise NotVersionedError if
+            an element of paths is not versioned in this tree and all of trees.
+        """
+        return find_ids_across_trees(paths, [self] + list(trees), require_versioned)
+
     def path2id(self, path):
         """Return the id for path in this tree."""
         with self.lock_read():
@@ -217,7 +269,7 @@
         # are not versioned.
         return set((p for p in paths if self.path2id(p) is None))
 
-    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+    def iter_entries_by_dir(self, specific_files=None, yield_parents=False):
         """Walk the tree in 'by_dir' order.
 
         This will yield each entry in the tree as a (path, entry) tuple.
@@ -226,20 +278,20 @@
         See Tree.iter_entries_by_dir for details.
 
         :param yield_parents: If True, yield the parents from the root leading
-            down to specific_file_ids that have been requested. This has no
-            impact if specific_file_ids is None.
+            down to specific_files that have been requested. This has no
+            impact if specific_files is None.
         """
         with self.lock_read():
-            if specific_file_ids is None:
-                inventory_file_ids = None
-            else:
+            if specific_files is not None:
                 inventory_file_ids = []
-                for tree_file_id in specific_file_ids:
-                    inventory, inv_file_id = self._unpack_file_id(tree_file_id)
+                for path in specific_files:
+                    inventory, inv_file_id = self._path2inv_file_id(path)
                     if not inventory is self.root_inventory: # for now
                         raise AssertionError("%r != %r" % (
                             inventory, self.root_inventory))
                     inventory_file_ids.append(inv_file_id)
+            else:
+                inventory_file_ids = None
             # FIXME: Handle nested trees
             return self.root_inventory.iter_entries_by_dir(
                 specific_file_ids=inventory_file_ids, yield_parents=yield_parents)
@@ -248,15 +300,148 @@
         with self.lock_read():
             inv, inv_file_id = self._path2inv_file_id(path, file_id)
             try:
-                return iter(viewvalues(inv[inv_file_id].children))
+                ie = inv[inv_file_id]
             except errors.NoSuchId:
                 raise errors.NoSuchFile(path)
-
-    def iter_children(self, file_id, path=None):
-        """See Tree.iter_children."""
-        entry = next(self.iter_entries_by_dir([file_id]))[1]
-        for child in viewvalues(getattr(entry, 'children', {})):
-            yield child.file_id
+            else:
+                if ie.kind != 'directory':
+                    raise errors.NotADirectory(path)
+                return iter(viewvalues(ie.children))
+
+    def _get_plan_merge_data(self, file_id, other, base):
+        from . import versionedfile
+        vf = versionedfile._PlanMergeVersionedFile(file_id)
+        last_revision_a = self._get_file_revision(
+                self.id2path(file_id), file_id, vf, 'this:')
+        last_revision_b = other._get_file_revision(
+                other.id2path(file_id), file_id, vf, 'other:')
+        if base is None:
+            last_revision_base = None
+        else:
+            last_revision_base = base._get_file_revision(
+                    base.id2path(file_id), file_id, vf, 'base:')
+        return vf, last_revision_a, last_revision_b, last_revision_base
+
+    def plan_file_merge(self, file_id, other, base=None):
+        """Generate a merge plan based on annotations.
+
+        If the file contains uncommitted changes in this tree, they will be
+        attributed to the 'current:' pseudo-revision.  If the file contains
+        uncommitted changes in the other tree, they will be assigned to the
+        'other:' pseudo-revision.
+        """
+        data = self._get_plan_merge_data(file_id, other, base)
+        vf, last_revision_a, last_revision_b, last_revision_base = data
+        return vf.plan_merge(last_revision_a, last_revision_b,
+                             last_revision_base)
+
+    def plan_file_lca_merge(self, file_id, other, base=None):
+        """Generate a merge plan based lca-newness.
+
+        If the file contains uncommitted changes in this tree, they will be
+        attributed to the 'current:' pseudo-revision.  If the file contains
+        uncommitted changes in the other tree, they will be assigned to the
+        'other:' pseudo-revision.
+        """
+        data = self._get_plan_merge_data(file_id, other, base)
+        vf, last_revision_a, last_revision_b, last_revision_base = data
+        return vf.plan_lca_merge(last_revision_a, last_revision_b,
+                                 last_revision_base)
+
+    def _get_file_revision(self, path, file_id, vf, tree_revision):
+        """Ensure that file_id, tree_revision is in vf to plan the merge."""
+        if getattr(self, '_repository', None) is None:
+            last_revision = tree_revision
+            parent_keys = [(file_id, t.get_file_revision(path, file_id)) for t in
+                self._iter_parent_trees()]
+            vf.add_lines((file_id, last_revision), parent_keys,
+                         self.get_file_lines(path, file_id))
+            repo = self.branch.repository
+            base_vf = repo.texts
+        else:
+            last_revision = self.get_file_revision(path, file_id)
+            base_vf = self._repository.texts
+        if base_vf not in vf.fallback_versionedfiles:
+            vf.fallback_versionedfiles.append(base_vf)
+        return last_revision
+
+
+def find_ids_across_trees(filenames, trees, require_versioned=True):
+    """Find the ids corresponding to specified filenames.
+
+    All matches in all trees will be used, and all children of matched
+    directories will be used.
+
+    :param filenames: The filenames to find file_ids for (if None, returns
+        None)
+    :param trees: The trees to find file_ids within
+    :param require_versioned: if true, all specified filenames must occur in
+        at least one tree.
+    :return: a set of file ids for the specified filenames and their children.
+    """
+    if not filenames:
+        return None
+    specified_path_ids = _find_ids_across_trees(filenames, trees,
+        require_versioned)
+    return _find_children_across_trees(specified_path_ids, trees)
+
+
+def _find_ids_across_trees(filenames, trees, require_versioned):
+    """Find the ids corresponding to specified filenames.
+
+    All matches in all trees will be used, but subdirectories are not scanned.
+
+    :param filenames: The filenames to find file_ids for
+    :param trees: The trees to find file_ids within
+    :param require_versioned: if true, all specified filenames must occur in
+        at least one tree.
+    :return: a set of file ids for the specified filenames
+    """
+    not_versioned = []
+    interesting_ids = set()
+    for tree_path in filenames:
+        not_found = True
+        for tree in trees:
+            file_id = tree.path2id(tree_path)
+            if file_id is not None:
+                interesting_ids.add(file_id)
+                not_found = False
+        if not_found:
+            not_versioned.append(tree_path)
+    if len(not_versioned) > 0 and require_versioned:
+        raise errors.PathsNotVersionedError(not_versioned)
+    return interesting_ids
+
+
+def _find_children_across_trees(specified_ids, trees):
+    """Return a set including specified ids and their children.
+
+    All matches in all trees will be used.
+
+    :param trees: The trees to find file_ids within
+    :return: a set containing all specified ids and their children
+    """
+    interesting_ids = set(specified_ids)
+    pending = interesting_ids
+    # now handle children of interesting ids
+    # we loop so that we handle all children of each id in both trees
+    while len(pending) > 0:
+        new_pending = set()
+        for file_id in pending:
+            for tree in trees:
+                try:
+                    path = tree.id2path(file_id)
+                except errors.NoSuchId:
+                    continue
+                try:
+                    for child in tree.iter_child_entries(path, file_id):
+                        if child.file_id not in interesting_ids:
+                            new_pending.add(child.file_id)
+                except errors.NotADirectory:
+                    pass
+        interesting_ids.update(new_pending)
+        pending = new_pending
+    return interesting_ids
 
 
 class MutableInventoryTree(MutableTree, InventoryTree):
@@ -382,10 +567,11 @@
             return entry[3]
         # Find a 'best fit' match if the filesystem is case-insensitive
         inv_path = self.tree._fix_case_of_inventory_path(inv_path)
-        file_id = self.tree.path2id(inv_path)
-        if file_id is not None:
-            return next(self.tree.iter_entries_by_dir([file_id]))[1]
-        return None
+        try:
+            return next(self.tree.iter_entries_by_dir(
+                    specific_files=[inv_path]))[1]
+        except StopIteration:
+            return None
 
     def _convert_to_directory(self, this_ie, inv_path):
         """Convert an entry to a directory.
@@ -408,9 +594,9 @@
         :param parent_ie: Parent inventory entry if known, or None.  If
             None, the parent is looked up by name and used if present, otherwise it
             is recursively added.
-        :param path: 
+        :param path: Filesystem path to add
         :param kind: Kind of new entry (file, directory, etc)
-        :param inv_path:
+        :param inv_path: Inventory path
         :return: Inventory entry for path and a list of paths which have been added.
         """
         # Nothing to do if path is already versioned.
@@ -643,7 +829,10 @@
 
     def get_file_revision(self, path, file_id=None):
         inv, inv_file_id = self._path2inv_file_id(path, file_id)
-        ie = inv[inv_file_id]
+        try:
+            ie = inv[inv_file_id]
+        except errors.NoSuchId:
+            raise errors.NoSuchFile(path)
         return ie.revision
 
     def is_executable(self, path, file_id=None):
@@ -689,7 +878,10 @@
 
     def kind(self, path, file_id=None):
         inv, inv_file_id = self._path2inv_file_id(path, file_id)
-        return inv[inv_file_id].kind
+        try:
+            return inv[inv_file_id].kind
+        except errors.NoSuchId:
+            raise errors.NoSuchFile(path)
 
     def path_content_summary(self, path):
         """See Tree.path_content_summary."""
@@ -710,9 +902,6 @@
             return None, False, None
         return entry.kind, entry.executable, None
 
-    def _file_size(self, entry, stat_value):
-        return entry.text_size
-
     def walkdirs(self, prefix=""):
         _directory = 'directory'
         inv, top_id = self._path2inv_file_id(prefix)
@@ -745,7 +934,7 @@
         """See Tree.iter_files_bytes.
 
         This version is implemented on top of Repository.iter_files_bytes"""
-        repo_desired_files = [(f, self.get_file_revision(self.id2path(f), f), i)
+        repo_desired_files = [(self.path2id(f), self.get_file_revision(f), i)
                               for f, i in desired_files]
         try:
             for result in self._repository.iter_files_bytes(repo_desired_files):

=== modified file 'breezy/bzr/knitpack_repo.py'
--- breezy/bzr/knitpack_repo.py	2018-02-17 00:06:35 +0000
+++ breezy/bzr/knitpack_repo.py	2018-03-24 17:48:04 +0000
@@ -620,12 +620,9 @@
 
         :param output_lines: Output full texts of copied items.
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             return self._do_copy_nodes(nodes, index_map, writer,
                 write_index, pb, output_lines=output_lines)
-        finally:
-            pb.finished()
 
     def _do_copy_nodes(self, nodes, index_map, writer, write_index, pb,
         output_lines=None):
@@ -685,13 +682,10 @@
         :param output_lines: Return lines present in the copied data as
             an iterator of line,version_id.
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             for result in self._do_copy_nodes_graph(index_map, writer,
                 write_index, output_lines, pb, readv_group_iter, total_items):
                 yield result
-        finally:
-            pb.finished()
 
     def _do_copy_nodes_graph(self, index_map, writer, write_index,
         output_lines, pb, readv_group_iter, total_items):

=== modified file 'breezy/bzr/remote.py'
--- breezy/bzr/remote.py	2017-11-23 20:02:55 +0000
+++ breezy/bzr/remote.py	2018-03-24 03:35:53 +0000
@@ -2692,7 +2692,7 @@
 
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
         with self.lock_write():
-            signature = gpg_strategy.sign(plaintext)
+            signature = gpg_strategy.sign(plaintext, gpg.MODE_CLEAR)
             self.add_signature_text(revision_id, signature)
 
     def add_signature_text(self, revision_id, signature):
@@ -2737,9 +2737,11 @@
             signature = self.get_signature_text(revision_id)
 
             testament = _mod_testament.Testament.from_revision(self, revision_id)
-            plaintext = testament.as_short_text()
 
-            return gpg_strategy.verify(signature, plaintext)
+            (status, key, signed_plaintext) = gpg_strategy.verify(signature)
+            if testament.as_short_text() != signed_plaintext:
+                return gpg.SIGNATURE_NOT_VALID, None
+            return (status, key)
 
     def item_keys_introduced_by(self, revision_ids, _files_pb=None):
         self._ensure_real()
@@ -3439,6 +3441,7 @@
 
     def _clear_cached_state(self):
         super(RemoteBranch, self)._clear_cached_state()
+        self._tags_bytes = None
         if self._real_branch is not None:
             self._real_branch._clear_cached_state()
 

=== modified file 'breezy/bzr/smart/repository.py'
--- breezy/bzr/smart/repository.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/smart/repository.py	2018-03-25 00:39:16 +0000
@@ -732,20 +732,19 @@
                 yield record
 
         self.seed_state()
-        pb = ui.ui_factory.nested_progress_bar()
-        rc = self._record_counter
-        try:
-            # Make and consume sub generators, one per substream type:
-            while self.first_bytes is not None:
-                substream = NetworkRecordStream(self.iter_substream_bytes())
-                # after substream is fully consumed, self.current_type is set
-                # to the next type, and self.first_bytes is set to the matching
-                # bytes.
-                yield self.current_type, wrap_and_count(pb, rc, substream)
-        finally:
-            if rc:
-                pb.update('Done', rc.max, rc.max)
-            pb.finished()
+        with ui.ui_factory.nested_progress_bar() as pb:
+            rc = self._record_counter
+            try:
+                # Make and consume sub generators, one per substream type:
+                while self.first_bytes is not None:
+                    substream = NetworkRecordStream(self.iter_substream_bytes())
+                    # after substream is fully consumed, self.current_type is set
+                    # to the next type, and self.first_bytes is set to the matching
+                    # bytes.
+                    yield self.current_type, wrap_and_count(pb, rc, substream)
+            finally:
+                if rc:
+                    pb.update('Done', rc.max, rc.max)
 
     def seed_state(self):
         """Prepare the _ByteStreamDecoder to decode from the pack stream."""

=== modified file 'breezy/bzr/versionedfile.py'
--- breezy/bzr/versionedfile.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/versionedfile.py	2018-03-25 00:39:16 +0000
@@ -669,7 +669,7 @@
         """
         raise NotImplementedError(self.iter_lines_added_or_present_in_versions)
 
-    def plan_merge(self, ver_a, ver_b):
+    def plan_merge(self, ver_a, ver_b, base=None):
         """Return pseudo-annotation indicating how the two versions merge.
 
         This is computed between versions a and b and their common

=== modified file 'breezy/bzr/vf_repository.py'
--- breezy/bzr/vf_repository.py	2018-02-17 02:02:38 +0000
+++ breezy/bzr/vf_repository.py	2018-03-24 23:49:53 +0000
@@ -786,11 +786,8 @@
         This is responsible for verifying the sha1 of inventories and
         creating a pending_keys set that covers data referenced by inventories.
         """
-        bar = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as bar:
             self._do_check_inventories(checker, bar)
-        finally:
-            bar.finished()
 
     def _do_check_inventories(self, checker, bar):
         """Helper for _check_inventories."""
@@ -1143,12 +1140,9 @@
         """
         revision_keys = self.revisions.keys()
         w = self.inventories
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             return self._serializer._find_text_key_references(
                 w.iter_lines_added_or_present_in_keys(revision_keys, pb=pb))
-        finally:
-            pb.finished()
 
     def _inventory_xml_lines_for_keys(self, keys):
         """Get a line iterator of the sort needed for findind references.
@@ -1270,12 +1264,9 @@
             ancestors = graph.get_parent_map(self.all_revision_ids())
         if text_key_references is None:
             text_key_references = self.find_text_key_references()
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             return self._do_generate_text_key_index(ancestors,
                 text_key_references, pb)
-        finally:
-            pb.finished()
 
     def _do_generate_text_key_index(self, ancestors, text_key_references, pb):
         """Helper for _generate_text_key_index to avoid deep nesting."""
@@ -2760,11 +2751,8 @@
             # Walk though all revisions; get inventory deltas, copy referenced
             # texts that delta references, insert the delta, revision and
             # signature.
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
+            with ui.ui_factory.nested_progress_bar() as pb:
                 self._fetch_all_revisions(revision_ids, pb)
-            finally:
-                pb.finished()
             return len(revision_ids), 0
 
     def _get_basis(self, first_revision_id):

=== modified file 'breezy/bzr/workingtree.py'
--- breezy/bzr/workingtree.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/workingtree.py	2018-03-25 00:39:16 +0000
@@ -367,7 +367,7 @@
             for parent_info, file_infos in self.walkdirs(directory):
                 for relpath, basename, kind, lstat, fileid, kind in file_infos:
                     # Is it versioned or ignored?
-                    if self.path2id(relpath):
+                    if self.is_versioned(relpath):
                         # Add nested content for deletion.
                         all_files.add(relpath)
                     else:
@@ -566,8 +566,8 @@
         ignore_globs = set()
         ignore_globs.update(ignores.get_runtime_ignores())
         ignore_globs.update(ignores.get_user_ignores())
-        if self.has_filename(breezy.IGNORE_FILENAME):
-            with self.get_file(breezy.IGNORE_FILENAME) as f:
+        if self.has_filename(self._format.ignore_filename):
+            with self.get_file(self._format.ignore_filename)
                 ignore_globs.update(ignores.parse_ignore_file(f))
         self._ignoreset = ignore_globs
         return ignore_globs
@@ -643,10 +643,7 @@
         file_ids are in a WorkingTree if they are in the working inventory
         and the working file exists.
         """
-        ret = set()
-        for path, ie in self.iter_entries_by_dir():
-            ret.add(ie.file_id)
-        return ret
+        return {ie.file_id for path, ie in self.iter_entries_by_dir()}
 
     def all_versioned_paths(self):
         return {path for path, ie in self.iter_entries_by_dir()}
@@ -819,7 +816,7 @@
 
                     try:
                         kind = parent_tree.kind(path, file_id)
-                    except errors.NoSuchId:
+                    except errors.NoSuchFile:
                         continue
                     if kind != 'file':
                         # Note: this is slightly unnecessary, because symlinks and
@@ -832,7 +829,7 @@
                         parent_tree.get_file_revision(parent_path, file_id))
                     if parent_text_key not in maybe_file_parent_keys:
                         maybe_file_parent_keys.append(parent_text_key)
-            graph = _mod_graph.Graph(self.branch.repository.texts)
+            graph = self.branch.repository.get_file_graph()
             heads = graph.heads(maybe_file_parent_keys)
             file_parent_keys = []
             for key in maybe_file_parent_keys:
@@ -842,7 +839,7 @@
             # Now we have the parents of this content
             annotator = self.branch.repository.texts.get_annotator()
             text = self.get_file_text(path, file_id)
-            this_key =(file_id, default_revision)
+            this_key = (file_id, default_revision)
             annotator.add_special_text(this_key, file_parent_keys, text)
             annotations = [(key[-1], line)
                            for key, line in annotator.annotate_flat(this_key)]
@@ -1690,13 +1687,13 @@
                 blocked_parent_ids.add(ie.file_id)
             yield path, ie
 
-    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+    def iter_entries_by_dir(self, specific_files=None,
+                            yield_parents=False):
         """See Tree.iter_entries_by_dir()"""
         # The only trick here is that if we supports_tree_reference then we
         # need to detect if a directory becomes a tree-reference.
         iterator = super(WorkingTree, self).iter_entries_by_dir(
-                specific_file_ids=specific_file_ids,
-                yield_parents=yield_parents)
+                specific_files=specific_files, yield_parents=yield_parents)
         if not self.supports_tree_reference():
             return iterator
         else:
@@ -1706,45 +1703,7 @@
 class WorkingTreeFormatMetaDir(bzrdir.BzrFormat, WorkingTreeFormat):
     """Base class for working trees that live in bzr meta directories."""
 
-    def __init__(self):
-        WorkingTreeFormat.__init__(self)
-        bzrdir.BzrFormat.__init__(self)
-
-    @classmethod
-    def find_format_string(klass, controldir):
-        """Return format name for the working tree object in controldir."""
-        try:
-            transport = controldir.get_workingtree_transport(None)
-            return transport.get_bytes("format")
-        except errors.NoSuchFile:
-            raise errors.NoWorkingTree(base=transport.base)
-
-    @classmethod
-    def find_format(klass, controldir):
-        """Return the format for the working tree object in controldir."""
-        format_string = klass.find_format_string(controldir)
-        return klass._find_format(format_registry, 'working tree',
-                format_string)
-
-    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
-            basedir=None):
-        WorkingTreeFormat.check_support_status(self,
-            allow_unsupported=allow_unsupported, recommend_upgrade=recommend_upgrade,
-            basedir=basedir)
-        bzrdir.BzrFormat.check_support_status(self, allow_unsupported=allow_unsupported,
-            recommend_upgrade=recommend_upgrade, basedir=basedir)
-
-    def get_controldir_for_branch(self):
-        """Get the control directory format for creating branches.
-
-        This is to support testing of working tree formats that can not exist
-        in the same control directory as a branch.
-        """
-        return self._matchingcontroldir
-
-
-class WorkingTreeFormatMetaDir(bzrdir.BzrFormat, WorkingTreeFormat):
-    """Base class for working trees that live in bzr meta directories."""
+    ignore_filename = '.bzrignore'
 
     def __init__(self):
         WorkingTreeFormat.__init__(self)

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2018-02-18 19:18:40 +0000
+++ breezy/bzr/workingtree_4.py	2018-03-25 00:39:16 +0000
@@ -1726,9 +1726,6 @@
         # sensible: the entry might not have come from us?
         return entry.kind, entry.executable, None
 
-    def _file_size(self, entry, stat_value):
-        return entry.text_size
-
     def filter_unversioned_files(self, paths):
         """Filter out paths that are not versioned.
 
@@ -1896,10 +1893,8 @@
         return inv[inv_file_id].text_size
 
     def get_file_text(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
         content = None
-        for _, content_iter in self.iter_files_bytes([(file_id, None)]):
+        for _, content_iter in self.iter_files_bytes([(path, None)]):
             if content is not None:
                 raise AssertionError('iter_files_bytes returned'
                     ' too many entries')
@@ -1921,11 +1916,11 @@
         This version is implemented on top of Repository.iter_files_bytes"""
         parent_index = self._get_parent_index()
         repo_desired_files = []
-        for file_id, identifier in desired_files:
-            entry = self._get_entry(file_id)
+        for path, identifier in desired_files:
+            entry = self._get_entry(path=path)
             if entry == (None, None):
-                raise errors.NoSuchId(self, file_id)
-            repo_desired_files.append((file_id, entry[1][parent_index][4],
+                raise errors.NoSuchFile(self, path)
+            repo_desired_files.append((entry[0][2], entry[1][parent_index][4],
                                        identifier))
         return self._repository.iter_files_bytes(repo_desired_files)
 
@@ -1965,7 +1960,7 @@
     def kind(self, path, file_id=None):
         entry = self._get_entry(file_id=file_id, path=path)[1]
         if entry is None:
-            raise errors.NoSuchId(tree=self, file_id=file_id)
+            raise errors.NoSuchFile(path)
         parent_index = self._get_parent_index()
         return dirstate.DirState._minikind_to_kind[entry[parent_index][0]]
 

=== modified file 'breezy/check.py'
--- breezy/check.py	2017-06-22 01:06:22 +0000
+++ breezy/check.py	2018-02-24 15:50:23 +0000
@@ -105,9 +105,7 @@
     def check(self, callback_refs=None, check_repo=True):
         if callback_refs is None:
             callback_refs = {}
-        self.repository.lock_read()
-        self.progress = ui.ui_factory.nested_progress_bar()
-        try:
+        with self.repository.lock_read(), ui.ui_factory.nested_progress_bar() as self.progress:
             self.progress.update(gettext('check'), 0, 4)
             if self.check_repo:
                 self.progress.update(gettext('checking revisions'), 0)
@@ -160,9 +158,6 @@
                         item._check(refs)
                     if isinstance(item, Branch):
                         self.other_results.append(item.check(refs))
-        finally:
-            self.progress.finished()
-            self.repository.unlock()
 
     def _check_revisions(self, revisions_iterator):
         """Check revision objects by decorating a generator.
@@ -307,11 +302,8 @@
         """Check all the weaves we can get our hands on.
         """
         weave_ids = []
-        storebar = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as storebar:
             self._check_weaves(storebar)
-        finally:
-            storebar.finished()
 
     def _check_weaves(self, storebar):
         storebar.update('text-index', 0, 2)

=== modified file 'breezy/commit.py'
--- breezy/commit.py	2017-12-03 00:46:19 +0000
+++ breezy/commit.py	2018-03-09 19:20:41 +0000
@@ -460,9 +460,7 @@
         # Make the working tree be up to date with the branch. This
         # includes automatic changes scheduled to be made to the tree, such
         # as updating its basis and unversioning paths that were missing.
-        self.work_tree.unversion(
-                {self.work_tree.id2path(file_id) for file_id in self.deleted_ids},
-                self.deleted_ids)
+        self.work_tree.unversion(self.deleted_paths)
         self._set_progress_stage("Updating the working tree")
         self.work_tree.update_basis_by_delta(self.rev_id,
              self.builder.get_basis_delta())
@@ -694,7 +692,7 @@
         """
         reporter = self.reporter
         report_changes = reporter.is_verbose()
-        deleted_ids = []
+        deleted_paths = []
         for change in iter_changes:
             if report_changes:
                 old_path = change[1][0]
@@ -706,7 +704,7 @@
                 # 'missing' path
                 if report_changes:
                     reporter.missing(new_path)
-                deleted_ids.append(change[0])
+                deleted_paths.append(change[1][1])
                 # Reset the new path (None) and new versioned flag (False)
                 change = (change[0], (change[1][0], None), change[2],
                     (change[3][0], False)) + change[4:]
@@ -731,8 +729,8 @@
                             # repositories.
                             reporter.snapshot_change(gettext('modified'), new_path)
             self._next_progress_entry()
-        # Unversion IDs that were found to be deleted
-        self.deleted_ids = deleted_ids
+        # Unversion files that were found to be deleted
+        self.deleted_paths = deleted_paths
 
     def _check_strict(self):
         # XXX: when we use iter_changes this would likely be faster if

=== modified file 'breezy/config.py'
--- breezy/config.py	2017-10-26 11:10:38 +0000
+++ breezy/config.py	2018-03-10 13:52:14 +0000
@@ -2706,7 +2706,7 @@
 Whether revisions associated with tags should be fetched.
 """))
 option_registry.register_lazy(
-    'bzr.transform.orphan_policy', 'breezy.transform', 'opt_transform_orphan')
+    'transform.orphan_policy', 'breezy.transform', 'opt_transform_orphan')
 option_registry.register(
     Option('bzr.workingtree.worth_saving_limit', default=10,
            from_unicode=int_from_store,  invalid='warning',

=== modified file 'breezy/controldir.py'
--- breezy/controldir.py	2017-11-28 17:13:29 +0000
+++ breezy/controldir.py	2018-03-04 19:06:44 +0000
@@ -415,7 +415,7 @@
                 # revision
                 revision_id = source.last_revision()
             repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.clone(self, revision_id=revision_id)
+            br_to = source.sprout(self, revision_id=revision_id)
             if source.get_push_location() is None or remember:
                 # FIXME: Should be done only if we succeed ? -- vila 2012-01-18
                 source.set_push_location(br_to.base)

=== modified file 'breezy/errors.py'
--- breezy/errors.py	2017-12-23 09:56:39 +0000
+++ breezy/errors.py	2018-03-01 05:20:51 +0000
@@ -245,15 +245,6 @@
         self.tree = tree
 
 
-class NoSuchIdInRepository(NoSuchId):
-
-    _fmt = ('The file id "%(file_id)s" is not present in the repository'
-            ' %(repository)r')
-
-    def __init__(self, repository, file_id):
-        BzrError.__init__(self, repository=repository, file_id=file_id)
-
-
 class NotStacked(BranchError):
 
     _fmt = "The branch '%(branch)s' is not stacked."
@@ -1867,6 +1858,14 @@
         self.tname = type(method_self).__name__
 
 
+class FetchLimitUnsupported(UnsupportedOperation):
+
+    fmt = ("InterBranch %(interbranch)r does not support fetching limits.")
+
+    def __init__(self, interbranch):
+        BzrError.__init__(self, interbranch=interbranch)
+
+
 class NonAsciiRevisionId(UnsupportedOperation):
     """Raised when a commit is attempting to set a non-ascii revision id
        but cant.

=== modified file 'breezy/export/dir_exporter.py'
--- breezy/export/dir_exporter.py	2018-02-18 19:18:40 +0000
+++ breezy/export/dir_exporter.py	2018-03-25 00:39:16 +0000
@@ -57,7 +57,7 @@
     for dp, tp, ie in _export_iter_entries(tree, subdir):
         fullpath = osutils.pathjoin(dest, dp)
         if ie.kind == "file":
-            to_fetch.append((ie.file_id, (dp, tp, ie.file_id)))
+            to_fetch.append((tp, (dp, tp, ie.file_id)))
         elif ie.kind == "directory":
             os.mkdir(fullpath)
         elif ie.kind == "symlink":
@@ -70,7 +70,7 @@
                     % (fullpath, symlink_target, e))
         else:
             raise errors.BzrError("don't know how to export {%s} of kind %r" %
-               (ie.file_id, ie.kind))
+               (tp, ie.kind))
 
         yield
     # The data returned here can be in any order, but we've already created all

=== modified file 'breezy/fetch.py'
--- breezy/fetch.py	2018-02-17 02:02:38 +0000
+++ breezy/fetch.py	2018-03-24 23:49:53 +0000
@@ -97,9 +97,8 @@
         # assert not missing
         self.count_total = 0
         self.file_ids_names = {}
-        pb = ui.ui_factory.nested_progress_bar()
-        pb.show_pct = pb.show_count = False
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
+            pb.show_pct = pb.show_count = False
             pb.update(gettext("Finding revisions"), 0, 2)
             search_result = self._revids_to_fetch()
             mutter('fetching: %s', search_result)
@@ -107,8 +106,6 @@
                 return
             pb.update(gettext("Fetching revisions"), 1, 2)
             self._fetch_everything_for_search(search_result)
-        finally:
-            pb.finished()
 
     def _fetch_everything_for_search(self, search):
         """Fetch all data for the given set of revisions."""
@@ -125,8 +122,7 @@
             raise errors.IncompatibleRepositories(
                 self.from_repository, self.to_repository,
                 "different rich-root support")
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             pb.update("Get stream source")
             source = self.from_repository._get_source(
                 self.to_repository._format)
@@ -151,8 +147,6 @@
                         resume_tokens,))
             pb.update("Finishing stream")
             self.sink.finished()
-        finally:
-            pb.finished()
 
     def _revids_to_fetch(self):
         """Determines the exact revisions needed from self.from_repository to
@@ -323,7 +317,7 @@
                 pass
             else:
                 try:
-                    parent_ids.append(tree.get_file_revision(u'', root_id))
+                    parent_ids.append(tree.get_file_revision(tree.id2path(root_id), root_id))
                 except errors.NoSuchId:
                     # not in the tree
                     pass

=== modified file 'breezy/filter_tree.py'
--- breezy/filter_tree.py	2017-11-12 17:53:35 +0000
+++ breezy/filter_tree.py	2018-03-24 00:45:37 +0000
@@ -47,7 +47,7 @@
     def get_file_text(self, path, file_id=None):
         chunks = self.backing_tree.get_file_lines(path, file_id)
         filters = self.filter_stack_callback(path)
-        context = ContentFilterContext(path, self, None)
+        context = ContentFilterContext(path, self)
         contents = filtered_output_bytes(chunks, filters, context)
         content = ''.join(contents)
         return content
@@ -58,14 +58,14 @@
     def is_executable(self, path, file_id=None):
         return self.backing_tree.is_executable(path, file_id)
 
-    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=None):
+    def iter_entries_by_dir(self, specific_files=None, yield_parents=None):
         # NB: This simply returns the parent tree's entries; the length may be
         # wrong but it can't easily be calculated without filtering the whole
         # text.  Currently all callers cope with this; perhaps they should be
         # updated to a narrower interface that only provides things guaranteed
         # cheaply available across all trees. -- mbp 20110705
         return self.backing_tree.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids,
+            specific_files=specific_files,
             yield_parents=yield_parents)
 
     def lock_read(self):

=== modified file 'breezy/filters/__init__.py'
--- breezy/filters/__init__.py	2017-11-12 13:53:51 +0000
+++ breezy/filters/__init__.py	2018-03-23 23:32:28 +0000
@@ -72,19 +72,16 @@
 class ContentFilterContext(object):
     """Object providing information that filters can use."""
 
-    def __init__(self, relpath=None, tree=None, entry=None):
+    def __init__(self, relpath=None, tree=None):
         """Create a context.
 
         :param relpath: the relative path or None if this context doesn't
            support that information.
         :param tree: the Tree providing this file or None if this context
            doesn't support that information.
-        :param entry: the InventoryEntry object if it is already known or
-           None if it should be derived if possible
         """
         self._relpath = relpath
         self._tree = tree
-        self._entry = entry
         # Cached values
         self._revision_id = None
         self._revision = None
@@ -97,24 +94,12 @@
         """Source Tree object."""
         return self._tree
 
-    def file_id(self):
-        """File-id of file."""
-        if self._entry is not None:
-            return self._entry.file_id
-        elif self._tree is None:
-            return None
-        else:
-            return self._tree.path2id(self._relpath)
-
     def revision_id(self):
         """Id of revision that last changed this file."""
         if self._revision_id is None:
-            if self._entry is not None:
-                self._revision_id = self._entry.revision
-            elif self._tree is not None:
-                file_id = self._tree.path2id(self._relpath)
-                self._entry = self._tree.inventory[file_id]
-                self._revision_id = self._entry.revision
+            if self._tree is not None:
+                self._revision_id = self._tree.get_file_revision(
+                        self._relpath)
         return self._revision_id
 
     def revision(self):

=== modified file 'breezy/gpg.py'
--- breezy/gpg.py	2017-07-04 20:03:11 +0000
+++ breezy/gpg.py	2018-03-24 01:56:08 +0000
@@ -52,6 +52,10 @@
 SIGNATURE_NOT_SIGNED = 3
 SIGNATURE_EXPIRED = 4
 
+MODE_NORMAL = 0
+MODE_DETACH = 1
+MODE_CLEAR = 2
+
 
 class GpgNotInstalled(errors.DependencyNotPresent):
 
@@ -99,8 +103,7 @@
     result = []
     all_verifiable = True
     total = len(revids)
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb:
         for i, (rev_id, verification_result, uid) in enumerate(
                 repository.verify_revision_signatures(
                     revids, strategy)):
@@ -111,8 +114,6 @@
                 all_verifiable = False
             if process_events_callback is not None:
                 process_events_callback()
-    finally:
-        pb.finished()
     return (count, result, all_verifiable)
 
 
@@ -126,10 +127,10 @@
     def __init__(self, ignored):
         """Real strategies take a configuration."""
 
-    def sign(self, content):
+    def sign(self, content, mode):
         raise SigningFailed('Signing is disabled.')
 
-    def verify(self, content, testament):
+    def verify(self, signed_data, signature=None):
         raise SignatureVerificationFailed('Signature verification is \
 disabled.')
 
@@ -149,12 +150,14 @@
     def __init__(self, ignored):
         """Real strategies take a configuration."""
 
-    def sign(self, content):
+    def sign(self, content, mode):
         return ("-----BEGIN PSEUDO-SIGNED CONTENT-----\n" + content +
                 "-----END PSEUDO-SIGNED CONTENT-----\n")
 
-    def verify(self, content, testament):
-        return SIGNATURE_VALID, None
+    def verify(self, signed_data, signature=None):
+        plain_text = signed_data.replace("-----BEGIN PSEUDO-SIGNED CONTENT-----\n", "")
+        plain_text = plain_text.replace("-----END PSEUDO-SIGNED CONTENT-----\n", "")
+        return SIGNATURE_VALID, None, plain_text
 
     def set_acceptable_keys(self, command_line_input):
         if command_line_input is not None:
@@ -190,11 +193,11 @@
         try:
             import gpg
             self.context = gpg.Context()
+            self.context.armor = True
+            self.context.signers = self._get_signing_keys()
         except ImportError as error:
             pass # can't use verify()
 
-        self.context.signers = self._get_signing_keys()
-
     def _get_signing_keys(self):
         import gpg
         keyname = self._config_stack.get('gpg_signing_key')
@@ -227,7 +230,7 @@
         except ImportError as error:
             return False
 
-    def sign(self, content):
+    def sign(self, content, mode):
         import gpg
         if isinstance(content, unicode):
             raise errors.BzrBadParameterUnicode('content')
@@ -235,29 +238,34 @@
         plain_text = gpg.Data(content)
         try:
             output, result = self.context.sign(
-                plain_text, mode=gpg.constants.sig.mode.CLEAR)
+                plain_text, mode={
+                    MODE_DETACH: gpg.constants.sig.mode.DETACH,
+                    MODE_CLEAR: gpg.constants.sig.mode.CLEAR,
+                    MODE_NORMAL: gpg.constants.sig.mode.NORMAL,
+                    }[mode])
         except gpg.errors.GPGMEError as error:
             raise SigningFailed(str(error))
 
         return output
 
-    def verify(self, content, testament):
+    def verify(self, signed_data, signature=None):
         """Check content has a valid signature.
 
-        :param content: the commit signature
-        :param testament: the valid testament string for the commit
+        :param signed_data; Signed data
+        :param signature: optional signature (if detached)
 
-        :return: SIGNATURE_VALID or a failed SIGNATURE_ value, key uid if valid
+        :return: SIGNATURE_VALID or a failed SIGNATURE_ value, key uid if valid, plain text
         """
         try:
             import gpg
         except ImportError as error:
             raise errors.GpgNotInstalled(error)
 
-        signature = gpg.Data(content)
-        sink = gpg.Data()
+        signed_data = gpg.Data(signed_data)
+        if signature:
+            signature = gpg.Data(signature)
         try:
-            plain_output, result = self.context.verify(signature)
+            plain_output, result = self.context.verify(signed_data, signature)
         except gpg.errors.BadSignatures as error:
             fingerprint = error.result.signatures[0].fpr
             if error.result.signatures[0].summary & gpg.constants.SIGSUM_KEY_EXPIRED:
@@ -265,35 +273,32 @@
                 if expires > error.result.signatures[0].timestamp:
                     # The expired key was not expired at time of signing.
                     # test_verify_expired_but_valid()
-                    return SIGNATURE_EXPIRED, fingerprint[-8:]
+                    return SIGNATURE_EXPIRED, fingerprint[-8:], None
                 else:
                     # I can't work out how to create a test where the signature
                     # was expired at the time of signing.
-                    return SIGNATURE_NOT_VALID, None
+                    return SIGNATURE_NOT_VALID, None, None
 
             # GPG does not know this key.
             # test_verify_unknown_key()
             if error.result.signatures[0].summary & gpg.constants.SIGSUM_KEY_MISSING:
-                return SIGNATURE_KEY_MISSING, fingerprint[-8:]
+                return SIGNATURE_KEY_MISSING, fingerprint[-8:], None
 
-            return SIGNATURE_NOT_VALID, None
+            return SIGNATURE_NOT_VALID, None, None
         except gpg.errors.GPGMEError as error:
-            raise SignatureVerificationFailed(error[2])
+            raise SignatureVerificationFailed(error)
 
         # No result if input is invalid.
         # test_verify_invalid()
         if len(result.signatures) == 0:
-            return SIGNATURE_NOT_VALID, None
+            return SIGNATURE_NOT_VALID, None, plain_output
+
         # User has specified a list of acceptable keys, check our result is in
         # it.  test_verify_unacceptable_key()
         fingerprint = result.signatures[0].fpr
         if self.acceptable_keys is not None:
             if not fingerprint in self.acceptable_keys:
-                return SIGNATURE_KEY_MISSING, fingerprint[-8:]
-        # Check the signature actually matches the testament.
-        # test_verify_bad_testament()
-        if testament != plain_output:
-            return SIGNATURE_NOT_VALID, None
+                return SIGNATURE_KEY_MISSING, fingerprint[-8:], plain_output
         # Yay gpg set the valid bit.
         # Can't write a test for this one as you can't set a key to be
         # trusted using gpg.
@@ -301,20 +306,20 @@
             key = self.context.get_key(fingerprint)
             name = key.uids[0].name
             email = key.uids[0].email
-            return SIGNATURE_VALID, name + " <" + email + ">"
+            return SIGNATURE_VALID, name.decode('utf-8') + u" <" + email.decode('utf-8') + u">", plain_output
         # Sigsum_red indicates a problem, unfortunatly I have not been able
         # to write any tests which actually set this.
         if result.signatures[0].summary & gpg.constants.SIGSUM_RED:
-            return SIGNATURE_NOT_VALID, None
+            return SIGNATURE_NOT_VALID, None, plain_output
         # Summary isn't set if sig is valid but key is untrusted but if user
         # has explicity set the key as acceptable we can validate it.
         if result.signatures[0].summary == 0 and self.acceptable_keys is not None:
             if fingerprint in self.acceptable_keys:
                 # test_verify_untrusted_but_accepted()
-                return SIGNATURE_VALID, None
+                return SIGNATURE_VALID, None, plain_output
         # test_verify_valid_but_untrusted()
         if result.signatures[0].summary == 0 and self.acceptable_keys is None:
-            return SIGNATURE_NOT_VALID, None
+            return SIGNATURE_NOT_VALID, None, plain_output
         # Other error types such as revoked keys should (I think) be caught by
         # SIGSUM_RED so anything else means something is buggy.
         raise SignatureVerificationFailed(

=== modified file 'breezy/ignores.py'
--- breezy/ignores.py	2018-02-15 19:38:33 +0000
+++ breezy/ignores.py	2018-03-04 19:11:21 +0000
@@ -190,7 +190,7 @@
     :return: None
     """
     # read in the existing ignores set
-    ifn = tree.abspath(breezy.IGNORE_FILENAME)
+    ifn = tree.abspath(tree._format.ignore_filename)
     if tree.has_filename(ifn):
         with open(ifn, 'rbU') as f:
             file_contents = f.read()
@@ -203,13 +203,13 @@
     else:
         file_contents = b""
         newline = os.linesep.encode()
-    
+
     sio = BytesIO(file_contents)
     try:
         ignores = parse_ignore_file(sio)
     finally:
         sio.close()
-    
+
     # write out the updated ignores set
     f = atomicfile.AtomicFile(ifn, 'wb')
     try:
@@ -225,5 +225,5 @@
     finally:
         f.close()
 
-    if not tree.is_versioned(breezy.IGNORE_FILENAME):
-        tree.add([breezy.IGNORE_FILENAME])
+    if not tree.is_versioned(tree._format.ignore_filename):
+        tree.add([tree._format.ignore_filename])

=== modified file 'breezy/info.py'
--- breezy/info.py	2017-07-24 01:09:25 +0000
+++ breezy/info.py	2018-03-24 13:36:30 +0000
@@ -290,9 +290,8 @@
     outfile.write('  %8d ignored\n' % ignore_cnt)
 
     dir_cnt = 0
-    root_id = working.get_root_id()
     for path, entry in working.iter_entries_by_dir():
-        if entry.kind == 'directory' and entry.file_id != root_id:
+        if entry.kind == 'directory' and path != '':
             dir_cnt += 1
     outfile.write('  %8d versioned %s\n' % (dir_cnt,
         plural(dir_cnt, 'subdirectory', 'subdirectories')))
@@ -469,8 +468,8 @@
         if branch is None and tree is not None:
             phrase = "branchless tree"
         else:
-            if (tree is not None and tree.user_url !=
-                branch.user_url):
+            if (tree is not None and tree.controldir.control_url !=
+                branch.controldir.control_url):
                 independence = ''
                 phrase = "Lightweight checkout"
             elif branch.get_bound_location() is not None:

=== modified file 'breezy/log.py'
--- breezy/log.py	2017-11-28 22:51:09 +0000
+++ breezy/log.py	2018-03-24 00:39:56 +0000
@@ -64,7 +64,6 @@
     config,
     controldir,
     diff,
-    errors,
     foreign,
     repository as _mod_repository,
     revision as _mod_revision,
@@ -74,6 +73,7 @@
 """)
 
 from . import (
+    errors,
     lazy_regex,
     registry,
     revisionspec,
@@ -85,14 +85,15 @@
     get_terminal_encoding,
     terminal_width,
     )
-from breezy.sixish import (
+from .sixish import (
     BytesIO,
     range,
     zip,
     )
-
-
-def find_touching_revisions(branch, file_id):
+from .tree import find_previous_path
+
+
+def find_touching_revisions(repository, last_revision, last_tree, last_path):
     """Yield a description of revisions which affect the file_id.
 
     Each returned element is (revno, revision_id, description)
@@ -103,40 +104,35 @@
     TODO: Perhaps some way to limit this to only particular revisions,
     or to traverse a non-mainline set of revisions?
     """
-    last_verifier = None
-    last_path = None
-    revno = 1
-    graph = branch.repository.get_graph()
-    history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
-        [_mod_revision.NULL_REVISION]))
-    for revision_id in reversed(history):
-        this_tree = branch.repository.revision_tree(revision_id)
-        try:
-            this_path = this_tree.id2path(file_id)
-        except errors.NoSuchId:
-            this_verifier = this_path = None
-        else:
-            this_verifier = this_tree.get_file_verifier(this_path, file_id)
+    last_verifier = last_tree.get_file_verifier(last_path)
+    graph = repository.get_graph()
+    history = list(graph.iter_lefthand_ancestry(last_revision, []))
+    revno = len(history)
+    for revision_id in history:
+        this_tree = repository.revision_tree(revision_id)
+        this_path = find_previous_path(last_tree, this_tree, last_path)
 
         # now we know how it was last time, and how it is in this revision.
         # are those two states effectively the same or not?
-
-        if not this_verifier and not last_verifier:
-            # not present in either
-            pass
-        elif this_verifier and not last_verifier:
-            yield revno, revision_id, "added " + this_path
-        elif not this_verifier and last_verifier:
-            # deleted here
-            yield revno, revision_id, "deleted " + last_path
+        if this_path is not None and last_path is None:
+            yield revno, revision_id, "deleted " + this_path
+            this_verifier = this_tree.get_file_verifier(this_path)
+        elif this_path is None and last_path is not None:
+            yield revno, revision_id, "added " + last_path
         elif this_path != last_path:
-            yield revno, revision_id, ("renamed %s => %s" % (last_path, this_path))
-        elif (this_verifier != last_verifier):
-            yield revno, revision_id, "modified " + this_path
+            yield revno, revision_id, ("renamed %s => %s" % (this_path, last_path))
+            this_verifier = this_tree.get_file_verifier(this_path)
+        else:
+            this_verifier = this_tree.get_file_verifier(this_path)
+            if (this_verifier != last_verifier):
+                yield revno, revision_id, "modified " + this_path
 
         last_verifier = this_verifier
         last_path = this_path
-        revno += 1
+        last_tree = this_tree
+        if last_path is None:
+            return
+        revno -= 1
 
 
 def show_log(branch,

=== modified file 'breezy/memorytree.py'
--- breezy/memorytree.py	2017-11-17 03:06:50 +0000
+++ breezy/memorytree.py	2018-03-24 13:49:31 +0000
@@ -131,12 +131,6 @@
         else:
             raise NotImplementedError('unknown kind')
 
-    def _file_size(self, entry, stat_value):
-        """See Tree._file_size."""
-        if entry is None:
-            return 0
-        return entry.text_size
-
     def get_parent_ids(self):
         """See Tree.get_parent_ids.
 
@@ -276,7 +270,12 @@
             # is not relevant to memory tree. Until that is done in unlock by
             # working tree, we cannot share the implementation.
             if file_ids is None:
-                file_ids = {self.path2id(path) for path in paths}
+                file_ids = set()
+                for path in paths:
+                    file_id = self.path2id(path)
+                    if file_id is None:
+                        raise errors.NoSuchFile(path)
+                    file_ids.add(file_id)
             for file_id in file_ids:
                 if self._inventory.has_id(file_id):
                     self._inventory.remove_recursive_id(file_id)

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2017-11-13 22:51:34 +0000
+++ breezy/merge.py	2018-03-24 18:22:08 +0000
@@ -56,12 +56,13 @@
 # TODO: Report back as changes are merged in
 
 
-def transform_tree(from_tree, to_tree, interesting_ids=None):
+def transform_tree(from_tree, to_tree, interesting_files=None):
     from_tree.lock_tree_write()
     operation = cleanup.OperationWithCleanups(merge_inner)
     operation.add_cleanup(from_tree.unlock)
     operation.run_simple(from_tree.branch, to_tree, from_tree,
-        ignore_zero=True, interesting_ids=interesting_ids, this_tree=from_tree)
+        ignore_zero=True, this_tree=from_tree,
+        interesting_files=interesting_files)
 
 
 class MergeHooks(hooks.Hooks):
@@ -94,7 +95,7 @@
     """PerFileMerger objects are used by plugins extending merge for breezy.
 
     See ``breezy.plugins.news_merge.news_merge`` for an example concrete class.
-    
+
     :ivar merger: The Merge3Merger performing the merge.
     """
 
@@ -104,7 +105,7 @@
 
     def merge_contents(self, merge_params):
         """Attempt to merge the contents of a single file.
-        
+
         :param merge_params: A breezy.merge.MergeFileHookParams
         :return: A tuple of (status, chunks), where status is one of
             'not_applicable', 'success', 'conflicted', or 'delete'.  If status
@@ -130,20 +131,6 @@
         """
         raise NotImplementedError(self.file_matches)
 
-    def get_filename(self, params, tree):
-        """Lookup the filename (i.e. basename, not path), given a Tree (e.g.
-        self.merger.this_tree) and a MergeFileHookParams.
-        """
-        return osutils.basename(tree.id2path(params.file_id))
-
-    def get_filepath(self, params, tree):
-        """Calculate the path to the file in a tree.
-
-        :param params: A MergeFileHookParams describing the file to merge
-        :param tree: a Tree, e.g. self.merger.this_tree.
-        """
-        return tree.id2path(params.file_id)
-
     def merge_contents(self, params):
         """Merge the contents of a single file."""
         # Check whether this custom merge logic should be used.
@@ -174,13 +161,13 @@
     classes should implement ``merge_text``.
 
     See ``breezy.plugins.news_merge.news_merge`` for an example concrete class.
-    
+
     :ivar affected_files: The configured file paths to merge.
 
     :cvar name_prefix: The prefix to use when looking up configuration
         details. <name_prefix>_merge_files describes the files targeted by the
         hook for example.
-        
+
     :cvar default_files: The default file paths to merge when no configuration
         is present.
     """
@@ -216,7 +203,7 @@
                 affected_files = self.default_files
             self.affected_files = affected_files
         if affected_files:
-            filepath = self.get_filepath(params, self.merger.this_tree)
+            filepath = params.this_path
             if filepath in affected_files:
                 return True
         return False
@@ -240,16 +227,21 @@
     There are some fields hooks can access:
 
     :ivar file_id: the file ID of the file being merged
+    :ivar base_path: Path in base tree
+    :ivar other_path: Path in other tree
+    :ivar this_path: Path in this tree
     :ivar trans_id: the transform ID for the merge of this file
     :ivar this_kind: kind of file_id in 'this' tree
     :ivar other_kind: kind of file_id in 'other' tree
     :ivar winner: one of 'this', 'other', 'conflict'
     """
 
-    def __init__(self, merger, file_id, trans_id, this_kind, other_kind,
+    def __init__(self, merger, file_id, paths, trans_id, this_kind, other_kind,
             winner):
         self._merger = merger
         self.file_id = file_id
+        self.paths = paths
+        self.base_path, self.other_path, self.this_path = paths
         self.trans_id = trans_id
         self.this_kind = this_kind
         self.other_kind = other_kind
@@ -262,17 +254,17 @@
     @decorators.cachedproperty
     def base_lines(self):
         """The lines of the 'base' version of the file."""
-        return self._merger.get_lines(self._merger.base_tree, self.file_id)
+        return self._merger.get_lines(self._merger.base_tree, self.base_path, self.file_id)
 
     @decorators.cachedproperty
     def this_lines(self):
         """The lines of the 'this' version of the file."""
-        return self._merger.get_lines(self._merger.this_tree, self.file_id)
+        return self._merger.get_lines(self._merger.this_tree, self.this_path, self.file_id)
 
     @decorators.cachedproperty
     def other_lines(self):
         """The lines of the 'other' version of the file."""
-        return self._merger.get_lines(self._merger.other_tree, self.file_id)
+        return self._merger.get_lines(self._merger.other_tree, self.other_path, self.file_id)
 
 
 class Merger(object):
@@ -295,7 +287,6 @@
         self.base_tree = base_tree
         self.ignore_zero = False
         self.backup_files = False
-        self.interesting_ids = None
         self.interesting_files = None
         self.show_base = False
         self.reprocess = False
@@ -610,7 +601,6 @@
     def make_merger(self):
         kwargs = {'working_tree': self.this_tree, 'this_tree': self.this_tree,
                   'other_tree': self.other_tree,
-                  'interesting_ids': self.interesting_ids,
                   'interesting_files': self.interesting_files,
                   'this_branch': self.this_branch,
                   'other_branch': self.other_branch,
@@ -661,7 +651,8 @@
                 other_branch = self.other_branch.reference_parent(file_id,
                                                                   relpath)
                 sub_merge.set_other_revision(other_revision, other_branch)
-                base_tree_path = self.base_tree.id2path(file_id)
+                base_tree_path = _mod_tree.find_previous_path(
+                    self.this_tree, self.base_tree, relpath)
                 base_revision = self.base_tree.get_reference_revision(
                     base_tree_path, file_id)
                 sub_merge.base_tree = \
@@ -718,9 +709,10 @@
     supports_reverse_cherrypick = True
     winner_idx = {"this": 2, "other": 1, "conflict": 1}
     supports_lca_trees = True
+    requires_file_merge_plan = False
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree,
-                 interesting_ids=None, reprocess=False, show_base=False,
+                 reprocess=False, show_base=False,
                  change_reporter=None, interesting_files=None, do_merge=True,
                  cherrypick=False, lca_trees=None, this_branch=None,
                  other_branch=None):
@@ -733,30 +725,22 @@
         :param this_branch: The branch associated with this_tree.  Defaults to
             this_tree.branch if not supplied.
         :param other_branch: The branch associated with other_tree, if any.
-        :param interesting_ids: The file_ids of files that should be
-            participate in the merge.  May not be combined with
-            interesting_files.
         :param: reprocess If True, perform conflict-reduction processing.
         :param show_base: If True, show the base revision in text conflicts.
             (incompatible with reprocess)
         :param change_reporter: An object that should report changes made
         :param interesting_files: The tree-relative paths of files that should
             participate in the merge.  If these paths refer to directories,
-            the contents of those directories will also be included.  May not
-            be combined with interesting_ids.  If neither interesting_files nor
-            interesting_ids is specified, all files may participate in the
+            the contents of those directories will also be included.  If not
+            specified, all files may participate in the
             merge.
         :param lca_trees: Can be set to a dictionary of {revision_id:rev_tree}
             if the ancestry was found to include a criss-cross merge.
             Otherwise should be None.
         """
         object.__init__(self)
-        if interesting_files is not None and interesting_ids is not None:
-            raise ValueError(
-                'specify either interesting_ids or interesting_files')
         if this_branch is None:
             this_branch = this_tree.branch
-        self.interesting_ids = interesting_ids
         self.interesting_files = interesting_files
         self.working_tree = working_tree
         self.this_tree = this_tree
@@ -827,22 +811,19 @@
         # One hook for each registered one plus our default merger
         hooks = [factory(self) for factory in factories] + [self]
         self.active_hooks = [hook for hook in hooks if hook is not None]
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
-            for num, (file_id, changed, parents3, names3,
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            for num, (file_id, changed, paths3, parents3, names3,
                       executable3) in enumerate(entries):
                 # Try merging each entry
                 child_pb.update(gettext('Preparing file merge'),
                                 num, len(entries))
-                self._merge_names(file_id, parents3, names3, resolver=resolver)
+                self._merge_names(file_id, paths3, parents3, names3, resolver=resolver)
                 if changed:
-                    file_status = self._do_merge_contents(file_id)
+                    file_status = self._do_merge_contents(paths3, file_id)
                 else:
                     file_status = 'unmodified'
-                self._merge_executable(file_id,
-                    executable3, file_status, resolver=resolver)
-        finally:
-            child_pb.finished()
+                self._merge_executable(paths3, file_id, executable3,
+                        file_status, resolver=resolver)
         self.tt.fixup_new_roots()
         self._finish_computing_transform()
 
@@ -851,12 +832,9 @@
 
         This is the second half of _compute_transform.
         """
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             fs_conflicts = transform.resolve_conflicts(self.tt, child_pb,
                 lambda t, c: transform.conflict_pass(t, c, self.other_tree))
-        finally:
-            child_pb.finished()
         if self.change_reporter is not None:
             from breezy import delta
             delta.report_changes(
@@ -878,27 +856,32 @@
         iterator = self.other_tree.iter_changes(self.base_tree,
                 specific_files=self.interesting_files,
                 extra_trees=[self.this_tree])
-        this_entries = dict((e.file_id, e) for p, e in
-                            self.this_tree.iter_entries_by_dir(
-                            self.interesting_ids))
+        this_interesting_files = self.this_tree.find_related_paths_across_trees(
+                self.interesting_files, trees=[self.other_tree])
+        this_entries = dict(self.this_tree.iter_entries_by_dir(
+                            specific_files=this_interesting_files))
         for (file_id, paths, changed, versioned, parents, names, kind,
              executable) in iterator:
-            if (self.interesting_ids is not None and
-                file_id not in self.interesting_ids):
-                continue
-            entry = this_entries.get(file_id)
-            if entry is not None:
-                this_name = entry.name
-                this_parent = entry.parent_id
-                this_executable = entry.executable
+            if paths[0] is not None:
+                this_path = _mod_tree.find_previous_path(
+                        self.base_tree, self.this_tree, paths[0])
+            else:
+                this_path = _mod_tree.find_previous_path(
+                        self.other_tree, self.this_tree, paths[1])
+            this_entry = this_entries.get(this_path)
+            if this_entry is not None:
+                this_name = this_entry.name
+                this_parent = this_entry.parent_id
+                this_executable = this_entry.executable
             else:
                 this_name = None
                 this_parent = None
                 this_executable = None
             parents3 = parents + (this_parent,)
             names3 = names + (this_name,)
+            paths3 = paths + (this_path, )
             executable3 = executable + (this_executable,)
-            result.append((file_id, changed, parents3, names3, executable3))
+            result.append((file_id, changed, paths3, parents3, names3, executable3))
         return result
 
     def _entries_lca(self):
@@ -909,10 +892,11 @@
 
         For the multi-valued entries, the format will be (BASE, [lca1, lca2])
 
-        :return: [(file_id, changed, parents, names, executable)], where:
+        :return: [(file_id, changed, paths, parents, names, executable)], where:
 
             * file_id: Simple file_id of the entry
             * changed: Boolean, True if the kind or contents changed else False
+            * paths: ((base, [path, in, lcas]), path_other, path_this)
             * parents: ((base, [parent_id, in, lcas]), parent_id_other,
                         parent_id_this)
             * names:   ((base, [name, in, lcas]), name_in_other, name_in_this)
@@ -923,10 +907,10 @@
             lookup_trees = [self.this_tree, self.base_tree]
             lookup_trees.extend(self._lca_trees)
             # I think we should include the lca trees as well
-            interesting_ids = self.other_tree.paths2ids(self.interesting_files,
-                                                        lookup_trees)
+            interesting_files = self.other_tree.find_related_paths_across_trees(
+                    self.interesting_files, lookup_trees)
         else:
-            interesting_ids = self.interesting_ids
+            interesting_files = None
         result = []
         walker = _mod_tree.MultiWalker(self.other_tree, self._lca_trees)
 
@@ -936,7 +920,10 @@
             # Is this modified at all from any of the other trees?
             if other_ie is None:
                 other_ie = _none_entry
-            if interesting_ids is not None and file_id not in interesting_ids:
+                other_path = None
+            else:
+                other_path = self.other_tree.id2path(file_id)
+            if interesting_files is not None and other_path not in interesting_files:
                 continue
 
             # If other_revision is found in any of the lcas, that means this
@@ -959,21 +946,30 @@
                     continue
 
             lca_entries = []
+            lca_paths = []
             for lca_path, lca_ie in lca_values:
                 if lca_ie is None:
                     lca_entries.append(_none_entry)
+                    lca_paths.append(None)
                 else:
                     lca_entries.append(lca_ie)
+                    lca_paths.append(path)
 
-            if base_inventory.has_id(file_id):
+            try:
                 base_ie = base_inventory[file_id]
-            else:
+            except errors.NoSuchId:
                 base_ie = _none_entry
+                base_path = None
+            else:
+                base_path = self.base_tree.id2path(file_id)
 
-            if this_inventory.has_id(file_id):
+            try:
                 this_ie = this_inventory[file_id]
-            else:
+            except errors.NoSuchId:
                 this_ie = _none_entry
+                this_path = None
+            else:
+                this_path = self.this_tree.id2path(file_id)
 
             lca_kinds = []
             lca_parent_ids = []
@@ -1004,15 +1000,18 @@
                         continue
                     content_changed = False
                 elif other_ie.kind is None or other_ie.kind == 'file':
-                    def get_sha1(ie, tree):
-                        if ie.kind != 'file':
-                            return None
-                        return tree.get_file_sha1(tree.id2path(file_id), file_id)
-                    base_sha1 = get_sha1(base_ie, self.base_tree)
-                    lca_sha1s = [get_sha1(ie, tree) for ie, tree
-                                 in zip(lca_entries, self._lca_trees)]
-                    this_sha1 = get_sha1(this_ie, self.this_tree)
-                    other_sha1 = get_sha1(other_ie, self.other_tree)
+                    def get_sha1(tree, path):
+                        if path is None:
+                            return None
+                        try:
+                            return tree.get_file_sha1(path, file_id)
+                        except errors.NoSuchFile:
+                            return None
+                    base_sha1 = get_sha1(self.base_tree, base_path)
+                    lca_sha1s = [get_sha1(tree, lca_path) for tree, lca_path
+                                 in zip(self._lca_trees, lca_paths)]
+                    this_sha1 = get_sha1(self.this_tree, this_path)
+                    other_sha1 = get_sha1(self.other_tree, other_path)
                     sha1_winner = self._lca_multi_way(
                         (base_sha1, lca_sha1s), other_sha1, this_sha1,
                         allow_overriding_lca=False)
@@ -1027,16 +1026,15 @@
                     if sha1_winner == 'this':
                         content_changed = False
                 elif other_ie.kind == 'symlink':
-                    def get_target(ie, tree):
+                    def get_target(ie, tree, path):
                         if ie.kind != 'symlink':
                             return None
-                        path = tree.id2path(file_id)
                         return tree.get_symlink_target(path, file_id)
-                    base_target = get_target(base_ie, self.base_tree)
-                    lca_targets = [get_target(ie, tree) for ie, tree
-                                   in zip(lca_entries, self._lca_trees)]
-                    this_target = get_target(this_ie, self.this_tree)
-                    other_target = get_target(other_ie, self.other_tree)
+                    base_target = get_target(base_ie, self.base_tree, base_path)
+                    lca_targets = [get_target(ie, tree, lca_path) for ie, tree, lca_path
+                                   in zip(lca_entries, self._lca_trees, lca_paths)]
+                    this_target = get_target(this_ie, self.this_tree, this_path)
+                    other_target = get_target(other_ie, self.other_tree, other_path)
                     target_winner = self._lca_multi_way(
                         (base_target, lca_targets),
                         other_target, this_target)
@@ -1060,6 +1058,8 @@
 
             # If we have gotten this far, that means something has changed
             result.append((file_id, content_changed,
+                           ((base_path, lca_paths),
+                            other_path, this_path),
                            ((base_ie.parent_id, lca_parent_ids),
                             other_ie.parent_id, this_ie.parent_id),
                            ((base_ie.name, lca_names),
@@ -1070,6 +1070,8 @@
         return result
 
     def write_modified(self, results):
+        if not self.working_tree.supports_merge_modified():
+            return
         modified_hashes = {}
         for path in results.modified_paths:
             wt_relpath = self.working_tree.relpath(path)
@@ -1097,33 +1099,30 @@
         return entry.name
 
     @staticmethod
-    def contents_sha1(tree, file_id):
+    def contents_sha1(tree, path, file_id=None):
         """Determine the sha1 of the file contents (used as a key method)."""
         try:
-            path = tree.id2path(file_id)
-        except errors.NoSuchId:
+            return tree.get_file_sha1(path, file_id)
+        except errors.NoSuchFile:
             return None
-        return tree.get_file_sha1(path, file_id)
 
     @staticmethod
-    def executable(tree, file_id):
+    def executable(tree, path, file_id=None):
         """Determine the executability of a file-id (used as a key method)."""
         try:
-            path = tree.id2path(file_id)
-        except errors.NoSuchId:
+            if tree.kind(path, file_id) != "file":
+                return False
+        except errors.NoSuchFile:
             return None
-        if tree.kind(path, file_id) != "file":
-            return False
         return tree.is_executable(path, file_id)
 
     @staticmethod
-    def kind(tree, path, file_id):
+    def kind(tree, path, file_id=None):
         """Determine the kind of a file-id (used as a key method)."""
         try:
-            path = tree.id2path(file_id)
-        except errors.NoSuchId:
+            return tree.kind(path, file_id)
+        except errors.NoSuchFile:
             return None
-        return tree.kind(path, file_id)
 
     @staticmethod
     def _three_way(base, other, this):
@@ -1210,13 +1209,14 @@
             else:
                 names.append(entry.name)
                 parents.append(entry.parent_id)
-        return self._merge_names(file_id, parents, names,
+        return self._merge_names(file_id, paths, parents, names,
                                  resolver=self._three_way)
 
-    def _merge_names(self, file_id, parents, names, resolver):
+    def _merge_names(self, file_id, paths, parents, names, resolver):
         """Perform a merge on file_id names and parents"""
         base_name, other_name, this_name = names
         base_parent, other_parent, this_parent = parents
+        unused_base_path, other_path, this_path = paths
 
         name_winner = resolver(*names)
 
@@ -1236,7 +1236,7 @@
             self._raw_conflicts.append(('path conflict', trans_id, file_id,
                                         this_parent, this_name,
                                         other_parent, other_name))
-        if not self.other_tree.has_id(file_id):
+        if other_path is None:
             # it doesn't matter whether the result was 'other' or
             # 'conflict'-- if it has no file id, we leave it alone.
             return
@@ -1258,12 +1258,10 @@
             self.tt.adjust_path(name, parent_trans_id,
                                 self.tt.trans_id_file_id(file_id))
 
-    def _do_merge_contents(self, file_id):
+    def _do_merge_contents(self, paths, file_id):
         """Performs a merge on file_id contents."""
-        def contents_pair(tree):
-            try:
-                path = tree.id2path(file_id)
-            except errors.NoSuchId:
+        def contents_pair(tree, path):
+            if path is None:
                 return (None, None)
             try:
                 kind = tree.kind(path, file_id)
@@ -1277,23 +1275,27 @@
                 contents = None
             return kind, contents
 
+        base_path, other_path, this_path = paths
         # See SPOT run.  run, SPOT, run.
         # So we're not QUITE repeating ourselves; we do tricky things with
         # file kind...
-        base_pair = contents_pair(self.base_tree)
-        other_pair = contents_pair(self.other_tree)
+        other_pair = contents_pair(self.other_tree, other_path)
+        this_pair = contents_pair(self.this_tree, this_path)
         if self._lca_trees:
-            this_pair = contents_pair(self.this_tree)
-            lca_pairs = [contents_pair(tree) for tree in self._lca_trees]
+            (base_path, lca_paths) = base_path
+            base_pair = contents_pair(self.base_tree, base_path)
+            lca_pairs = [contents_pair(tree, path)
+                         for tree, path in zip(self._lca_trees, lca_paths)]
             winner = self._lca_multi_way((base_pair, lca_pairs), other_pair,
                                          this_pair, allow_overriding_lca=False)
         else:
+            base_pair = contents_pair(self.base_tree, base_path)
             if base_pair == other_pair:
                 winner = 'this'
             else:
                 # We delayed evaluating this_pair as long as we can to avoid
                 # unnecessary sha1 calculation
-                this_pair = contents_pair(self.this_tree)
+                this_pair = contents_pair(self.this_tree, this_path)
                 winner = self._three_way(base_pair, other_pair, this_pair)
         if winner == 'this':
             # No interesting changes introduced by OTHER
@@ -1301,7 +1303,9 @@
         # We have a hypothetical conflict, but if we have files, then we
         # can try to merge the content
         trans_id = self.tt.trans_id_file_id(file_id)
-        params = MergeFileHookParams(self, file_id, trans_id, this_pair[0],
+        params = MergeFileHookParams(
+            self, file_id, (base_path, other_path,
+            this_path), trans_id, this_pair[0],
             other_pair[0], winner)
         hooks = self.active_hooks
         hook_status = 'not_applicable'
@@ -1325,9 +1329,7 @@
             inhibit_content_conflict = False
             if params.this_kind is None: # file_id is not in THIS
                 # Is the name used for a different file_id ?
-                dupe_path = self.other_tree.id2path(file_id)
-                this_id = self.this_tree.path2id(dupe_path)
-                if this_id is not None:
+                if self.this_tree.is_versioned(other_path):
                     # Two entries for the same path
                     keep_this = True
                     # versioning the merged file will trigger a duplicate
@@ -1335,14 +1337,12 @@
                     self.tt.version_file(file_id, trans_id)
                     transform.create_from_tree(
                         self.tt, trans_id, self.other_tree,
-                        self.other_tree.id2path(file_id), file_id=file_id,
+                        other_path, file_id=file_id,
                         filter_tree_path=self._get_filter_tree_path(file_id))
                     inhibit_content_conflict = True
             elif params.other_kind is None: # file_id is not in OTHER
                 # Is the name used for a different file_id ?
-                dupe_path = self.this_tree.id2path(file_id)
-                other_id = self.other_tree.path2id(dupe_path)
-                if other_id is not None:
+                if self.other_tree.is_versioned(this_path):
                     # Two entries for the same path again, but here, the other
                     # entry will also be merged.  We simply inhibit the
                     # 'content' conflict creation because we know OTHER will
@@ -1356,8 +1356,9 @@
                     self.tt.unversion_file(trans_id)
                 # This is a contents conflict, because none of the available
                 # functions could merge it.
-                file_group = self._dump_conflicts(name, parent_id, file_id,
-                                                  set_version=True)
+                file_group = self._dump_conflicts(
+                        name, (base_path, other_path, this_path), parent_id,
+                        file_id, set_version=True)
                 self._raw_conflicts.append(('contents conflict', file_group))
         elif hook_status == 'success':
             self.tt.create_file(lines, trans_id)
@@ -1368,7 +1369,8 @@
             self._raw_conflicts.append(('text conflict', trans_id))
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            self._dump_conflicts(name, parent_id, file_id)
+            self._dump_conflicts(
+                name, (base_path, other_path, this_path), parent_id, file_id)
         elif hook_status == 'delete':
             self.tt.unversion_file(trans_id)
             result = "deleted"
@@ -1378,7 +1380,7 @@
             pass
         else:
             raise AssertionError('unknown hook_status: %r' % (hook_status,))
-        if not self.this_tree.has_id(file_id) and result == "modified":
+        if not this_path and result == "modified":
             self.tt.version_file(file_id, trans_id)
         if not keep_this:
             # The merge has been performed and produced a new content, so the
@@ -1390,14 +1392,14 @@
         """Replace this contents with other."""
         file_id = merge_hook_params.file_id
         trans_id = merge_hook_params.trans_id
-        if self.other_tree.has_id(file_id):
+        if merge_hook_params.other_path is not None:
             # OTHER changed the file
             transform.create_from_tree(
                 self.tt, trans_id, self.other_tree,
-                self.other_tree.id2path(file_id), file_id=file_id,
+                merge_hook_params.other_path, file_id=file_id,
                 filter_tree_path=self._get_filter_tree_path(file_id))
             return 'done', None
-        elif self.this_tree.has_id(file_id):
+        elif merge_hook_params.this_path is not None:
             # OTHER deleted the file
             return 'delete', None
         else:
@@ -1408,7 +1410,7 @@
     def merge_contents(self, merge_hook_params):
         """Fallback merge logic after user installed hooks."""
         # This function is used in merge hooks as the fallback instance.
-        # Perhaps making this function and the functions it calls be a 
+        # Perhaps making this function and the functions it calls be a
         # a separate class would be better.
         if merge_hook_params.winner == 'other':
             # OTHER is a straight winner, so replace this contents with other
@@ -1418,38 +1420,35 @@
             # BASE is a file, or both converted to files, so at least we
             # have agreement that output should be a file.
             try:
-                self.text_merge(merge_hook_params.file_id,
-                    merge_hook_params.trans_id)
+                self.text_merge(merge_hook_params.trans_id,
+                                merge_hook_params.paths, merge_hook_params.file_id)
             except errors.BinaryFile:
                 return 'not_applicable', None
             return 'done', None
         else:
             return 'not_applicable', None
 
-    def get_lines(self, tree, file_id):
+    def get_lines(self, tree, path, file_id=None):
         """Return the lines in a file, or an empty list."""
+        if path is None:
+            return []
         try:
-            path = tree.id2path(file_id)
-        except errors.NoSuchId:
+            kind = tree.kind(path, file_id)
+        except errors.NoSuchFile:
             return []
         else:
+            if kind != 'file':
+                return []
             return tree.get_file_lines(path, file_id)
 
-    def text_merge(self, file_id, trans_id):
+    def text_merge(self, trans_id, paths, file_id):
         """Perform a three-way text merge on a file_id"""
         # it's possible that we got here with base as a different type.
         # if so, we just want two-way text conflicts.
-        try:
-            base_path = self.base_tree.id2path(file_id)
-        except errors.NoSuchId:
-            base_lines = []
-        else:
-            if self.base_tree.kind(base_path, file_id) == "file":
-                base_lines = self.get_lines(self.base_tree, file_id)
-            else:
-                base_lines = []
-        other_lines = self.get_lines(self.other_tree, file_id)
-        this_lines = self.get_lines(self.this_tree, file_id)
+        base_path, other_path, this_path = paths
+        base_lines = self.get_lines(self.base_tree, base_path, file_id)
+        other_lines = self.get_lines(self.other_tree, other_path, file_id)
+        this_lines = self.get_lines(self.this_tree, this_path, file_id)
         m3 = merge3.Merge3(base_lines, this_lines, other_lines,
                            is_cherrypick=self.cherrypick)
         start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
@@ -1478,12 +1477,11 @@
             self._raw_conflicts.append(('text conflict', trans_id))
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            file_group = self._dump_conflicts(name, parent_id, file_id,
+            file_group = self._dump_conflicts(name, paths, parent_id, file_id,
                                               this_lines, base_lines,
                                               other_lines)
             file_group.append(trans_id)
 
-
     def _get_filter_tree_path(self, file_id):
         if self.this_tree.supports_content_filtering():
             # We get the path from the working tree if it exists.
@@ -1497,7 +1495,7 @@
         # Skip the id2path lookup for older formats
         return None
 
-    def _dump_conflicts(self, name, parent_id, file_id, this_lines=None,
+    def _dump_conflicts(self, name, paths, parent_id, file_id, this_lines=None,
                         base_lines=None, other_lines=None, set_version=False,
                         no_base=False):
         """Emit conflict files.
@@ -1505,10 +1503,11 @@
         determined automatically.  If set_version is true, the .OTHER, .THIS
         or .BASE (in that order) will be created as versioned files.
         """
-        data = [('OTHER', self.other_tree, other_lines),
-                ('THIS', self.this_tree, this_lines)]
+        base_path, other_path, this_path = paths
+        data = [('OTHER', self.other_tree, other_path, other_lines),
+                ('THIS', self.this_tree, this_path, this_lines)]
         if not no_base:
-            data.append(('BASE', self.base_tree, base_lines))
+            data.append(('BASE', self.base_tree, base_path, base_lines))
 
         # We need to use the actual path in the working tree of the file here,
         # ignoring the conflict suffixes
@@ -1525,12 +1524,8 @@
 
         versioned = False
         file_group = []
-        for suffix, tree, lines in data:
-            try:
-                path = tree.id2path(file_id)
-            except errors.NoSuchId:
-                pass
-            else:
+        for suffix, tree, path, lines in data:
+            if path is not None:
                 trans_id = self._conflict_file(
                         name, parent_id, path, tree, file_id, suffix, lines,
                         filter_tree_path)
@@ -1551,24 +1546,25 @@
                 filter_tree_path=filter_tree_path)
         return trans_id
 
-    def merge_executable(self, file_id, file_status):
+    def merge_executable(self, paths, file_id, file_status):
         """Perform a merge on the execute bit."""
-        executable = [self.executable(t, file_id) for t in (self.base_tree,
-                      self.other_tree, self.this_tree)]
-        self._merge_executable(file_id, executable, file_status,
+        executable = [self.executable(t, p, file_id) for t, p in zip([self.base_tree,
+                      self.other_tree, self.this_tree], paths)]
+        self._merge_executable(paths, file_id, executable, file_status,
                                resolver=self._three_way)
 
-    def _merge_executable(self, file_id, executable, file_status,
+    def _merge_executable(self, paths, file_id, executable, file_status,
                           resolver):
         """Perform a merge on the execute bit."""
         base_executable, other_executable, this_executable = executable
+        base_path, other_path, this_path = paths
         if file_status == "deleted":
             return
         winner = resolver(*executable)
         if winner == "conflict":
         # There must be a None in here, if we have a conflict, but we
         # need executability since file status was not deleted.
-            if self.executable(self.other_tree, file_id) is None:
+            if self.executable(self.other_tree, other_path, file_id) is None:
                 winner = "this"
             else:
                 winner = "other"
@@ -1580,11 +1576,11 @@
         if winner == "this":
             executability = this_executable
         else:
-            if self.other_tree.has_id(file_id):
+            if other_path is not None:
                 executability = other_executable
-            elif self.this_tree.has_id(file_id):
+            elif this_path is not None:
                 executability = this_executable
-            elif self.base_tree_has_id(file_id):
+            elif base_path is not None:
                 executability = base_executable
         if executability is not None:
             trans_id = self.tt.trans_id_file_id(file_id)
@@ -1667,6 +1663,7 @@
     supports_show_base = False
     supports_reverse_cherrypick = False
     history_based = True
+    requires_file_merge_plan = True
 
     def _generate_merge_plan(self, file_id, base):
         return self.this_tree.plan_file_merge(file_id, self.other_tree,
@@ -1697,11 +1694,12 @@
             base_lines = None
         return lines, base_lines
 
-    def text_merge(self, file_id, trans_id):
+    def text_merge(self, trans_id, paths, file_id):
         """Perform a (weave) text merge for a given file and file-id.
         If conflicts are encountered, .THIS and .OTHER files will be emitted,
         and a conflict will be noted.
         """
+        base_path, other_path, this_path = paths
         lines, base_lines = self._merged_lines(file_id)
         lines = list(lines)
         # Note we're checking whether the OUTPUT is binary in this case,
@@ -1713,7 +1711,7 @@
             self._raw_conflicts.append(('text conflict', trans_id))
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            file_group = self._dump_conflicts(name, parent_id, file_id,
+            file_group = self._dump_conflicts(name, paths, parent_id, file_id,
                                               no_base=False,
                                               base_lines=base_lines)
             file_group.append(trans_id)
@@ -1721,6 +1719,8 @@
 
 class LCAMerger(WeaveMerger):
 
+    requires_file_merge_plan = True
+
     def _generate_merge_plan(self, file_id, base):
         return self.this_tree.plan_file_lca_merge(file_id, self.other_tree,
                                                   base=base)
@@ -1728,41 +1728,38 @@
 class Diff3Merger(Merge3Merger):
     """Three-way merger using external diff3 for text merging"""
 
-    def dump_file(self, temp_dir, name, tree, file_id):
+    requires_file_merge_plan = False
+
+    def dump_file(self, temp_dir, name, tree, path, file_id=None):
         out_path = osutils.pathjoin(temp_dir, name)
-        out_file = open(out_path, "wb")
-        try:
-            in_file = tree.get_file(tree.id2path(file_id), file_id)
+        with open(out_path, "wb") as out_file:
+            in_file = tree.get_file(path, file_id=None)
             for line in in_file:
                 out_file.write(line)
-        finally:
-            out_file.close()
         return out_path
 
-    def text_merge(self, file_id, trans_id):
+    def text_merge(self, trans_id, paths, file_id):
         """Perform a diff3 merge using a specified file-id and trans-id.
         If conflicts are encountered, .BASE, .THIS. and .OTHER conflict files
         will be dumped, and a will be conflict noted.
         """
         import breezy.patch
+        base_path, other_path, this_path = paths
         temp_dir = osutils.mkdtemp(prefix="bzr-")
         try:
             new_file = osutils.pathjoin(temp_dir, "new")
-            this = self.dump_file(temp_dir, "this", self.this_tree, file_id)
-            base = self.dump_file(temp_dir, "base", self.base_tree, file_id)
-            other = self.dump_file(temp_dir, "other", self.other_tree, file_id)
+            this = self.dump_file(temp_dir, "this", self.this_tree, this_path, file_id)
+            base = self.dump_file(temp_dir, "base", self.base_tree, base_path, file_id)
+            other = self.dump_file(temp_dir, "other", self.other_tree, other_path, file_id)
             status = breezy.patch.diff3(new_file, this, base, other)
             if status not in (0, 1):
                 raise errors.BzrError("Unhandled diff3 exit code")
-            f = open(new_file, 'rb')
-            try:
+            with open(new_file, 'rb') as f:
                 self.tt.create_file(f, trans_id)
-            finally:
-                f.close()
             if status == 1:
                 name = self.tt.final_name(trans_id)
                 parent_id = self.tt.final_parent(trans_id)
-                self._dump_conflicts(name, parent_id, file_id)
+                self._dump_conflicts(name, paths, parent_id, file_id)
                 self._raw_conflicts.append(('text conflict', trans_id))
         finally:
             osutils.rmtree(temp_dir)
@@ -1819,7 +1816,7 @@
         self.merge_type = Merge3Merger
         self.show_base = False
         self.reprocess = False
-        self.interesting_ids = None
+        self.interesting_files = None
         self.merge_type = _MergeTypeParameterizer(MergeIntoMergeType,
               target_subdir=self._target_subdir,
               source_subpath=self._source_subpath)
@@ -1837,7 +1834,7 @@
 
 class _MergeTypeParameterizer(object):
     """Wrap a merge-type class to provide extra parameters.
-    
+
     This is hack used by MergeIntoMerger to pass some extra parameters to its
     merge_type.  Merger.do_merge() sets up its own set of parameters to pass to
     the 'merge_type' member.  It is difficult override do_merge without
@@ -1878,17 +1875,15 @@
         super(MergeIntoMergeType, self).__init__(*args, **kwargs)
 
     def _compute_transform(self):
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             entries = self._entries_to_incorporate()
             entries = list(entries)
-            for num, (entry, parent_id, path) in enumerate(entries):
+            for num, (entry, parent_id, relpath) in enumerate(entries):
                 child_pb.update(gettext('Preparing file merge'), num, len(entries))
                 parent_trans_id = self.tt.trans_id_file_id(parent_id)
+                path = osutils.pathjoin(self._source_subpath, relpath)
                 trans_id = transform.new_by_entry(path, self.tt, entry,
                     parent_trans_id, self.other_tree)
-        finally:
-            child_pb.finished()
         self._finish_computing_transform()
 
     def _entries_to_incorporate(self):
@@ -1933,7 +1928,6 @@
 def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
                 backup_files=False,
                 merge_type=Merge3Merger,
-                interesting_ids=None,
                 show_base=False,
                 reprocess=False,
                 other_rev_id=None,
@@ -1954,13 +1948,8 @@
                     change_reporter=change_reporter)
     merger.backup_files = backup_files
     merger.merge_type = merge_type
-    merger.interesting_ids = interesting_ids
     merger.ignore_zero = ignore_zero
-    if interesting_files:
-        if interesting_ids:
-            raise ValueError('Only supply interesting_ids'
-                             ' or interesting_files')
-        merger.interesting_files = interesting_files
+    merger.interesting_files = interesting_files
     merger.show_base = show_base
     merger.reprocess = reprocess
     merger.other_rev_id = other_rev_id

=== modified file 'breezy/merge_directive.py'
--- breezy/merge_directive.py	2017-10-27 00:18:42 +0000
+++ breezy/merge_directive.py	2018-03-11 22:13:09 +0000
@@ -251,7 +251,7 @@
         :return: a string
         """
         my_gpg = gpg.GPGStrategy(branch.get_config_stack())
-        return my_gpg.sign(''.join(self.to_lines()))
+        return my_gpg.sign(''.join(self.to_lines()), gpg.MODE_CLEAR)
 
     def to_email(self, mail_to, branch, sign=False):
         """Serialize as an email message.

=== modified file 'breezy/multiparent.py'
--- breezy/multiparent.py	2017-06-04 18:09:30 +0000
+++ breezy/multiparent.py	2018-02-24 15:50:23 +0000
@@ -403,8 +403,7 @@
             raise ValueError()
         revisions = set(vf.versions())
         total = len(revisions)
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             while len(revisions) > 0:
                 added = set()
                 for revision in revisions:
@@ -430,8 +429,6 @@
                     pb.update(gettext('Importing revisions'),
                               (total - len(revisions)) + len(added), total)
                 revisions = [r for r in revisions if r not in added]
-        finally:
-            pb.finished()
 
     def select_snapshots(self, vf):
         """Determine which versions to add as snapshots"""

=== modified file 'breezy/plugins/changelog_merge/changelog_merge.py'
--- breezy/plugins/changelog_merge/changelog_merge.py	2017-05-25 21:59:11 +0000
+++ breezy/plugins/changelog_merge/changelog_merge.py	2018-03-24 01:29:53 +0000
@@ -23,6 +23,7 @@
 from ... import (
     debug,
     merge,
+    osutils,
     urlutils,
     )
 from ...merge3 import Merge3
@@ -63,18 +64,25 @@
 
     name_prefix = "changelog"
 
-    def get_filepath(self, params, tree):
-        """Calculate the path to the file in a tree.
-
-        This is overridden to return just the basename, rather than full path,
-        so that e.g. if the config says ``changelog_merge_files = ChangeLog``,
-        then all ChangeLog files in the tree will match (not just one in the
-        root of the tree).
-        
-        :param params: A MergeHookParams describing the file to merge
-        :param tree: a Tree, e.g. self.merger.this_tree.
-        """
-        return urlutils.basename(tree.id2path(params.file_id))
+    def file_matches(self, params):
+        affected_files = self.affected_files
+        if affected_files is None:
+            config = self.merger.this_branch.get_config()
+            # Until bzr provides a better policy for caching the config, we
+            # just add the part we're interested in to the params to avoid
+            # reading the config files repeatedly (breezy.conf, location.conf,
+            # branch.conf).
+            config_key = self.name_prefix + '_merge_files'
+            affected_files = config.get_user_option_as_list(config_key)
+            if affected_files is None:
+                # If nothing was specified in the config, use the default.
+                affected_files = self.default_files
+            self.affected_files = affected_files
+        if affected_files:
+            filepath = osutils.basename(params.this_path)
+            if filepath in affected_files:
+                return True
+        return False
 
     def merge_text(self, params):
         """Merge changelog changes.

=== modified file 'breezy/plugins/changelog_merge/tests/test_changelog_merge.py'
--- breezy/plugins/changelog_merge/tests/test_changelog_merge.py	2017-05-21 18:10:28 +0000
+++ breezy/plugins/changelog_merge/tests/test_changelog_merge.py	2018-03-18 02:14:58 +0000
@@ -191,8 +191,9 @@
         # won't write the new value to disk where get_user_option can get it).
         merger.this_branch.get_config().set_user_option(
             'changelog_merge_files', 'ChangeLog')
-        merge_hook_params = merge.MergeFileHookParams(merger, 'clog-id', None,
-            'file', 'file', 'conflict')
+        merge_hook_params = merge.MergeFileHookParams(merger, 'clog-id',
+                ['ChangeLog', 'ChangeLog', 'ChangeLog'], None,
+                'file', 'file', 'conflict')
         changelog_merger = changelog_merge.ChangeLogMerger(merger)
         return changelog_merger, merge_hook_params
 

=== modified file 'breezy/plugins/fastimport/bzr_commit_handler.py'
--- breezy/plugins/fastimport/bzr_commit_handler.py	2018-02-17 02:02:38 +0000
+++ breezy/plugins/fastimport/bzr_commit_handler.py	2018-03-24 23:49:53 +0000
@@ -374,12 +374,9 @@
                 % (kind, path))
             return
         # Record it
-        if inv.has_id(file_id):
+        try:
             old_ie = inv[file_id]
-            if old_ie.kind == 'directory':
-                self.record_delete(path, old_ie)
-            self.record_changed(path, ie, parent_id)
-        else:
+        except errors.NoSuchId:
             try:
                 self.record_new(path, ie)
             except:
@@ -387,6 +384,10 @@
                     % (path, ie, self.command.id))
                 print("parent's children are:\n%r\n" % (ie.parent_id.children,))
                 raise
+        else:
+            if old_ie.kind == 'directory':
+                self.record_delete(path, old_ie)
+            self.record_changed(path, ie, parent_id)
 
     def _ensure_directory(self, path, inv):
         """Ensure that the containing directory exists for 'path'"""

=== modified file 'breezy/plugins/fastimport/revision_store.py'
--- breezy/plugins/fastimport/revision_store.py	2017-11-12 20:07:32 +0000
+++ breezy/plugins/fastimport/revision_store.py	2018-03-10 13:56:26 +0000
@@ -280,8 +280,11 @@
             list(parent_candidate_entries))
         heads = []
         for inv in self._rev_parent_invs:
-            if inv.has_id(ie.file_id):
+            try:
                 old_rev = inv[ie.file_id].revision
+            except errors.NoSuchId:
+                pass
+            else:
                 if old_rev in head_set:
                     rev_id = inv[ie.file_id].revision
                     heads.append(rev_id)

=== modified file 'breezy/plugins/grep/grep.py'
--- breezy/plugins/grep/grep.py	2017-11-12 17:53:47 +0000
+++ breezy/plugins/grep/grep.py	2018-03-24 10:24:48 +0000
@@ -356,9 +356,8 @@
             rev = RevisionSpec_revid.from_string("revid:"+revid)
             tree = rev.as_tree(branch)
             for path in opts.path_list:
-                path_for_id = osutils.pathjoin(relpath, path)
-                id = tree.path2id(path_for_id)
-                if not id:
+                tree_path = osutils.pathjoin(relpath, path)
+                if not tree.has_filename(tree_path):
                     trace.warning("Skipped unknown file '%s'." % path)
                     continue
 
@@ -366,7 +365,7 @@
                     path_prefix = path
                     dir_grep(tree, path, relpath, opts, revno, path_prefix)
                 else:
-                    versioned_file_grep(tree, id, '.', path, opts, revno)
+                    versioned_file_grep(tree, tree_path, '.', path, opts, revno)
 
 
 def workingtree_grep(opts):
@@ -408,8 +407,8 @@
     from_dir = osutils.pathjoin(relpath, path)
     if opts.from_root:
         # start searching recursively from root
-        from_dir=None
-        recursive=True
+        from_dir = None
+        recursive = True
 
     to_grep = []
     to_grep_append = to_grep.append
@@ -435,7 +434,7 @@
                     #                but it's what the old code seemed to do
                     outputter.write_cached_lines(cache_id, revno)
                 else:
-                    to_grep_append((fid, (fp, fid)))
+                    to_grep_append((osutils.pathjoin(from_dir if from_dir else '', fp), (fp, fid)))
             else:
                 # we are grepping working tree.
                 if from_dir is None:
@@ -454,7 +453,7 @@
     if revno != None: # grep versioned files
         for (path, fid), chunks in tree.iter_files_bytes(to_grep):
             path = _make_display_path(relpath, path)
-            _file_grep(chunks[0], path, opts, revno, path_prefix,
+            _file_grep(''.join(chunks), path, opts, revno, path_prefix,
                 tree.get_file_revision(path, fid))
 
 
@@ -473,12 +472,12 @@
     return path
 
 
-def versioned_file_grep(tree, id, relpath, path, opts, revno, path_prefix = None):
+def versioned_file_grep(tree, tree_path, relpath, path, opts, revno, path_prefix = None):
     """Create a file object for the specified id and pass it on to _file_grep.
     """
 
     path = _make_display_path(relpath, path)
-    file_text = tree.get_file_text(relpath, id)
+    file_text = tree.get_file_text(tree_path)
     _file_grep(file_text, path, opts, revno, path_prefix)
 
 

=== modified file 'breezy/plugins/launchpad/cmds.py'
--- breezy/plugins/launchpad/cmds.py	2018-02-09 23:09:53 +0000
+++ breezy/plugins/launchpad/cmds.py	2018-02-24 15:50:23 +0000
@@ -271,9 +271,7 @@
         from . import lp_api
         import webbrowser
         b = _mod_branch.Branch.open_containing('.')[0]
-        pb = ui.ui_factory.nested_progress_bar()
-        b.lock_read()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb, b.lock_read():
             if revision is None:
                 revision_id = b.last_revision()
             else:
@@ -284,9 +282,6 @@
             trace.note(gettext('%d proposals(s) found.') % len(merged))
             for mp in merged:
                 webbrowser.open(lp_api.canonical_url(mp))
-        finally:
-            b.unlock()
-            pb.finished()
 
     def _find_proposals(self, revision_id, pb):
         from . import (lp_api, lp_registration)

=== modified file 'breezy/plugins/launchpad/lp_registration.py'
--- breezy/plugins/launchpad/lp_registration.py	2017-09-26 00:55:04 +0000
+++ breezy/plugins/launchpad/lp_registration.py	2018-03-08 23:16:56 +0000
@@ -74,8 +74,7 @@
         self.verbose = verbose
         url = self._scheme + "://" + host + handler
         request = _urllib2_wrappers.Request("POST", url, request_body)
-        # FIXME: _urllib2_wrappers will override user-agent with its own
-        # request.add_header("User-Agent", self.user_agent)
+        request.add_header("User-Agent", self.user_agent)
         request.add_header("Content-Type", "text/xml")
 
         response = self._opener.open(request)
@@ -122,7 +121,7 @@
         if transport is None:
             uri_type = urllib.splittype(self.service_url)[0]
             transport = XMLRPCTransport(uri_type)
-            transport.user_agent = 'bzr/%s (xmlrpclib/%s)' \
+            transport.user_agent = 'Breezy/%s (xmlrpclib/%s)' \
                     % (_breezy_version, xmlrpclib.__version__)
         self.transport = transport
 

=== modified file 'breezy/plugins/po_merge/po_merge.py'
--- breezy/plugins/po_merge/po_merge.py	2017-05-22 00:56:52 +0000
+++ breezy/plugins/po_merge/po_merge.py	2018-03-18 02:14:58 +0000
@@ -99,7 +99,7 @@
             # Return early if there is no options defined
             return False
         po_dir = None
-        po_path = self.get_filepath(params, self.merger.this_tree)
+        po_path = params.this_path
         for po_dir in self.po_dirs:
             glob = osutils.pathjoin(po_dir, self.po_glob)
             if fnmatch.fnmatch(po_path, glob):

=== modified file 'breezy/plugins/stats/cmds.py'
--- breezy/plugins/stats/cmds.py	2017-11-12 13:53:51 +0000
+++ breezy/plugins/stats/cmds.py	2018-03-24 00:31:49 +0000
@@ -143,10 +143,9 @@
 
     email_users = {} # user@email.com => User Name
     combo_count = {}
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb:
         trace.note('getting revisions')
-        revisions = a_repo.iter_revisions(revids)
+        revisions = list(a_repo.iter_revisions(revids))
         for count, (revid, rev) in enumerate(revisions):
             pb.update('checking', count, len(revids))
             for author in rev.get_apparent_authors():
@@ -156,25 +155,19 @@
                 email_users.setdefault(email, set()).add(username)
                 combo = (username, email)
                 combo_count[combo] = combo_count.setdefault(combo, 0) + 1
-    finally:
-        pb.finished()
-    return revisions, collapse_email_and_users(email_users, combo_count)
+    return ((rev for (revid, rev) in revisions),
+            collapse_email_and_users(email_users, combo_count))
 
 
 def get_info(a_repo, revision):
     """Get all of the information for a particular revision"""
-    pb = ui.ui_factory.nested_progress_bar()
-    a_repo.lock_read()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb, a_repo.lock_read():
         trace.note('getting ancestry')
         graph = a_repo.get_graph()
         ancestry = [
             r for (r, ps) in graph.iter_ancestry([revision])
             if ps is not None and r != NULL_REVISION]
         revs, canonical_committer = get_revisions_and_committers(a_repo, ancestry)
-    finally:
-        a_repo.unlock()
-        pb.finished()
 
     return collapse_by_person(revs, canonical_committer)
 
@@ -184,16 +177,11 @@
 
     This lets us figure out what has actually changed between 2 revisions.
     """
-    pb = ui.ui_factory.nested_progress_bar()
-    a_repo.lock_read()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb, a_repo.lock_read():
         graph = a_repo.get_graph()
         trace.note('getting ancestry diff')
         ancestry = graph.find_difference(start_rev, end_rev)[1]
         revs, canonical_committer = get_revisions_and_committers(a_repo, ancestry)
-    finally:
-        a_repo.unlock()
-        pb.finished()
 
     return collapse_by_person(revs, canonical_committer)
 
@@ -314,8 +302,7 @@
 def gather_class_stats(repository, revs):
     ret = {}
     total = 0
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb:
         with repository.lock_read():
             i = 0
             for delta in repository.get_deltas_for_revisions(revs):
@@ -326,8 +313,6 @@
                     ret[c] += 1
                     total += 1
                 i += 1
-    finally:
-        pb.finished()
     return ret, total
 
 
@@ -367,8 +352,7 @@
         ancestry = [r for (r, ps) in graph.iter_ancestry([revid])
                     if ps is not None and r != NULL_REVISION]
         revs = repository.get_revisions(ancestry)
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             iterator = zip(revs, repository.get_deltas_for_revisions(revs))
             for i, (rev, delta) in enumerate(iterator):
                 pb.update("analysing revisions", i, len(revs))
@@ -380,8 +364,6 @@
                         if not author in ret[c]:
                             ret[c][author] = 0
                         ret[c][author] += 1
-        finally:
-            pb.finished()
     def sort_class(name):
         return [author
             for author, _  in sorted(ret[name].items(), key=classify_key)]

=== modified file 'breezy/plugins/stats/test_stats.py'
--- breezy/plugins/stats/test_stats.py	2018-02-18 15:21:06 +0000
+++ breezy/plugins/stats/test_stats.py	2018-03-25 00:39:16 +0000
@@ -45,6 +45,7 @@
         self.assertEqual({('Fero', ''): ('Fero', ''),
                           ('FERO', ''): ('Fero', ''),
                          }, committers)
+        self.assertEquals(['1', '2', '3'], sorted([r.revision_id for r in revs]))
 
 
 class TestCollapseByPerson(TestCase):

=== modified file 'breezy/plugins/upload/cmds.py'
--- breezy/plugins/upload/cmds.py	2017-11-12 20:07:32 +0000
+++ breezy/plugins/upload/cmds.py	2018-03-04 15:59:37 +0000
@@ -174,15 +174,15 @@
                     dir = os.path.dirname(dir)
         return ignored
 
-    def upload_file(self, relpath, id, mode=None):
+    def upload_file(self, old_relpath, new_relpath, id, mode=None):
         if mode is None:
-            if self.tree.is_executable(relpath, id):
+            if self.tree.is_executable(new_relpath, id):
                 mode = 0o775
             else:
                 mode = 0o664
         if not self.quiet:
-            self.outf.write('Uploading %s\n' % relpath)
-        self._up_put_bytes(relpath, self.tree.get_file_text(relpath, id), mode)
+            self.outf.write('Uploading %s\n' % old_relpath)
+        self._up_put_bytes(old_relpath, self.tree.get_file_text(new_relpath, id), mode)
 
     def upload_file_robustly(self, relpath, id, mode=None):
         """Upload a file, clearing the way on the remote side.
@@ -200,7 +200,7 @@
                 self._up_delete_tree(relpath)
         except errors.PathError:
             pass
-        self.upload_file(relpath, id, mode)
+        self.upload_file(relpath, relpath, id, mode)
 
     def make_remote_dir(self, relpath, mode=None):
         if mode is None:
@@ -364,7 +364,7 @@
                 if content_change:
                     # We update the old_path content because renames and
                     # deletions are differed.
-                    self.upload_file(old_path, id)
+                    self.upload_file(old_path, new_path, id)
                 if kind == 'symlink':
                     if not self.quiet:
                         self.outf.write('Not renaming remote symlink %s to %s\n'
@@ -387,7 +387,7 @@
                     raise NotImplementedError
 
                 if new_kind == 'file':
-                    self.upload_file(path, id)
+                    self.upload_file(path, path, id)
                 elif new_kind is 'directory':
                     self.make_remote_dir(path)
                 else:
@@ -399,7 +399,7 @@
                         self.outf.write('Ignoring %s\n' % path)
                     continue
                 if kind == 'file':
-                    self.upload_file(path, id)
+                    self.upload_file(path, path, id)
                 elif kind == 'directory':
                     self.make_remote_dir(path)
                 elif kind == 'symlink':
@@ -417,8 +417,8 @@
                     if not self.quiet:
                         self.outf.write('Ignoring %s\n' % path)
                     continue
-                if kind is 'file':
-                    self.upload_file(path, id)
+                if kind == 'file':
+                    self.upload_file(path, path, id)
                 else:
                     raise NotImplementedError
 

=== modified file 'breezy/plugins/weave_fmt/bzrdir.py'
--- breezy/plugins/weave_fmt/bzrdir.py	2018-02-17 00:06:35 +0000
+++ breezy/plugins/weave_fmt/bzrdir.py	2018-03-24 17:48:04 +0000
@@ -241,15 +241,12 @@
         self.controldir = to_convert
         if pb is not None:
             warnings.warn(gettext("pb parameter to convert() is deprecated"))
-        self.pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as self.pb:
             ui.ui_factory.note(gettext('starting upgrade from format 4 to 5'))
             if isinstance(self.controldir.transport, local.LocalTransport):
                 self.controldir.get_workingtree_transport(None).delete('stat-cache')
             self._convert_to_weaves()
             return ControlDir.open(self.controldir.user_url)
-        finally:
-            self.pb.finished()
 
     def _convert_to_weaves(self):
         ui.ui_factory.note(gettext(
@@ -508,13 +505,10 @@
     def convert(self, to_convert, pb):
         """See Converter.convert()."""
         self.controldir = to_convert
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as pb:
             ui.ui_factory.note(gettext('starting upgrade from format 5 to 6'))
             self._convert_to_prefixed()
             return ControlDir.open(self.controldir.user_url)
-        finally:
-            pb.finished()
 
     def _convert_to_prefixed(self):
         from .store import TransportStore

=== modified file 'breezy/plugins/weave_fmt/repository.py'
--- breezy/plugins/weave_fmt/repository.py	2018-02-18 19:18:40 +0000
+++ breezy/plugins/weave_fmt/repository.py	2018-03-25 00:39:16 +0000
@@ -800,8 +800,7 @@
                 pass
             # FIXME do not peek!
             if self.source._transport.listable():
-                pb = ui.ui_factory.nested_progress_bar()
-                try:
+                with ui.ui_factory.nested_progress_bar() as pb:
                     self.target.texts.insert_record_stream(
                         self.source.texts.get_record_stream(
                             self.source.texts.keys(), 'topological', False))
@@ -817,8 +816,6 @@
                         self.source.revisions.get_record_stream(
                             self.source.revisions.keys(),
                             'topological', True))
-                finally:
-                    pb.finished()
             else:
                 self.target.fetch(self.source, revision_id=revision_id)
 

=== modified file 'breezy/plugins/weave_fmt/workingtree.py'
--- breezy/plugins/weave_fmt/workingtree.py	2017-08-29 13:25:07 +0000
+++ breezy/plugins/weave_fmt/workingtree.py	2018-03-04 19:11:21 +0000
@@ -18,6 +18,8 @@
 
 from __future__ import absolute_import
 
+import breezy
+
 from ... import (
     conflicts as _mod_conflicts,
     errors,
@@ -65,6 +67,8 @@
 
     supports_versioned_directories = True
 
+    ignore_filename = '.bzrignore'
+
     def get_format_description(self):
         """See WorkingTreeFormat.get_format_description()."""
         return "Working tree format 2"

=== modified file 'breezy/progress.py'
--- breezy/progress.py	2017-05-21 18:10:28 +0000
+++ breezy/progress.py	2018-02-24 15:50:23 +0000
@@ -164,6 +164,13 @@
         else:
             self.ui_factory.clear_term()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.finished()
+        return False
+
 
 class DummyProgress(object):
     """Progress-bar standin that does nothing.

=== modified file 'breezy/reconcile.py'
--- breezy/reconcile.py	2017-06-10 16:40:42 +0000
+++ breezy/reconcile.py	2018-03-01 01:45:44 +0000
@@ -76,14 +76,14 @@
           branch history was correct, True if the branch history needed to be
           re-normalized.
         """
-        self.pb = ui.ui_factory.nested_progress_bar()
-        try:
-            self._reconcile()
-        finally:
-            self.pb.finished()
+        operation = cleanup.OperationWithCleanups(self._reconcile)
+        self.add_cleanup = operation.add_cleanup
+        operation.run_simple()
 
     def _reconcile(self):
         """Helper function for performing reconciliation."""
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.add_cleanup(self.pb.finished)
         self._reconcile_branch()
         self._reconcile_repository()
 

=== modified file 'breezy/rename_map.py'
--- breezy/rename_map.py	2018-02-15 19:38:33 +0000
+++ breezy/rename_map.py	2018-03-24 10:24:48 +0000
@@ -26,6 +26,7 @@
 from .sixish import (
     BytesIO,
     viewitems,
+    viewvalues,
     )
 from .ui import ui_factory
 
@@ -64,9 +65,8 @@
         :param tree: The tree containing the files.
         :param file_ids: A list of file_ids to perform the updates for.
         """
-        desired_files = [(f, f) for f in file_ids]
-        task = ui_factory.nested_progress_bar()
-        try:
+        desired_files = [(tree.id2path(f), f) for f in file_ids]
+        with ui_factory.nested_progress_bar() as task:
             for num, (file_id, contents) in enumerate(
                 tree.iter_files_bytes(desired_files)):
                 task.update(gettext('Calculating hashes'), num, len(file_ids))
@@ -74,8 +74,6 @@
                 s.writelines(contents)
                 s.seek(0)
                 self.add_edge_hashes(s.readlines(), file_id)
-        finally:
-            task.finished()
 
     def hitcounts(self, lines):
         """Count the number of hash hits for each tag, for the given lines.
@@ -104,14 +102,11 @@
         :return: A list of tuples of count, path, file_id.
         """
         all_hits = []
-        task = ui_factory.nested_progress_bar()
-        try:
+        with ui_factory.nested_progress_bar() as task:
             for num, path in enumerate(paths):
                 task.update(gettext('Determining hash hits'), num, len(paths))
                 hits = self.hitcounts(self.tree.get_file_lines(path))
                 all_hits.extend((v, path, k) for k, v in viewitems(hits))
-        finally:
-            task.finished()
         return all_hits
 
     def file_match(self, paths):
@@ -178,10 +173,9 @@
         missing_files = set()
         missing_parents = {}
         candidate_files = set()
-        task = ui_factory.nested_progress_bar()
-        iterator = self.tree.iter_changes(basis, want_unversioned=True,
-                                          pb=task)
-        try:
+        with ui_factory.nested_progress_bar() as task:
+            iterator = self.tree.iter_changes(basis, want_unversioned=True,
+                                              pb=task)
             for (file_id, paths, changed_content, versioned, parent, name,
                  kind, executable) in iterator:
                 if kind[1] is None and versioned[1]:
@@ -202,34 +196,28 @@
                             for child in children:
                                 if child[2] == 'file':
                                     candidate_files.add(child[0])
-        finally:
-            task.finished()
         return missing_files, missing_parents, candidate_files
 
     @classmethod
-    def guess_renames(klass, tree, dry_run=False):
+    def guess_renames(klass, from_tree, to_tree, dry_run=False):
         """Guess which files to rename, and perform the rename.
 
         We assume that unversioned files and missing files indicate that
         versioned files have been renamed outside of Bazaar.
 
-        :param tree: A write-locked working tree.
+        :param from_tree: A tree to compare from
+        :param to_tree: A write-locked working tree.
         """
         required_parents = {}
-        task = ui_factory.nested_progress_bar()
-        try:
+        with ui_factory.nested_progress_bar() as task:
             pp = progress.ProgressPhase('Guessing renames', 4, task)
-            basis = tree.basis_tree()
-            basis.lock_read()
-            try:
-                rn = klass(tree)
+            with from_tree.lock_read():
+                rn = klass(to_tree)
                 pp.next_phase()
                 missing_files, missing_parents, candidate_files = (
-                    rn._find_missing_files(basis))
+                    rn._find_missing_files(from_tree))
                 pp.next_phase()
-                rn.add_file_edge_hashes(basis, missing_files)
-            finally:
-                basis.unlock()
+                rn.add_file_edge_hashes(from_tree, missing_files)
             pp.next_phase()
             matches = rn.file_match(candidate_files)
             parents_matches = matches
@@ -244,15 +232,20 @@
             for old, new, file_id, entry in delta:
                 trace.note( gettext("{0} => {1}").format(old, new) )
             if not dry_run:
-                tree.add(required_parents)
-                tree.apply_inventory_delta(delta)
-        finally:
-            task.finished()
+                to_tree.add(required_parents)
+                to_tree.apply_inventory_delta(delta)
 
     def _make_inventory_delta(self, matches):
         delta = []
         file_id_matches = dict((f, p) for p, f in viewitems(matches))
-        for old_path, entry in self.tree.iter_entries_by_dir(file_id_matches):
+        file_id_query = []
+        for f in viewvalues(matches):
+            try:
+                file_id_query.append(self.tree.id2path(f))
+            except errors.NoSuchId:
+                pass
+        for old_path, entry in self.tree.iter_entries_by_dir(
+                specific_files=file_id_query):
             new_path = file_id_matches[entry.file_id]
             parent_path, new_name = osutils.split(new_path)
             parent_id = matches.get(parent_path)

=== modified file 'breezy/repository.py'
--- breezy/repository.py	2018-02-18 19:18:40 +0000
+++ breezy/repository.py	2018-03-25 00:39:16 +0000
@@ -887,7 +887,7 @@
 
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
         with self.lock_write():
-            signature = gpg_strategy.sign(plaintext)
+            signature = gpg_strategy.sign(plaintext, gpg.MODE_CLEAR)
             self.add_signature_text(revision_id, signature)
 
     def add_signature_text(self, revision_id, signature):
@@ -1104,11 +1104,12 @@
                 return gpg.SIGNATURE_NOT_SIGNED, None
             signature = self.get_signature_text(revision_id)
 
-            testament = _mod_testament.Testament.from_revision(
-                    self, revision_id)
-            plaintext = testament.as_short_text()
+            testament = _mod_testament.Testament.from_revision(self, revision_id)
 
-            return gpg_strategy.verify(signature, plaintext)
+            (status, key, signed_plaintext) = gpg_strategy.verify(signature)
+            if testament.as_short_text() != signed_plaintext:
+                return gpg.SIGNATURE_NOT_VALID, None
+            return (status, key)
 
     def verify_revision_signatures(self, revision_ids, gpg_strategy):
         """Verify revision signatures for a number of revisions.
@@ -1579,33 +1580,32 @@
         :param to_convert: The disk object to convert.
         :param pb: a progress bar to use for progress information.
         """
-        pb = ui.ui_factory.nested_progress_bar()
-        self.count = 0
-        self.total = 4
-        # this is only useful with metadir layouts - separated repo content.
-        # trigger an assertion if not such
-        repo._format.get_format_string()
-        self.repo_dir = repo.controldir
-        pb.update(gettext('Moving repository to repository.backup'))
-        self.repo_dir.transport.move('repository', 'repository.backup')
-        backup_transport =  self.repo_dir.transport.clone('repository.backup')
-        repo._format.check_conversion_target(self.target_format)
-        self.source_repo = repo._format.open(self.repo_dir,
-            _found=True,
-            _override_transport=backup_transport)
-        pb.update(gettext('Creating new repository'))
-        converted = self.target_format.initialize(self.repo_dir,
-                                                  self.source_repo.is_shared())
-        converted.lock_write()
-        try:
-            pb.update(gettext('Copying content'))
-            self.source_repo.copy_content_into(converted)
-        finally:
-            converted.unlock()
-        pb.update(gettext('Deleting old repository content'))
-        self.repo_dir.transport.delete_tree('repository.backup')
-        ui.ui_factory.note(gettext('repository converted'))
-        pb.finished()
+        with ui.ui_factory.nested_progress_bar() as pb:
+            self.count = 0
+            self.total = 4
+            # this is only useful with metadir layouts - separated repo content.
+            # trigger an assertion if not such
+            repo._format.get_format_string()
+            self.repo_dir = repo.controldir
+            pb.update(gettext('Moving repository to repository.backup'))
+            self.repo_dir.transport.move('repository', 'repository.backup')
+            backup_transport =  self.repo_dir.transport.clone('repository.backup')
+            repo._format.check_conversion_target(self.target_format)
+            self.source_repo = repo._format.open(self.repo_dir,
+                _found=True,
+                _override_transport=backup_transport)
+            pb.update(gettext('Creating new repository'))
+            converted = self.target_format.initialize(self.repo_dir,
+                                                      self.source_repo.is_shared())
+            converted.lock_write()
+            try:
+                pb.update(gettext('Copying content'))
+                self.source_repo.copy_content_into(converted)
+            finally:
+                converted.unlock()
+            pb.update(gettext('Deleting old repository content'))
+            self.repo_dir.transport.delete_tree('repository.backup')
+            ui.ui_factory.note(gettext('repository converted'))
 
 
 def _strip_NULL_ghosts(revision_graph):

=== modified file 'breezy/revisiontree.py'
--- breezy/revisiontree.py	2017-11-12 17:53:35 +0000
+++ breezy/revisiontree.py	2018-03-04 15:59:37 +0000
@@ -69,9 +69,7 @@
         raise NotImplementedError(self.get_file_revision)
 
     def get_file_text(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        for (identifier, content) in self.iter_files_bytes([(file_id, None)]):
+        for (identifier, content) in self.iter_files_bytes([(path, None)]):
             ret = "".join(content)
         return ret
 

=== modified file 'breezy/shelf_ui.py'
--- breezy/shelf_ui.py	2017-11-13 22:07:48 +0000
+++ breezy/shelf_ui.py	2018-03-24 10:24:48 +0000
@@ -324,7 +324,7 @@
         if self.reporter.invert_diff:
             target_lines = work_tree_lines
         else:
-            path = self.target_tree.path2id(file_id)
+            path = self.target_tree.id2path(file_id)
             target_lines = self.target_tree.get_file_lines(path, file_id)
         textfile.check_text_lines(work_tree_lines)
         textfile.check_text_lines(target_lines)

=== modified file 'breezy/tests/__init__.py'
--- breezy/tests/__init__.py	2018-02-15 19:38:33 +0000
+++ breezy/tests/__init__.py	2018-03-24 15:16:49 +0000
@@ -1959,11 +1959,12 @@
             os.chdir(working_dir)
 
         try:
-            result = self.apply_redirected(
-                ui.ui_factory.stdin,
-                stdout, stderr,
-                _mod_commands.run_bzr_catch_user_errors,
-                args)
+            with ui.ui_factory:
+                result = self.apply_redirected(
+                    ui.ui_factory.stdin,
+                    stdout, stderr,
+                    _mod_commands.run_bzr_catch_user_errors,
+                    args)
         finally:
             logger.removeHandler(handler)
             ui.ui_factory = old_ui_factory
@@ -2693,7 +2694,7 @@
     def make_branch_and_memory_tree(self, relpath, format=None):
         """Create a branch on the default transport and a MemoryTree for it."""
         b = self.make_branch(relpath, format=format)
-        return memorytree.MemoryTree.create_on_branch(b)
+        return b.create_memorytree()
 
     def make_branch_builder(self, relpath, format=None):
         branch = self.make_branch(relpath, format=format)

=== modified file 'breezy/tests/blackbox/test_branch.py'
--- breezy/tests/blackbox/test_branch.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/blackbox/test_branch.py	2018-03-25 00:39:16 +0000
@@ -81,7 +81,7 @@
         out, err = self.run_bzr(
             'init --format=development-colo file:b,branch=orig')
         self.assertEqual(
-            """Created a lightweight checkout (format: development-colo)\n""",
+            """Created a standalone tree (format: development-colo)\n""",
             out)
         self.assertEqual('', err)
         out, err = self.run_bzr(

=== modified file 'breezy/tests/blackbox/test_cat.py'
--- breezy/tests/blackbox/test_cat.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/blackbox/test_cat.py	2018-03-25 00:39:16 +0000
@@ -79,8 +79,7 @@
         # tree and the working tree
         self.build_tree_contents([('a-rev-tree', b'foo\n'),
             ('c-rev', b'baz\n'), ('d-rev', b'bar\n'), ('e-rev', b'qux\n')])
-        tree.lock_write()
-        try:
+        with tree.lock_write():
             tree.add(['a-rev-tree', 'c-rev', 'd-rev', 'e-rev'])
             tree.commit('add test files', rev_id=b'first')
             # remove currently uses self._write_inventory -
@@ -92,9 +91,6 @@
             tree.rename_one('e-rev', 'old-rev')
             self.build_tree_contents([('e-rev', b'new\n')])
             tree.add(['e-rev'])
-        finally:
-            # calling brz as another process require free lock on win32
-            tree.unlock()
 
         # 'b-tree' is not present in the old tree.
         self.run_bzr_error(["^brz: ERROR: u?'b-tree' "
@@ -103,8 +99,8 @@
 
         # get to the old file automatically
         out, err = self.run_bzr('cat d-rev')
+        self.assertEqual('', err)
         self.assertEqual('bar\n', out)
-        self.assertEqual('', err)
 
         out, err = \
                 self.run_bzr('cat a-rev-tree --name-from-revision')

=== modified file 'breezy/tests/blackbox/test_config.py'
--- breezy/tests/blackbox/test_config.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/blackbox/test_config.py	2018-03-25 00:39:16 +0000
@@ -116,20 +116,20 @@
             ''')
 
     def test_registry_value_all(self):
-        self.breezy_config.set_user_option('bzr.transform.orphan_policy',
+        self.breezy_config.set_user_option('transform.orphan_policy',
                                            u'move')
         script.run_script(self, '''\
             $ brz config -d tree
             breezy:
               [DEFAULT]
-              bzr.transform.orphan_policy = move
+              transform.orphan_policy = move
             ''')
 
     def test_registry_value_one(self):
-        self.breezy_config.set_user_option('bzr.transform.orphan_policy',
+        self.breezy_config.set_user_option('transform.orphan_policy',
                                            u'move')
         script.run_script(self, '''\
-            $ brz config -d tree bzr.transform.orphan_policy
+            $ brz config -d tree transform.orphan_policy
             move
             ''')
 

=== modified file 'breezy/tests/blackbox/test_init.py'
--- breezy/tests/blackbox/test_init.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/blackbox/test_init.py	2018-03-25 00:39:16 +0000
@@ -57,7 +57,7 @@
     def test_init_colocated(self):
         """Smoke test for constructing a colocated branch."""
         out, err = self.run_bzr('init --format=development-colo file:,branch=abranch')
-        self.assertEqual("""Created a lightweight checkout (format: development-colo)\n""",
+        self.assertEqual("""Created a standalone tree (format: development-colo)\n""",
             out)
         self.assertEqual('', err)
         out, err = self.run_bzr('branches')

=== modified file 'breezy/tests/blackbox/test_merge.py'
--- breezy/tests/blackbox/test_merge.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/blackbox/test_merge.py	2018-03-25 00:39:16 +0000
@@ -148,7 +148,7 @@
                              '=======\n'
                              'D\n'
                              '>>>>>>> MERGE-SOURCE\n',
-                             tree.get_file_text(tree.id2path(b'f-id')))
+                             tree.get_file_text('fname'))
 
     def test_merge_explicit_reprocess_show_base(self):
         tree, other = self.create_conflicting_branches()
@@ -170,7 +170,7 @@
                              'B\n'
                              'D\n'
                              '>>>>>>> MERGE-SOURCE\n',
-                             tree.get_file_text(tree.id2path(b'f-id')))
+                             tree.get_file_text('fname'))
 
     def test_merge_override_show_base(self):
         tree, other = self.create_conflicting_branches()
@@ -188,7 +188,7 @@
                              'B\n'
                              'D\n'
                              '>>>>>>> MERGE-SOURCE\n',
-                             tree.get_file_text(tree.id2path(b'f-id')))
+                             tree.get_file_text('fname'))
 
     def test_merge_with_missing_file(self):
         """Merge handles missing file conflicts"""

=== modified file 'breezy/tests/matchers.py'
--- breezy/tests/matchers.py	2017-08-26 13:58:53 +0000
+++ breezy/tests/matchers.py	2018-03-24 13:41:51 +0000
@@ -28,6 +28,7 @@
 
 __all__ = [
     'HasLayout',
+    'HasPathRelations',
     'MatchesAncestry',
     'ContainsNoVfsCalls',
     'ReturnsUnlockable',
@@ -47,6 +48,7 @@
 from ..sixish import (
     text_type,
     )
+from ..tree import find_previous_path
 
 from testtools.matchers import Equals, Mismatch, Matcher
 
@@ -182,6 +184,75 @@
         return Equals(entries).match(actual)
 
 
+class HasPathRelations(Matcher):
+    """Matcher verifies that paths have a relation to those in another tree.
+
+    :ivar previous_tree: tree to compare to
+    :ivar previous_entries: List of expected entries, as (path, previous_path) pairs.
+    """
+
+    def __init__(self, previous_tree, previous_entries):
+        Matcher.__init__(self)
+        self.previous_tree = previous_tree
+        self.previous_entries = previous_entries
+
+    def get_path_map(self, tree):
+        """Get the (path, previous_path) pairs for the current tree."""
+        with tree.lock_read(), self.previous_tree.lock_read():
+            for path, ie in tree.iter_entries_by_dir():
+                if tree.supports_rename_tracking():
+                    previous_path = find_previous_path(tree, self.previous_tree, path)
+                else:
+                    if self.previous_tree.is_versioned(path):
+                        previous_path = path
+                    else:
+                        previous_path = None
+                if previous_path:
+                    kind = self.previous_tree.kind(previous_path)
+                    if kind == 'directory':
+                        previous_path += '/'
+                if ie.parent_id is None:
+                    yield (u"", previous_path)
+                else:
+                    yield (path+ie.kind_character(), previous_path)
+
+    @staticmethod
+    def _strip_unreferenced_directories(entries):
+        """Strip all directories that don't (in)directly contain any files.
+
+        :param entries: List of path strings or (path, previous_path) tuples to process
+        """
+        directory_used = set()
+        directories = []
+        for (path, previous_path) in entries:
+            if not path or path[-1] == "/":
+                # directory
+                directories.append((path, previous_path))
+            else:
+                # Yield the referenced parent directories
+                for direntry in directories:
+                    if osutils.is_inside(direntry[0], path):
+                        directory_used.add(direntry[0])
+        for (path, previous_path) in entries:
+            if (not path.endswith("/")) or path in directory_used:
+                yield (path, previous_path)
+
+    def __str__(self):
+        return 'HasPathRelations(%r, %r)' % (self.previous_tree, self.previous_entries)
+
+    def match(self, tree):
+        actual = list(self.get_path_map(tree))
+        if not tree.has_versioned_directories():
+            entries = list(self._strip_unreferenced_directories(self.previous_entries))
+        else:
+            entries = self.previous_entries
+        if not tree.supports_rename_tracking():
+            entries = [
+                (path, path if self.previous_tree.is_versioned(path) else None)
+                for (path, previous_path) in entries]
+        return Equals(entries).match(actual)
+
+
 class RevisionHistoryMatches(Matcher):
     """A matcher that checks if a branch has a specific revision history.
 

=== modified file 'breezy/tests/per_branch/test_branch.py'
--- breezy/tests/per_branch/test_branch.py	2018-02-18 19:18:40 +0000
+++ breezy/tests/per_branch/test_branch.py	2018-03-25 00:39:16 +0000
@@ -34,6 +34,7 @@
     revision,
     shelf,
     tests,
+    tree as _mod_tree,
     )
 from breezy.bzr import (
     branch as _mod_bzrbranch,
@@ -162,7 +163,7 @@
         rev1 = wt_a.commit('commit one')
 
         branch_b = wt_a.branch.controldir.sprout('b', revision_id=rev1).open_branch()
-        self.assertEqual(wt_a.branch.base, branch_b.get_parent())
+        self.assertEqual(wt_a.branch.user_url, branch_b.get_parent())
         return branch_b
 
     def test_clone_branch_nickname(self):
@@ -424,7 +425,12 @@
         branch_a = tree_a.branch
         checkout_b = branch_a.create_checkout('b')
         self.assertEqual(b'null:', checkout_b.last_revision())
-        rev1 = checkout_b.commit('rev1')
+        try:
+            rev1 = checkout_b.commit('rev1')
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                    'roundtripping between %r and %r not supported' %
+                    (checkout_b.branch, checkout_b.branch.get_master_branch()))
         self.assertEqual(rev1, branch_a.last_revision())
         self.assertNotEqual(checkout_b.branch.base, branch_a.base)
 
@@ -492,6 +498,10 @@
         self.assertFalse(revision.NULL_REVISION in must_fetch)
         self.assertFalse(revision.NULL_REVISION in should_fetch)
 
+    def test_create_memorytree(self):
+        tree = self.make_branch_and_tree('a')
+        self.assertIsInstance(tree.branch.create_memorytree(), _mod_tree.Tree)
+
 
 class TestBranchFormat(per_branch.TestCaseWithBranch):
 
@@ -628,9 +638,22 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        made_branch = self.make_branch('.')
+        made_controldir = self.make_controldir('.')
+        made_controldir.create_repository()
+        if made_controldir._format.colocated_branches:
+            # Formats that support colocated branches sometimes have a default
+            # branch that is a reference branch (such as Git). Cope with
+            # those by creating a different colocated branch.
+            name = 'foo'
+        else:
+            name = None
+        try:
+            made_branch = made_controldir.create_branch(name)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable('Uninitializable branch format')
+
         self.assertEqual(None,
-            made_branch._format.get_reference(made_branch.controldir))
+            made_branch._format.get_reference(made_branch.controldir, name))
 
     def test_set_reference(self):
         """set_reference on all regular branches should be callable."""
@@ -649,7 +672,7 @@
             pass
         else:
             ref = this_branch._format.get_reference(this_branch.controldir)
-            self.assertEqual(ref, other_branch.base)
+            self.assertEqual(ref, other_branch.user_url)
 
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
@@ -1022,9 +1045,6 @@
         br = self.make_branch('branch')
         self.assertIsInstance(br.user_url, str)
         self.assertEqual(br.user_url, br.user_transport.base)
-        # for all current bzrdir implementations the user dir must be 
-        # above the control dir but we might need to relax that?
-        self.assertEqual(br.control_url.find(br.user_url), 0)
         self.assertEqual(br.control_url, br.control_transport.base)
 
 

=== modified file 'breezy/tests/per_branch/test_commit.py'
--- breezy/tests/per_branch/test_commit.py	2018-01-12 08:52:43 +0000
+++ breezy/tests/per_branch/test_commit.py	2018-03-20 07:41:20 +0000
@@ -194,7 +194,8 @@
             dir_id = tree.path2id('dir')
             tree.add('dir/subfile')
             dir_subfile_id = tree.path2id('dir/subfile')
-            tree.mkdir('to_be_unversioned')
+            tree.put_file_bytes_non_atomic('to_be_unversioned', 'blah')
+            tree.add(['to_be_unversioned'])
             to_be_unversioned_id = tree.path2id('to_be_unversioned')
             tree.put_file_bytes_non_atomic('dir/subfile', 'def')
             revid1 = tree.commit('first revision')
@@ -212,11 +213,19 @@
             revid2 = tree.commit('second revision')
 
         expected_delta = delta.TreeDelta()
-        expected_delta.added = [('added_dir', added_dir_id, 'directory')]
-        expected_delta.removed = [('to_be_unversioned',
-                                   to_be_unversioned_id, 'directory')]
-        expected_delta.renamed = [('dir/subfile', 'dir/subfile_renamed',
-                                   dir_subfile_id, 'file', False, False)]
+        if tree.has_versioned_directories():
+            expected_delta.added.append(('added_dir', added_dir_id, 'directory'))
+        if tree.supports_rename_tracking():
+            expected_delta.removed = [('to_be_unversioned',
+                                       to_be_unversioned_id, 'file')]
+            expected_delta.renamed = [('dir/subfile', 'dir/subfile_renamed',
+                                       dir_subfile_id, 'file', False, False)]
+        else:
+            expected_delta.added.append(('dir/subfile_renamed',
+                                        tree.path2id('dir/subfile_renamed'), 'file'))
+            expected_delta.removed = [
+                    ('dir/subfile', dir_subfile_id, 'file'),
+                    ('to_be_unversioned', to_be_unversioned_id, 'file')]
         expected_delta.modified=[('rootfile', rootfile_id, 'file', True,
                                   False)]
         self.assertEqual([('pre_commit', 1, revid1, 2, revid2,

=== modified file 'breezy/tests/per_branch/test_http.py'
--- breezy/tests/per_branch/test_http.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/per_branch/test_http.py	2018-03-04 02:57:55 +0000
@@ -75,4 +75,4 @@
         # from, even if that branch has an invalid parent.
         branch_b = self.get_branch_with_invalid_parent()
         branch_c = branch_b.controldir.sprout('c').open_branch()
-        self.assertEqual(branch_b.base, branch_c.get_parent())
+        self.assertEqual(branch_b.user_url, branch_c.get_parent())

=== modified file 'breezy/tests/per_branch/test_pull.py'
--- breezy/tests/per_branch/test_pull.py	2017-08-07 19:15:59 +0000
+++ breezy/tests/per_branch/test_pull.py	2018-03-01 00:15:57 +0000
@@ -121,8 +121,8 @@
         tree_a = self.make_branch_and_tree('tree_a')
         tree_a.commit('message 1')
         tree_b = tree_a.controldir.sprout('tree_b').open_workingtree()
-        rev2a = tree_a.commit('message 2')
-        rev2b = tree_b.commit('message 2')
+        rev2a = tree_a.commit('message 2a')
+        rev2b = tree_b.commit('message 2b')
         self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
         self.assertRaises(errors.DivergedBranches,
                           tree_a.branch.pull, tree_b.branch,

=== modified file 'breezy/tests/per_branch/test_push.py'
--- breezy/tests/per_branch/test_push.py	2017-12-03 21:05:52 +0000
+++ breezy/tests/per_branch/test_push.py	2018-02-27 18:44:59 +0000
@@ -223,13 +223,16 @@
         # See https://bugs.launchpad.net/bzr/+bug/465517
         t = self.get_transport('target')
         t.ensure_base()
-        bzrdir = self.bzrdir_format.initialize_on_transport(t)
+        try:
+            bzrdir = self.bzrdir_format.initialize_on_transport(t)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable('cannot initialize this format')
         try:
             bzrdir.open_branch()
         except errors.NotBranchError:
             pass
         else:
-            raise tests.TestNotApplicable('older formats can\'t have a repo'
+            raise tests.TestNotApplicable('some formats can\'t have a repo'
                                           ' without a branch')
         try:
             source = self.make_branch_builder('source',

=== modified file 'breezy/tests/per_branch/test_sprout.py'
--- breezy/tests/per_branch/test_sprout.py	2017-11-12 20:07:32 +0000
+++ breezy/tests/per_branch/test_sprout.py	2018-03-04 02:57:55 +0000
@@ -43,7 +43,7 @@
     def test_sprout_branch_parent(self):
         source = self.make_branch('source')
         target = source.controldir.sprout(self.get_url('target')).open_branch()
-        self.assertEqual(source.controldir.root_transport.base, target.get_parent())
+        self.assertEqual(source.user_url, target.get_parent())
 
     def test_sprout_uses_bzrdir_branch_format(self):
         # branch.sprout(bzrdir) is defined as using the branch format selected

=== modified file 'breezy/tests/per_controldir/test_controldir.py'
--- breezy/tests/per_controldir/test_controldir.py	2017-11-23 02:44:41 +0000
+++ breezy/tests/per_controldir/test_controldir.py	2018-03-24 14:47:01 +0000
@@ -450,9 +450,7 @@
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("control directory does not "
                 "support branch references")
-        self.assertEqual(
-            referenced_branch.controldir.root_transport.abspath('') + '/',
-            dir.get_branch_reference())
+        self.assertEqual(referenced_branch.user_url, dir.get_branch_reference())
 
     def test_set_branch_reference_on_existing_reference(self):
         """set_branch_reference creates a branch reference"""
@@ -467,7 +465,7 @@
                 "support branch references")
         reference = dir.set_branch_reference(referenced_branch2)
         self.assertEqual(
-            referenced_branch2.controldir.root_transport.abspath('') + '/',
+            referenced_branch2.user_url,
             dir.get_branch_reference())
 
     def test_set_branch_reference_on_existing_branch(self):
@@ -481,7 +479,7 @@
             raise TestNotApplicable("control directory does not "
                 "support branch references")
         self.assertEqual(
-            referenced_branch.controldir.root_transport.abspath('') + '/',
+            referenced_branch.user_url,
             dir.get_branch_reference())
 
     def test_get_branch_reference_on_reference(self):
@@ -494,13 +492,21 @@
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("control directory does not "
                 "support branch references")
-        self.assertEqual(referenced_branch.controldir.root_transport.abspath('') + '/',
+        self.assertEqual(referenced_branch.user_url,
             dir.get_branch_reference())
 
     def test_get_branch_reference_on_non_reference(self):
         """get_branch_reference should return None for non-reference branches."""
-        branch = self.make_branch('referenced')
-        self.assertEqual(None, branch.controldir.get_branch_reference())
+        dir = self.make_controldir('referenced')
+        dir.create_repository()
+        if dir._format.colocated_branches:
+            # The default branch in a controldir might be a reference branch
+            # (e.g. for Git), so let's create another one.
+            name = 'foo'
+        else:
+            name = None
+        branch = dir.create_branch(name)
+        self.assertEqual(None, branch.controldir.get_branch_reference(name))
 
     def test_get_branch_reference_no_branch(self):
         """get_branch_reference should not mask NotBranchErrors."""
@@ -1607,11 +1613,8 @@
             # if its default updatable there must be an updater
             # (we force the latest known format as downgrades may not be
             # available
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
+            with ui.ui_factory.nested_progress_bar() as pb:
                 dir._format.get_converter(format=dir._format).convert(dir, pb)
-            finally:
-                pb.finished()
             # and it should pass 'check' now.
             check.check_dwim(self.get_url('.'), False, True, True)
 
@@ -1670,16 +1673,19 @@
                     "format does not support branch references")
         unused_repo = thisdir.create_repository()
         master.lock_write()
-        unused_repo.lock_write()
-        try:
+        with unused_repo.lock_write():
             # two yes's : branch and repository. If the repo in this
             # dir is inappropriately accessed, 3 will be needed, and
             # we'll see that because the stream will be fully consumed
             breezy.ui.ui_factory = CannedInputUIFactory([True, True, True])
             # determine if the repository will have been locked;
             this_repo_locked = \
-                thisdir.open_repository().get_physical_lock_status()
-            master.controldir.break_lock()
+                thisdir.find_repository().get_physical_lock_status()
+            try:
+                master.controldir.break_lock()
+            except NotImplementedError:
+                # bzrdir does not support break_lock
+                raise TestNotApplicable("format does not support breaking locks")
             if this_repo_locked:
                 # only two ys should have been read
                 self.assertEqual([True], breezy.ui.ui_factory.responses)
@@ -1696,8 +1702,6 @@
                 # lock should not have touched it.
                 repo = thisdir.open_repository()
                 self.assertRaises(errors.LockContention, repo.lock_write)
-        finally:
-            unused_repo.unlock()
         self.assertRaises(errors.LockBroken, master.unlock)
 
     def test_break_lock_tree(self):

=== modified file 'breezy/tests/per_controldir_colo/test_supported.py'
--- breezy/tests/per_controldir_colo/test_supported.py	2017-06-10 00:52:08 +0000
+++ breezy/tests/per_controldir_colo/test_supported.py	2018-03-04 03:27:09 +0000
@@ -18,6 +18,7 @@
 
 from breezy.branch import Branch
 from breezy import (
+    branchbuilder,
     errors,
     tests,
     urlutils,
@@ -32,10 +33,18 @@
 
 class TestColocatedBranchSupport(per_controldir.TestCaseWithControlDir):
 
+    def create_branch(self, bzrdir, name=None):
+        branch = bzrdir.create_branch(name=name)
+        # Create a commit on the branch, just because some formats
+        # have nascent branches that don't hit disk.
+        bb = branchbuilder.BranchBuilder(branch=branch)
+        bb.build_commit()
+        return branch
+
     def test_destroy_colocated_branch(self):
         branch = self.make_branch('branch')
         bzrdir = branch.controldir
-        colo_branch = bzrdir.create_branch('colo')
+        colo_branch = self.create_branch(bzrdir, 'colo')
         try:
             bzrdir.destroy_branch("colo")
         except (errors.UnsupportedOperation, errors.TransportNotPossible):
@@ -76,8 +85,8 @@
             raise tests.TestNotApplicable(
                 'Control dir does not support creating new branches.')
         made_control.create_repository()
-        made_branch = made_control.create_branch(name="colo")
-        other_branch = made_control.create_branch(name="othercolo")
+        made_branch = self.create_branch(made_control, name="colo")
+        other_branch = self.create_branch(made_control, name="othercolo")
         self.assertIsInstance(made_branch, Branch)
         self.assertEqual(made_control, made_branch.controldir)
         self.assertNotEqual(made_branch.user_url, other_branch.user_url)
@@ -121,28 +130,29 @@
             raise tests.TestNotApplicable(
                 'Control dir does not support creating new branches.')
         made_control.create_repository()
-        made_branch = made_control.create_branch(name=u"col\xe9")
-        self.assertTrue(
-            u"col\xe9" in [b.name for b in made_control.list_branches()])
+        made_branch = self.create_branch(made_control, name=u"col\xe9")
+        self.assertIn(
+            u"col\xe9", [b.name for b in made_control.list_branches()])
         made_branch = Branch.open(made_branch.user_url)
         self.assertEqual(u"col\xe9", made_branch.name)
         made_control.destroy_branch(u"col\xe9")
 
     def test_get_branches(self):
         repo = self.make_repository('branch-1')
-        target_branch = repo.controldir.create_branch(name='foo')
-        self.assertEqual(['foo'], list(repo.controldir.get_branches()))
+        self.assertNotIn('foo', list(repo.controldir.get_branches()))
+        target_branch = self.create_branch(repo.controldir, name='foo')
+        self.assertIn('foo', list(repo.controldir.get_branches()))
         self.assertEqual(target_branch.base,
                          repo.controldir.get_branches()['foo'].base)
 
     def test_branch_name_with_slash(self):
         repo = self.make_repository('branch-1')
         try:
-            target_branch = repo.controldir.create_branch(name='foo/bar')
+            target_branch = self.create_branch(repo.controldir, name='foo/bar')
         except errors.InvalidBranchName:
             raise tests.TestNotApplicable(
                 "format does not support branches with / in their name")
-        self.assertEqual(['foo/bar'], list(repo.controldir.get_branches()))
+        self.assertIn('foo/bar', list(repo.controldir.get_branches()))
         self.assertEqual(
             target_branch.base, repo.controldir.open_branch(name='foo/bar').base)
 
@@ -154,5 +164,5 @@
         except errors.IncompatibleFormat:
             raise tests.TestNotApplicable(
                 'Control dir does not support creating branch references.')
-        self.assertEqual(referenced.base,
-            repo.controldir.get_branch_reference('foo'))
+        self.assertEqual(referenced.user_url,
+                         repo.controldir.get_branch_reference('foo'))

=== modified file 'breezy/tests/per_interbranch/test_copy_content_into.py'
--- breezy/tests/per_interbranch/test_copy_content_into.py	2018-02-18 19:18:40 +0000
+++ breezy/tests/per_interbranch/test_copy_content_into.py	2018-03-25 00:39:16 +0000
@@ -17,6 +17,8 @@
 """Tests for breezy.branch.InterBranch.copy_content_into."""
 
 from breezy import branch
+from breezy.errors import NoRoundtrippingSupport
+from breezy.tests import TestNotApplicable
 from breezy.tests.per_interbranch import (
     StubMatchingInter,
     StubWithFormat,
@@ -30,7 +32,12 @@
         self.tree1 = self.make_from_branch_and_tree('tree1')
         rev1 = self.tree1.commit('one')
         branch2 = self.make_to_branch('tree2')
-        branch2.repository.fetch(self.tree1.branch.repository)
+        try:
+            branch2.repository.fetch(self.tree1.branch.repository)
+        except NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless cross-vcs fetch from %r to %r unsupported' %
+                (self.tree1.branch, branch2))
         self.tree1.branch.copy_content_into(branch2, revision_id=rev1)
 
     def test_inter_is_used(self):

=== modified file 'breezy/tests/per_interbranch/test_fetch.py'
--- breezy/tests/per_interbranch/test_fetch.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_interbranch/test_fetch.py	2018-03-25 00:39:16 +0000
@@ -16,7 +16,9 @@
 
 """Tests for InterBranch.fetch."""
 
+from breezy.errors import FetchLimitUnsupported, NoRoundtrippingSupport
 from breezy.revision import NULL_REVISION
+from breezy.tests import TestNotApplicable
 from breezy.tests.per_interbranch import (
     TestCaseWithInterBranch,
     )
@@ -33,7 +35,12 @@
         rev1 = wt.commit('lala!', allow_pointless=False)
 
         b2 = self.make_to_branch('b2')
-        b2.fetch(b1)
+        try:
+            b2.fetch(b1)
+        except NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless cross-vcs fetch %r to %r not supported' %
+                (b1, b2))
 
         # fetch does not update the last revision
         self.assertEqual(NULL_REVISION, b2.last_revision())
@@ -55,7 +62,14 @@
         builder.finish_series()
         b1 = builder.get_branch()
         b2 = self.make_to_branch('b2')
-        b2.fetch(b1, limit=1)
+        try:
+            b2.fetch(b1, limit=1)
+        except FetchLimitUnsupported:
+            raise TestNotApplicable('interbranch does not support fetch limits')
+        except NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless cross-vcs fetch %r to %r not supported' %
+                (b1, b2))
 
         # fetch does not update the last revision
         self.assertEqual(NULL_REVISION, b2.last_revision())
@@ -72,7 +86,14 @@
         rev1 = wt.commit('lala!', allow_pointless=False)
 
         b2 = self.make_to_branch('b2')
-        b2.fetch(b1, limit=1)
+        try:
+            b2.fetch(b1, limit=1)
+        except FetchLimitUnsupported:
+            raise TestNotApplicable('interbranch does not support fetch limits')
+        except NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless cross-vcs fetch %r to %r not supported' %
+                (b1, b2))
 
         self.assertEqual(
             {rev1},

=== modified file 'breezy/tests/per_interbranch/test_pull.py'
--- breezy/tests/per_interbranch/test_pull.py	2017-08-07 19:09:47 +0000
+++ breezy/tests/per_interbranch/test_pull.py	2018-03-08 03:01:49 +0000
@@ -21,6 +21,7 @@
 from breezy import errors
 from breezy.memorytree import MemoryTree
 from breezy.revision import NULL_REVISION
+from breezy.tests import TestNotApplicable
 from breezy.tests.per_interbranch import TestCaseWithInterBranch
 
 
@@ -35,9 +36,19 @@
         # become the revision-history.
         parent = self.make_from_branch_and_tree('parent')
         parent.commit('1st post', allow_pointless=True)
-        mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        try:
+            mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         mine.commit('my change', allow_pointless=True)
-        parent.merge_from_branch(mine.branch)
+        try:
+            parent.merge_from_branch(mine.branch)
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         p2 = parent.commit('merge my change')
         mine.pull(parent.branch)
         self.assertEqual(p2, mine.branch.last_revision())
@@ -49,12 +60,22 @@
         # directly accessible.
         parent = self.make_from_branch_and_tree('parent')
         parent.commit('1st post', allow_pointless=True)
-        mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        try:
+            mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         mine.commit('my change', allow_pointless=True)
         other = self.sprout_to(parent.controldir, 'other').open_workingtree()
         other.merge_from_branch(mine.branch)
         other.commit('merge my change')
-        parent.merge_from_branch(other.branch)
+        try:
+            parent.merge_from_branch(other.branch)
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         p2 = parent.commit('merge other')
         mine.pull(parent.branch)
         self.assertEqual(p2, mine.branch.last_revision())
@@ -64,10 +85,20 @@
         master_tree = self.make_from_branch_and_tree('master')
         master_tree.commit('master')
         checkout = master_tree.branch.create_checkout('checkout')
-        other = self.sprout_to(master_tree.branch.controldir, 'other').open_workingtree()
+        try:
+            other = self.sprout_to(master_tree.branch.controldir, 'other').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         rev2 = other.commit('other commit')
         # now pull, which should update both checkout and master.
-        checkout.branch.pull(other.branch)
+        try:
+            checkout.branch.pull(other.branch)
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         self.assertEqual(rev2, checkout.branch.last_revision())
         self.assertEqual(rev2, master_tree.branch.last_revision())
 
@@ -77,7 +108,11 @@
         other = self.sprout_to(master_tree.branch.controldir, 'other').open_branch()
         # move the branch out of the way on disk to cause a connection
         # error.
-        master_tree.branch.controldir.destroy_branch()
+        try:
+            master_tree.branch.controldir.destroy_branch()
+        except errors.UnsupportedOperation:
+            raise TestNotApplicable(
+                'control format does not support destroying default branch')
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                           checkout.branch.pull, other)
@@ -85,9 +120,19 @@
     def test_pull_returns_result(self):
         parent = self.make_from_branch_and_tree('parent')
         p1 = parent.commit('1st post')
-        mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        try:
+            mine = self.sprout_to(parent.controldir, 'mine').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         m1 = mine.commit('my change')
-        result = parent.branch.pull(mine.branch)
+        try:
+            result = parent.branch.pull(mine.branch)
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         self.assertIsNot(None, result)
         self.assertIs(mine.branch, result.source_branch)
         self.assertIs(parent.branch, result.target_branch)
@@ -102,10 +147,21 @@
     def test_pull_overwrite(self):
         tree_a = self.make_from_branch_and_tree('tree_a')
         tree_a.commit('message 1')
-        tree_b = self.sprout_to(tree_a.controldir, 'tree_b').open_workingtree()
-        rev2a = tree_a.commit('message 2')
-        rev2b = tree_b.commit('message 2')
-        self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
+        try:
+            tree_b = self.sprout_to(tree_a.controldir, 'tree_b').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
+
+        rev2a = tree_a.commit('message 2a')
+        rev2b = tree_b.commit('message 2b')
+        try:
+            self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
+        except errors.NoRoundtrippingSupport:
+            raise TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         self.assertRaises(errors.DivergedBranches,
                           tree_a.branch.pull, tree_b.branch,
                           overwrite=False, stop_revision=rev2b)

=== modified file 'breezy/tests/per_interbranch/test_push.py'
--- breezy/tests/per_interbranch/test_push.py	2017-11-21 20:40:14 +0000
+++ breezy/tests/per_interbranch/test_push.py	2018-03-06 21:57:52 +0000
@@ -58,9 +58,19 @@
         # become the revision-history.
         mine = self.make_from_branch_and_tree('mine')
         mine.commit('1st post', allow_pointless=True)
-        other = self.sprout_to(mine.controldir, 'other').open_workingtree()
+        try:
+            other = self.sprout_to(mine.controldir, 'other').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         m1 = other.commit('my change', allow_pointless=True)
-        mine.merge_from_branch(other.branch)
+        try:
+            mine.merge_from_branch(other.branch)
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         p2 = mine.commit('merge my change')
         result = mine.branch.push(other.branch)
         self.assertEqual(p2, other.branch.last_revision())
@@ -75,12 +85,22 @@
         # directly accessible.
         mine = self.make_from_branch_and_tree('mine')
         p1 = mine.commit('1st post', allow_pointless=True)
-        target = self.sprout_to(mine.controldir, 'target').open_workingtree()
+        try:
+            target = self.sprout_to(mine.controldir, 'target').open_workingtree()
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         m1 = target.commit('my change', allow_pointless=True)
         other = self.sprout_to(mine.controldir, 'other').open_workingtree()
         other.merge_from_branch(target.branch)
         o2 = other.commit('merge my change')
-        mine.merge_from_branch(other.branch)
+        try:
+            mine.merge_from_branch(other.branch)
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         p2 = mine.commit('merge other')
         mine.branch.push(target.branch)
         self.assertEqual(p2, target.branch.last_revision())
@@ -96,7 +116,12 @@
             return
         rev1 = checkout.commit('master')
 
-        other_bzrdir = self.sprout_from(master_tree.branch.controldir, 'other')
+        try:
+            other_bzrdir = self.sprout_from(master_tree.branch.controldir, 'other')
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         other = other_bzrdir.open_workingtree()
         rev2 = other.commit('other commit')
         # now push, which should update both checkout and master.
@@ -130,15 +155,29 @@
         source.add(['a'])
         source.commit('a')
 
-        source.branch.lock_read()
         try:
-            target.lock_write()
-            try:
+            with source.branch.lock_read(), target.lock_write():
                 source.branch.push(target, stop_revision=source.last_revision())
-            finally:
-                target.unlock()
-        finally:
-            source.branch.unlock()
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
+
+    def test_push_uses_read_lock_lossy(self):
+        """Push should only need a read lock on the source side."""
+        source = self.make_from_branch_and_tree('source')
+        target = self.make_to_branch('target')
+
+        self.build_tree(['source/a'])
+        source.add(['a'])
+        source.commit('a')
+
+        try:
+            with source.branch.lock_read(), target.lock_write():
+                source.branch.push(target, stop_revision=source.last_revision(), lossy=True)
+        except errors.LossyPushToSameVCS:
+            raise tests.TestNotApplicable(
+                'push between branches of same format')
 
     def test_push_within_repository(self):
         """Push from one branch to another inside the same repository."""
@@ -172,10 +211,13 @@
         tree.commit('a')
 
         to_branch = self.make_to_branch('repo/branch')
-        tree.branch.push(to_branch)
-
-        self.assertEqual(tree.branch.last_revision(),
-                         to_branch.last_revision())
+        try:
+            tree.branch.push(to_branch)
+        except errors.NoRoundtrippingSupport:
+            tree.branch.push(to_branch, lossy=True)
+        else:
+            self.assertEqual(tree.branch.last_revision(),
+                             to_branch.last_revision())
 
     def test_push_overwrite_of_non_tip_with_stop_revision(self):
         """Combining the stop_revision and overwrite options works.
@@ -186,7 +228,12 @@
         target = self.make_to_branch('target')
 
         source.commit('1st commit')
-        source.branch.push(target)
+        try:
+            source.branch.push(target)
+        except errors.NoRoundtrippingSupport:
+            raise tests.TestNotApplicable(
+                'lossless push between %r and %r not supported' %
+                (self.branch_format_from, self.branch_format_to))
         rev2 = source.commit('2nd commit')
         source.commit('3rd commit')
 
@@ -212,7 +259,11 @@
         #   - rev-2, no changes
         #   - rev-3, modifies the file.
         repo = self.make_repository('repo', shared=True, format='1.6')
-        builder = self.make_from_branch_builder('repo/local')
+        try:
+            builder = self.make_from_branch_builder('repo/local')
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable(
+                'BranchBuilder can not initialize some formats')
         builder.start_series()
         revid1 = builder.build_snapshot(None, [
             ('add', ('', 'root-id', 'directory', '')),
@@ -263,13 +314,16 @@
         remote_bzrdir = local.controldir.sprout(
             self.get_url('remote'), revision_id=third)
         remote = remote_bzrdir.open_branch()
+        if not remote.repository._format.supports_full_versioned_files:
+            raise tests.TestNotApplicable(
+                'remote is not a VersionedFile repository')
         # Push fourth revision
         self.reset_smart_call_log()
         self.disableOptimisticGetParentMap()
         self.assertFalse(local.is_locked())
         local.push(remote)
         hpss_call_names = [item.call.method for item in self.hpss_calls]
-        self.assertTrue('Repository.insert_stream_1.19' in hpss_call_names)
+        self.assertIn('Repository.insert_stream_1.19', hpss_call_names)
         insert_stream_idx = hpss_call_names.index(
             'Repository.insert_stream_1.19')
         calls_after_insert_stream = hpss_call_names[insert_stream_idx:]

=== modified file 'breezy/tests/per_intertree/test_compare.py'
--- breezy/tests/per_intertree/test_compare.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_intertree/test_compare.py	2018-03-25 00:39:16 +0000
@@ -557,7 +557,9 @@
 
     @staticmethod
     def get_path_entry(tree, file_id):
-        iterator = tree.iter_entries_by_dir(specific_file_ids=[file_id])
+        with tree.lock_read():
+            path = tree.id2path(file_id)
+        iterator = tree.iter_entries_by_dir(specific_files=[path])
         try:
             return next(iterator)
         except StopIteration:

=== modified file 'breezy/tests/per_intertree/test_file_content_matches.py'
--- breezy/tests/per_intertree/test_file_content_matches.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_intertree/test_file_content_matches.py	2018-03-25 00:39:16 +0000
@@ -32,7 +32,9 @@
         tree2.add('file', b'file-id-2')
         tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
         inter = self.intertree_class(tree1, tree2)
-        self.assertTrue(inter.file_content_matches(b'file-id-1', b'file-id-2'))
+        self.assertTrue(inter.file_content_matches(
+            'file', 'file', b'file-id-1', b'file-id-2'))
+        self.assertTrue(inter.file_content_matches('file', 'file'))
 
     def test_different_contents_and_same_verifier(self):
         tree1 = self.make_branch_and_tree('1')
@@ -45,4 +47,7 @@
         tree2.add('file', b'file-id-2')
         tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
         inter = self.intertree_class(tree1, tree2)
-        self.assertFalse(inter.file_content_matches(b'file-id-1', b'file-id-2'))
+        self.assertFalse(inter.file_content_matches(
+            'file', 'file', b'file-id-1', b'file-id-2'))
+        self.assertFalse(inter.file_content_matches(
+            'file', 'file'))

=== modified file 'breezy/tests/per_repository/test_commit_builder.py'
--- breezy/tests/per_repository/test_commit_builder.py	2018-02-10 14:27:08 +0000
+++ breezy/tests/per_repository/test_commit_builder.py	2018-03-24 19:06:10 +0000
@@ -181,7 +181,7 @@
 
     def test_commit_unchanged_root_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
-        old_revision_id = tree.commit('')
+        old_revision_id = tree.commit('oldrev')
         tree.lock_write()
         builder = tree.branch.get_commit_builder([old_revision_id])
         try:
@@ -190,7 +190,7 @@
             # pointless commit.
             self.assertFalse(builder.any_changes())
             builder.finish_inventory()
-            builder.commit('')
+            builder.commit('rev')
             builder_tree = builder.revision_tree()
             new_root_id = builder_tree.get_root_id()
             new_root_revision = builder_tree.get_file_revision(u'')
@@ -231,7 +231,7 @@
         rev_tree = builder.revision_tree()
         rev_tree.lock_read()
         self.addCleanup(rev_tree.unlock)
-        self.assertFalse(rev_tree.path2id('foo'))
+        self.assertFalse(rev_tree.is_versioned('foo'))
 
     def test_revision_tree_record_iter_changes(self):
         tree = self.make_branch_and_tree(".")
@@ -284,26 +284,25 @@
         # commiting without changing the root does not change the
         # last modified except on non-rich-root-repositories.
         tree = self.make_branch_and_tree('.')
-        rev1 = tree.commit('')
-        rev2 = tree.commit('')
+        rev1 = tree.commit('rev1')
+        rev2 = tree.commit('rev2')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
         self.assertEqual(rev1, tree1.get_file_revision(u''))
         if tree.branch.repository.supports_rich_root():
-            self.assertEqual(rev1,tree2.get_file_revision(u''))
+            self.assertEqual(rev1, tree2.get_file_revision(u''))
         else:
             self.assertEqual(rev2, tree2.get_file_revision(u''))
 
-    def _add_commit_check_unchanged(self, tree, name, mini_commit=None):
+    def _add_commit_check_unchanged(self, tree, name):
         tree.add([name])
-        self._commit_check_unchanged(tree, name, tree.path2id(name),
-            mini_commit=mini_commit)
+        self._commit_check_unchanged(tree, name, tree.path2id(name))
 
-    def _commit_check_unchanged(self, tree, name, file_id, mini_commit=None):
-        rev1 = tree.commit('')
-        rev2 = mini_commit(tree, name, name, False, False)
+    def _commit_check_unchanged(self, tree, name, file_id):
+        rev1 = tree.commit('rev1')
+        rev2 = self.mini_commit_record_iter_changes(tree, name, name, False, False)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.get_file_revision(tree1.id2path(file_id)))
-        self.assertEqual(rev1, tree2.get_file_revision(tree2.id2path(file_id)))
+        self.assertEqual(rev1, tree1.get_file_revision(name))
+        self.assertEqual(rev1, tree2.get_file_revision(name))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         self.assertFileGraph(expected_graph, tree, (file_id, rev1))
@@ -311,45 +310,41 @@
     def test_last_modified_revision_after_commit_dir_unchanged(self):
         # committing without changing a dir does not change the last modified.
         tree = self.make_branch_and_tree('.')
+        if not tree.has_versioned_directories():
+            raise tests.TestNotApplicable(
+                    'Format does not support versioned directories')
         self.build_tree(['dir/'])
-        self._add_commit_check_unchanged(tree, 'dir',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_check_unchanged(tree, 'dir')
 
     def test_last_modified_revision_after_commit_dir_contents_unchanged(self):
         # committing without changing a dir does not change the last modified
         # of the dir even the dirs contents are changed.
         tree = self.make_branch_and_tree('.')
-        self.build_tree(['dir/'])
-        tree.add(['dir'])
+        self.build_tree(['dir/', 'dir/orig'])
+        tree.add(['dir', 'dir/orig'])
         dir_id = tree.path2id('dir')
-        rev1 = tree.commit('')
+        rev1 = tree.commit('rev1')
         self.build_tree(['dir/content'])
         tree.add(['dir/content'])
-        rev2 = tree.commit('')
+        rev2 = tree.commit('rev2')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        if self.repository_format.supports_versioned_directories:
-            # In VCSes that don't support empty directories, 'dir' doesn't
-            # exist in rev1.
-            self.assertEqual(rev1, tree1.get_file_revision('dir'))
+        self.assertEqual(rev1, tree1.get_file_revision('dir'))
         self.assertEqual(rev1, tree2.get_file_revision('dir'))
-        expected_graph = {}
-        expected_graph[(dir_id, rev1)] = ()
+        expected_graph = {(dir_id, rev1): ()}
         self.assertFileGraph(expected_graph, tree, (dir_id, rev1))
 
     def test_last_modified_revision_after_commit_file_unchanged(self):
         # committing without changing a file does not change the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
-        self._add_commit_check_unchanged(tree, 'file',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_check_unchanged(tree, 'file')
 
     def test_last_modified_revision_after_commit_link_unchanged(self):
         # committing without changing a link does not change the last modified.
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_check_unchanged(tree, 'link',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_check_unchanged(tree, 'link')
 
     def test_last_modified_revision_after_commit_reference_unchanged(self):
         # committing without changing a subtree does not change the last
@@ -359,47 +354,47 @@
         try:
             tree.add_reference(subtree)
             self._commit_check_unchanged(tree, 'reference',
-                subtree.get_root_id(),
-                mini_commit=self.mini_commit_record_iter_changes)
+                subtree.get_root_id())
         except errors.UnsupportedOperation:
             return
 
     def _add_commit_renamed_check_changed(self, tree, name,
-        expect_fs_hash=False, mini_commit=None):
+        expect_fs_hash=False):
         def rename():
             tree.rename_one(name, 'new_' + name)
-        self._add_commit_change_check_changed(tree, name, rename,
-            expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
+        self._add_commit_change_check_changed(tree,
+                (name, 'new_' + name), rename,
+            expect_fs_hash=expect_fs_hash)
 
-    def _commit_renamed_check_changed(self, tree, name, file_id,
-        expect_fs_hash=False, mini_commit=None):
+    def _commit_renamed_check_changed(self, tree, name,
+        expect_fs_hash=False):
         def rename():
             tree.rename_one(name, 'new_' + name)
-        self._commit_change_check_changed(tree, name, file_id, rename,
-            expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
+        self._commit_change_check_changed(tree, [name, 'new_' + name],
+                rename, expect_fs_hash=expect_fs_hash)
 
     def test_last_modified_revision_after_rename_dir_changes(self):
         # renaming a dir changes the last modified.
         tree = self.make_branch_and_tree('.')
+        if not tree.has_versioned_directories():
+            raise tests.TestNotApplicable(
+                    'Format does not support versioned directories')
         self.build_tree(['dir/'])
-        self._add_commit_renamed_check_changed(tree, 'dir',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_renamed_check_changed(tree, 'dir')
 
     def test_last_modified_revision_after_rename_file_changes(self):
         # renaming a file changes the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
         self._add_commit_renamed_check_changed(tree, 'file',
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+            expect_fs_hash=True)
 
     def test_last_modified_revision_after_rename_link_changes(self):
         # renaming a link changes the last modified.
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_renamed_check_changed(tree, 'link',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_renamed_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_rename_ref_changes(self):
         # renaming a reference changes the last modified.
@@ -407,62 +402,58 @@
         subtree = self.make_reference('reference')
         try:
             tree.add_reference(subtree)
-            self._commit_renamed_check_changed(tree, 'reference',
-                subtree.get_root_id(),
-                mini_commit=self.mini_commit_record_iter_changes)
+            self._commit_renamed_check_changed(tree, 'reference')
         except errors.UnsupportedOperation:
             return
 
     def _add_commit_reparent_check_changed(self, tree, name,
-        expect_fs_hash=False, mini_commit=None):
+        expect_fs_hash=False):
         self.build_tree(['newparent/'])
         tree.add(['newparent'])
         def reparent():
             tree.rename_one(name, 'newparent/new_' + name)
-        self._add_commit_change_check_changed(tree, name, reparent,
-            expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
+        self._add_commit_change_check_changed(
+                tree, (name, 'newparent/new_' + name), reparent,
+            expect_fs_hash=expect_fs_hash)
 
     def test_last_modified_revision_after_reparent_dir_changes(self):
         # reparenting a dir changes the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/'])
-        self._add_commit_reparent_check_changed(tree, 'dir',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_reparent_check_changed(tree, 'dir')
 
     def test_last_modified_revision_after_reparent_file_changes(self):
         # reparenting a file changes the last modified.
         tree = self.make_branch_and_tree('.')
         self.build_tree(['file'])
         self._add_commit_reparent_check_changed(tree, 'file',
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+            expect_fs_hash=True)
 
     def test_last_modified_revision_after_reparent_link_changes(self):
         # reparenting a link changes the last modified.
         self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
-        self._add_commit_reparent_check_changed(tree, 'link',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_reparent_check_changed(tree, 'link')
 
-    def _add_commit_change_check_changed(self, tree, name, changer,
-            expect_fs_hash=False, mini_commit=None):
-        tree.add([name])
-        file_id = tree.path2id(name)
-        self.assertIsNot(None, file_id)
+    def _add_commit_change_check_changed(self, tree, names, changer,
+            expect_fs_hash=False):
+        tree.add([names[0]])
+        self.assertTrue(tree.is_versioned(names[0]))
         self._commit_change_check_changed(
-            tree, name, file_id,
-            changer, expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
+            tree, names,
+            changer, expect_fs_hash=expect_fs_hash)
 
-    def _commit_change_check_changed(self, tree, name, file_id, changer,
-        expect_fs_hash=False, mini_commit=None):
-        rev1 = tree.commit('')
+    def _commit_change_check_changed(self, tree, names, changer,
+                                     expect_fs_hash=False):
+        rev1 = tree.commit('rev1')
         changer()
-        rev2 = mini_commit(tree, name, tree.id2path(file_id),
-            expect_fs_hash=expect_fs_hash)
+        rev2 = self.mini_commit_record_iter_changes(
+                tree, names[0], names[1], expect_fs_hash=expect_fs_hash)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.get_file_revision(tree1.id2path(file_id)))
-        self.assertEqual(rev2, tree2.get_file_revision(tree2.id2path(file_id)))
+        self.assertEqual(rev1, tree1.get_file_revision(names[0]))
+        self.assertEqual(rev2, tree2.get_file_revision(names[1]))
+        file_id = tree1.path2id(names[0])
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         expected_graph[(file_id, rev2)] = ((file_id, rev1),)
@@ -473,7 +464,7 @@
         """Perform a miniature commit looking for record entry results.
 
         This version uses the record_iter_changes interface.
-        
+
         :param tree: The tree to commit.
         :param name: The path in the basis tree of the tree being committed.
         :param new_name: The path in the tree being committed.
@@ -484,41 +475,39 @@
         :param expect_fs_hash: If true, looks for a fs hash output from
             record_iter_changes.
         """
-        tree.lock_write()
-        try:
+        with tree.lock_write():
             # mini manual commit here so we can check the return of
             # record_iter_changes
             parent_ids = tree.get_parent_ids()
             builder = tree.branch.get_commit_builder(parent_ids)
-            parent_tree = tree.basis_tree()
-            parent_tree.lock_read()
-            self.addCleanup(parent_tree.unlock)
-            parent_trees = [parent_tree]
-            for parent_id in parent_ids[1:]:
-                parent_trees.append(tree.branch.repository.revision_tree(
-                    parent_id))
-            changes = list(tree.iter_changes(parent_tree))
-            result = list(builder.record_iter_changes(tree, parent_ids[0],
-                changes))
-            file_id = tree.path2id(new_name)
-            self.assertIsNot(None, file_id)
-            if expect_fs_hash:
-                tree_file_stat = tree.get_file_with_stat(new_name)
-                tree_file_stat[0].close()
-                self.assertLength(1, result)
-                result = result[0]
-                self.assertEqual(result[:2], (file_id, new_name))
-                self.assertEqual(result[2][0], tree.get_file_sha1(new_name))
-                self.assertEqualStat(result[2][1], tree_file_stat[1])
-            else:
-                self.assertEqual([], result)
-            builder.finish_inventory()
-            if tree.branch.repository._format.supports_full_versioned_files:
-                inv_key = (builder._new_revision_id,)
-                inv_sha1 = tree.branch.repository.inventories.get_sha1s(
-                                [inv_key])[inv_key]
-                self.assertEqual(inv_sha1, builder.inv_sha1)
-            rev2 = builder.commit('')
+            try:
+                parent_tree = tree.basis_tree()
+                with parent_tree.lock_read():
+                    changes = list(tree.iter_changes(parent_tree))
+                result = list(builder.record_iter_changes(tree, parent_ids[0],
+                    changes))
+                file_id = tree.path2id(new_name)
+                self.assertIsNot(None, file_id)
+                if expect_fs_hash:
+                    tree_file_stat = tree.get_file_with_stat(new_name)
+                    tree_file_stat[0].close()
+                    self.assertLength(1, result)
+                    result = result[0]
+                    self.assertEqual(result[:2], (file_id, new_name))
+                    self.assertEqual(result[2][0], tree.get_file_sha1(new_name))
+                    self.assertEqualStat(result[2][1], tree_file_stat[1])
+                else:
+                    self.assertEqual([], result)
+                builder.finish_inventory()
+                if tree.branch.repository._format.supports_full_versioned_files:
+                    inv_key = (builder._new_revision_id,)
+                    inv_sha1 = tree.branch.repository.inventories.get_sha1s(
+                                    [inv_key])[inv_key]
+                    self.assertEqual(inv_sha1, builder.inv_sha1)
+                rev2 = builder.commit('rev2')
+            except BaseException:
+                builder.abort()
+                raise
             delta = builder.get_basis_delta()
             delta_dict = dict((change[2], change) for change in delta)
             version_recorded = (file_id in delta_dict and
@@ -529,21 +518,19 @@
             else:
                 self.assertFalse(version_recorded)
 
-            new_inventory = builder.revision_tree().root_inventory
-            new_entry = new_inventory[file_id]
+            revtree = builder.revision_tree()
+            new_entry = revtree.iter_entries_by_dir(specific_files=[new_name]).next()[1]
+
             if delta_against_basis:
-                expected_delta = (name, new_name, file_id, new_entry)
+                if tree.supports_rename_tracking() or name == new_name:
+                    expected_delta = (name, new_name, file_id, new_entry)
+                else:
+                    expected_delta = (None, new_name, file_id, new_entry)
                 self.assertEqual(expected_delta, delta_dict[file_id])
             else:
                 expected_delta = None
                 self.assertFalse(version_recorded)
             tree.set_parent_ids([rev2])
-        except:
-            builder.abort()
-            tree.unlock()
-            raise
-        else:
-            tree.unlock()
         return rev2
 
     def assertFileGraph(self, expected_graph, tree, tip):
@@ -560,9 +547,8 @@
         self.build_tree(['file'])
         def change_file():
             tree.put_file_bytes_non_atomic('file', 'new content')
-        self._add_commit_change_check_changed(tree, 'file', change_file,
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._add_commit_change_check_changed(tree, ('file', 'file'), change_file,
+            expect_fs_hash=True)
 
     def _test_last_mod_rev_after_content_link_changes(
         self, link, target, newtarget):
@@ -574,8 +560,7 @@
             os.unlink(link)
             os.symlink(newtarget, link)
         self._add_commit_change_check_changed(
-            tree, link, change_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+            tree, (link, link), change_link)
 
     def test_last_modified_rev_after_content_link_changes(self):
         self._test_last_mod_rev_after_content_link_changes(
@@ -588,27 +573,26 @@
 
     def _commit_sprout(self, tree, name):
         tree.add([name])
-        rev_id = tree.commit('')
+        rev_id = tree.commit('rev')
         return rev_id, tree.controldir.sprout('t2').open_workingtree()
 
-    def _rename_in_tree(self, tree, name):
+    def _rename_in_tree(self, tree, name, message):
         tree.rename_one(name, 'new_' + name)
-        return tree.commit('')
+        return tree.commit(message)
 
-    def _commit_sprout_rename_merge(self, tree1, name, expect_fs_hash=False,
-        mini_commit=None):
+    def _commit_sprout_rename_merge(self, tree1, name, expect_fs_hash=False):
         """Do a rename in both trees."""
         rev1, tree2 = self._commit_sprout(tree1, name)
         file_id = tree2.path2id(name)
         self.assertIsNot(None, file_id)
         # change both sides equally
-        rev2 = self._rename_in_tree(tree1, name)
-        rev3 = self._rename_in_tree(tree2, name)
+        rev2 = self._rename_in_tree(tree1, name, 'rev2')
+        rev3 = self._rename_in_tree(tree2, name, 'rev3')
         tree1.merge_from_branch(tree2.branch)
-        rev4 = mini_commit(tree1, 'new_' + name, 'new_' + name,
+        rev4 = self.mini_commit_record_iter_changes(tree1, 'new_' + name, 'new_' + name,
             expect_fs_hash=expect_fs_hash)
         tree3, = self._get_revtrees(tree1, [rev4])
-        self.assertEqual(rev4, tree3.get_file_revision(tree3.id2path(file_id)))
+        self.assertEqual(rev4, tree3.get_file_revision('new_' + name))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         expected_graph[(file_id, rev2)] = ((file_id, rev1),)
@@ -619,27 +603,26 @@
     def test_last_modified_revision_after_merge_dir_changes(self):
         # merge a dir changes the last modified.
         tree1 = self.make_branch_and_tree('t1')
+        if not tree1.has_versioned_directories():
+            raise tests.TestNotApplicable(
+                    'Format does not support versioned directories')
         self.build_tree(['t1/dir/'])
-        self._commit_sprout_rename_merge(tree1, 'dir',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge(tree1, 'dir')
 
     def test_last_modified_revision_after_merge_file_changes(self):
         # merge a file changes the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/file'])
-        self._commit_sprout_rename_merge(tree1, 'file', expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge(tree1, 'file', expect_fs_hash=True)
 
     def test_last_modified_revision_after_merge_link_changes(self):
         # merge a link changes the last modified.
         self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
-        self._commit_sprout_rename_merge(tree1, 'link',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge(tree1, 'link')
 
-    def _commit_sprout_rename_merge_converged(self, tree1, name,
-        mini_commit=None):
+    def _commit_sprout_rename_merge_converged(self, tree1, name):
         # Make a merge which just incorporates a change from a branch:
         # The per-file graph is straight line, and no alteration occurs
         # in the inventory.
@@ -648,15 +631,16 @@
         file_id = tree2.path2id(name)
         self.assertIsNot(None, file_id)
         # change on the other side to merge back
-        rev2 = self._rename_in_tree(tree2, name)
+        rev2 = self._rename_in_tree(tree2, name, 'rev2')
         tree1.merge_from_branch(tree2.branch)
         def _check_graph(in_tree, changed_in_tree):
-            rev3 = mini_commit(in_tree, name, 'new_' + name, False,
-                delta_against_basis=changed_in_tree)
+            rev3 = self.mini_commit_record_iter_changes(
+                    in_tree, name, 'new_' + name, False,
+                    delta_against_basis=changed_in_tree)
             tree3, = self._get_revtrees(in_tree, [rev2])
             self.assertEqual(
                     rev2,
-                    tree3.get_file_revision(tree3.id2path(file_id), file_id))
+                    tree3.get_file_revision('new_' + name, file_id))
             expected_graph = {}
             expected_graph[(file_id, rev1)] = ()
             expected_graph[(file_id, rev2)] = ((file_id, rev1),)
@@ -666,27 +650,27 @@
         # change to name, branch tree1 and give it an unrelated change, then
         # merge that to t2.
         other_tree = tree1.controldir.sprout('t3').open_workingtree()
-        other_rev = other_tree.commit('')
+        other_rev = other_tree.commit('other_rev')
         tree2.merge_from_branch(other_tree.branch)
         _check_graph(tree2, False)
 
-    def _commit_sprout_make_merge(self, tree1, make, mini_commit=None):
+    def _commit_sprout_make_merge(self, tree1, make):
         # Make a merge which incorporates the addition of a new object to
         # another branch. The per-file graph shows no additional change
         # in the merge because its a straight line.
-        rev1 = tree1.commit('')
+        rev1 = tree1.commit('rev1')
         tree2 = tree1.controldir.sprout('t2').open_workingtree()
         # make and commit on the other side to merge back
         make('t2/name')
         tree2.add(['name'])
-        file_id = tree2.path2id('name')
-        self.assertIsNot(None, file_id)
-        rev2 = tree2.commit('')
+        self.assertTrue(tree2.is_versioned('name'))
+        rev2 = tree2.commit('rev2')
         tree1.merge_from_branch(tree2.branch)
-        rev3 = mini_commit(tree1, None, 'name', False)
+        rev3 = self.mini_commit_record_iter_changes(tree1, None, 'name', False)
         tree3, = self._get_revtrees(tree1, [rev2])
         # in rev2, name should be only changed in rev2
-        self.assertEqual(rev2, tree3.get_file_revision(tree3.id2path(file_id)))
+        self.assertEqual(rev2, tree3.get_file_revision('name'))
+        file_id = tree2.path2id('name')
         expected_graph = {}
         expected_graph[(file_id, rev2)] = ()
         self.assertFileGraph(expected_graph, tree1, (file_id, rev2))
@@ -694,42 +678,42 @@
     def test_last_modified_revision_after_converged_merge_dir_unchanged(self):
         # merge a dir that changed preserves the last modified.
         tree1 = self.make_branch_and_tree('t1')
+        if not tree1.has_versioned_directories():
+            raise tests.TestNotApplicable(
+                    'Format does not support versioned directories')
         self.build_tree(['t1/dir/'])
-        self._commit_sprout_rename_merge_converged(tree1, 'dir',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge_converged(tree1, 'dir')
 
     def test_last_modified_revision_after_converged_merge_file_unchanged(self):
         # merge a file that changed preserves the last modified.
         tree1 = self.make_branch_and_tree('t1')
         self.build_tree(['t1/file'])
-        self._commit_sprout_rename_merge_converged(tree1, 'file',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge_converged(tree1, 'file')
 
     def test_last_modified_revision_after_converged_merge_link_unchanged(self):
         # merge a link that changed preserves the last modified.
         self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
-        self._commit_sprout_rename_merge_converged(tree1, 'link',
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_rename_merge_converged(tree1, 'link')
 
     def test_last_modified_revision_after_merge_new_dir_unchanged(self):
         # merge a new dir does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_dir,
-            mini_commit=self.mini_commit_record_iter_changes)
+        if not tree1.has_versioned_directories():
+            raise tests.TestNotApplicable(
+                    'Format does not support versioned directories')
+        self._commit_sprout_make_merge(tree1, self.make_dir)
 
     def test_last_modified_revision_after_merge_new_file_unchanged(self):
         # merge a new file does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_file,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_make_merge(tree1, self.make_file)
 
     def test_last_modified_revision_after_merge_new_link_unchanged(self):
         # merge a new link does not change the last modified.
         tree1 = self.make_branch_and_tree('t1')
-        self._commit_sprout_make_merge(tree1, self.make_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._commit_sprout_make_merge(tree1, self.make_link)
 
     def make_dir(self, name):
         self.build_tree([name + '/'])
@@ -746,8 +730,7 @@
         tree.commit('foo')
         return tree
 
-    def _check_kind_change(self, make_before, make_after, expect_fs_hash=False,
-        mini_commit=None):
+    def _check_kind_change(self, make_before, make_after, expect_fs_hash=False):
         tree = self.make_branch_and_tree('.')
         path = 'name'
         make_before(path)
@@ -759,8 +742,8 @@
                 osutils.delete_any(path)
             make_after(path)
 
-        self._add_commit_change_check_changed(tree, path, change_kind,
-            expect_fs_hash=expect_fs_hash, mini_commit=mini_commit)
+        self._add_commit_change_check_changed(tree, (path, path), change_kind,
+            expect_fs_hash=expect_fs_hash)
 
     def test_last_modified_dir_file(self):
         if not self.repository_format.supports_versioned_directories:
@@ -770,8 +753,7 @@
                 'format does not support versioned directories')
         try:
             self._check_kind_change(self.make_dir, self.make_file,
-                expect_fs_hash=True,
-                mini_commit=self.mini_commit_record_iter_changes)
+                expect_fs_hash=True)
         except errors.UnsupportedKindChange:
             raise tests.TestSkipped(
                 "tree does not support changing entry kind from "
@@ -784,8 +766,7 @@
             raise tests.TestNotApplicable(
                 'format does not support versioned directories')
         try:
-            self._check_kind_change(self.make_dir, self.make_link,
-                mini_commit=self.mini_commit_record_iter_changes)
+            self._check_kind_change(self.make_dir, self.make_link)
         except errors.UnsupportedKindChange:
             raise tests.TestSkipped(
                 "tree does not support changing entry kind from "
@@ -793,8 +774,7 @@
 
     def test_last_modified_link_file(self):
         self._check_kind_change(self.make_link, self.make_file,
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+            expect_fs_hash=True)
 
     def test_last_modified_link_dir(self):
         if not self.repository_format.supports_versioned_directories:
@@ -803,8 +783,7 @@
             raise tests.TestNotApplicable(
                 'format does not support versioned directories')
 
-        self._check_kind_change(self.make_link, self.make_dir,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._check_kind_change(self.make_link, self.make_dir)
 
     def test_last_modified_file_dir(self):
         if not self.repository_format.supports_versioned_directories:
@@ -813,12 +792,10 @@
             raise tests.TestNotApplicable(
                 'format does not support versioned directories')
 
-        self._check_kind_change(self.make_file, self.make_dir,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._check_kind_change(self.make_file, self.make_dir)
 
     def test_last_modified_file_link(self):
-        self._check_kind_change(self.make_file, self.make_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+        self._check_kind_change(self.make_file, self.make_link)
 
     def test_get_commit_builder_with_invalid_revprops(self):
         branch = self.make_branch('.')

=== modified file 'breezy/tests/per_repository/test_fetch.py'
--- breezy/tests/per_repository/test_fetch.py	2018-01-12 08:52:43 +0000
+++ breezy/tests/per_repository/test_fetch.py	2018-03-08 23:32:04 +0000
@@ -126,6 +126,9 @@
         remote_format = isinstance(repo, remote.RemoteRepository)
         if not repo._format.rich_root_data and not remote_format:
             return # not relevant
+        if not repo._format.supports_full_versioned_files:
+            raise TestNotApplicable(
+                'format does not support full versioned files')
         builder = self.make_branch_builder('source', format='1.9')
         builder.start_series()
         for revision_id, parent_ids, actions in snapshots:

=== modified file 'breezy/tests/per_repository/test_repository.py'
--- breezy/tests/per_repository/test_repository.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_repository/test_repository.py	2018-03-25 00:39:16 +0000
@@ -958,7 +958,7 @@
         revtree = branch.repository.revision_tree(rev1)
         revtree.lock_read()
         self.addCleanup(revtree.unlock)
-        contents = revtree.get_file_text(u'', FOO_ID)
+        contents = revtree.get_file_text(revtree.id2path(FOO_ID), FOO_ID)
         self.assertEqual(contents, 'contents of repo/foo\n')
 
     def test_create_bundle(self):

=== modified file 'breezy/tests/per_repository/test_signatures.py'
--- breezy/tests/per_repository/test_signatures.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_repository/test_signatures.py	2018-03-25 00:39:16 +0000
@@ -123,7 +123,7 @@
                          '-----END PSEUDO-SIGNED CONTENT-----\n',
                          repo.get_signature_text(a))
         self.assertEqual(
-            (gpg.SIGNATURE_VALID, None, ),
+            (gpg.SIGNATURE_VALID, None),
             repo.verify_revision_signature(a, strategy))
 
     def test_verify_revision_signatures(self):

=== modified file 'breezy/tests/per_repository_vf/test_fileid_involved.py'
--- breezy/tests/per_repository_vf/test_fileid_involved.py	2017-11-16 10:35:32 +0000
+++ breezy/tests/per_repository_vf/test_fileid_involved.py	2018-03-20 00:30:39 +0000
@@ -429,10 +429,9 @@
     os.chmod() doesn't work on windows. But TreeTransform can mark or
     unmark a file as executable.
     """
-    file_id = wt.path2id(path)
     tt = transform.TreeTransform(wt)
     try:
-        tt.set_executability(executable, tt.trans_id_tree_file_id(file_id))
+        tt.set_executability(executable, tt.trans_id_tree_path(path))
         tt.apply()
     finally:
         tt.finalize()

=== modified file 'breezy/tests/per_tree/__init__.py'
--- breezy/tests/per_tree/__init__.py	2018-02-17 02:02:38 +0000
+++ breezy/tests/per_tree/__init__.py	2018-03-24 23:49:53 +0000
@@ -277,56 +277,6 @@
         tt.apply()
         return self.workingtree_to_test_tree(tree)
 
-    def get_tree_with_utf8(self, tree):
-        """Generate a tree with a utf8 revision and unicode paths."""
-        self._create_tree_with_utf8(tree)
-        return self.workingtree_to_test_tree(tree)
-
-    def _create_tree_with_utf8(self, tree):
-        """Generate a tree with a utf8 revision and unicode paths."""
-        self.requireFeature(features.UnicodeFilenameFeature)
-        # We avoid combining characters in file names here, normalization
-        # checks (as performed by some file systems (OSX) are outside the scope
-        # of these tests).  We use the euro sign \N{Euro Sign} or \u20ac in
-        # unicode strings or '\xe2\x82\ac' (its utf-8 encoding) in raw strings.
-        paths = [u'',
-                 u'fo\N{Euro Sign}o',
-                 u'ba\N{Euro Sign}r/',
-                 u'ba\N{Euro Sign}r/ba\N{Euro Sign}z',
-                ]
-        # bzr itself does not create unicode file ids, but we want them for
-        # testing.
-        file_ids = [b'TREE_ROOT',
-                    b'fo\xe2\x82\xaco-id',
-                    b'ba\xe2\x82\xacr-id',
-                    b'ba\xe2\x82\xacz-id',
-                   ]
-        self.build_tree(paths[1:])
-        if tree.get_root_id() is None:
-            # Some trees do not have a root yet.
-            tree.add(paths, file_ids)
-        else:
-            # Some trees will already have a root
-            tree.set_root_id(file_ids[0])
-            tree.add(paths[1:], file_ids[1:])
-        try:
-            tree.commit(u'in\xedtial', rev_id=u'r\xe9v-1'.encode('utf8'))
-        except errors.NonAsciiRevisionId:
-            raise tests.TestSkipped('non-ascii revision ids not supported')
-
-    def get_tree_with_merged_utf8(self, tree):
-        """Generate a tree with utf8 ancestors."""
-        self._create_tree_with_utf8(tree)
-        tree2 = tree.controldir.sprout('tree2').open_workingtree()
-        self.build_tree([u'tree2/ba\N{Euro Sign}r/qu\N{Euro Sign}x'])
-        tree2.add([u'ba\N{Euro Sign}r/qu\N{Euro Sign}x'],
-                  [u'qu\N{Euro Sign}x-id'.encode('utf-8')])
-        tree2.commit(u'to m\xe9rge', rev_id=u'r\xe9v-2'.encode('utf8'))
-
-        tree.merge_from_branch(tree2.branch)
-        tree.commit(u'm\xe9rge', rev_id=u'r\xe9v-3'.encode('utf8'))
-        return self.workingtree_to_test_tree(tree)
-
 
 def make_scenarios(transport_server, transport_readonly_server, formats):
     """Generate test suites for each Tree implementation in breezy.

=== modified file 'breezy/tests/per_tree/test_get_root_id.py'
--- breezy/tests/per_tree/test_get_root_id.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_tree/test_get_root_id.py	2018-03-25 00:39:16 +0000
@@ -39,13 +39,11 @@
 
     def test_get_root_id_default(self):
         tree = self.make_tree_with_default_root_id()
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        self.assertIsNot(None, tree.get_root_id())
+        with tree.lock_read():
+            self.assertIsNot(None, tree.get_root_id())
 
     def test_get_root_id_fixed(self):
         tree = self.make_tree_with_fixed_root_id()
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        self.assertEqual('custom-tree-root-id', tree.get_root_id())
+        with tree.lock_read():
+            self.assertEqual('custom-tree-root-id', tree.get_root_id())
 

=== modified file 'breezy/tests/per_tree/test_get_symlink_target.py'
--- breezy/tests/per_tree/test_get_symlink_target.py	2017-11-12 20:07:32 +0000
+++ breezy/tests/per_tree/test_get_symlink_target.py	2018-03-24 13:36:19 +0000
@@ -50,7 +50,7 @@
         self.assertEqual('/baz/bing',
                          tree.get_symlink_target('abs_link'))
         self.assertEqual('foo',
-                         tree.get_symlink_target('link', tree.path2id('link')))
+                         tree.get_symlink_target('link'))
 
     def test_get_unicode_symlink_target(self):
         self.requireFeature(features.SymlinkFeature)
@@ -59,7 +59,6 @@
         target = u'targ\N{Euro Sign}t'
         os.symlink(target,  u'tree/\u03b2_link'.encode(osutils._fs_enc))
         tree.add([u'\u03b2_link'])
-        file_id = tree.path2id(u'\u03b2_link')
         tree.lock_read()
         self.addCleanup(tree.unlock)
         actual = tree.get_symlink_target(u'\u03b2_link')

=== modified file 'breezy/tests/per_tree/test_inv.py'
--- breezy/tests/per_tree/test_inv.py	2017-08-01 01:44:41 +0000
+++ breezy/tests/per_tree/test_inv.py	2018-03-22 08:44:28 +0000
@@ -33,8 +33,8 @@
     )
 
 
-def get_entry(tree, file_id):
-    return tree.iter_entries_by_dir([file_id]).next()[1]
+def get_entry(tree, path):
+    return tree.iter_entries_by_dir(specific_files=[path]).next()[1]
 
 
 class TestInventoryWithSymlinks(per_tree.TestCaseWithTree):
@@ -52,7 +52,7 @@
             raise TestSkipped(
                 'symlinks not accurately represented in working trees and'
                 ' preview trees')
-        entry = get_entry(self.tree, self.tree.path2id('symlink'))
+        entry = get_entry(self.tree, 'symlink')
         self.assertEqual(entry.symlink_target, 'link-target')
 
     def test_symlink_target_tree(self):
@@ -64,7 +64,7 @@
         self.assertIs(None, self.tree.get_file_size('symlink'))
 
     def test_symlink(self):
-        entry = get_entry(self.tree, self.tree.path2id('symlink'))
+        entry = get_entry(self.tree, 'symlink')
         self.assertEqual(entry.kind, 'symlink')
         self.assertEqual(None, entry.text_size)
 
@@ -76,6 +76,9 @@
         self.build_tree(['tree/dir/', 'tree/dir/file'])
         work_tree.add(['dir', 'dir/file'])
         tree = self._convert_tree(work_tree)
+        if not isinstance(tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEqual({tree.path2id('dir'), tree.path2id('dir/file')},
@@ -88,6 +91,9 @@
         work_tree.commit('commit old state')
         work_tree.remove('file')
         tree = self._convert_tree(work_tree)
+        if not isinstance(tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEqual(set([]), tree.paths2ids(['file'],

=== modified file 'breezy/tests/per_tree/test_list_files.py'
--- breezy/tests/per_tree/test_list_files.py	2018-01-12 08:52:43 +0000
+++ breezy/tests/per_tree/test_list_files.py	2018-02-26 13:36:15 +0000
@@ -60,12 +60,8 @@
         expected = [('', 'V', 'directory', tree.path2id('')),
                     ('a', 'V', 'file', tree.path2id('a')),
                    ]
-        if tree.has_versioned_directories():
-            expected.append(
-                ('b', 'V', 'directory', tree.path2id('b')))
-        else:
-            expected.append(
-                ('b', '?', 'directory', None))
+        expected.append(
+            ('b', 'V', 'directory', tree.path2id('b')))
         with tree.lock_read():
             actual = [(path, status, kind, file_id)
                 for path, status, kind, file_id, ie in
@@ -76,12 +72,8 @@
         work_tree = self.make_branch_and_tree('wt')
         tree = self.get_tree_no_parents_abc_content(work_tree)
         expected = [('a', 'V', 'file', tree.path2id('a'))]
-        if tree.has_versioned_directories():
-            expected.append(
-                ('b', 'V', 'directory', tree.path2id('b')))
-        else:
-            expected.append(
-                ('b', '?', 'directory', None))
+        expected.append(
+            ('b', 'V', 'directory', tree.path2id('b')))
         tree.lock_read()
         try:
             actual = [(path, status, kind, file_id)
@@ -110,11 +102,8 @@
         work_tree = self.make_branch_and_tree('wt')
         tree = self.get_tree_no_parents_abc_content(work_tree)
         expected = [('a', 'V', 'file', tree.path2id('a'))]
-        if tree.has_versioned_directories():
-            expected.append(
-                ('b', 'V', 'directory', tree.path2id('b')))
-        else:
-            expected.append(('b', '?', 'directory', None))
+        expected.append(
+            ('b', 'V', 'directory', tree.path2id('b')))
 
         with tree.lock_read():
             actual = [(path, status, kind, file_id)

=== modified file 'breezy/tests/per_tree/test_test_trees.py'
--- breezy/tests/per_tree/test_test_trees.py	2018-02-15 19:38:33 +0000
+++ breezy/tests/per_tree/test_test_trees.py	2018-03-24 15:27:01 +0000
@@ -16,8 +16,11 @@
 
 """Tests for the test trees used by the per_tree tests."""
 
+from breezy import errors
 from breezy.tests import per_tree
 from breezy.tests import (
+    TestNotApplicable,
+    TestSkipped,
     features,
     )
 
@@ -137,7 +140,10 @@
         self.addCleanup(tree.unlock)
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
-        self.assertEqual([], list(tree.unknowns()))
+        if tree.has_versioned_directories():
+            self.assertEqual([], list(tree.unknowns()))
+        else:
+            self.assertEqual(['b'], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         expected_paths = (
                 ['', 'a'] +
@@ -162,25 +168,45 @@
         self.addCleanup(tree.unlock)
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
-        self.assertEqual([], list(tree.unknowns()))
+        if tree.has_versioned_directories():
+            self.assertEqual([], list(tree.unknowns()))
+        else:
+            self.assertEqual(['1top-dir/1dir-in-1topdir'], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         tree_root = tree.path2id('')
-        self.assertEqual(
-            {tree.path2id(p) for p in [
-                '', '0file', '1top-dir', '1top-dir/1dir-in-1topdir',
-                '1top-dir/0file-in-1topdir', 'symlink', u'2utf\u1234file']},
-            set(tree.all_file_ids()))
-        # note that the order of the paths and fileids is deliberately
-        # mismatched to ensure that the result order is path based.
-        self.assertEqual(
-            [('', 'directory'),
-             ('0file', 'file'),
-             ('1top-dir', 'directory'),
-             (u'2utf\u1234file', 'file'),
-             ('symlink', 'symlink'),
-             ('1top-dir/0file-in-1topdir', 'file'),
-             ('1top-dir/1dir-in-1topdir', 'directory')],
-            [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
+        if tree.has_versioned_directories():
+            self.assertEqual(
+                {tree.path2id(p) for p in [
+                    '', '0file', '1top-dir', '1top-dir/1dir-in-1topdir',
+                    '1top-dir/0file-in-1topdir', 'symlink', u'2utf\u1234file']},
+                set(tree.all_file_ids()))
+            # note that the order of the paths and fileids is deliberately
+            # mismatched to ensure that the result order is path based.
+            self.assertEqual(
+                [('', 'directory'),
+                 ('0file', 'file'),
+                 ('1top-dir', 'directory'),
+                 (u'2utf\u1234file', 'file'),
+                 ('symlink', 'symlink'),
+                 ('1top-dir/0file-in-1topdir', 'file'),
+                 ('1top-dir/1dir-in-1topdir', 'directory')],
+                [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
+        else:
+            self.assertEqual(
+                {tree.path2id(p) for p in [
+                    '', '0file', '1top-dir', 
+                    '1top-dir/0file-in-1topdir', 'symlink', u'2utf\u1234file']},
+                set(tree.all_file_ids()))
+            # note that the order of the paths and fileids is deliberately
+            # mismatched to ensure that the result order is path based.
+            self.assertEqual(
+                [('', 'directory'),
+                 ('0file', 'file'),
+                 ('1top-dir', 'directory'),
+                 (u'2utf\u1234file', 'file'),
+                 ('symlink', 'symlink'),
+                 ('1top-dir/0file-in-1topdir', 'file')],
+                [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
 
     def test_tree_with_subdirs_and_all_content_types_wo_symlinks(self):
         # currently this test tree requires unicode. It might be good
@@ -191,27 +217,90 @@
         self.addCleanup(tree.unlock)
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
-        self.assertEqual([], list(tree.unknowns()))
+        if tree.has_versioned_directories():
+            self.assertEqual([], list(tree.unknowns()))
+        else:
+            self.assertEqual(['1top-dir/1dir-in-1topdir'], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         tree_root = tree.path2id('')
-        self.assertEqual(
-            {'', '0file', '1top-dir', '1top-dir/0file-in-1topdir',
-             '1top-dir/1dir-in-1topdir', u'2utf\u1234file'},
-            set(tree.all_versioned_paths()))
-        # note that the order of the paths and fileids is deliberately
-        # mismatched to ensure that the result order is path based.
-        self.assertEqual(
-            [('', 'directory'),
-             ('0file', 'file'),
-             ('1top-dir', 'directory'),
-             (u'2utf\u1234file', 'file'),
-             ('1top-dir/0file-in-1topdir', 'file'),
-             ('1top-dir/1dir-in-1topdir', 'directory')],
-            [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
+        if tree.has_versioned_directories():
+            self.assertEqual(
+                {'', '0file', '1top-dir', '1top-dir/0file-in-1topdir',
+                 '1top-dir/1dir-in-1topdir', u'2utf\u1234file'},
+                set(tree.all_versioned_paths()))
+            # note that the order of the paths and fileids is deliberately
+            # mismatched to ensure that the result order is path based.
+            self.assertEqual(
+                [('', 'directory'),
+                 ('0file', 'file'),
+                 ('1top-dir', 'directory'),
+                 (u'2utf\u1234file', 'file'),
+                 ('1top-dir/0file-in-1topdir', 'file'),
+                 ('1top-dir/1dir-in-1topdir', 'directory')],
+                [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
+        else:
+            self.assertEqual(
+                {'', '0file', '1top-dir', '1top-dir/0file-in-1topdir',
+                 u'2utf\u1234file'},
+                set(tree.all_versioned_paths()))
+            # note that the order of the paths and fileids is deliberately
+            # mismatched to ensure that the result order is path based.
+            self.assertEqual(
+                [('', 'directory'),
+                 ('0file', 'file'),
+                 ('1top-dir', 'directory'),
+                 (u'2utf\u1234file', 'file'),
+                 ('1top-dir/0file-in-1topdir', 'file')],
+                [(path, node.kind) for path, node in tree.iter_entries_by_dir()])
+
+    def _create_tree_with_utf8(self, tree):
+        self.requireFeature(features.UnicodeFilenameFeature)
+
+        # We avoid combining characters in file names here, normalization
+        # checks (as performed by some file systems (OSX) are outside the scope
+        # of these tests).  We use the euro sign \N{Euro Sign} or \u20ac in
+        # unicode strings or '\xe2\x82\ac' (its utf-8 encoding) in raw strings.
+        paths = [u'',
+                 u'fo\N{Euro Sign}o',
+                 u'ba\N{Euro Sign}r/',
+                 u'ba\N{Euro Sign}r/ba\N{Euro Sign}z',
+                ]
+        # bzr itself does not create unicode file ids, but we want them for
+        # testing.
+        file_ids = ['TREE_ROOT',
+                    'fo\xe2\x82\xaco-id',
+                    'ba\xe2\x82\xacr-id',
+                    'ba\xe2\x82\xacz-id',
+                   ]
+        self.build_tree(paths[1:])
+        if tree.get_root_id() is None:
+            # Some trees do not have a root yet.
+            tree.add(paths, file_ids)
+        else:
+            # Some trees will already have a root
+            if tree.supports_setting_file_ids():
+                tree.set_root_id(file_ids[0])
+                tree.add(paths[1:], file_ids[1:])
+            else:
+                tree.add(paths[1:])
+        if tree.branch.repository._format.supports_setting_revision_ids:
+            try:
+                tree.commit(u'in\xedtial', rev_id=u'r\xe9v-1'.encode('utf8'))
+            except errors.NonAsciiRevisionId:
+                raise TestSkipped('non-ascii revision ids not supported')
+        else:
+            tree.commit(u'in\xedtial')
+
+        return tree
 
     def test_tree_with_utf8(self):
         tree = self.make_branch_and_tree('.')
-        tree = self.get_tree_with_utf8(tree)
+        if not tree.supports_setting_file_ids():
+            raise TestNotApplicable(
+                'format does not support custom file ids')
+        self._create_tree_with_utf8(tree)
+
+        tree = self.workingtree_to_test_tree(tree)
 
         revision_id = u'r\xe9v-1'.encode('utf8')
         root_id = 'TREE_ROOT'
@@ -224,11 +313,8 @@
                         (u'ba\N{Euro Sign}r/ba\N{Euro Sign}z',
                          baz_id, bar_id, revision_id),
                        ]
-        tree.lock_read()
-        try:
+        with tree.lock_read():
             path_entries = list(tree.iter_entries_by_dir())
-        finally:
-            tree.unlock()
 
         for expected, (path, ie) in zip(path_and_ids, path_entries):
             self.assertEqual(expected[0], path) # Paths should match
@@ -254,8 +340,31 @@
             self.assertIsInstance(last_revision(), str)
 
     def test_tree_with_merged_utf8(self):
-        tree = self.make_branch_and_tree('.')
-        tree = self.get_tree_with_merged_utf8(tree)
+        wt = self.make_branch_and_tree('.')
+
+        self._create_tree_with_utf8(wt)
+
+        tree2 = wt.controldir.sprout('tree2').open_workingtree()
+        self.build_tree([u'tree2/ba\N{Euro Sign}r/qu\N{Euro Sign}x'])
+        if wt.supports_setting_file_ids():
+            tree2.add([u'ba\N{Euro Sign}r/qu\N{Euro Sign}x'],
+                      [u'qu\N{Euro Sign}x-id'.encode('utf-8')])
+        else:
+            tree2.add([u'ba\N{Euro Sign}r/qu\N{Euro Sign}x'])
+        if wt.branch.repository._format.supports_setting_revision_ids:
+            tree2.commit(u'to m\xe9rge', rev_id=u'r\xe9v-2'.encode('utf8'))
+        else:
+            tree2.commit(u'to m\xe9rge')
+
+        self.assertTrue(tree2.is_versioned(u'ba\N{Euro Sign}r/qu\N{Euro Sign}x'))
+        wt.merge_from_branch(tree2.branch)
+        self.assertTrue(wt.is_versioned(u'ba\N{Euro Sign}r/qu\N{Euro Sign}x'))
+
+        if wt.branch.repository._format.supports_setting_revision_ids:
+            wt.commit(u'm\xe9rge', rev_id=u'r\xe9v-3'.encode('utf8'))
+        else:
+            wt.commit(u'm\xe9rge')
+        tree = self.workingtree_to_test_tree(wt)
 
         revision_id_1 = u'r\xe9v-1'.encode('utf8')
         revision_id_2 = u'r\xe9v-2'.encode('utf8')
@@ -272,33 +381,26 @@
                         (u'ba\N{Euro Sign}r/qu\N{Euro Sign}x',
                          qux_id, bar_id, revision_id_2),
                        ]
-        tree.lock_read()
-        try:
+        with tree.lock_read():
             path_entries = list(tree.iter_entries_by_dir())
-        finally:
-            tree.unlock()
 
         for (epath, efid, eparent, erev), (path, ie) in zip(path_and_ids,
                                                             path_entries):
             self.assertEqual(epath, path) # Paths should match
             self.assertIsInstance(path, unicode)
-            self.assertEqual(efid, ie.file_id)
-            self.assertIsInstance(ie.file_id, str)
-            self.assertEqual(eparent, ie.parent_id)
+            self.assertIsInstance(ie.file_id, bytes)
+            if wt.supports_setting_file_ids():
+                self.assertEqual(efid, ie.file_id)
+                self.assertEqual(eparent, ie.parent_id)
             if eparent is not None:
-                self.assertIsInstance(ie.parent_id, str)
-            # WorkingTree's return None for the last modified revision
-            if ie.revision is not None:
-                self.assertIsInstance(ie.revision, str)
-                if epath == '':
-                    # Some trees will preserve the revision id of the tree root,
-                    # but not all will
-                    continue
-                self.assertEqual(erev, ie.revision)
-        self.assertEqual(len(path_and_ids), len(path_entries))
+                self.assertIsInstance(ie.parent_id, bytes)
+        self.assertEqual(len(path_and_ids), len(path_entries),
+                         "%r vs %r" % (
+                             [p for (p, f, pf, r) in path_and_ids],
+                             [p for (p, e) in path_entries]))
         get_revision_id = getattr(tree, 'get_revision_id', None)
         if get_revision_id is not None:
-            self.assertIsInstance(get_revision_id(), str)
+            self.assertIsInstance(get_revision_id(), bytes)
         last_revision = getattr(tree, 'last_revision', None)
         if last_revision is not None:
-            self.assertIsInstance(last_revision(), str)
+            self.assertIsInstance(last_revision(), bytes)

=== modified file 'breezy/tests/per_tree/test_tree.py'
--- breezy/tests/per_tree/test_tree.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_tree/test_tree.py	2018-03-25 00:39:16 +0000
@@ -56,6 +56,8 @@
         work_b = work_a.controldir.sprout('wtb').open_workingtree()
         self.build_tree_contents([('wta/file', b'b\nc\nd\ne\n')])
         tree_a = self.workingtree_to_test_tree(work_a)
+        if getattr(tree_a, 'plan_file_merge', None) is None:
+            raise tests.TestNotApplicable('Tree does not support plan_file_merge')
         tree_a.lock_read()
         self.addCleanup(tree_a.unlock)
         self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
@@ -120,6 +122,12 @@
         tree = self.make_branch_and_tree('tree')
         self.assertTrue(tree.is_versioned(''))
         self.assertFalse(tree.is_versioned('blah'))
+        self.build_tree(['tree/dir/', 'tree/dir/file'])
+        self.assertFalse(tree.is_versioned('dir'))
+        self.assertFalse(tree.is_versioned('dir/'))
+        tree.add(['dir', 'dir/file'])
+        self.assertTrue(tree.is_versioned('dir'))
+        self.assertTrue(tree.is_versioned('dir/'))
 
 
 class TestFileIds(TestCaseWithTree):
@@ -224,15 +232,15 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         extracted = dict((i, ''.join(b)) for i, b in
-                         tree.iter_files_bytes([(tree.path2id('foo'), 'id1'),
-                                                (tree.path2id('bar'), 'id2'),
-                                                (tree.path2id('baz'), 'id3')]))
+                         tree.iter_files_bytes([('foo', 'id1'),
+                                                ('bar', 'id2'),
+                                                ('baz', 'id3')]))
         self.assertEqual('foo', extracted['id1'])
         self.assertEqual('bar', extracted['id2'])
         self.assertEqual('baz', extracted['id3'])
-        self.assertRaises(errors.NoSuchId, lambda: list(
+        self.assertRaises(errors.NoSuchFile, lambda: list(
                           tree.iter_files_bytes(
-                          [('qux-id', 'file1-notpresent')])))
+                          [('qux', 'file1-notpresent')])))
 
 
 class TestConflicts(TestCaseWithTree):
@@ -351,4 +359,12 @@
     def test_has_versioned_directories(self):
         work_tree = self.make_branch_and_tree('tree')
         tree = self._convert_tree(work_tree)
-        self.assertSubset([tree.has_versioned_directories()], (True, False))
+        self.assertIn(tree.has_versioned_directories(), (True, False))
+
+
+class TestSupportsRenameTracking(TestCaseWithTree):
+
+    def test_supports_rename_tracking(self):
+        work_tree = self.make_branch_and_tree('tree')
+        tree = self._convert_tree(work_tree)
+        self.assertSubset([tree.supports_rename_tracking()], (True, False))

=== modified file 'breezy/tests/per_tree/test_walkdirs.py'
--- breezy/tests/per_tree/test_walkdirs.py	2018-02-03 13:39:29 +0000
+++ breezy/tests/per_tree/test_walkdirs.py	2018-03-17 21:00:51 +0000
@@ -38,14 +38,17 @@
              [('1top-dir/0file-in-1topdir', '0file-in-1topdir',
                'file', None, tree.path2id('1top-dir/0file-in-1topdir'), 'file'),
               ('1top-dir/1dir-in-1topdir', '1dir-in-1topdir',
-               'directory', None, tree.path2id('1top-dir/1dir-in-1topdir'), 'directory'),
-              ]),
+               'directory',
+               None if tree.has_versioned_directories() else os.stat(tree.abspath('1top-dir/1dir-in-1topdir')),
+               tree.path2id('1top-dir/1dir-in-1topdir'),
+               'directory' if tree.has_versioned_directories() else None,
+              )]),
             (('1top-dir/1dir-in-1topdir', tree.path2id('1top-dir/1dir-in-1topdir')),
              []),
             ]
         if symlinks:
             dirblocks[0][1].append(('symlink', 'symlink', 'symlink', None,
-                                    'symlink', 'symlink'))
+                                    tree.path2id('symlink'), 'symlink'))
         return dirblocks
 
     def test_walkdir_root(self):
@@ -105,8 +108,10 @@
         if tree.path2id('file') is None:
             raise tests.TestNotApplicable(
                 'Tree type cannot represent dangling ids.')
-        expected = [(('', work_tree.path2id('')), [
-            ('dir', 'dir', 'unknown', None, dir_id, 'directory'),
-            ('file', 'file', 'unknown', None, file_id, 'file')]),
-            (('dir', dir_id), [])]
+        expected = [(('', work_tree.path2id('')), ([
+            ('dir', 'dir', 'unknown', None, dir_id, 'directory')]
+            if tree.has_versioned_directories() else []) +
+            [('file', 'file', 'unknown', None, file_id, 'file')])]
+        if tree.has_versioned_directories():
+            expected.append((('dir', dir_id), []))
         self.assertEqual(expected, list(tree.walkdirs()))

=== modified file 'breezy/tests/per_uifactory/__init__.py'
--- breezy/tests/per_uifactory/__init__.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/per_uifactory/__init__.py	2018-03-24 17:10:12 +0000
@@ -44,6 +44,7 @@
 from ..ui_testing import (
     StringIOWithEncoding,
     StringIOAsTTY,
+    TextUIFactory,
     )
 
 
@@ -124,11 +125,15 @@
 
     def setUp(self):
         super(TestTextUIFactory, self).setUp()
-        self.stdin = StringIOWithEncoding()
-        self.stdout = StringIOWithEncoding()
-        self.stderr = StringIOWithEncoding()
-        self.factory = ui.text.TextUIFactory(self.stdin, self.stdout,
-            self.stderr)
+        self.factory = self._create_ui_factory()
+        self.factory.__enter__()
+        self.addCleanup(self.factory.__exit__, None, None, None)
+        self.stdin = self.factory.stdin
+        self.stdout = self.factory.stdout
+        self.stderr = self.factory.stderr
+
+    def _create_ui_factory(self):
+        return TextUIFactory(u'')
 
     def _check_note(self, note_text):
         self.assertEqual("%s\n" % note_text,
@@ -172,16 +177,11 @@
 
 class TestTTYTextUIFactory(TestTextUIFactory):
 
-    def setUp(self):
-        super(TestTTYTextUIFactory, self).setUp()
-
+    def _create_ui_factory(self):
         # Remove 'TERM' == 'dumb' which causes us to *not* treat output as a
         # real terminal, even though isatty returns True
         self.overrideEnv('TERM', None)
-        self.stderr = StringIOAsTTY()
-        self.stdout = StringIOAsTTY()
-        self.factory = ui.text.TextUIFactory(self.stdin, self.stdout,
-            self.stderr)
+        return TextUIFactory(u'', StringIOAsTTY(), StringIOAsTTY())
 
     def _check_log_transport_activity_display(self):
         self.assertEqual('', self.stdout.getvalue())

=== modified file 'breezy/tests/per_workingtree/test_add.py'
--- breezy/tests/per_workingtree/test_add.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_add.py	2018-03-25 00:39:16 +0000
@@ -23,7 +23,7 @@
 from breezy.bzr import (
     inventory,
     )
-from breezy.tests.matchers import HasLayout
+from breezy.tests.matchers import HasLayout, HasPathRelations
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -33,14 +33,15 @@
         """Check that the tree has the correct layout."""
         self.assertThat(tree, HasLayout(expected))
 
+    def assertPathRelations(self, previous_tree, tree, relations):
+        self.assertThat(tree, HasPathRelations(previous_tree, relations))
+
     def test_add_one(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['one'])
         tree.add('one')
-        root_id = tree.get_root_id()
 
-        self.assertTreeLayout(
-            [('', root_id), ('one', tree.path2id('one'))], tree)
+        self.assertTreeLayout(['', 'one'], tree)
 
     def test_add_existing_id(self):
         """Adding an entry with a pre-existing id raises DuplicateFileId"""
@@ -51,9 +52,8 @@
         tree.add(['a'])
         self.assertRaises(errors.DuplicateFileId,
                           tree.add, ['b'], [tree.path2id('a')])
-        root_id = tree.get_root_id()
         # And the entry should not have been added.
-        self.assertTreeLayout([('', root_id), ('a', tree.path2id('a'))], tree)
+        self.assertTreeLayout(['', 'a'], tree)
 
     def test_add_old_id(self):
         """We can add an old id, as long as it doesn't exist now."""
@@ -64,31 +64,26 @@
         tree.add(['a'])
         file_id = tree.path2id('a')
         tree.commit('first')
-        root_id = tree.get_root_id()
         # And the entry should not have been added.
         tree.unversion(['a'])
         tree.add(['b'], [file_id])
-        self.assertTreeLayout([('', root_id), ('b', file_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', file_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b', 'a')])
 
     def test_add_one_list(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['one'])
         tree.add(['one'])
-        root_id = tree.get_root_id()
 
-        self.assertTreeLayout(
-            [('', root_id), ('one', tree.path2id('one'))], tree)
+        self.assertTreeLayout(['', 'one'], tree)
 
     def test_add_one_new_id(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['one'])
         tree.add(['one'])
-        root_id = tree.get_root_id()
-        one_id = tree.path2id('one')
 
-        self.assertTreeLayout([('', root_id), ('one', one_id)], tree)
+        self.assertTreeLayout(['', 'one'], tree)
 
     def test_add_unicode(self):
         tree = self.make_branch_and_tree('.')
@@ -97,10 +92,8 @@
         except UnicodeError:
             raise tests.TestSkipped('Filesystem does not support filename.')
         tree.add([u'f\xf6'])
-        root_id = tree.get_root_id()
-        foo_id = tree.path2id(u'f\xf6')
 
-        self.assertTreeLayout([('', root_id), (u'f\xf6', foo_id)], tree)
+        self.assertTreeLayout(['', u'f\xf6'], tree)
 
     def test_add_subdir_with_ids(self):
         tree = self.make_branch_and_tree('.')
@@ -122,24 +115,17 @@
         tree.add(['dir'])
         tree.add(['dir/subdir'])
         tree.add(['dir/subdir/foo'])
-        root_id = tree.get_root_id()
 
-        self.assertTreeLayout([
-            ('', root_id),
-            ('dir/', tree.path2id('dir')),
-            ('dir/subdir/', tree.path2id('dir/subdir')),
-            ('dir/subdir/foo', tree.path2id('dir/subdir/foo'))],
-            tree)
+        self.assertTreeLayout(
+            ['', 'dir/', 'dir/subdir/', 'dir/subdir/foo'], tree)
 
     def test_add_multiple(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'dir/', 'dir/subdir/', 'dir/subdir/foo'])
         tree.add(['a', 'b', 'dir', 'dir/subdir', 'dir/subdir/foo'])
-        root_id = tree.get_root_id()
 
-        self.assertTreeLayout([
-            (p, tree.path2id(p))
-            for p in ['', 'a', 'b', 'dir/', 'dir/subdir/', 'dir/subdir/foo']],
+        self.assertTreeLayout(
+            ['', 'a', 'b', 'dir/', 'dir/subdir/', 'dir/subdir/foo'],
             tree)
 
     def test_add_multiple_with_file_ids(self):
@@ -149,41 +135,44 @@
         self.build_tree(['a', 'b', 'dir/', 'dir/subdir/', 'dir/subdir/foo'])
         tree.add(['a', 'b', 'dir', 'dir/subdir', 'dir/subdir/foo'],
                  ['a-id', 'b-id', 'dir-id', 'subdir-id', 'foo-id'])
-        root_id = tree.get_root_id()
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', tree.get_root_id()), ('a', 'a-id'), ('b', 'b-id'),
                                ('dir/', 'dir-id'), ('dir/subdir/', 'subdir-id'),
                                ('dir/subdir/foo', 'foo-id')], tree)
 
     def test_add_invalid(self):
         tree = self.make_branch_and_tree('.')
+        if not tree._format.supports_versioned_directories:
+            raise tests.TestNotApplicable(
+                'format does not support versioned directories')
         self.build_tree(['dir/', 'dir/subdir/', 'dir/subdir/foo'])
-        root_id = tree.get_root_id()
 
         self.assertRaises(errors.NotVersionedError,
                           tree.add, ['dir/subdir'])
-        self.assertTreeLayout([('', root_id)], tree)
+        self.assertTreeLayout([''], tree)
 
     def test_add_after_remove(self):
         tree = self.make_branch_and_tree('.')
+        if not tree._format.supports_versioned_directories:
+            raise tests.TestNotApplicable(
+                'format does not support versioned directories')
         self.build_tree(['dir/', 'dir/subdir/', 'dir/subdir/foo'])
-        root_id = tree.get_root_id()
         tree.add(['dir'])
         tree.commit('dir')
         tree.unversion(['dir'])
         self.assertRaises(errors.NotVersionedError,
-                          tree.add, ['dir/subdir'])
+                          tree.add, ['dir/subdir/foo'])
 
     def test_add_root(self):
         # adding the root should be a no-op, or at least not
         # do anything whacky.
         tree = self.make_branch_and_tree('.')
-        tree.lock_write()
-        tree.add('')
-        self.assertEqual([''], list(tree.all_versioned_paths()))
-        # the root should have been changed to be a new unique root.
-        self.assertNotEqual(inventory.ROOT_ID, tree.path2id(''))
-        tree.unlock()
+        with tree.lock_write():
+            tree.add('')
+            self.assertEqual([''], list(tree.all_versioned_paths()))
+            # the root should have been changed to be a new unique root.
+            if tree._format.supports_setting_file_ids:
+                self.assertNotEqual(inventory.ROOT_ID, tree.path2id(''))
 
     def test_add_previously_added(self):
         # adding a path that was previously added should work

=== modified file 'breezy/tests/per_workingtree/test_basis_inventory.py'
--- breezy/tests/per_workingtree/test_basis_inventory.py	2017-08-07 19:09:47 +0000
+++ breezy/tests/per_workingtree/test_basis_inventory.py	2018-03-05 01:29:18 +0000
@@ -33,6 +33,10 @@
 
         # Make sure the basis file is created by a commit
         t = self.make_branch_and_tree('.')
+        if getattr(t, 'root_inventory', None) is None:
+            raise TestNotApplicable("not applicable to %r"
+                % (self.workingtree_format,))
+
         b = t.branch
         with open('a', 'wb') as f: f.write('a\n')
         t.add('a')
@@ -68,6 +72,11 @@
             raise TestNotApplicable("not applicable to %r"
                 % (self.workingtree_format,))
         t = self.make_branch_and_tree('.')
+        if getattr(t, 'root_inventory', None) is None:
+            raise TestNotApplicable("not applicable to %r"
+                % (self.workingtree_format,))
+
+
         b = t.branch
         with open('a', 'wb') as f: f.write('a\n')
         t.add('a')

=== modified file 'breezy/tests/per_workingtree/test_check_state.py'
--- breezy/tests/per_workingtree/test_check_state.py	2017-05-21 18:10:28 +0000
+++ breezy/tests/per_workingtree/test_check_state.py	2018-03-20 20:43:18 +0000
@@ -81,8 +81,11 @@
         tree = self.make_initial_tree()
         foo_id = tree.path2id('foo')
         tree.rename_one('foo', 'baz')
-        self.assertEqual(None, tree.path2id('foo'))
-        self.assertEqual(foo_id, tree.path2id('baz'))
+        self.assertFalse(tree.is_versioned('foo'))
+        if tree.supports_rename_tracking():
+            self.assertEqual(foo_id, tree.path2id('baz'))
+        else:
+            self.assertTrue(tree.is_versioned('baz'))
         tree.reset_state()
         # After reset, we should have forgotten about the rename, but we won't
         # have

=== modified file 'breezy/tests/per_workingtree/test_commit.py'
--- breezy/tests/per_workingtree/test_commit.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_commit.py	2018-03-25 00:39:16 +0000
@@ -33,6 +33,7 @@
     CannotCommitSelectedFileMerge,
     PointlessCommit,
     )
+from breezy.tests.matchers import HasPathRelations
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 from breezy.tests.testui import ProgressRecordingUIFactory
 
@@ -66,51 +67,37 @@
         tree_a = self.make_branch_and_tree('a')
         self.build_tree(['a/dir/', 'a/dir/f1', 'a/dir/f2', 'a/dir2/'])
         tree_a.add(['dir', 'dir/f1', 'dir/f2', 'dir2'])
-        dir2_id =  tree_a.path2id('dir2')
-        f1_id = tree_a.path2id('dir/f1')
         rev_id1 = tree_a.commit('init')
+        revtree = tree_a.branch.repository.revision_tree(rev_id1)
         # Rename one entry out of this directory
         tree_a.rename_one('dir/f1', 'dir2/a')
         osutils.rmtree('a/dir')
         tree_a.commit('autoremoved')
 
-        with tree_a.lock_read():
-            root_id = tree_a.get_root_id()
-            paths = [(path, ie.file_id)
-                     for path, ie in tree_a.iter_entries_by_dir()]
         # The only paths left should be the root
-        self.assertEqual([('', root_id),
-                          ('dir2', dir2_id),
-                          ('dir2/a', f1_id),
-                         ], paths)
+        self.assertThat(
+            tree_a, HasPathRelations(
+                revtree,
+                [('', ''), ('dir2/', 'dir2/'), ('dir2/a', 'dir/f1')]))
 
     def test_no_autodelete_alternate_renamed(self):
         # Test for bug #114615
         tree_a = self.make_branch_and_tree('A')
         self.build_tree(['A/a/', 'A/a/m', 'A/a/n'])
         tree_a.add(['a', 'a/m', 'a/n'])
-        a_id = tree_a.path2id('a')
-        m_id = tree_a.path2id('a/m')
-        n_id = tree_a.path2id('a/n')
         tree_a.commit('init')
 
-        with tree_a.lock_read():
-            root_id = tree_a.get_root_id()
-
         tree_b = tree_a.controldir.sprout('B').open_workingtree()
         self.build_tree(['B/xyz/'])
         tree_b.add(['xyz'])
-        xyz_id = tree_b.path2id('xyz')
         tree_b.rename_one('a/m', 'xyz/m')
         osutils.rmtree('B/a')
         tree_b.commit('delete in B')
 
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('xyz', xyz_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+        self.assertThat(
+                tree_b,
+                HasPathRelations(
+                    tree_a, [('', ''), ('xyz/', None), ('xyz/m', 'a/m')]))
 
         self.build_tree_contents([('A/a/n', b'new contents for n\n')])
         tree_a.commit('change n in A')
@@ -119,14 +106,13 @@
         # (in A) and removed (in B), so 'a' needs to be restored.
         num_conflicts = tree_b.merge_from_branch(tree_a.branch)
         self.assertEqual(3, num_conflicts)
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('a', a_id),
-                          ('xyz', xyz_id),
-                          ('a/n.OTHER', n_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+
+        self.assertThat(
+                tree_b, HasPathRelations(
+                    tree_a,
+                    [('', ''), ('a/', 'a/'), ('xyz/', None),
+                     ('a/n.OTHER', 'a/n'), ('xyz/m', 'a/m')]))
+
         osutils.rmtree('B/a')
         try:
             # bzr resolve --all
@@ -136,12 +122,11 @@
             # effect.
             pass
         tree_b.commit('autoremove a, without touching xyz/m')
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('xyz', xyz_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+
+        self.assertThat(
+                tree_b, HasPathRelations(
+                    tree_a,
+                    [('', ''), ('xyz/', None), ('xyz/m', 'a/m')]))
 
     def test_commit_exclude_pending_merge_fails(self):
         """Excludes are a form of partial commit."""
@@ -173,15 +158,13 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         changes = list(tree.iter_changes(tree.basis_tree()))
-        self.assertEqual(2, len(changes))
-        self.assertEqual((None, 'b'), changes[0][1])
-        self.assertEqual((None, 'c'), changes[1][1])
+        self.assertEqual([(None, 'b'), (None, 'c')], [c[1] for c in changes])
 
     def test_commit_exclude_subtree_of_selected(self):
         tree = self.make_branch_and_tree('.')
-        self.build_tree(['a/', 'a/b'])
+        self.build_tree(['a/', 'a/b', 'a/c'])
         tree.smart_add(['.'])
-        tree.commit('test', specific_files=['a'], exclude=['a/b'])
+        tree.commit('test', specific_files=['a', 'a/c'], exclude=['a/b'])
         # If a/b was excluded it will still be 'added' in status.
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -235,6 +218,7 @@
         wt = self.make_branch_and_tree('.')
         wt.add(['a'], None, ['file'])
         a_id = wt.path2id('a')
+        self.assertEqual('a', wt.id2path(a_id))
         def fail_message(obj):
             raise errors.BzrCommandError("empty commit message")
         self.assertRaises(errors.BzrCommandError, wt.commit,
@@ -362,7 +346,7 @@
         this_dir.delete_tree('b')
         wt.lock_write()
         wt.commit('commit deleted rename')
-        self.assertTrue(wt.has_id(a_id))
+        self.assertTrue(wt.is_versioned('a'))
         self.assertFalse(wt.has_or_had_id(b_id))
         self.assertFalse(wt.has_or_had_id(c_id))
         self.assertTrue(wt.has_filename('a'))

=== modified file 'breezy/tests/per_workingtree/test_executable.py'
--- breezy/tests/per_workingtree/test_executable.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_executable.py	2018-03-25 00:39:16 +0000
@@ -56,7 +56,7 @@
                 the inventory is empty, just that the tree doesn't have them
         """
         tree.lock_read()
-        if not ignore_inv:
+        if not ignore_inv and getattr(tree, 'root_inventory', None):
             self.assertEqual(
                 [('', tree.root_inventory.root)],
                 list(tree.root_inventory.iter_entries()))
@@ -68,14 +68,14 @@
 
     def commit_and_branch(self):
         """Commit the current tree, and create a second tree"""
-        self.wt.commit('adding a,b', rev_id=b'r1')
+        r1 = self.wt.commit('adding a,b')
         # Now make sure that 'bzr branch' also preserves the
         # executable bit
-        dir2 = self.wt.branch.controldir.sprout('b2', revision_id='r1')
+        dir2 = self.wt.branch.controldir.sprout('b2', revision_id=r1)
         wt2 = dir2.open_workingtree()
-        self.assertEqual(['r1'], wt2.get_parent_ids())
-        self.assertEqual('r1', wt2.branch.last_revision())
-        return wt2
+        self.assertEqual([r1], wt2.get_parent_ids())
+        self.assertEqual(r1, wt2.branch.last_revision())
+        return wt2, r1
 
     def test_01_is_executable(self):
         """Make sure that the tree was created and has the executable bit set"""
@@ -88,14 +88,14 @@
 
     def test_03_after_commit(self):
         """Commit the change, and check the history"""
-        self.wt.commit('adding a,b', rev_id=b'r1')
+        r1 = self.wt.commit('adding a,b')
 
-        rev_tree = self.wt.branch.repository.revision_tree('r1')
+        rev_tree = self.wt.branch.repository.revision_tree(r1)
         self.check_exist(rev_tree)
 
     def test_04_after_removed(self):
         """Make sure reverting removed files brings them back correctly"""
-        self.wt.commit('adding a,b', rev_id=b'r1')
+        r1 = self.wt.commit('adding a,b')
 
         # Make sure the entries are gone
         os.remove('b1/a')
@@ -105,24 +105,24 @@
         # Make sure that revert is able to bring them back,
         # and sets 'a' back to being executable
 
-        rev_tree = self.wt.branch.repository.revision_tree('r1')
+        rev_tree = self.wt.branch.repository.revision_tree(r1)
 
         self.wt.revert(['a', 'b'], rev_tree, backups=False)
         self.check_exist(self.wt)
 
     def test_05_removed_and_committed(self):
         """Check that reverting to an earlier commit restores them"""
-        self.wt.commit('adding a,b', rev_id=b'r1')
+        r1 = self.wt.commit('adding a,b')
 
         # Now remove them again, and make sure that after a
         # commit, they are still marked correctly
         os.remove('b1/a')
         os.remove('b1/b')
-        self.wt.commit('removed', rev_id=b'r2')
+        r2 = self.wt.commit('removed')
 
         self.check_empty(self.wt)
 
-        rev_tree = self.wt.branch.repository.revision_tree('r1')
+        rev_tree = self.wt.branch.repository.revision_tree(r1)
         # Now revert back to the previous commit
         self.wt.revert(old_tree=rev_tree, backups=False)
 
@@ -131,38 +131,38 @@
     def test_06_branch(self):
         """branch b1=>b2 should preserve the executable bits"""
         # TODO: Maybe this should be a blackbox test
-        wt2 = self.commit_and_branch()
+        wt2, r1 = self.commit_and_branch()
 
         self.check_exist(wt2)
 
     def test_07_pull(self):
         """Test that pull will handle bits correctly"""
-        wt2 = self.commit_and_branch()
+        wt2, r1 = self.commit_and_branch()
 
         os.remove('b1/a')
         os.remove('b1/b')
-        self.wt.commit('removed', rev_id=b'r2')
+        r2 = self.wt.commit('removed')
 
         # now wt2 can pull and the files should be removed
 
         # Make sure pull will delete the files
         wt2.pull(self.wt.branch)
-        self.assertEqual(['r2'], wt2.get_parent_ids())
-        self.assertEqual('r2', wt2.branch.last_revision())
+        self.assertEqual([r2], wt2.get_parent_ids())
+        self.assertEqual(r2, wt2.branch.last_revision())
         self.check_empty(wt2)
 
         # Now restore the files on the first branch and commit
         # so that the second branch can pull the changes
         # and make sure that the executable bit has been copied
-        rev_tree = self.wt.branch.repository.revision_tree('r1')
+        rev_tree = self.wt.branch.repository.revision_tree(r1)
         self.wt.revert(old_tree=rev_tree, backups=False)
-        self.wt.commit('resurrected', rev_id=b'r3')
+        r3 = self.wt.commit('resurrected')
 
         self.check_exist(self.wt)
 
         wt2.pull(self.wt.branch)
-        self.assertEqual(['r3'], wt2.get_parent_ids())
-        self.assertEqual('r3', wt2.branch.last_revision())
+        self.assertEqual([r3], wt2.get_parent_ids())
+        self.assertEqual(r3, wt2.branch.last_revision())
         self.check_exist(wt2)
 
     def test_08_no_op_revert(self):
@@ -170,8 +170,8 @@
 
         The bits shouldn't swap.
         """
-        self.wt.commit('adding a,b', rev_id=b'r1')
-        rev_tree = self.wt.branch.repository.revision_tree('r1')
+        r1 = self.wt.commit('adding a,b')
+        rev_tree = self.wt.branch.repository.revision_tree(r1)
         self.wt.revert(old_tree=rev_tree, backups=False)
         self.check_exist(self.wt)
 
@@ -180,8 +180,8 @@
             self.wt._is_executable_from_path_and_stat_from_basis
         rev_id1 = self.wt.commit('one')
         rev_tree1 = self.wt.branch.repository.revision_tree(rev_id1)
-        a_executable = rev_tree1.root_inventory[self.a_id].executable
-        b_executable = rev_tree1.root_inventory[self.b_id].executable
+        a_executable = rev_tree1.is_executable('a')
+        b_executable = rev_tree1.is_executable('b')
         self.assertIsNot(None, a_executable)
         self.assertTrue(a_executable)
         self.assertIsNot(None, b_executable)

=== modified file 'breezy/tests/per_workingtree/test_flush.py'
--- breezy/tests/per_workingtree/test_flush.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_flush.py	2018-03-25 00:39:16 +0000
@@ -52,7 +52,7 @@
         tree.lock_write()
         try:
             old_root = tree.get_root_id()
-            tree.set_root_id(b'new-root')
+            tree.add('')
             # to detect that the inventory is written by flush, we
             # first check that it was not written yet.
             reference_tree = tree.controldir.open_workingtree()
@@ -61,7 +61,7 @@
             tree.flush()
             # and check it was written using another reference tree
             reference_tree = tree.controldir.open_workingtree()
-            self.assertEqual('new-root', reference_tree.get_root_id())
+            self.assertIsNot(None, reference_tree.get_root_id())
         finally:
             tree.unlock()
 

=== modified file 'breezy/tests/per_workingtree/test_get_parent_ids.py'
--- breezy/tests/per_workingtree/test_get_parent_ids.py	2017-06-10 00:17:06 +0000
+++ breezy/tests/per_workingtree/test_get_parent_ids.py	2018-03-24 13:45:51 +0000
@@ -17,6 +17,7 @@
 
 """Tests for interface conformance of 'workingtree.get_parent_ids'"""
 
+from breezy.tests import TestNotApplicable
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -39,6 +40,10 @@
         """Test the correspondence between set pending merges and get_parent_ids."""
         wt = self.make_branch_and_tree('.')
         self.assertEqual([], wt.get_parent_ids())
+        if not wt._format. supports_righthand_parent_id_as_ghost:
+            raise TestNotApplicable(
+                    'format does not support right hand side parents '
+                    'that are ghosts')
         # the first pending merge replaces the 'last revision' because
         # 'last revision' is shorthand for 'left most parent'
         wt.add_pending_merge('foo@azkhazan-123123-abcabc')

=== modified file 'breezy/tests/per_workingtree/test_inv.py'
--- breezy/tests/per_workingtree/test_inv.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_inv.py	2018-03-25 00:39:16 +0000
@@ -178,6 +178,6 @@
         # wt.current_dirstate()'s idea about what files are where.
         ie = base.inventory[b'subdir-id']
         self.assertEqual('directory', ie.kind)
-        path, ie = next(base.iter_entries_by_dir([b'subdir-id']))
+        path, ie = next(base.iter_entries_by_dir(specific_files=['subdir']))
         self.assertEqual('subdir', path)
         self.assertEqual('tree-reference', ie.kind)

=== modified file 'breezy/tests/per_workingtree/test_is_ignored.py'
--- breezy/tests/per_workingtree/test_is_ignored.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_is_ignored.py	2018-03-25 00:39:16 +0000
@@ -15,21 +15,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from breezy import config, ignores
+from breezy import config, ignores, tests
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
 class TestIsIgnored(TestCaseWithWorkingTree):
 
+    def setUp(self):
+        super(TestIsIgnored, self).setUp()
+        if self.workingtree_format.ignore_filename != '.bzrignore':
+            raise tests.TestNotApplicable(
+                'format does not use .bzrignore for ignore patterns')
+
     def _set_user_ignore_content(self, ignores):
         """Create user ignore file and set its content to ignores."""
         config.ensure_config_dir_exists()
         user_ignore_file = config.user_ignore_config_filename()
-        f = open(user_ignore_file, 'wb')
-        try:
+        with open(user_ignore_file, 'wb') as f:
             f.write(ignores)
-        finally:
-            f.close()
 
     def test_is_ignored(self):
         tree = self.make_branch_and_tree('.')
@@ -76,7 +79,7 @@
         self.assertEqual("path/from/ro?t", tree.is_ignored('path/from/root'))
         self.assertEqual("path/from/ro?t", tree.is_ignored('path/from/roat'))
         self.assertEqual(None, tree.is_ignored('roat'))
-        
+
         self.assertEqual('**/piffle.py', tree.is_ignored('piffle.py'))
         self.assertEqual('**/piffle.py', tree.is_ignored('a/piffle.py'))
         self.assertEqual(None, tree.is_ignored('b/piffle.py')) # exclusion
@@ -96,7 +99,7 @@
         self.assertEqual('*bar', tree.is_ignored(r'foo\nbar'))
         self.assertEqual('*bar', tree.is_ignored('bar'))
         self.assertEqual('*bar', tree.is_ignored('.bar'))
-        
+
         self.assertEqual(None, tree.is_ignored('bazbar')) # exclusion
 
         self.assertEqual('?foo', tree.is_ignored('afoo'))
@@ -110,7 +113,7 @@
 
         self.assertEqual('dir1/?f2', tree.is_ignored('dir1/ff2'))
         self.assertEqual('dir1/?f2', tree.is_ignored('dir1/.f2'))
-        
+
         self.assertEqual('RE:dir2/.*\.wombat', tree.is_ignored('dir2/foo.wombat'))
         self.assertEqual(None, tree.is_ignored('dir2/foo'))
 

=== modified file 'breezy/tests/per_workingtree/test_locking.py'
--- breezy/tests/per_workingtree/test_locking.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_locking.py	2018-03-25 00:39:16 +0000
@@ -149,7 +149,7 @@
         # moment trigger inventory writes and thus will not
         # let us trigger a read-when-dirty situation.
         old_root = tree.get_root_id()
-        tree.set_root_id(b'new-root')
+        tree.add('')
         # to detect that the inventory is written by unlock, we
         # first check that it was not written yet.
         # TODO: This requires taking a read lock while we are holding the above
@@ -164,7 +164,7 @@
         tree.unlock()
         # and check it was written using another reference tree
         reference_tree = tree.controldir.open_workingtree()
-        self.assertEqual('new-root', reference_tree.get_root_id())
+        self.assertIsNot(None, reference_tree.get_root_id())
 
     def test_unlock_from_tree_write_lock_flushes(self):
         self._test_unlock_with_lock_method("lock_tree_write")

=== modified file 'breezy/tests/per_workingtree/test_merge_from_branch.py'
--- breezy/tests/per_workingtree/test_merge_from_branch.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_merge_from_branch.py	2018-03-25 00:39:16 +0000
@@ -38,8 +38,8 @@
         self.tree_from = self.make_branch_and_tree('from')
         self.first_rev = self.tree_from.commit('first post')
         self.tree_to = self.tree_from.controldir.sprout('to').open_workingtree()
-        self.second_rev = self.tree_from.commit('second rev', allow_pointless=True)
-        self.to_second_rev = self.tree_to.commit('second rev', allow_pointless=True)
+        self.second_rev = self.tree_from.commit('second rev on from', allow_pointless=True)
+        self.to_second_rev = self.tree_to.commit('second rev on to', allow_pointless=True)
 
     def test_smoking_merge(self):
         """Smoke test of merge_from_branch."""
@@ -111,7 +111,7 @@
         self.build_tree_contents([('this/foo', b'baz')])
         this.commit('content -> baz')
         class QuxMerge(merge.Merge3Merger):
-            def text_merge(self, file_id, trans_id):
+            def text_merge(self, trans_id, paths, file_id):
                 self.tt.create_file('qux', trans_id)
         this.merge_from_branch(other.branch, merge_type=QuxMerge)
         self.assertEqual('qux', this.get_file_text('foo'))
@@ -122,29 +122,28 @@
     def make_inner_branch(self):
         bld_inner = self.make_branch_builder('inner')
         bld_inner.start_series()
-        bld_inner.build_snapshot(
+        rev1 = bld_inner.build_snapshot(
             None,
             [('add', ('', 'inner-root-id', 'directory', '')),
              ('add', ('dir', 'dir-id', 'directory', '')),
              ('add', ('dir/file1', 'file1-id', 'file', 'file1 content\n')),
              ('add', ('file3', 'file3-id', 'file', 'file3 content\n')),
-             ], revision_id='1')
-        bld_inner.build_snapshot(
-            ['1'],
+             ])
+        rev4 = bld_inner.build_snapshot(
+            [rev1],
             [('add', ('file4', 'file4-id', 'file', 'file4 content\n'))
-             ], revision_id='4')
-        bld_inner.build_snapshot(
-            ['4'], [('rename', ('file4', 'dir/file4'))], revision_id='5')
-        bld_inner.build_snapshot(
-            ['1'], [('modify', ('file3-id', 'new file3 contents\n')),],
-            revision_id='3')
-        bld_inner.build_snapshot(
-            ['1'],
+             ])
+        rev5 = bld_inner.build_snapshot(
+            [rev4], [('rename', ('file4', 'dir/file4'))])
+        rev3 = bld_inner.build_snapshot(
+            [rev1], [('modify', ('file3-id', 'new file3 contents\n')),])
+        rev2 = bld_inner.build_snapshot(
+            [rev1],
             [('add', ('dir/file2', 'file2-id', 'file', 'file2 content\n')),
              ], revision_id='2')
         bld_inner.finish_series()
         br = bld_inner.get_branch()
-        return br
+        return br, [rev1, rev2, rev3, rev4, rev5]
 
     def assertTreeLayout(self, expected, tree):
         tree.lock_read()
@@ -161,18 +160,18 @@
         self.build_tree_contents([('outer/foo', b'foo')])
         outer.add('foo')
         outer.commit('added foo')
-        inner = self.make_inner_branch()
-        outer.merge_from_branch(inner, to_revision=b'1', from_revision=b'null:')
+        inner, revs = self.make_inner_branch()
+        outer.merge_from_branch(inner, to_revision=revs[0], from_revision=b'null:')
         #retain original root id.
         outer.set_root_id(outer.basis_tree().get_root_id())
         outer.commit('merge inner branch')
         outer.mkdir('dir-outer', b'dir-outer-id')
         outer.move(['dir', 'file3'], to_dir='dir-outer')
         outer.commit('rename imported dir and file3 to dir-outer')
-        return outer, inner
+        return outer, inner, revs
 
     def test_file1_deleted_in_dir(self):
-        outer, inner = self.make_outer_tree()
+        outer, inner, revs = self.make_outer_tree()
         outer.remove(['dir-outer/dir/file1'], keep_files=False)
         outer.commit('delete file1')
         outer.merge_from_branch(inner)
@@ -186,7 +185,7 @@
 
     def test_file3_deleted_in_root(self):
         # Reproduce bug #375898
-        outer, inner = self.make_outer_tree()
+        outer, inner, revs = self.make_outer_tree()
         outer.remove(['dir-outer/file3'], keep_files=False)
         outer.commit('delete file3')
         outer.merge_from_branch(inner)
@@ -200,10 +199,10 @@
 
 
     def test_file3_in_root_conflicted(self):
-        outer, inner = self.make_outer_tree()
+        outer, inner, revs = self.make_outer_tree()
         outer.remove(['dir-outer/file3'], keep_files=False)
         outer.commit('delete file3')
-        nb_conflicts = outer.merge_from_branch(inner, to_revision='3')
+        nb_conflicts = outer.merge_from_branch(inner, to_revision=revs[2])
         self.assertEqual(4, nb_conflicts)
         self.assertTreeLayout(['dir-outer',
                                'dir-outer/dir',
@@ -218,8 +217,8 @@
                               outer)
 
     def test_file4_added_in_root(self):
-        outer, inner = self.make_outer_tree()
-        nb_conflicts = outer.merge_from_branch(inner, to_revision='4')
+        outer, inner, revs = self.make_outer_tree()
+        nb_conflicts = outer.merge_from_branch(inner, to_revision=revs[3])
         # file4 could not be added to its original root, so it gets added to
         # the new root with a conflict.
         self.assertEqual(1, nb_conflicts)
@@ -232,9 +231,9 @@
                               outer)
 
     def test_file4_added_then_renamed(self):
-        outer, inner = self.make_outer_tree()
+        outer, inner, revs = self.make_outer_tree()
         # 1 conflict, because file4 can't be put into the old root
-        self.assertEqual(1, outer.merge_from_branch(inner, to_revision='4'))
+        self.assertEqual(1, outer.merge_from_branch(inner, to_revision=revs[3]))
         try:
             outer.set_conflicts(conflicts.ConflictList())
         except errors.UnsupportedOperation:
@@ -244,7 +243,7 @@
             pass
         outer.commit('added file4')
         # And now file4 gets renamed into an existing dir
-        nb_conflicts = outer.merge_from_branch(inner, to_revision='5')
+        nb_conflicts = outer.merge_from_branch(inner, to_revision=revs[4])
         self.assertEqual(1, nb_conflicts)
         self.assertTreeLayout(['dir-outer',
                                'dir-outer/dir',

=== modified file 'breezy/tests/per_workingtree/test_move.py'
--- breezy/tests/per_workingtree/test_move.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_move.py	2018-03-25 00:39:16 +0000
@@ -24,7 +24,7 @@
     tests,
     )
 
-from breezy.tests.matchers import HasLayout
+from breezy.tests.matchers import HasLayout, HasPathRelations
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 from breezy.tests import (
     features,
@@ -33,6 +33,9 @@
 
 class TestMove(TestCaseWithWorkingTree):
 
+    def assertPathRelations(self, previous_tree, tree, relations):
+        self.assertThat(tree, HasPathRelations(previous_tree, relations))
+
     def assertTreeLayout(self, expected, tree):
         """Check that the tree has the correct layout."""
         self.assertThat(tree, HasLayout(expected))
@@ -125,11 +128,7 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'b', 'c', 'd'])
         tree.add(['a', 'c', 'd'])
-        a_id = tree.path2id('a')
-        c_id = tree.path2id('c')
-        d_id = tree.path2id('d')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.move, ['c', 'b', 'd'], 'a')
         self.assertRaises(errors.BzrMoveFailedError,
@@ -138,53 +137,41 @@
                           tree.move, ['d', 'c', 'b'], 'a')
         if osutils.lexists('a/c'):
             # If 'c' was actually moved, then 'd' should have also been moved
-            self.assertTreeLayout([('', root_id), ('a/', a_id),
-                                   ('a/c', c_id),  ('a/d', d_id)], tree)
+            self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a/', 'a/'), ('a/c', 'c'), ('a/d', 'd')])
         else:
-            self.assertTreeLayout([('', root_id), ('a/', a_id), ('c', c_id),
-                                   ('d', d_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c', c_id),
-                               ('d', d_id)], tree.basis_tree())
+            self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ( 'a/', 'a/'), ('c', 'c'), ('d', 'd')])
         tree._validate()
 
     def test_move_over_deleted(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'b'])
         tree.add(['a', 'a/b', 'b'])
-        a_id = tree.path2id('a')
-        ab_id = tree.path2id('a/b')
-        b_id = tree.path2id('b')
         tree.commit('initial')
 
-        root_id = tree.get_root_id()
         tree.remove(['a/b'], keep_files=False)
         self.assertEqual([('b', 'a/b')], tree.move(['b'], 'a'))
-        self.assertTreeLayout([('', root_id),
-                               ('a/', a_id),
-                               ('a/b', b_id),
-                              ], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a/', 'a/'), ('a/b', 'b')])
         tree._validate()
 
     def test_move_subdir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
         tree.add(['a', 'b', 'b/c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('b/c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
-        a_contents = tree.get_file_text('a', a_id)
-        self.assertEqual([('a', 'b/a')],
-            tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/a', a_id),
-                               ('b/c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/'), ('b/c', 'b/c')])
+        a_contents = tree.get_file_text('a')
+        self.assertEqual([('a', 'b/a')], tree.move(['a'], 'b'))
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/a', 'a'), ('b/c', 'b/c')])
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/a')
         tree._validate()
@@ -193,18 +180,13 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
         tree.add(['a', 'b', 'b/c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('b/c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
-        c_contents = tree.get_file_text('b/c', c_id)
+        c_contents = tree.get_file_text('b/c')
         self.assertEqual([('b/c', 'c')],
             tree.move(['b/c'], ''))
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/'), ('c', 'b/c')])
         self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'c')
         tree._validate()
@@ -213,25 +195,21 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/a', 'c'])
         tree.add(['a', 'b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         # Target already exists
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.move, ['c', 'a'], 'b')
         # 'c' may or may not have been moved, but either way the tree should
         # maintain a consistent state.
         if osutils.lexists('c'):
-            self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                                   ('c', c_id)], tree)
+            self.assertPathRelations(
+                    tree.basis_tree(), tree,
+                    [('', ''), ('a', 'a'), ('b/', 'b/'), ('c', 'c')])
         else:
             self.assertPathExists('b/c')
-            self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                                   ('b/c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('c', c_id)], tree.basis_tree())
+            self.assertPathRelations(
+                    tree.basis_tree(), tree,
+                    [('', ''), ('a', 'a'), ('b/', 'b/'), ('b/c', 'c')])
         tree._validate()
 
     def test_move_onto_self(self):
@@ -248,7 +226,6 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a'])
         tree.add(['a'])
-        a_id = tree.path2id('a')
         tree.commit('initial')
 
         self.assertRaises(errors.BzrMoveFailedError,
@@ -259,98 +236,76 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
         # We don't need after=True as long as source is missing and target
         # exists.
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/a', a_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/a', 'a')])
         tree._validate()
 
     def test_move_after_with_after(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
         # Passing after=True should work as well
-        self.assertEqual([('a', 'b/a')],
-            tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/a', a_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertEqual([('a', 'b/a')], tree.move(['a'], 'b', after=True))
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/a', 'a')])
         tree._validate()
 
     def test_move_after_no_target(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         # Passing after when the file hasn't been move raises an exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.move, ['a'], 'b', after=True)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+
+        self.assertTreeLayout(['', 'a','b/'], tree.basis_tree())
         tree._validate()
 
     def test_move_after_source_and_dest(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/a'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         # TODO: jam 20070225 I would usually use 'rb', but assertFileEqual
         #       uses 'r'.
-        a_file = open('a', 'r')
-        try:
+        with open('a', 'r') as a_file:
             a_text = a_file.read()
-        finally:
-            a_file.close()
-        ba_file = open('b/a', 'r')
-        try:
+        with open('b/a', 'r') as ba_file:
             ba_text = ba_file.read()
-        finally:
-            ba_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertTreeLayout(['', 'a', 'b/'], tree)
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.move, ['a'], 'b', after=False)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertTreeLayout(['', 'a', 'b/'], tree)
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(ba_text, 'b/a')
         # But you can pass after=True
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/a', a_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/a', 'a')])
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(ba_text, 'b/a')
@@ -359,19 +314,14 @@
     def test_move_directory(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c/', 'a/c/d', 'e/'])
-        tree.add(['a', 'a/b', 'a/c', 'a/c/d', 'e'],
-                 ['a-id', 'b-id', 'c-id', 'd-id', 'e-id'])
+        tree.add(['a', 'a/b', 'a/c', 'a/c/d', 'e'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
-        self.assertEqual([('a', 'e/a')],
-            tree.move(['a'], 'e'))
-        self.assertTreeLayout([('', root_id), ('e/', 'e-id'), ('e/a/', 'a-id'),
-                               ('e/a/b', 'b-id'), ('e/a/c/', 'c-id'),
-                               ('e/a/c/d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('e/', 'e-id'),
-                               ('a/b', 'b-id'), ('a/c/', 'c-id'),
-                               ('a/c/d', 'd-id')], tree.basis_tree())
+        self.assertEqual([('a', 'e/a')], tree.move(['a'], 'e'))
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('e/', 'e/'), ('e/a/', 'a/'), ('e/a/b', 'a/b'),
+                 ('e/a/c/', 'a/c/'), ('e/a/c/d', 'a/c/d')])
         tree._validate()
 
     def test_move_directory_into_parent(self):
@@ -380,178 +330,179 @@
                 "test requires versioned directories")
         tree = self.make_branch_and_tree('.')
         self.build_tree(['c/', 'c/b/', 'c/b/d/'])
-        tree.add(['c', 'c/b', 'c/b/d'],
-                 ['c-id', 'b-id', 'd-id'])
+        tree.add(['c', 'c/b', 'c/b/d'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         self.assertEqual([('c/b', 'b')],
                          tree.move(['c/b'], ''))
-        self.assertTreeLayout([('', root_id),
-                               ('b/', 'b-id'),
-                               ('c/', 'c-id'),
-                               ('b/d/', 'd-id'),
-                              ], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''),
+                 ('b/', 'c/b/'),
+                 ('c/', 'c/'),
+                 ('b/d/', 'c/b/d/')])
         tree._validate()
 
     def test_move_directory_with_children_in_subdir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c/', 'd/'])
-        tree.add(['a', 'a/b', 'a/c', 'd'],
-                 ['a-id', 'b-id', 'c-id', 'd-id'])
+        tree.add(['a', 'a/b', 'a/c', 'd'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'a/c/b')
-        self.assertTreeLayout([('', root_id),
-                               ('a/', 'a-id'),
-                               ('d/', 'd-id'),
-                               ('a/c/', 'c-id'),
-                               ('a/c/b', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('a/', 'a/'),
+             ('d/', 'd/'),
+             ('a/c/', 'a/c/'),
+             ('a/c/b', 'a/b'),
+             ])
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
-        self.assertTreeLayout([('', root_id),
-                               ('d/', 'd-id'),
-                               ('d/a/', 'a-id'),
-                               ('d/a/c/', 'c-id'),
-                               ('d/a/c/b', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('d/', 'd/'),
+             ('d/a/', 'a/'),
+             ('d/a/c/', 'a/c/'),
+             ('d/a/c/b', 'a/b'),
+             ])
         tree._validate()
 
     def test_move_directory_with_deleted_children(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c', 'a/d', 'b/'])
-        tree.add(['a', 'b', 'a/b', 'a/c', 'a/d'],
-                 ['a-id', 'b-id', 'ab-id', 'ac-id', 'ad-id'])
+        tree.add(['a', 'b', 'a/b', 'a/c', 'a/d'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         tree.remove(['a/b', 'a/d'])
 
         self.assertEqual([('a', 'b/a')],
                          tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id),
-                               ('b/', 'b-id'),
-                               ('b/a/', 'a-id'),
-                               ('b/a/c', 'ac-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('b/', 'b/'),
+             ('b/a/', 'a/'),
+             ('b/a/c', 'a/c')])
         tree._validate()
 
     def test_move_directory_with_new_children(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/c', 'b/'])
         tree.add(['a', 'b', 'a/c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        ac_id = tree.path2id('a/c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         self.build_tree(['a/b', 'a/d'])
         tree.add(['a/b', 'a/d'])
-        ab_id = tree.path2id('a/b')
-        ad_id = tree.path2id('a/d')
 
         self.assertEqual([('a', 'b/a')], tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id),
-                               ('b/', b_id),
-                               ('b/a/', a_id),
-                               ('b/a/b', ab_id),
-                               ('b/a/c', ac_id),
-                               ('b/a/d', ad_id),
-                              ], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''),
+                 ('b/', 'b/'),
+                 ('b/a/', 'a/'),
+                 ('b/a/b', None),
+                 ('b/a/c', 'a/c'),
+                 ('b/a/d', None),
+                 ])
         tree._validate()
 
     def test_move_directory_with_moved_children(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c', 'd', 'e/'])
-        tree.add(['a', 'a/b', 'a/c', 'd', 'e'],
-                 ['a-id', 'b-id', 'c-id', 'd-id', 'e-id'])
+        tree.add(['a', 'a/b', 'a/c', 'd', 'e'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         self.assertEqual([('a/b', 'b')],
                          tree.move(['a/b'], ''))
-        self.assertTreeLayout([('', root_id),
-                               ('a/', 'a-id'),
-                               ('b', 'b-id'),
-                               ('d', 'd-id'),
-                               ('e/', 'e-id'),
-                               ('a/c', 'c-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('a/', 'a/'),
+             ('b', 'a/b'),
+             ('d', 'd'),
+             ('e/', 'e/'),
+             ('a/c', 'a/c'),
+             ])
         self.assertEqual([('d', 'a/d')],
                          tree.move(['d'], 'a'))
-        self.assertTreeLayout([('', root_id),
-                               ('a/', 'a-id'),
-                               ('b', 'b-id'),
-                               ('e/', 'e-id'),
-                               ('a/c', 'c-id'),
-                               ('a/d', 'd-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('a/', 'a/'),
+             ('b', 'a/b'),
+             ('e/', 'e/'),
+             ('a/c', 'a/c'),
+             ('a/d', 'd'),
+             ])
         self.assertEqual([('a', 'e/a')],
                          tree.move(['a'], 'e'))
-        self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('e/', 'e-id'),
-                               ('e/a/', 'a-id'),
-                               ('e/a/c', 'c-id'),
-                               ('e/a/d', 'd-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('b', 'a/b'),
+             ('e/', 'e/'),
+             ('e/a/', 'a/'),
+             ('e/a/c', 'a/c'),
+             ('e/a/d', 'd'),
+             ])
         tree._validate()
 
     def test_move_directory_with_renamed_child(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c', 'd/'])
-        tree.add(['a', 'a/b', 'a/c', 'd'],
-                 ['a-id', 'b-id', 'c-id', 'd-id'])
+        tree.add(['a', 'a/b', 'a/c', 'd'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'a/d')
-        self.assertTreeLayout([('', root_id),
-                               ('a/', 'a-id'),
-                               ('d/', 'd-id'),
-                               ('a/c', 'c-id'),
-                               ('a/d', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('a/', 'a/'),
+             ('d/', 'd/'),
+             ('a/c', 'a/c'),
+             ('a/d', 'a/b')])
+
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
-        self.assertTreeLayout([('', root_id),
-                               ('d/', 'd-id'),
-                               ('d/a/', 'a-id'),
-                               ('d/a/c', 'c-id'),
-                               ('d/a/d', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''),
+             ('d/', 'd/'),
+             ('d/a/', 'a/'),
+             ('d/a/c', 'a/c'),
+             ('d/a/d', 'a/b')])
         tree._validate()
 
     def test_move_directory_with_swapped_children(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c', 'a/d', 'e/'])
-        tree.add(['a', 'a/b', 'a/c', 'a/d', 'e'],
-                 ['a-id', 'b-id', 'c-id', 'd-id', 'e-id'])
+        tree.add(['a', 'a/b', 'a/c', 'a/d', 'e'])
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'a/bb')
         tree.rename_one('a/d', 'a/b')
         tree.rename_one('a/bb', 'a/d')
-        self.assertTreeLayout([('', root_id),
-                               ('a/', 'a-id'),
-                               ('e/', 'e-id'),
-                               ('a/b', 'd-id'),
-                               ('a/c', 'c-id'),
-                               ('a/d', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''),
+                 ('a/', 'a/'),
+                 ('e/', 'e/'),
+                 ('a/b', 'a/d'),
+                 ('a/c', 'a/c'),
+                 ('a/d', 'a/b')])
         self.assertEqual([('a', 'e/a')],
                          tree.move(['a'], 'e'))
-        self.assertTreeLayout([('', root_id),
-                               ('e/', 'e-id'),
-                               ('e/a/', 'a-id'),
-                               ('e/a/b', 'd-id'),
-                               ('e/a/c', 'c-id'),
-                               ('e/a/d', 'b-id'),
-                              ], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''),
+                 ('e/', 'e/'),
+                 ('e/a/', 'a/'),
+                 ('e/a/b', 'a/d'),
+                 ('e/a/c', 'a/c'),
+                 ('e/a/d', 'a/b')])
         tree._validate()
 
     def test_move_moved(self):
@@ -559,25 +510,18 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'c/'])
         tree.add(['a', 'a/b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('a/b')
-        c_id = tree.path2id('c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         self.assertEqual([('a/b', 'c/b')],
             tree.move(['a/b'], 'c'))
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('c/b', b_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('a/b', b_id)], tree.basis_tree())
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''), ('a/', 'a/'), ('c/', 'c/'), ('c/b', 'a/b')])
 
-        self.assertEqual([('c/b', 'b')],
-            tree.move(['c/b'], ''))
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('b', b_id),
-                               ('c/', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('a/b', b_id)], tree.basis_tree())
+        self.assertEqual([('c/b', 'b')], tree.move(['c/b'], ''))
+        self.assertPathRelations(
+            tree.basis_tree(), tree,
+            [('', ''), ('a/', 'a/'), ('b', 'a/b'), ('c/', 'c/')])
         tree._validate()
 
     def test_move_to_unversioned_non_ascii_dir(self):
@@ -586,10 +530,13 @@
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a", u"\xA7/"])
         tree.add(["a"])
-        e = self.assertRaises(errors.BzrMoveFailedError,
-            tree.move, ["a"], u"\xA7")
-        self.assertIsInstance(e.extra, errors.NotVersionedError)
-        self.assertEqual(e.extra.path, u"\xA7")
+        if tree.has_versioned_directories():
+            e = self.assertRaises(errors.BzrMoveFailedError,
+                tree.move, ["a"], u"\xA7")
+            self.assertIsInstance(e.extra, errors.NotVersionedError)
+            self.assertEqual(e.extra.path, u"\xA7")
+        else:
+            tree.move(["a"], u"\xA7")
 
     def test_move_unversioned_non_ascii(self):
         """Check error when moving an unversioned non-ascii file"""

=== modified file 'breezy/tests/per_workingtree/test_nested_specifics.py'
--- breezy/tests/per_workingtree/test_nested_specifics.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_nested_specifics.py	2018-03-25 00:39:16 +0000
@@ -79,5 +79,5 @@
 
     def test_iter_entries_by_dir_autodetects_subtree(self):
         tree = self.prepare_with_subtree()
-        path, ie = next(tree.iter_entries_by_dir(['subtree-id']))
+        path, ie = next(tree.iter_entries_by_dir(specific_files=['subtree']))
         self.assertEqual('tree-reference', ie.kind)

=== modified file 'breezy/tests/per_workingtree/test_parents.py'
--- breezy/tests/per_workingtree/test_parents.py	2018-02-03 13:39:29 +0000
+++ breezy/tests/per_workingtree/test_parents.py	2018-03-24 13:45:31 +0000
@@ -28,10 +28,14 @@
     InventoryDirectory,
     InventoryLink,
     )
-from ...bzr.inventorytree import InventoryRevisionTree
+from ...bzr.inventorytree import (
+    InventoryRevisionTree,
+    InventoryTree,
+    )
 from ...sixish import (
     BytesIO,
     )
+from ...tests import TestNotApplicable
 from ..per_workingtree import TestCaseWithWorkingTree
 from .. import (
     features,
@@ -102,9 +106,14 @@
         # remove the tree's history
         uncommit(t.branch, tree=t)
         rev_tree = t.branch.repository.revision_tree(revision_in_repo)
-        t.set_parent_trees([(revision_in_repo, rev_tree),
-            ('another-missing', None)])
-        self.assertConsistentParents([revision_in_repo, 'another-missing'], t)
+        if t._format.supports_righthand_parent_id_as_ghost:
+            t.set_parent_trees([(revision_in_repo, rev_tree),
+                ('another-missing', None)])
+            self.assertConsistentParents([revision_in_repo, 'another-missing'], t)
+        else:
+            self.assertRaises(errors.GhostRevisionUnusableHere,
+                t.set_parent_trees, [(revision_in_repo, rev_tree),
+                ('another-missing', None)])
 
     def test_set_three_parents(self):
         t = self.make_branch_and_tree('.')
@@ -154,8 +163,12 @@
         # remove the tree's history
         uncommit(t.branch, tree=t)
         rev_tree = t.branch.repository.revision_tree(revision_in_repo)
-        t.set_parent_ids([revision_in_repo, 'another-missing'])
-        self.assertConsistentParents([revision_in_repo, 'another-missing'], t)
+        if t._format.supports_righthand_parent_id_as_ghost:
+            t.set_parent_ids([revision_in_repo, 'another-missing'])
+            self.assertConsistentParents([revision_in_repo, 'another-missing'], t)
+        else:
+            self.assertRaises(errors.GhostRevisionUnusableHere,
+                t.set_parent_ids, [revision_in_repo, 'another-missing'])
 
     def test_set_three_parents_ids(self):
         t = self.make_branch_and_tree('.')
@@ -316,8 +329,12 @@
         """Test adding the second parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         first_revision = tree.commit('first post')
-        tree.add_parent_tree_id('second')
-        self.assertConsistentParents([first_revision, 'second'], tree)
+        if tree._format.supports_righthand_parent_id_as_ghost:
+            tree.add_parent_tree_id('second')
+            self.assertConsistentParents([first_revision, 'second'], tree)
+        else:
+            self.assertRaises(errors.GhostRevisionUnusableHere,
+                    tree.add_parent_tree_id, 'second')
 
     def test_add_first_parent_tree(self):
         """Test adding the first parent id"""
@@ -360,8 +377,12 @@
         """Test adding the second parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         first_revision = tree.commit('first post')
-        tree.add_parent_tree(('second', None))
-        self.assertConsistentParents([first_revision, 'second'], tree)
+        if tree._format.supports_righthand_parent_id_as_ghost:
+            tree.add_parent_tree(('second', None))
+            self.assertConsistentParents([first_revision, 'second'], tree)
+        else:
+            self.assertRaises(errors.GhostRevisionUnusableHere,
+                    tree.add_parent_tree, ('second', None))
 
 
 class UpdateToOneParentViaDeltaTests(TestCaseWithWorkingTree):
@@ -375,11 +396,8 @@
 
     def assertDeltaApplicationResultsInExpectedBasis(self, tree, revid, delta,
         expected_inventory):
-        tree.lock_write()
-        try:
+        with tree.lock_write():
             tree.update_basis_by_delta(revid, delta)
-        finally:
-            tree.unlock()
         # check the last revision was adjusted to rev_id
         self.assertEqual(revid, tree.last_revision())
         # check the parents are what we expect
@@ -387,11 +405,8 @@
         # check that the basis tree has the inventory we expect from applying
         # the delta.
         result_basis = tree.basis_tree()
-        result_basis.lock_read()
-        try:
+        with result_basis.lock_read():
             self.assertEqual(expected_inventory, result_basis.root_inventory)
-        finally:
-            result_basis.unlock()
 
     def make_inv_delta(self, old, new):
         """Make an inventory delta from two inventories."""
@@ -413,6 +428,9 @@
 
     def fake_up_revision(self, tree, revid, shape):
 
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable("test requires inventory tree")
+
         class ShapeTree(InventoryRevisionTree):
 
             def __init__(self, shape):

=== modified file 'breezy/tests/per_workingtree/test_paths2ids.py'
--- breezy/tests/per_workingtree/test_paths2ids.py	2018-02-03 13:39:29 +0000
+++ breezy/tests/per_workingtree/test_paths2ids.py	2018-03-22 08:44:28 +0000
@@ -22,6 +22,7 @@
 """
 
 from breezy import errors
+from breezy.bzr.inventorytree import InventoryTree
 from breezy.tests import (
     features,
     TestNotApplicable,
@@ -62,16 +63,30 @@
 
     def test_paths_none_result_none(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
         tree.lock_read()
         self.assertEqual(None, tree.paths2ids(None))
         tree.unlock()
 
     def test_find_single_root(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
+
         self.assertExpectedIds([tree.path2id('')], tree, [''])
 
     def test_find_tree_and_clone_roots(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
+
         clone = tree.controldir.clone('clone').open_workingtree()
         clone.lock_tree_write()
         clone_root_id = 'new-id'
@@ -136,6 +151,11 @@
           new-child because its under dir in new.
         """
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
+
         self.build_tree(
             ['tree/dir/', 'tree/dir/child-moves', 'tree/dir/child-stays',
              'tree/dir/child-goes'])
@@ -161,6 +181,11 @@
 
     def test_unversioned_one_tree(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
+
         self.build_tree(['tree/unversioned'])
         self.assertExpectedIds([], tree, ['unversioned'], require_versioned=False)
         tree.lock_read()
@@ -172,6 +197,10 @@
         # should not raise an error: it must be unversioned in *all* trees to
         # error.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
         if not tree.supports_setting_file_ids():
             raise TestNotApplicable('tree does not support setting file ids')
         tree.commit('make basis')
@@ -185,6 +214,10 @@
         # should not raise an error: it must be unversioned in *all* trees to
         # error.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
         tree.commit('make basis')
         basis = tree.basis_tree()
         self.assertExpectedIds([], tree, ['unversioned'], [basis],
@@ -201,6 +234,10 @@
     def test_unversioned_non_ascii_one_tree(self):
         self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('.')
+        if not isinstance(tree, InventoryTree):
+            raise TestNotApplicable(
+                "test not applicable on non-inventory tests")
+
         self.build_tree([u"\xa7"])
         self.assertExpectedIds([], tree, [u"\xa7"], require_versioned=False)
         self.addCleanup(tree.lock_read().unlock)

=== modified file 'breezy/tests/per_workingtree/test_pull.py'
--- breezy/tests/per_workingtree/test_pull.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_pull.py	2018-03-25 00:39:16 +0000
@@ -103,7 +103,7 @@
         trunk = self.make_branch_deleting_dir('trunk')
         work = trunk.controldir.sprout('work', revision_id='2').open_workingtree()
         work.branch.get_config_stack().set(
-            'bzr.transform.orphan_policy', 'move')
+            'transform.orphan_policy', 'move')
         # Add some unversioned files in dir
         self.build_tree(['work/dir/foo',
                          'work/dir/subdir/',

=== modified file 'breezy/tests/per_workingtree/test_remove.py'
--- breezy/tests/per_workingtree/test_remove.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_remove.py	2018-03-25 00:39:16 +0000
@@ -36,7 +36,12 @@
         tree = self.get_tree(files)
         tree.add(files)
         tree.commit(message)
-        self.assertInWorkingTree(files)
+        if not tree.has_versioned_directories():
+            self.assertInWorkingTree(
+                [f for f in files if not f.endswith("/")])
+            self.assertPathExists(files)
+        else:
+            self.assertInWorkingTree(files)
         return tree
 
     def assertRemovedAndDeleted(self, files):
@@ -51,7 +56,6 @@
         """Check that files and directories are unversioned but not deleted."""
         tree = self.get_tree(TestRemove.files)
         tree.add(TestRemove.files)
-        self.assertInWorkingTree(TestRemove.files)
 
         tree.remove(TestRemove.files)
         self.assertRemovedAndNotDeleted(TestRemove.files)
@@ -76,7 +80,6 @@
         """Removal of newly added files must back them up."""
         tree = self.get_tree(TestRemove.files)
         tree.add(TestRemove.files)
-        self.assertInWorkingTree(TestRemove.files)
         tree.remove(TestRemove.files, keep_files=False)
         self.assertNotInWorkingTree(TestRemove.files)
         self.assertPathExists(TestRemove.backup_files)
@@ -97,7 +100,6 @@
         tree = self.get_committed_tree(TestRemove.files)
         for f in TestRemove.rfiles:
             osutils.delete_any(f)
-        self.assertInWorkingTree(TestRemove.files)
         self.assertPathDoesNotExist(TestRemove.files)
         tree.remove(TestRemove.files, keep_files=False)
         self.assertRemovedAndDeleted(TestRemove.files)
@@ -110,7 +112,6 @@
         for f in TestRemove.rfiles:
             tree.rename_one(f, f+'x')
         rfilesx = ['bx/cx', 'bx', 'ax', 'dx']
-        self.assertInWorkingTree(rfilesx)
         self.assertPathExists(rfilesx)
 
         tree.remove(rfilesx, keep_files=False)
@@ -126,20 +127,21 @@
         rfilesx = ['bx/cx', 'bx', 'ax', 'dx']
         self.build_tree_contents([('ax', b'changed and renamed!'),
                                   ('bx/cx', b'changed and renamed!')])
-        self.assertInWorkingTree(rfilesx)
         self.assertPathExists(rfilesx)
 
         tree.remove(rfilesx, keep_files=False)
         self.assertNotInWorkingTree(rfilesx)
         self.assertPathExists(['bx.~1~/cx.~1~', 'bx.~1~', 'ax.~1~'])
-        self.assertPathDoesNotExist('dx.~1~') # unchanged file
+        if tree.supports_rename_tracking():
+            self.assertPathDoesNotExist('dx.~1~') # unchanged file
+        else:
+            self.assertPathExists('dx.~1~') # renamed, so appears changed
         tree._validate()
 
     def test_force_remove_changed_files(self):
         """Check that changed files are removed and deleted when forced."""
         tree = self.get_tree(TestRemove.files)
         tree.add(TestRemove.files)
-        self.assertInWorkingTree(TestRemove.files)
 
         tree.remove(TestRemove.files, keep_files=False, force=True)
         self.assertRemovedAndDeleted(TestRemove.files)
@@ -201,7 +203,6 @@
         tree = self.get_tree(files)
         tree.add(files)
         ignores.add_runtime_ignores(["*ignored*"])
-        self.assertInWorkingTree(files)
         self.assertNotEqual(None, tree.is_ignored(files[0]))
 
         tree.remove(files, keep_files=False)
@@ -211,7 +212,7 @@
 
     def test_dont_remove_directory_with_unknowns(self):
         """Directories with unknowns should be backed up."""
-        directories = ['a/', 'b/', 'c/', 'c/c/']
+        directories = ['a/', 'b/', 'c/', 'c/c/', 'c/blah']
         tree = self.get_committed_tree(directories)
 
         self.build_tree(['a/unknown_file'])
@@ -288,7 +289,7 @@
     def test_remove_directory_with_renames(self):
         """Delete directory with renames in or out."""
 
-        files = ['a/', 'a/file', 'a/directory/', 'b/']
+        files = ['a/', 'a/file', 'a/directory/', 'a/directory/stuff', 'b/']
         files_to_move = ['a/file', 'a/directory/']
 
         tree = self.get_committed_tree(files)

=== modified file 'breezy/tests/per_workingtree/test_rename_one.py'
--- breezy/tests/per_workingtree/test_rename_one.py	2018-02-03 13:39:29 +0000
+++ breezy/tests/per_workingtree/test_rename_one.py	2018-03-12 23:28:22 +0000
@@ -26,16 +26,15 @@
 from breezy.tests import (
     features,
     )
-from breezy.tests.matchers import HasLayout
+from breezy.tests.matchers import HasPathRelations
 
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
 class TestRenameOne(TestCaseWithWorkingTree):
 
-    def assertTreeLayout(self, expected, tree):
-        """Check that the tree has the correct layout."""
-        self.assertThat(tree, HasLayout(expected))
+    def assertPathRelations(self, previous_tree, tree, relations):
+        self.assertThat(tree, HasPathRelations(previous_tree, relations))
 
     def test_rename_one_target_not_dir(self):
         tree = self.make_branch_and_tree('.')
@@ -79,17 +78,13 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
-        a_contents = tree.get_file_text('a', a_id)
+        a_contents = tree.get_file_text('a')
         tree.rename_one('a', 'foo')
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('foo', a_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('foo', 'a')])
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'foo')
 
@@ -97,17 +92,13 @@
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a', 'tree/b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
-        a_contents = tree.get_file_text('a', a_id)
+        a_contents = tree.get_file_text('a')
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/foo', a_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/foo', 'a')])
         self.assertPathDoesNotExist('tree/a')
         self.assertFileEqual(a_contents, 'tree/b/foo')
 
@@ -115,21 +106,15 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
         tree.add(['a', 'b', 'b/c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('b/c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
-        a_contents = tree.get_file_text('a', a_id)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/'), ('b/c', 'b/c')])
+        a_contents = tree.get_file_text('a')
         tree.rename_one('a', 'b/d')
-        self.assertTreeLayout([('', root_id), ('b/', b_id), ('b/c', c_id),
-                               ('b/d', a_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/c', 'b/c'), ('b/d', 'a')])
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/d')
 
@@ -137,17 +122,12 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
         tree.add(['a', 'b', 'b/c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('b/c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
-        c_contents = tree.get_file_text('b/c', c_id)
+        c_contents = tree.get_file_text('b/c')
         tree.rename_one('b/c', 'd')
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('d', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('b/c', c_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/'), ('d', 'b/c')])
         self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'd')
 
@@ -155,18 +135,13 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/a', 'c'])
         tree.add(['a', 'b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('c')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         # Target already exists
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/a')
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('c', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id),
-                               ('c', c_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/'), ('c', 'c')])
 
     def test_rename_one_onto_existing(self):
         tree = self.make_branch_and_tree('.')
@@ -199,54 +174,51 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
+
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b/', b_id),
-                               ('b/foo', a_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/foo', 'a')])
 
     def test_rename_one_after_with_after(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        if tree.has_versioned_directories():
+            self.assertPathRelations(tree.basis_tree(), tree,
+                    [('', ''), ('a', 'a'), ('b/', 'b/')])
+        else:
+            self.assertPathRelations(tree.basis_tree(), tree,
+                    [('', ''), ('a', 'a')])
+
         # Passing after=True should work as well
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b/', b_id),
-                               ('b/foo', a_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/foo', 'a')])
 
     def test_rename_one_after_dest_versioned(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a'])
         tree.add(['a'])
-        a_id = tree.path2id('a')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b')
         tree.add(['b'])
-        b_id = tree.path2id('b')
-
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b', b_id)],
-                              tree)
+
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b', None)])
+
         e = self.assertRaises(
                 errors.BzrMoveFailedError, tree.rename_one, 'a', 'b')
         self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
@@ -256,14 +228,13 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.unlink('a')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b', 'b')])
+
         e = self.assertRaises(
                 errors.BzrMoveFailedError,
                 tree.rename_one, 'a', 'b', after=True)
@@ -274,93 +245,80 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a'])
         tree.add(['a'])
-        a_id = tree.path2id('a')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b')
         tree.add(['b'])
-        b_id = tree.path2id('b')
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b', None)])
         tree.rename_one('a', 'b', after=True)
-        self.assertTreeLayout([('', root_id), ('b', a_id)], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b', 'a')])
 
     def test_rename_one_after_source_removed(self):
         """Rename even if the source was already unversioned."""
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
         tree.remove(['a'])
 
-        self.assertTreeLayout([('', root_id), ('b/', b_id)], tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/')])
+
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b/', b_id),
-                               ('b/foo', a_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/foo', 'a')])
 
     def test_rename_one_after_no_target(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         # Passing after when the file hasn't been rename_one raises an
         # exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.rename_one, 'a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
 
     def test_rename_one_after_source_and_dest(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/foo'])
         tree.add(['a', 'b'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         # TODO: jam 20070225 I would usually use 'rb', but assertFileEqual
         #       uses 'r'.
-        a_file = open('a', 'r')
-        try:
+        with open('a', 'r') as a_file:
             a_text = a_file.read()
-        finally:
-            a_file.close()
-        foo_file = open('b/foo', 'r')
-        try:
+        with open('b/foo', 'r') as foo_file:
             foo_text = foo_file.read()
-        finally:
-            foo_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/foo', after=False)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree)
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a', 'a'), ('b/', 'b/')])
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(foo_text, 'b/foo')
         # But you can pass after=True
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b/', b_id),
-                               ('b/foo', a_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a', a_id), ('b/', b_id)],
-                              tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('b/', 'b/'), ('b/foo', 'a')])
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(foo_text, 'b/foo')
@@ -369,44 +327,32 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c/', 'a/c/d', 'e/'])
         tree.add(['a', 'a/b', 'a/c', 'a/c/d', 'e'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('a/b')
-        c_id = tree.path2id('a/c')
-        d_id = tree.path2id('a/c/d')
-        e_id = tree.path2id('e')
         tree.commit('initial')
-        root_id = tree.get_root_id()
 
         tree.rename_one('a', 'e/f')
-        self.assertTreeLayout([('', root_id), ('e/', e_id), ('e/f/', a_id),
-                               ('e/f/b', b_id), ('e/f/c/', c_id),
-                               ('e/f/c/d', d_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('e/', e_id),
-                               ('a/b', b_id), ('a/c/', c_id),
-                               ('a/c/d', d_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('e/', 'e/'), ('e/f/', 'a/'),
+                 ('e/f/b', 'a/b'), ('e/f/c/', 'a/c/'),
+                 ('e/f/c/d', 'a/c/d')])
 
     def test_rename_one_moved(self):
         """Moving a moved entry works as expected."""
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'c/'])
         tree.add(['a', 'a/b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('a/b')
-        c_id = tree.path2id('c')
         tree.commit('initial')
         root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'c/foo')
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('c/foo', b_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('a/b', b_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a/', 'a/'), ('c/', 'c/'), ('c/foo', 'a/b')])
 
         tree.rename_one('c/foo', 'bar')
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('bar', b_id),
-                               ('c/', c_id)], tree)
-        self.assertTreeLayout([('', root_id), ('a/', a_id), ('c/', c_id),
-                               ('a/b', b_id)], tree.basis_tree())
+        self.assertPathRelations(
+                tree.basis_tree(), tree,
+                [('', ''), ('a/', 'a/'), ('bar', 'a/b'), ('c/', 'c/')])
 
     def test_rename_to_denormalised_fails(self):
         if osutils.normalizes_filenames():
@@ -434,10 +380,13 @@
         tree = self.make_branch_and_tree(".")
         self.build_tree(["a", u"\xA7/"])
         tree.add(["a"])
-        e = self.assertRaises(errors.BzrMoveFailedError,
-            tree.rename_one, "a", u"\xA7/a")
-        self.assertIsInstance(e.extra, errors.NotVersionedError)
-        self.assertEqual(e.extra.path, u"\xA7")
+        if tree.has_versioned_directories():
+            e = self.assertRaises(errors.BzrMoveFailedError,
+                tree.rename_one, "a", u"\xA7/a")
+            self.assertIsInstance(e.extra, errors.NotVersionedError)
+            self.assertEqual(e.extra.path, u"\xA7")
+        else:
+            tree.rename_one("a", u"\xA7/a")
 
     def test_rename_over_already_versioned_non_ascii(self):
         """Check error renaming over an already versioned non-ascii file"""

=== modified file 'breezy/tests/per_workingtree/test_revision_tree.py'
--- breezy/tests/per_workingtree/test_revision_tree.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/per_workingtree/test_revision_tree.py	2018-03-25 00:39:16 +0000
@@ -101,29 +101,29 @@
         self.build_tree(files, line_endings='binary',
                         transport=tree.controldir.root_transport)
         tree.add(files)
-        tree.commit('a, b and b/c', rev_id=b'base')
+        base_revid = tree.commit('a, b and b/c')
         tree2 = tree.controldir.sprout(relpath + '2').open_workingtree()
         # Delete 'a' in tree
         tree.remove('a', keep_files=False)
-        tree.commit('remove a', rev_id=b'this')
+        this_revid = tree.commit('remove a')
         # Delete 'c' in tree2
         tree2.remove('b/c', keep_files=False)
         tree2.remove('b', keep_files=False)
-        tree2.commit('remove b/c', rev_id=b'other')
+        other_revid = tree2.commit('remove b/c')
         # Merge tree2 into tree
         tree.merge_from_branch(tree2.branch)
-        return tree
+        return tree, [base_revid, this_revid, other_revid]
 
     def test_kind_parent_tree(self):
-        tree = self.make_branch_with_merged_deletions()
+        tree, [base_revid, this_revid, other_revid] = self.make_branch_with_merged_deletions()
         tree.lock_read()
         self.addCleanup(tree.unlock)
         parents = tree.get_parent_ids()
-        self.assertEqual(['this', 'other'], parents)
+        self.assertEqual([this_revid, other_revid], parents)
         basis = tree.revision_tree(parents[0])
         basis.lock_read()
         self.addCleanup(basis.unlock)
-        self.assertRaises(errors.NoSuchId, basis.kind, 'a')
+        self.assertRaises(errors.NoSuchFile, basis.kind, 'a')
         self.assertEqual(['directory', 'file'],
                          [basis.kind('b'), basis.kind('b/c')])
         try:
@@ -134,6 +134,6 @@
                 % type(tree))
         other.lock_read()
         self.addCleanup(other.unlock)
-        self.assertRaises(errors.NoSuchId, other.kind, 'b')
-        self.assertRaises(errors.NoSuchId, other.kind, 'c')
+        self.assertRaises(errors.NoSuchFile, other.kind, 'b')
+        self.assertRaises(errors.NoSuchFile, other.kind, 'c')
         self.assertEqual('file', other.kind('a'))

=== modified file 'breezy/tests/per_workingtree/test_shelf_manager.py'
--- breezy/tests/per_workingtree/test_shelf_manager.py	2017-09-11 02:55:57 +0000
+++ breezy/tests/per_workingtree/test_shelf_manager.py	2018-03-12 03:16:19 +0000
@@ -21,7 +21,6 @@
     tests,
     )
 from breezy.workingtree import ShelvingUnsupported
-from breezy.tests.matchers import HasLayout
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 

=== modified file 'breezy/tests/per_workingtree/test_smart_add.py'
--- breezy/tests/per_workingtree/test_smart_add.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_smart_add.py	2018-03-25 00:39:16 +0000
@@ -138,7 +138,7 @@
 
         self.build_tree(build_paths)
         wt = self.make_branch_and_tree('.')
-        if wt.user_url != wt.branch.user_url:
+        if wt.controldir.user_url != wt.branch.controldir.user_url:
             # Lightweight checkout, make sure we have a repo location.
             wt.branch.controldir.root_transport.mkdir('original')
         child_tree = self.make_branch_and_tree('original/child')
@@ -185,8 +185,12 @@
         self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS',
                         'inertiatic/foo.pyc'])
         added, ignored = wt.smart_add(u'.')
-        self.assertSubset(('inertiatic', 'inertiatic/esp', 'inertiatic/CVS'),
-                          added)
+        if wt.has_versioned_directories():
+            self.assertSubset(('inertiatic', 'inertiatic/esp', 'inertiatic/CVS'),
+                              added)
+        else:
+            self.assertSubset(('inertiatic/esp', 'inertiatic/CVS'),
+                              added)
         self.assertSubset(('*.py[co]',), ignored)
         self.assertSubset(('inertiatic/foo.pyc',), ignored['*.py[co]'])
 

=== modified file 'breezy/tests/per_workingtree/test_symlinks.py'
--- breezy/tests/per_workingtree/test_symlinks.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_symlinks.py	2018-03-25 00:39:16 +0000
@@ -65,9 +65,15 @@
             ('tree/dir/',),
             ('tree/dir/file', b'content'),
             ])
-        self.assertEqual(
-            tree.smart_add(['tree/link/file']),
-            ([u'dir', u'dir/file'], {}))
+        if tree.has_versioned_directories():
+            self.assertEqual(
+                tree.smart_add(['tree/link/file']),
+                ([u'dir', u'dir/file'], {}))
+        else:
+            self.assertEqual(
+                tree.smart_add(['tree/link/file']),
+                ([u'dir/file'], {}))
+
         # should add the actual parent directory, not the apparent parent
         # (which is actually a symlink)
         self.assertTrue(tree.is_versioned('dir/file'))
@@ -102,11 +108,8 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEqual([], list(tree.iter_changes(tree.basis_tree())))
-        if tree._format.supports_versioned_directories:
-            self.assertEqual(
-                ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
-        else:
-            self.assertEqual([], list(tree.list_files()))
+        self.assertEqual(
+            ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
 
     def test_dir_changes_to_symlink(self):
         # <https://bugs.launchpad.net/bzr/+bug/192859>:

=== modified file 'breezy/tests/per_workingtree/test_unversion.py'
--- breezy/tests/per_workingtree/test_unversion.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_unversion.py	2018-03-25 00:39:16 +0000
@@ -19,6 +19,7 @@
 from breezy import (
     errors,
     )
+from breezy.tests.matchers import HasPathRelations
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -44,29 +45,33 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['del/', 'del/sub/', 'del/sub/b'])
         tree.add(['del', 'del/sub', 'del/sub/b'])
-        b_id = tree.path2id('del/sub/b')
-        tree.commit('setup')
+        revid = tree.commit('setup')
+        revtree = tree.branch.repository.revision_tree(revid)
         tree.rename_one('del/sub', 'sub')
-        self.assertEqual('sub/b', tree.id2path(b_id))
-        tree.unversion(['del', 'sub/b'])
-        self.assertRaises(errors.NoSuchId, tree.id2path, b_id)
+        self.assertThat(
+                tree,
+                HasPathRelations(revtree, [('', ''), ('del/', 'del/'), ('sub/', 'del/sub/'), ('sub/b', 'del/sub/b')]))
+        if tree.has_versioned_directories():
+            tree.unversion(['del', 'sub/b'])
+        else:
+            tree.unversion(['sub/b'])
+        self.assertThat(
+                tree,
+                HasPathRelations(revtree, [('', ''), ('sub/', 'del/sub/')]))
 
     def test_unversion_several_files(self):
         """After unversioning several files, they should not be versioned."""
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'c'])
         tree.add(['a', 'b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('b')
-        c_id = tree.path2id('c')
         # within a lock unversion should take effect
         tree.lock_write()
         self.assertTrue(tree.is_versioned('a'))
         tree.unversion(['a', 'b'])
         self.assertFalse(tree.is_versioned('a'))
-        self.assertFalse(tree.has_id(a_id))
-        self.assertFalse(tree.has_id(b_id))
-        self.assertTrue(tree.has_id(c_id))
+        self.assertFalse(tree.is_versioned('a'))
+        self.assertFalse(tree.is_versioned('b'))
+        self.assertTrue(tree.is_versioned('c'))
         self.assertTrue(tree.has_filename('a'))
         self.assertTrue(tree.has_filename('b'))
         self.assertTrue(tree.has_filename('c'))
@@ -75,9 +80,9 @@
         # to be sure.
         tree = tree.controldir.open_workingtree()
         self.addCleanup(tree.lock_read().unlock)
-        self.assertFalse(tree.has_id(a_id))
-        self.assertFalse(tree.has_id(b_id))
-        self.assertTrue(tree.has_id(c_id))
+        self.assertFalse(tree.is_versioned('a'))
+        self.assertFalse(tree.is_versioned('b'))
+        self.assertTrue(tree.is_versioned('c'))
         self.assertTrue(tree.has_filename('a'))
         self.assertTrue(tree.has_filename('b'))
         self.assertTrue(tree.has_filename('c'))
@@ -87,19 +92,15 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'c'])
         tree.add(['a', 'a/b', 'c'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('a/b')
-        c_id = tree.path2id('c')
         # within a lock unversion should take effect
-        tree.lock_write()
-        tree.unversion(['a'])
-        self.assertFalse(tree.has_id(a_id))
-        self.assertFalse(tree.has_id(b_id))
-        self.assertTrue(tree.has_id(c_id))
-        self.assertTrue(tree.has_filename('a'))
-        self.assertTrue(tree.has_filename('a/b'))
-        self.assertTrue(tree.has_filename('c'))
-        tree.unlock()
+        with tree.lock_write():
+            tree.unversion(['a'])
+            self.assertFalse(tree.is_versioned('a'))
+            self.assertFalse(tree.is_versioned('a/b'))
+            self.assertTrue(tree.is_versioned('c'))
+            self.assertTrue(tree.has_filename('a'))
+            self.assertTrue(tree.has_filename('a/b'))
+            self.assertTrue(tree.has_filename('c'))
 
     def test_unversion_subtree_and_children(self):
         """Passing a child id will raise NoSuchId.
@@ -109,16 +110,12 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b', 'a/c', 'd'])
         tree.add(['a', 'a/b', 'a/c', 'd'])
-        a_id = tree.path2id('a')
-        b_id = tree.path2id('a/b')
-        c_id = tree.path2id('a/c')
-        d_id = tree.path2id('d')
         with tree.lock_write():
             tree.unversion(['a/b', 'a'])
-            self.assertFalse(tree.has_id(a_id))
-            self.assertFalse(tree.has_id(b_id))
-            self.assertFalse(tree.has_id(c_id))
-            self.assertTrue(tree.has_id(d_id))
+            self.assertFalse(tree.is_versioned('a'))
+            self.assertFalse(tree.is_versioned('a/b'))
+            self.assertFalse(tree.is_versioned('a/c'))
+            self.assertTrue(tree.is_versioned('d'))
             # The files are still on disk
             self.assertTrue(tree.has_filename('a'))
             self.assertTrue(tree.has_filename('a/b'))
@@ -130,72 +127,47 @@
         self.build_tree(['a/dir/', 'a/dir/f1', 'a/dir/f2', 'a/dir/f3',
                          'a/dir2/'])
         tree.add(['dir', 'dir/f1', 'dir/f2', 'dir/f3', 'dir2'])
-        dir_id = tree.path2id('dir')
-        dir2_id = tree.path2id('dir2')
-        f1_id = tree.path2id('dir/f1')
-        f2_id = tree.path2id('dir/f2')
-        f3_id = tree.path2id('dir/f3')
         rev_id1 = tree.commit('init')
+        revtree = tree.branch.repository.revision_tree(rev_id1)
         # Start off by renaming entries, and then unversion a bunch of entries
         # https://bugs.launchpad.net/bzr/+bug/114615
         tree.rename_one('dir/f1', 'dir/a')
         tree.rename_one('dir/f2', 'dir/z')
         tree.move(['dir/f3'], 'dir2')
 
-        tree.lock_read()
-        try:
-            root_id = tree.get_root_id()
-            paths = [(path, ie.file_id)
-                     for path, ie in tree.iter_entries_by_dir()]
-        finally:
-            tree.unlock()
-        self.assertEqual([('', root_id),
-                          ('dir', dir_id),
-                          ('dir2', dir2_id),
-                          ('dir/a', f1_id),
-                          ('dir/z', f2_id),
-                          ('dir2/f3', f3_id),
-                         ], paths)
+        self.assertThat(
+                tree, HasPathRelations(
+                    revtree,
+                    [('', ''), ('dir/', 'dir/'), ('dir2/', 'dir2/'),
+                     ('dir/a', 'dir/f1'), ('dir/z', 'dir/f2'),
+                     ('dir2/f3', 'dir/f3')]))
 
         tree.unversion({'dir'})
-        paths = [(path, ie.file_id)
-                 for path, ie in tree.iter_entries_by_dir()]
-
-        self.assertEqual([('', root_id),
-                          ('dir2', dir2_id),
-                          ('dir2/f3', f3_id),
-                         ], paths)
+        self.assertThat(
+                tree,
+                HasPathRelations(
+                    revtree,
+                    [('', ''), ('dir2/', 'dir2/'), ('dir2/f3', 'dir/f3')]))
 
     def test_unversion_after_conflicted_merge(self):
         # Test for bug #114615
         tree_a = self.make_branch_and_tree('A')
         self.build_tree(['A/a/', 'A/a/m', 'A/a/n'])
         tree_a.add(['a', 'a/m', 'a/n'])
-        a_id = tree_a.path2id('a')
-        m_id = tree_a.path2id('a/m')
-        n_id = tree_a.path2id('a/n')
         tree_a.commit('init')
 
-        tree_a.lock_read()
-        try:
-            root_id = tree_a.get_root_id()
-        finally:
-            tree_a.unlock()
-
         tree_b = tree_a.controldir.sprout('B').open_workingtree()
         self.build_tree(['B/xyz/'])
         tree_b.add(['xyz'])
-        xyz_id = tree_b.path2id('xyz')
         tree_b.rename_one('a/m', 'xyz/m')
         tree_b.unversion(['a'])
         tree_b.commit('delete in B')
 
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('xyz', xyz_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+        self.assertThat(
+                tree_b,
+                HasPathRelations(
+                    tree_a,
+                    [('', ''), ('xyz/', None), ('xyz/m', 'a/m')]))
 
         self.build_tree_contents([('A/a/n', b'new contents for n\n')])
         tree_a.commit('change n in A')
@@ -205,18 +177,17 @@
         # because 'a' is still an existing directory
         num_conflicts = tree_b.merge_from_branch(tree_a.branch)
         self.assertEqual(4, num_conflicts)
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('a', a_id),
-                          ('xyz', xyz_id),
-                          ('a/n.OTHER', n_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+
+        self.assertThat(
+                tree_b,
+                HasPathRelations(
+                    tree_a,
+                    [('', ''), ('a/', 'a/'), ('xyz/', None),
+                     ('a/n.OTHER', 'a/n'), ('xyz/m', 'a/m')]))
+
         tree_b.unversion(['a'])
-        paths = [(path, ie.file_id)
-                 for path, ie in tree_b.iter_entries_by_dir()]
-        self.assertEqual([('', root_id),
-                          ('xyz', xyz_id),
-                          ('xyz/m', m_id),
-                         ], paths)
+        self.assertThat(
+                tree_b,
+                HasPathRelations(
+                    tree_a,
+                    [('', ''), ('xyz/', None), ('xyz/m', 'a/m')]))

=== modified file 'breezy/tests/per_workingtree/test_walkdirs.py'
--- breezy/tests/per_workingtree/test_walkdirs.py	2017-11-29 11:52:27 +0000
+++ breezy/tests/per_workingtree/test_walkdirs.py	2018-03-07 02:28:01 +0000
@@ -109,16 +109,20 @@
 
         expected_dirblocks = [
             (('', tree.path2id('')),
-             [dirblocks[1].as_tuple(), dirblocks[3].as_tuple(),
-              dirblocks[0].as_tuple()]
+             ([dirblocks[1].as_tuple(), dirblocks[3].as_tuple()]
+                 if (tree.has_versioned_directories() or file_status != self.missing) else []) +
+             [dirblocks[0].as_tuple()]
             ),
             (dirblocks[1].as_dir_tuple(),
              [dirblocks[2].as_tuple()]
             ),
-            (dirblocks[3].as_dir_tuple(),
-             []
-            ),
             ]
+        if (tree.has_versioned_directories() or
+            file_status != self.missing):
+            expected_dirblocks.append(
+                (dirblocks[3].as_dir_tuple(),
+                 []
+                ))
         if prefix:
             expected_dirblocks = [e for e in expected_dirblocks
                 if len(e) > 0 and len(e[0]) > 0 and e[0][0] == prefix]
@@ -196,14 +200,18 @@
         link2_stat = os.lstat('link2')
         expected_dirblocks = [
              (('', tree.path2id('')),
-              [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), 'directory'),
-               ('dir2', 'dir2', 'symlink', dir2_stat, tree.path2id('dir2'), 'directory'),
+              [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'),
+                  'directory' if tree.has_versioned_directories() else None),
+               ('dir2', 'dir2', 'symlink', dir2_stat, tree.path2id('dir2'),
+                   'directory' if tree.has_versioned_directories() else None),
                ('file1', 'file1', 'directory', file1_stat, tree.path2id('file1'), 'file'),
                ('file2', 'file2', 'symlink', file2_stat, tree.path2id('file2'), 'file'),
                ('link1', 'link1', 'file', link1_stat, tree.path2id('link1'), 'symlink'),
                ('link2', 'link2', 'directory', link2_stat, tree.path2id('link2'), 'symlink'),
               ]
-             ),
+             )]
+        if tree.has_versioned_directories():
+            expected_dirblocks.extend([
              (('dir1', tree.path2id('dir1')),
               [
               ]
@@ -211,7 +219,8 @@
              (('dir2', tree.path2id('dir2')),
               [
               ]
-             ),
+             )])
+        expected_dirblocks.extend([
              (('file1', None),
               [
               ]
@@ -220,10 +229,9 @@
               [
               ]
              ),
-            ]
-        tree.lock_read()
-        result = list(tree.walkdirs())
-        tree.unlock()
+            ])
+        with tree.lock_read():
+            result = list(tree.walkdirs())
         # check each return value for debugging ease.
         for pos, item in enumerate(expected_dirblocks):
             self.assertEqual(item, result[pos])
@@ -242,24 +250,36 @@
         self.build_tree(changed_paths)
         dir1_stat = os.lstat('dir1')
         file1_stat = os.lstat('file1')
-        expected_dirblocks = [
-             (('', tree.path2id('')),
-              [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), 'directory'),
-               ('file1', 'file1', 'directory', file1_stat, tree.path2id('file1'), 'file'),
-              ]
-             ),
-             (('dir1', tree.path2id('dir1')),
-              [
-              ]
-             ),
-             (('file1', None),
-              [
-              ]
-             ),
-            ]
-        tree.lock_read()
-        result = list(tree.walkdirs())
-        tree.unlock()
+        if tree.has_versioned_directories():
+            expected_dirblocks = [
+                 (('', tree.path2id('')),
+                  [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), 'directory'),
+                   ('file1', 'file1', 'directory', file1_stat, tree.path2id('file1'), 'file'),
+                  ]
+                 ),
+                 (('dir1', tree.path2id('dir1')),
+                  [
+                  ]
+                 ),
+                 (('file1', None),
+                  [
+                  ]
+                 ),
+                ]
+        else:
+            expected_dirblocks = [
+                 (('', tree.path2id('')),
+                  [('dir1', 'dir1', 'file', dir1_stat, tree.path2id('dir1'), None),
+                   ('file1', 'file1', 'directory', file1_stat, tree.path2id('file1'), 'file'),
+                  ]
+                 ),
+                 (('file1', None),
+                  [
+                  ]
+                 ),
+                ]
+        with tree.lock_read():
+            result = list(tree.walkdirs())
         # check each return value for debugging ease.
         for pos, item in enumerate(expected_dirblocks):
             self.assertEqual(item, result[pos])

=== modified file 'breezy/tests/per_workingtree/test_workingtree.py'
--- breezy/tests/per_workingtree/test_workingtree.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/per_workingtree/test_workingtree.py	2018-03-25 00:39:16 +0000
@@ -89,10 +89,10 @@
         tree.lock_read()
         files = list(tree.list_files())
         tree.unlock()
-        self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
-        self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
+        self.assertEqual(files.pop(0), ('dir', '?', 'directory', None, TreeDirectory()))
+        self.assertEqual(files.pop(0), ('file', '?', 'file', None, TreeFile()))
         if has_symlinks():
-            self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+            self.assertEqual(files.pop(0), ('symlink', '?', 'symlink', None, TreeLink()))
 
     def test_list_files_sorted(self):
         tree = self.make_branch_and_tree('.')
@@ -110,22 +110,36 @@
             ('zz_dir', 'directory'),
             ], files)
 
-        tree.add(['dir', 'zz_dir'])
-        tree.lock_read()
-        files = [(path, kind) for (path, v, kind, file_id, entry)
-                               in tree.list_files()]
-        tree.unlock()
-        self.assertEqual([
-            ('a', 'file'),
-            ('dir', 'directory'),
-            ('dir/b', 'file'),
-            ('dir/file', 'file'),
-            ('dir/subdir', 'directory'),
-            ('dir/subfile', 'file'),
-            ('file', 'file'),
-            ('zz_dir', 'directory'),
-            ('zz_dir/subfile', 'file'),
-            ], files)
+        with tree.lock_write():
+            if tree.has_versioned_directories():
+                tree.add(['dir', 'zz_dir'])
+                files = [(path, kind) for (path, v, kind, file_id, entry)
+                                       in tree.list_files()]
+                self.assertEqual([
+                    ('a', 'file'),
+                    ('dir', 'directory'),
+                    ('dir/b', 'file'),
+                    ('dir/file', 'file'),
+                    ('dir/subdir', 'directory'),
+                    ('dir/subfile', 'file'),
+                    ('file', 'file'),
+                    ('zz_dir', 'directory'),
+                    ('zz_dir/subfile', 'file'),
+                    ], files)
+            else:
+                tree.add(['dir/b'])
+                files = [(path, kind) for (path, v, kind, file_id, entry)
+                                       in tree.list_files()]
+                self.assertEqual([
+                    ('a', 'file'),
+                    ('dir', 'directory'),
+                    ('dir/b', 'file'),
+                    ('dir/file', 'file'),
+                    ('dir/subdir', 'directory'),
+                    ('dir/subfile', 'file'),
+                    ('file', 'file'),
+                    ('zz_dir', 'directory'),
+                    ], files)
 
     def test_list_files_kind_change(self):
         tree = self.make_branch_and_tree('tree')
@@ -137,9 +151,14 @@
         self.addCleanup(tree.unlock)
         result = list(tree.list_files())
         self.assertEqual(1, len(result))
-        self.assertEqual(
-                ('filename', 'V', 'directory', tree.path2id('filename')),
-                result[0][:4])
+        if tree.has_versioned_directories():
+            self.assertEqual(
+                    ('filename', 'V', 'directory', tree.path2id('filename')),
+                    result[0][:4])
+        else:
+            self.assertEqual(
+                    ('filename', '?', 'directory', None),
+                    result[0][:4])
 
     def test_get_config_stack(self):
         # Smoke test that all working trees succeed getting a config
@@ -692,11 +711,22 @@
         with tree.lock_write():
             tree.add(['somefile'])
             d = {tree.path2id('somefile'): osutils.sha_string(b'hello')}
-            tree.set_merge_modified(d)
+            if tree.supports_merge_modified():
+                tree.set_merge_modified(d)
+                mm = tree.merge_modified()
+                self.assertEqual(mm, d)
+            else:
+                self.assertRaises(
+                    errors.UnsupportedOperation,
+                    tree.set_merge_modified, d)
+                mm = tree.merge_modified()
+                self.assertEqual(mm, {})
+        if tree.supports_merge_modified():
             mm = tree.merge_modified()
             self.assertEqual(mm, d)
-        mm = tree.merge_modified()
-        self.assertEqual(mm, d)
+        else:
+            mm = tree.merge_modified()
+            self.assertEqual(mm, {})
 
     def test_conflicts(self):
         from breezy.tests.test_conflicts import example_conflicts
@@ -1173,8 +1203,8 @@
         # tricky to figure out how to create an illegal filename.
         # \xb5 is an illegal path because it should be \xc2\xb5 for UTF-8
         tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/subdir/'])
-        tree.add('subdir')
+        self.build_tree(['tree/subdir/', 'tree/subdir/somefile'])
+        tree.add(['subdir', 'subdir/somefile'])
 
         f = open('tree/subdir/m\xb5', 'wb')
         try:
@@ -1190,7 +1220,7 @@
 
         e = self.assertListRaises(errors.BadFilenameEncoding,
                                   tree.iter_changes, tree.basis_tree(),
-                                                     want_unversioned=True)
+                                  want_unversioned=True)
         # We should display the relative path
         self.assertEqual('subdir/m\xb5', e.filename)
         self.assertEqual(osutils._fs_enc, e.fs_encoding)
@@ -1203,9 +1233,6 @@
         wt = self.make_branch_and_tree('wt')
         self.assertIsInstance(wt.user_url, str)
         self.assertEqual(wt.user_url, wt.user_transport.base)
-        # for all current bzrdir implementations the user dir must be 
-        # above the control dir but we might need to relax that?
-        self.assertEqual(wt.control_url.find(wt.user_url), 0)
         self.assertEqual(wt.control_url, wt.control_transport.base)
 
 

=== modified file 'breezy/tests/stub_sftp.py'
--- breezy/tests/stub_sftp.py	2017-06-05 22:09:44 +0000
+++ breezy/tests/stub_sftp.py	2018-03-24 17:54:01 +0000
@@ -489,8 +489,7 @@
             # Normalize the path or it will be wrongly escaped
             self._homedir = osutils.normpath(self._homedir)
         else:
-            # But unix SFTP servers should just deal in bytestreams
-            self._homedir = self._homedir.encode('utf-8')
+            self._homedir = self._homedir
         if self._server_homedir is None:
             self._server_homedir = self._homedir
         self._root = '/'

=== modified file 'breezy/tests/test_bundle.py'
--- breezy/tests/test_bundle.py	2018-02-18 19:18:40 +0000
+++ breezy/tests/test_bundle.py	2018-03-25 00:39:16 +0000
@@ -663,7 +663,7 @@
                 , 'sub/sub'
                 ])
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('exe-1')
+        trans_id = tt.trans_id_tree_path('executable')
         tt.set_executability(False, trans_id)
         tt.apply()
         self.tree1.commit('removed', rev_id=b'a@cset-0-3')
@@ -731,7 +731,7 @@
             self.assertEqual(link_target, bund_tree.get_symlink_target(link_name))
 
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id(link_id)
+        trans_id = tt.trans_id_tree_path(link_name)
         tt.adjust_path('link2', tt.root, trans_id)
         tt.delete_contents(trans_id)
         tt.create_symlink(new_link_target, trans_id)
@@ -745,7 +745,7 @@
                              bund_tree.get_symlink_target('link2'))
 
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id(link_id)
+        trans_id = tt.trans_id_tree_path('link2')
         tt.delete_contents(trans_id)
         tt.create_symlink('jupiter', trans_id)
         tt.apply()
@@ -753,7 +753,7 @@
         bundle = self.get_valid_bundle('l@cset-0-2', 'l@cset-0-3')
 
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id(link_id)
+        trans_id = tt.trans_id_tree_path('link2')
         tt.delete_contents(trans_id)
         tt.apply()
         self.tree1.commit('Delete symlink', rev_id=b'l@cset-0-4')
@@ -783,7 +783,7 @@
 
         # Delete
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('binary-1')
+        trans_id = tt.trans_id_tree_path('file')
         tt.delete_contents(trans_id)
         tt.apply()
         self.tree1.commit('delete binary', rev_id=b'b@cset-0-2')
@@ -791,7 +791,7 @@
 
         # Rename & modify
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('binary-2')
+        trans_id = tt.trans_id_tree_path('file2')
         tt.adjust_path('file3', tt.root, trans_id)
         tt.delete_contents(trans_id)
         tt.create_file('file\rcontents\x00\n\x00', trans_id)
@@ -801,7 +801,7 @@
 
         # Modify
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('binary-2')
+        trans_id = tt.trans_id_tree_path('file3')
         tt.delete_contents(trans_id)
         tt.create_file('\x00file\rcontents', trans_id)
         tt.apply()
@@ -820,7 +820,7 @@
         self.tree1.commit('create file', rev_id=b'a@lmod-0-1')
 
         tt = TreeTransform(self.tree1)
-        trans_id = tt.trans_id_tree_file_id('file')
+        trans_id = tt.trans_id_tree_path('file')
         tt.delete_contents(trans_id)
         tt.create_file('file2', trans_id)
         tt.apply()
@@ -828,7 +828,7 @@
 
         other = self.get_checkout('a@lmod-0-1')
         tt = TreeTransform(other)
-        trans_id = tt.trans_id_tree_file_id('file')
+        trans_id = tt.trans_id_tree_path('file2')
         tt.delete_contents(trans_id)
         tt.create_file('file2', trans_id)
         tt.apply()

=== modified file 'breezy/tests/test_bzrdir.py'
--- breezy/tests/test_bzrdir.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_bzrdir.py	2018-03-25 00:39:16 +0000
@@ -864,7 +864,7 @@
         tree.commit('Initial commit')
         # The following line force the orhaning to reveal bug #634470
         tree.branch.get_config_stack().set(
-            'bzr.transform.orphan_policy', 'move')
+            'transform.orphan_policy', 'move')
         tree.controldir.destroy_workingtree()
         # FIXME: subtree/.bzr is left here which allows the test to pass (or
         # fail :-( ) -- vila 20100909

=== modified file 'breezy/tests/test_commit.py'
--- breezy/tests/test_commit.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/test_commit.py	2018-03-25 00:39:16 +0000
@@ -435,7 +435,8 @@
                 message="base", allow_pointless=True, rev_id=b'B',
                 working_tree=wt)
             def sign(text):
-                return breezy.gpg.LoopbackGPGStrategy(None).sign(text)
+                return breezy.gpg.LoopbackGPGStrategy(None).sign(
+                        text, breezy.gpg.MODE_CLEAR)
             self.assertEqual(sign(Testament.from_revision(branch.repository,
                                                           'B').as_short_text()),
                              branch.repository.get_signature_text('B'))

=== modified file 'breezy/tests/test_gpg.py'
--- breezy/tests/test_gpg.py	2017-07-04 20:03:11 +0000
+++ breezy/tests/test_gpg.py	2018-03-12 00:47:15 +0000
@@ -224,8 +224,7 @@
 """
         my_gpg = gpg.GPGStrategy(FakeConfig())
         my_gpg.set_acceptable_keys("bazaar@example.com")
-        self.assertEqual((gpg.SIGNATURE_VALID, None), my_gpg.verify(content,
-                            plain))
+        self.assertEqual((gpg.SIGNATURE_VALID, None, plain), my_gpg.verify(content))
 
     def test_verify_unacceptable_key(self):
         self.requireFeature(features.gpg)
@@ -255,8 +254,8 @@
 """
         my_gpg = gpg.GPGStrategy(FakeConfig())
         my_gpg.set_acceptable_keys("foo@example.com")
-        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'E3080E45'),
-                         my_gpg.verify(content, plain))
+        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'E3080E45', plain),
+                         my_gpg.verify(content))
 
     def test_verify_valid_but_untrusted(self):
         self.requireFeature(features.gpg)
@@ -285,40 +284,7 @@
 sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
 """
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
-                            plain))
-
-    def test_verify_bad_testament(self):
-        self.requireFeature(features.gpg)
-        self.import_keys()
-
-        content = """-----BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA1
-
-bazaar-ng testament short form 1
-revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
-sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
------BEGIN PGP SIGNATURE-----
-Version: GnuPG v1.4.11 (GNU/Linux)
-
-iQEcBAEBAgAGBQJN+ekFAAoJEIdoGx7jCA5FGtEH/i+XxJRvqU6wdBtLVrGBMAGk
-FZ5VP+KyXYtymSbgSstj/vM12NeMIeFs3xGnNnYuX1MIcY6We5TKtCH0epY6ym5+
-6g2Q2QpQ5/sT2d0mWzR0K4uVngmxVQaXTdk5PdZ40O7ULeDLW6CxzxMHyUL1rsIx
-7UBUTBh1O/1n3ZfD99hUkm3hVcnsN90uTKH59zV9NWwArU0cug60+5eDKJhSJDbG
-rIwlqbFAjDZ7L/48e+IaYIJwBZFzMBpJKdCxzALLtauMf+KK8hGiL2hrRbWm7ty6
-NgxfkMYOB4rDPdSstT35N+5uBG3n/UzjxHssi0svMfVETYYX40y57dm2eZQXFp8=
-=iwsn
------END PGP SIGNATURE-----
-"""
-        plain = """bazaar-ng testament short form 1
-revision-id: doctor@example.com-20110527185938-hluafawphszb8dl1
-sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
-"""
-        my_gpg = gpg.GPGStrategy(FakeConfig())
-        my_gpg.set_acceptable_keys("bazaar@example.com")
-        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
-                            plain))
-
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None, plain), my_gpg.verify(content))
 
     def test_verify_revoked_signature(self):
         self.requireFeature(features.gpg)
@@ -341,8 +307,7 @@
         plain = """asdf\n"""
         my_gpg = gpg.GPGStrategy(FakeConfig())
         my_gpg.set_acceptable_keys("test@example.com")
-        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
-                            plain))
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None, None), my_gpg.verify(content))
 
     def test_verify_invalid(self):
         self.requireFeature(features.gpg)
@@ -366,8 +331,8 @@
 sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
 """
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None),
-                            my_gpg.verify(content, plain))
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None, plain),
+                            my_gpg.verify(content))
 
     def test_verify_expired_but_valid(self):
         self.requireFeature(features.gpg)
@@ -388,13 +353,9 @@
 =uHen
 -----END PGP SIGNATURE-----
 """
-        plain = """bazaar-ng testament short form 1
-revision-id: test@example.com-20110801100657-f1dr1nompeex723z
-sha1: 59ab434be4c2d5d646dee84f514aa09e1b72feeb
-"""
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual((gpg.SIGNATURE_EXPIRED, u'4F8D1513'),
-                            my_gpg.verify(content, plain))
+        self.assertEqual((gpg.SIGNATURE_EXPIRED, u'4F8D1513', None),
+                            my_gpg.verify(content))
 
     def test_verify_unknown_key(self):
         self.requireFeature(features.gpg)
@@ -415,10 +376,9 @@
 =RNR5
 -----END PGP SIGNATURE-----
 """
-        plain = "asdf\n"
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'5D51E56F'),
-                            my_gpg.verify(content, plain))
+        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'5D51E56F', None),
+                            my_gpg.verify(content))
 
     def test_set_acceptable_keys(self):
         self.requireFeature(features.gpg)
@@ -454,9 +414,8 @@
 
     def test_sign(self):
         self.assertRaises(gpg.SigningFailed,
-                          gpg.DisabledGPGStrategy(None).sign, 'content')
+                          gpg.DisabledGPGStrategy(None).sign, 'content', gpg.MODE_CLEAR)
 
     def test_verify(self):
         self.assertRaises(gpg.SignatureVerificationFailed,
-                          gpg.DisabledGPGStrategy(None).verify, 'content',
-                          'testament')
+                          gpg.DisabledGPGStrategy(None).verify, 'content')

=== modified file 'breezy/tests/test_info.py'
--- breezy/tests/test_info.py	2017-06-10 16:40:42 +0000
+++ breezy/tests/test_info.py	2018-03-07 00:05:06 +0000
@@ -19,6 +19,7 @@
 from .. import (
     branch as _mod_branch,
     controldir,
+    errors,
     info,
     tests,
     workingtree,
@@ -137,9 +138,11 @@
             repo, None, None))
 
     def test_describe_tree_format(self):
-        for key in controldir.format_registry.keys():
+        for key, format in controldir.format_registry.iteritems():
             if key in controldir.format_registry.aliases():
                 continue
+            if not format().supports_workingtrees:
+                continue
             self.assertTreeDescription(key)
 
     def test_describe_checkout_format(self):
@@ -151,6 +154,9 @@
             # legacy: weave does not support checkouts
             if key == 'weave':
                 continue
+            # foreign: git checkouts can actually be bzr controldirs
+            if key in ('git', 'git-bare'):
+                continue
             if controldir.format_registry.get_info(key).experimental:
                 # We don't require that experimental formats support checkouts
                 # or describe correctly in the UI.

=== modified file 'breezy/tests/test_log.py'
--- breezy/tests/test_log.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_log.py	2018-03-25 00:39:16 +0000
@@ -317,31 +317,22 @@
 
 
 class TestFormatSignatureValidity(tests.TestCaseWithTransport):
-    class UTFLoopbackGPGStrategy(gpg.LoopbackGPGStrategy):
-        def verify(self, content, testament):
-            return (gpg.SIGNATURE_VALID,
-                u'UTF8 Test \xa1\xb1\xc1\xd1\xe1\xf1 <jrandom@example.com>')
-
-    def has_signature_for_revision_id(self, revision_id):
-        return True
-
-    def get_signature_text(self, revision_id):
-        return ''
+
+    def verify_revision_signature(self, revid, gpg_strategy):
+        return (gpg.SIGNATURE_VALID,
+            u'UTF8 Test \xa1\xb1\xc1\xd1\xe1\xf1 <jrandom@example.com>')
 
     def test_format_signature_validity_utf(self):
         """Check that GPG signatures containing UTF-8 names are formatted
         correctly."""
-        # Monkey patch to use our UTF-8 generating GPGStrategy
-        self.overrideAttr(gpg, 'GPGStrategy', self.UTFLoopbackGPGStrategy)
         wt = self.make_branch_and_tree('.')
         revid = wt.commit('empty commit')
         repo = wt.branch.repository
         # Monkey patch out checking if this rev is actually signed, since we
         # can't sign it without a heavier TestCase and LoopbackGPGStrategy
         # doesn't care anyways.
-        self.overrideAttr(repo, 'has_signature_for_revision_id',
-                self.has_signature_for_revision_id)
-        self.overrideAttr(repo, 'get_signature_text', self.get_signature_text)
+        self.overrideAttr(repo, 'verify_revision_signature',
+                self.verify_revision_signature)
         out = log.format_signature_validity(revid, wt.branch)
         self.assertEqual(
 u'valid signature from UTF8 Test \xa1\xb1\xc1\xd1\xe1\xf1 <jrandom@example.com>',

=== modified file 'breezy/tests/test_matchers.py'
--- breezy/tests/test_matchers.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_matchers.py	2018-03-25 00:39:16 +0000
@@ -151,6 +151,31 @@
             set(mismatch.describe().split(" != ")))
 
 
+class TestHasPathRelations(TestCaseWithTransport):
+
+    def test__str__(self):
+        t = self.make_branch_and_tree('.')
+        matcher = HasPathRelations(t, [("a", "b")])
+        self.assertEqual("HasPathRelations(%r, [('a', 'b')])" % t, str(matcher))
+
+    def test_match(self):
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'])
+        self.assertThat(t, HasPathRelations(t,
+            [('', ''),
+             ('a', 'a'),
+             ('b/', 'b/'),
+             ('b/c', 'b/c')]))
+
+    def test_mismatch(self):
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'])
+        mismatch = HasPathRelations(t, [('a', 'a')]).match(t)
+        self.assertIsNot(None, mismatch)
+
+
 class TestContainsNoVfsCalls(TestCase):
 
     def _make_call(self, method, args):

=== modified file 'breezy/tests/test_merge.py'
--- breezy/tests/test_merge.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/test_merge.py	2018-03-25 00:39:16 +0000
@@ -1265,8 +1265,7 @@
         builder.build_snapshot(['B-id', 'C-id'], [], revision_id='D-id')
         return builder
 
-    def make_Merger(self, builder, other_revision_id,
-                    interesting_files=None, interesting_ids=None):
+    def make_Merger(self, builder, other_revision_id, interesting_files=None):
         """Make a Merger object from a branch builder"""
         mem_tree = memorytree.MemoryTree.create_on_branch(builder.get_branch())
         mem_tree.lock_write()
@@ -1274,8 +1273,6 @@
         merger = _mod_merge.Merger.from_revision_ids(
             mem_tree, other_revision_id)
         merger.set_interesting_files(interesting_files)
-        # It seems there is no matching function for set_interesting_ids
-        merger.interesting_ids = interesting_ids
         merger.merge_type = _mod_merge.Merge3Merger
         return merger
 
@@ -1400,10 +1397,9 @@
 class TestMergerEntriesLCA(TestMergerBase):
 
     def make_merge_obj(self, builder, other_revision_id,
-                       interesting_files=None, interesting_ids=None):
+                       interesting_files=None):
         merger = self.make_Merger(builder, other_revision_id,
-            interesting_files=interesting_files,
-            interesting_ids=interesting_ids)
+            interesting_files=interesting_files)
         return merger.make_merger()
 
     def test_simple(self):
@@ -1435,6 +1431,7 @@
         # BASE, lca1, lca2, OTHER, THIS
         root_id = 'a-root-id'
         self.assertEqual([('a-id', True,
+                           ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((False, [False, False]), False, False)),
@@ -1480,6 +1477,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('bar-id', True,
+                           ((None, [u'bar', u'bar']), u'bar', u'bar'),
                            ((None, [root_id, root_id]), root_id, root_id),
                            ((None, [u'bar', u'bar']), u'bar', u'bar'),
                            ((None, [False, False]), False, False)),
@@ -1512,6 +1510,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('a-id', True,
+                           ((u'a', [u'a', u'a']), u'a', None),
                            ((root_id, [root_id, root_id]), root_id, None),
                            ((u'a', [u'a', u'a']), u'a', None),
                            ((False, [False, False]), False, None)),
@@ -1562,6 +1561,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('a-id', True,
+                           ((u'a', [u'a', u'a']), None, u'a'),
                            ((root_id, [root_id, root_id]), None, root_id),
                            ((u'a', [u'a', u'a']), None, u'a'),
                            ((False, [False, False]), None, False)),
@@ -1627,6 +1627,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('foo-id', True,
+                           ((u'foo', [u'foo', None]), None, u'foo'),
                            ((root_id, [root_id, None]), None, root_id),
                            ((u'foo', [u'foo', None]), None, 'foo'),
                            ((False, [False, None]), None, False)),
@@ -1681,6 +1682,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('a-id', True,
+                           ((None, [None, None]), u'a', None),
                            ((None, [None, None]), root_id, None),
                            ((None, [None, None]), u'a', None),
                            ((None, [None, None]), False, None)),
@@ -1843,6 +1845,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('foo-id', True,
+                           ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((False, [False, False]), False, False)),
@@ -1880,6 +1883,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('foo-id', True,
+                           ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((False, [False, False]), False, False)),
@@ -1941,6 +1945,7 @@
         root_id = 'a-root-id'
         # The content was not changed, only the path
         self.assertEqual([('a-id', False,
+                           ((u'a', [u'b', u'b']), u'b', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'b', u'a'),
                            ((False, [False, False]), False, False)),
@@ -1966,6 +1971,7 @@
         root_id = 'a-root-id'
         # Only the kind was changed (content)
         self.assertEqual([('a-id', True,
+                           ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((False, [False, False]), False, False)),
@@ -2011,6 +2017,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('b-id', True,
+                           ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((False, [False, False]), False, False)),
@@ -2038,6 +2045,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('b-id', True,
+                           ((u'b', [u'b', u'b']), u'b', u'c'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'c'),
                            ((False, [False, False]), False, False)),
@@ -2067,6 +2075,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('c-id', True,
+                           ((u'c', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'c', [u'b', u'b']), u'b', u'b'),
                            ((False, [False, False]), False, False)),
@@ -2094,12 +2103,13 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('b-id', True,
+                           ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'c', u'b']), u'b', u'b'),
                            ((False, [False, False]), False, False)),
                          ], entries)
 
-    def test_interesting_ids(self):
+    def test_interesting_files(self):
         # Two files modified, but we should filter one of them
         builder = self.get_builder()
         builder.build_snapshot(None,
@@ -2114,10 +2124,11 @@
              ('modify', ('b-id', 'new-content\n'))], revision_id='E-id')
         builder.build_snapshot(['B-id', 'C-id'], [], revision_id='D-id')
         merge_obj = self.make_merge_obj(builder, 'E-id',
-                                        interesting_ids=['b-id'])
+                                        interesting_files=['b'])
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('b-id', True,
+                           ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((False, [False, False]), False, False)),
@@ -2253,7 +2264,7 @@
         wt = self.get_wt_from_builder(builder)
         tt = transform.TreeTransform(wt)
         try:
-            tt.set_executability(True, tt.trans_id_tree_file_id('foo-id'))
+            tt.set_executability(True, tt.trans_id_tree_path('foo'))
             tt.apply()
         except:
             tt.finalize()
@@ -2432,6 +2443,7 @@
         entries = list(merge_obj._entries_lca())
         # No content change, just a path change
         self.assertEqual([(b'foo-id', False,
+                           ((u'foo', [u'blah', u'blah']), u'blah', u'barry'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'barry', u'foo']), u'blah', u'barry'),
                            ((False, [False, False]), False, False)),
@@ -2536,6 +2548,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = wt.path2id('')
         self.assertEqual([(b'foo-id', True,
+                           ((None, [u'foo', None]), u'foo', u'foo'),
                            ((None, [root_id, None]), root_id, root_id),
                            ((None, [u'foo', None]), u'foo', u'foo'),
                            ((None, [False, None]), False, False)),
@@ -2597,6 +2610,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = wt.path2id('')
         self.assertEqual([(b'foo-id', True,
+                           ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((False, [False, False]), False, False)),
@@ -2720,10 +2734,12 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('a-id', False,
+                           ((u'a', [u'c', u'c']), u'c', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'b']), u'c', u'b'),
                            ((False, [False, False]), False, False)),
                           ('foo-id', True,
+                           ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((False, [False, False]), False, False)),
@@ -2838,6 +2854,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('sub-tree-root', False,
+                           ((u'sub', [u'alt_sub', u'alt_sub']), u'alt_sub', u'sub'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
                            ((False, [False, False]), False, False)),
@@ -2882,6 +2899,7 @@
         entries = list(merge_obj._entries_lca())
         root_id = 'a-root-id'
         self.assertEqual([('sub-tree-root', False,
+                           ((u'sub', [u'alt_sub', u'alt_sub']), u'alt_sub', u'sub'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
                            ((False, [False, False]), False, False)),

=== modified file 'breezy/tests/test_merge_core.py'
--- breezy/tests/test_merge_core.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_merge_core.py	2018-03-25 00:39:16 +0000
@@ -77,8 +77,8 @@
             if option is True:
                 new_file(tt)
 
-    def merge(self, merge_type=Merge3Merger, interesting_ids=None, **kwargs):
-        merger = self.make_merger(merge_type, interesting_ids, **kwargs)
+    def merge(self, merge_type=Merge3Merger, interesting_files=None, **kwargs):
+        merger = self.make_merger(merge_type, interesting_files, **kwargs)
         merger.do_merge()
         return merger.cooked_conflicts
 
@@ -86,7 +86,7 @@
         merger = self.make_merger(Merge3Merger, None, this_revision_tree=True)
         return merger.make_preview_transform()
 
-    def make_merger(self, merge_type, interesting_ids,
+    def make_merger(self, merge_type, interesting_files,
             this_revision_tree=False, **kwargs):
         self.base_tt.apply()
         self.base.commit('base commit')
@@ -110,7 +110,7 @@
         else:
             this_tree = self.this
         merger = merge_type(this_tree, self.this, self.base, other_basis,
-                            interesting_ids=interesting_ids, do_merge=False,
+                            interesting_files=interesting_files, do_merge=False,
                             this_branch=self.this.branch, **kwargs)
         return merger
 
@@ -251,7 +251,7 @@
         builder.change_contents("1", other="text4")
         builder.add_file("2", builder.tree_root, "name2", "hello1", True)
         builder.change_contents("2", other="text4")
-        builder.merge(interesting_ids=["1"])
+        builder.merge(interesting_files=["name1"])
         self.assertEqual(builder.this.get_file("name1").read(), "text4" )
         self.assertEqual(builder.this.get_file("name2").read(), "hello1" )
         builder.cleanup()
@@ -432,7 +432,7 @@
         builder.change_contents("1", other="text3")
         builder.add_file("2", builder.tree_root, "name1.new", "text2", True)
         builder.merge()
-        os.lstat(builder.this.id2abspath("2"))
+        os.lstat(builder.this.abspath(builder.this.id2path("2")))
         builder.cleanup()
 
     def test_spurious_conflict(self):

=== modified file 'breezy/tests/test_rename_map.py'
--- breezy/tests/test_rename_map.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_rename_map.py	2018-03-25 00:39:16 +0000
@@ -117,7 +117,7 @@
         tree.add('file', b'file-id')
         tree.commit('Added file')
         os.rename('tree/file', 'tree/file2')
-        RenameMap.guess_renames(tree)
+        RenameMap.guess_renames(tree.basis_tree(), tree)
         self.assertEqual('file2', tree.id2path(b'file-id'))
 
     def test_guess_renames_handles_directories(self):
@@ -128,7 +128,7 @@
         tree.add(['dir', 'dir/file'], [b'dir-id', b'file-id'])
         tree.commit('Added file')
         os.rename('tree/dir', 'tree/dir2')
-        RenameMap.guess_renames(tree)
+        RenameMap.guess_renames(tree.basis_tree(), tree)
         self.assertEqual('dir2/file', tree.id2path(b'file-id'))
         self.assertEqual('dir2', tree.id2path(b'dir-id'))
 
@@ -143,7 +143,7 @@
                  [b'topdir-id', b'middledir-id', b'file-id'])
         tree.commit('Added files.')
         os.rename('tree/topdir', 'tree/topdir2')
-        RenameMap.guess_renames(tree)
+        RenameMap.guess_renames(tree.basis_tree(), tree)
         self.assertEqual('topdir2', tree.id2path(b'topdir-id'))
 
     def test_guess_renames_preserves_children(self):
@@ -158,7 +158,7 @@
                  [b'foo-id', b'bar-id', b'empty-id'])
         tree.commit('rev1')
         os.rename('tree/foo', 'tree/baz')
-        RenameMap.guess_renames(tree)
+        RenameMap.guess_renames(tree.basis_tree(), tree)
         self.assertEqual('baz/empty', tree.id2path(b'empty-id'))
 
     def test_guess_renames_dry_run(self):
@@ -169,7 +169,7 @@
         tree.add('file', b'file-id')
         tree.commit('Added file')
         os.rename('tree/file', 'tree/file2')
-        RenameMap.guess_renames(tree, dry_run=True)
+        RenameMap.guess_renames(tree.basis_tree(), tree, dry_run=True)
         self.assertEqual('file', tree.id2path(b'file-id'))
 
     @staticmethod
@@ -194,11 +194,12 @@
         tree.add('file', b'file-id')
         tree.commit('Added file')
         os.rename('tree/file', 'tree/file2')
-        notes = self.captureNotes(RenameMap.guess_renames, tree,
-                                  dry_run=True)[0]
+        notes = self.captureNotes(
+                RenameMap.guess_renames, tree.basis_tree(), tree,
+                dry_run=True)[0]
         self.assertEqual('file => file2', ''.join(notes))
-        notes = self.captureNotes(RenameMap.guess_renames, tree,
-                                  dry_run=False)[0]
+        notes = self.captureNotes(RenameMap.guess_renames, tree.basis_tree(),
+                tree, dry_run=False)[0]
         self.assertEqual('file => file2', ''.join(notes))
 
     def test_guess_rename_handles_new_directories(self):
@@ -211,7 +212,8 @@
         tree.commit('Added file')
         os.mkdir('folder')
         os.rename('file', 'folder/file2')
-        notes = self.captureNotes(RenameMap.guess_renames, tree)[0]
+        notes = self.captureNotes(
+                RenameMap.guess_renames, tree.basis_tree(), tree)[0]
         self.assertEqual('file => folder/file2', ''.join(notes))
 
         tree.unlock()

=== modified file 'breezy/tests/test_repository.py'
--- breezy/tests/test_repository.py	2018-02-18 19:18:40 +0000
+++ breezy/tests/test_repository.py	2018-03-25 00:39:16 +0000
@@ -473,11 +473,8 @@
         repo_dir = bzrdir.BzrDirMetaFormat1().initialize('repository')
         repo = TestRepositoryFormat1().initialize(repo_dir)
         converter = repository.CopyConverter(target_format)
-        pb = breezy.ui.ui_factory.nested_progress_bar()
-        try:
+        with breezy.ui.ui_factory.nested_progress_bar() as pb:
             converter.convert(repo, pb)
-        finally:
-            pb.finished()
         repo = repo_dir.open_repository()
         self.assertTrue(isinstance(target_format, repo._format.__class__))
 

=== modified file 'breezy/tests/test_revert.py'
--- breezy/tests/test_revert.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_revert.py	2018-03-25 00:39:16 +0000
@@ -97,7 +97,7 @@
     def test_preserve_execute(self):
         tree = self.tree_with_executable()
         tt = transform.TreeTransform(tree)
-        newfile = tt.trans_id_tree_file_id('newfile-id')
+        newfile = tt.trans_id_tree_path('newfile')
         tt.delete_contents(newfile)
         tt.create_file('Woooorld!', newfile)
         tt.apply()
@@ -112,7 +112,7 @@
     def test_revert_executable(self):
         tree = self.tree_with_executable()
         tt = transform.TreeTransform(tree)
-        newfile = tt.trans_id_tree_file_id('newfile-id')
+        newfile = tt.trans_id_tree_path('newfile')
         tt.set_executability(False, newfile)
         tt.apply()
         tree.lock_write()

=== modified file 'breezy/tests/test_sftp_transport.py'
--- breezy/tests/test_sftp_transport.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_sftp_transport.py	2018-03-25 00:39:16 +0000
@@ -239,7 +239,7 @@
         self.vfs_transport_server = create_server
         f = open('a_file', 'wb')
         try:
-            f.write('foobar\n')
+            f.write(b'foobar\n')
         finally:
             f.close()
 

=== modified file 'breezy/tests/test_shelf.py'
--- breezy/tests/test_shelf.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_shelf.py	2018-03-25 00:39:16 +0000
@@ -206,7 +206,7 @@
 
     def check_shelve_creation(self, creator, tree):
         self.assertRaises(StopIteration,
-                          next, tree.iter_entries_by_dir([b'foo-id']))
+                          next, tree.iter_entries_by_dir(specific_files=['foo']))
         s_trans_id = creator.shelf_transform.trans_id_file_id(b'foo-id')
         self.assertEqual(b'foo-id',
                          creator.shelf_transform.final_file_id(s_trans_id))
@@ -349,7 +349,7 @@
         creator.shelve_creation(b'foo-id')
         creator.transform()
         self.assertRaises(StopIteration,
-                          next, tree.iter_entries_by_dir([b'foo-id']))
+                          next, tree.iter_entries_by_dir(specific_files=['foo']))
         self.assertShelvedFileEqual('', creator, b'foo-id')
         s_trans_id = creator.shelf_transform.trans_id_file_id(b'foo-id')
         self.assertEqual(b'foo-id',

=== modified file 'breezy/tests/test_transform.py'
--- breezy/tests/test_transform.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/test_transform.py	2018-03-25 00:39:16 +0000
@@ -168,7 +168,7 @@
         self.assertIs(self.wt.is_executable('name2'), False)
         self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
         self.assertEqual(len(modified_paths), 3)
-        tree_mod_paths = [self.wt.id2abspath(f) for f in
+        tree_mod_paths = [self.wt.abspath(self.wt.id2path(f)) for f in
                           ('ozzie', 'my_pretties', 'my_pretties2')]
         self.assertSubset(tree_mod_paths, modified_paths)
         # is it safe to finalize repeatedly?
@@ -453,7 +453,7 @@
         self.assertEqual(self.wt.path2id('name'), 'my_pretties')
         self.assertEqual('contents', file(self.wt.abspath('name')).read())
         transform2, root = self.get_transform()
-        oz_id = transform2.trans_id_tree_file_id('oz-id')
+        oz_id = transform2.trans_id_tree_path('oz')
         newtip = transform2.new_file('tip', oz_id, 'other', 'tip-id')
         result = transform2.find_conflicts()
         fp = FinalPaths(transform2)
@@ -467,12 +467,12 @@
         transform2.finalize()
         transform3 = TreeTransform(self.wt)
         self.addCleanup(transform3.finalize)
-        oz_id = transform3.trans_id_tree_file_id('oz-id')
+        oz_id = transform3.trans_id_tree_path('oz')
         transform3.delete_contents(oz_id)
         self.assertEqual(transform3.find_conflicts(),
                          [('missing parent', oz_id)])
         root_id = transform3.root
-        tip_id = transform3.trans_id_tree_file_id('tip-id')
+        tip_id = transform3.trans_id_tree_path('oz/tip')
         transform3.adjust_path('tip', root_id, tip_id)
         transform3.apply()
 
@@ -601,7 +601,7 @@
         start.new_directory('a', root, 'a')
         start.apply()
         transform, root = self.get_transform()
-        transform.delete_versioned(transform.trans_id_tree_file_id('a'))
+        transform.delete_versioned(transform.trans_id_tree_path('a'))
         transform.new_directory('a', root, 'a')
         transform.apply()
 
@@ -616,7 +616,7 @@
         unversion.unversion_file(parent)
         self.assertEqual(unversion.find_conflicts(),
                          [('unversioned parent', parent_id)])
-        file_id = unversion.trans_id_tree_file_id(b'child-id')
+        file_id = unversion.trans_id_tree_path('parent/child')
         unversion.unversion_file(file_id)
         unversion.apply()
 
@@ -635,23 +635,23 @@
         mangle_tree, root = self.get_transform()
         root = mangle_tree.root
         #swap names
-        name1 = mangle_tree.trans_id_tree_file_id('name1')
-        name2 = mangle_tree.trans_id_tree_file_id('name2')
+        name1 = mangle_tree.trans_id_tree_path('name1')
+        name2 = mangle_tree.trans_id_tree_path('name2')
         mangle_tree.adjust_path('name2', root, name1)
         mangle_tree.adjust_path('name1', root, name2)
 
         #tests for deleting parent directories
-        ddir = mangle_tree.trans_id_tree_file_id('ddir')
+        ddir = mangle_tree.trans_id_tree_path('dying_directory')
         mangle_tree.delete_contents(ddir)
-        dfile = mangle_tree.trans_id_tree_file_id('dfile')
+        dfile = mangle_tree.trans_id_tree_path('dying_directory/dying_file')
         mangle_tree.delete_versioned(dfile)
         mangle_tree.unversion_file(dfile)
-        mfile = mangle_tree.trans_id_tree_file_id('mfile')
+        mfile = mangle_tree.trans_id_tree_path('dying_directory/moving_file')
         mangle_tree.adjust_path('mfile', root, mfile)
 
         #tests for adding parent directories
         newdir = mangle_tree.new_directory('new_directory', root, 'newdir')
-        mfile2 = mangle_tree.trans_id_tree_file_id('mfile2')
+        mfile2 = mangle_tree.trans_id_tree_path('moving_file2')
         mangle_tree.adjust_path('mfile2', newdir, mfile2)
         mangle_tree.new_file('newfile', newdir, 'hello3', 'dfile')
         self.assertEqual(mangle_tree.final_file_id(mfile2), 'mfile2')
@@ -677,8 +677,8 @@
         create_tree.new_file('blackbox.py', newdir, 'hello1', 'blackbox-id')
         create_tree.apply()
         mangle_tree, root = self.get_transform()
-        selftest = mangle_tree.trans_id_tree_file_id('selftest-id')
-        blackbox = mangle_tree.trans_id_tree_file_id('blackbox-id')
+        selftest = mangle_tree.trans_id_tree_path('selftest')
+        blackbox = mangle_tree.trans_id_tree_path('selftest/blackbox.py')
         mangle_tree.adjust_path('test', root, selftest)
         mangle_tree.adjust_path('test_too_much', root, selftest)
         mangle_tree.set_executability(True, blackbox)
@@ -693,9 +693,9 @@
                              'test_too_much-id')
         create_tree.apply()
         mangle_tree, root = self.get_transform()
-        breezy = mangle_tree.trans_id_tree_file_id('breezy-id')
-        tests = mangle_tree.trans_id_tree_file_id('tests-id')
-        test_too_much = mangle_tree.trans_id_tree_file_id('test_too_much-id')
+        breezy = mangle_tree.trans_id_tree_path('breezy')
+        tests = mangle_tree.trans_id_tree_path('breezy/tests')
+        test_too_much = mangle_tree.trans_id_tree_path('breezy/tests/blackbox/test_too_much.py')
         mangle_tree.adjust_path('selftest', breezy, tests)
         mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
         mangle_tree.set_executability(True, test_too_much)
@@ -708,8 +708,8 @@
                              'test_too_much-id')
         create_tree.apply()
         mangle_tree, root = self.get_transform()
-        tests = mangle_tree.trans_id_tree_file_id('tests-id')
-        test_too_much = mangle_tree.trans_id_tree_file_id('test_too_much-id')
+        tests = mangle_tree.trans_id_tree_path('tests')
+        test_too_much = mangle_tree.trans_id_tree_path('tests/test_too_much.py')
         mangle_tree.adjust_path('selftest', root, tests)
         mangle_tree.adjust_path('blackbox.py', tests, test_too_much)
         mangle_tree.set_executability(True, test_too_much)
@@ -722,11 +722,11 @@
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents, root = self.get_transform()
-        file = delete_contents.trans_id_tree_file_id('name1')
+        file = delete_contents.trans_id_tree_path('name1')
         delete_contents.delete_contents(file)
         delete_contents.apply()
         move_id, root = self.get_transform()
-        name1 = move_id.trans_id_tree_file_id('name1')
+        name1 = move_id.trans_id_tree_path('name1')
         newdir = move_id.new_directory('dir', root, 'newdir')
         move_id.adjust_path('name2', newdir, name1)
         move_id.apply()
@@ -739,7 +739,7 @@
         create_tree.apply()
         delete_contents = TreeTransform(self.wt)
         self.addCleanup(delete_contents.finalize)
-        file = delete_contents.trans_id_tree_file_id('name1')
+        file = delete_contents.trans_id_tree_path('name1')
         delete_contents.delete_contents(file)
         delete_contents.apply()
         delete_contents.finalize()
@@ -747,7 +747,7 @@
         self.addCleanup(replace.finalize)
         name2 = replace.new_file('name2', root, 'hello2', 'name1')
         conflicts = replace.find_conflicts()
-        name1 = replace.trans_id_tree_file_id('name1')
+        name1 = replace.trans_id_tree_path('name1')
         self.assertEqual(conflicts, [('duplicate id', name1, name2)])
         resolve_conflicts(replace)
         replace.apply()
@@ -819,11 +819,11 @@
         # set up duplicate entry, duplicate id
         new_dorothy = conflicts.new_file('dorothy', root, 'dorothy',
                                          'dorothy-id')
-        old_dorothy = conflicts.trans_id_tree_file_id('dorothy-id')
-        oz = conflicts.trans_id_tree_file_id('oz-id')
+        old_dorothy = conflicts.trans_id_tree_path('dorothy')
+        oz = conflicts.trans_id_tree_path('oz')
         # set up DeletedParent parent conflict
         conflicts.delete_versioned(oz)
-        emerald = conflicts.trans_id_tree_file_id('emerald-id')
+        emerald = conflicts.trans_id_tree_path('oz/emeraldcity')
         # set up MissingParent conflict
         munchkincity = conflicts.trans_id_file_id('munchkincity-id')
         conflicts.adjust_path('munchkincity', root, munchkincity)
@@ -975,8 +975,8 @@
         create.new_directory('oz', root, 'oz-id')
         create.apply()
         cyclone, root = self.get_transform()
-        oz = cyclone.trans_id_tree_file_id('oz-id')
-        house = cyclone.trans_id_tree_file_id('house-id')
+        oz = cyclone.trans_id_tree_path('oz')
+        house = cyclone.trans_id_tree_path('house')
         cyclone.adjust_path('house', oz, house)
         cyclone.apply()
 
@@ -1080,7 +1080,7 @@
         self.addCleanup(self.wt.unlock)
         self.assertTrue(self.wt.is_executable('file1'))
         transform, root = self.get_transform()
-        file1_id = transform.trans_id_tree_file_id('file1-id')
+        file1_id = transform.trans_id_tree_path('file1')
         transform.delete_contents(file1_id)
         transform.create_file('contents2', file1_id)
         transform.apply()
@@ -1124,7 +1124,7 @@
         transform, root = self.get_transform()
         try:
             self.assertEqual([], list(transform.iter_changes()))
-            old = transform.trans_id_tree_file_id('id-1')
+            old = transform.trans_id_tree_path('old')
             transform.unversion_file(old)
             self.assertEqual([('id-1', ('old', None), False, (True, False),
                 ('eert_toor', 'eert_toor'), ('old', 'old'), ('file', 'file'),
@@ -1162,7 +1162,7 @@
         transform, root = self.get_transform()
         try:
             old = transform.trans_id_tree_path('old')
-            subdir = transform.trans_id_tree_file_id('subdir-id')
+            subdir = transform.trans_id_tree_path('subdir')
             new = transform.trans_id_tree_path('new')
             self.assertEqual([], list(transform.iter_changes()))
 
@@ -1280,7 +1280,7 @@
         transform, root = self.get_transform()
         try:
             old = transform.trans_id_tree_path('old')
-            subdir = transform.trans_id_tree_file_id('subdir-id')
+            subdir = transform.trans_id_tree_path('subdir')
             self.assertEqual([], list(transform.iter_changes()))
             transform.delete_contents(subdir)
             transform.create_directory(subdir)
@@ -1651,7 +1651,7 @@
         self.wt.set_root_id(root_id)
         self.b = self.wt.branch
         self.tt = TreeTransform(self.wt)
-        self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
+        self.root = self.tt.trans_id_tree_path('')
 
 
 def conflict_text(tree, merge):
@@ -1675,7 +1675,7 @@
         with TransformPreview(tree) as tt:
             tt.unversion_file(tt.root)
             tt.version_file(b'new-id', tt.root)
-            foo_trans_id = tt.trans_id_tree_file_id(b'foo-id')
+            foo_trans_id = tt.trans_id_tree_path('foo')
             foo_tuple = ('foo', foo_trans_id)
             root_tuple = ('', tt.root)
             self.assertEqual([root_tuple, foo_tuple], tt._inventory_altered())
@@ -1687,7 +1687,7 @@
         with TransformPreview(tree) as tt:
             tt.unversion_file(tt.root)
             tt.version_file(tree.get_root_id(), tt.root)
-            foo_trans_id = tt.trans_id_tree_file_id(b'foo-id')
+            foo_trans_id = tt.trans_id_tree_path('foo')
             self.assertEqual([], tt._inventory_altered())
 
 
@@ -1763,7 +1763,7 @@
         merge_modified = this.wt.merge_modified()
         self.assertSubset(merge_modified, modified)
         self.assertEqual(len(merge_modified), len(modified))
-        with file(this.wt.id2abspath('a'), 'wb') as f: f.write('booga')
+        with file(this.wt.abspath(this.wt.id2path('a')), 'wb') as f: f.write('booga')
         modified.pop(0)
         merge_modified = this.wt.merge_modified()
         self.assertSubset(merge_modified, modified)
@@ -2160,7 +2160,7 @@
         self.requireFeature(HardlinkFeature)
         source = self.create_ab_tree()
         tt = TreeTransform(source)
-        trans_id = tt.trans_id_tree_file_id('file1-id')
+        trans_id = tt.trans_id_tree_path('file1')
         tt.set_executability(True, trans_id)
         tt.apply()
         self.assertTrue(source.is_executable('file1'))
@@ -2669,7 +2669,7 @@
 
     def test_resolve_create_parent_for_versioned_file(self):
         wt, tt = self.make_tt_with_versioned_dir()
-        dir_tid = tt.trans_id_tree_file_id(b'dir-id')
+        dir_tid = tt.trans_id_tree_path('dir')
         file_tid = tt.new_file('file', dir_tid, 'Contents', file_id=b'file-id')
         tt.delete_contents(dir_tid)
         tt.unversion_file(dir_tid)
@@ -2680,7 +2680,7 @@
 
     def test_non_versioned_file_create_conflict(self):
         wt, tt = self.make_tt_with_versioned_dir()
-        dir_tid = tt.trans_id_tree_file_id(b'dir-id')
+        dir_tid = tt.trans_id_tree_path('dir')
         tt.new_file('file', dir_tid, 'Contents')
         tt.delete_contents(dir_tid)
         tt.unversion_file(dir_tid)
@@ -2837,7 +2837,7 @@
         work_tree.add('file', b'file-id')
         preview = TransformPreview(work_tree)
         self.addCleanup(preview.finalize)
-        file_trans_id = preview.trans_id_tree_file_id(b'file-id')
+        file_trans_id = preview.trans_id_tree_path('file')
         preview.adjust_path('renamed', preview.root, file_trans_id)
         preview_tree = preview.get_preview_tree()
         preview_mtime = preview_tree.get_file_mtime('renamed', b'file-id')
@@ -2937,13 +2937,14 @@
         self.assertFalse(preview_tree.is_versioned('old_name/child'))
         self.assertEqual(b'child-id', preview_tree.path2id('new_name/child'))
 
-    def assertMatchingIterEntries(self, tt, specific_file_ids=None):
+    def assertMatchingIterEntries(self, tt, specific_files=None):
         preview_tree = tt.get_preview_tree()
         preview_result = list(preview_tree.iter_entries_by_dir(
-                              specific_file_ids))
+                              specific_files=specific_files))
         tree = tt._tree
         tt.apply()
-        actual_result = list(tree.iter_entries_by_dir(specific_file_ids))
+        actual_result = list(tree.iter_entries_by_dir(
+            specific_files=specific_files))
         self.assertEqual(actual_result, preview_result)
 
     def test_iter_entries_by_dir_new(self):
@@ -2977,13 +2978,13 @@
                        tt.trans_id_file_id(b'moved-id'))
         self.assertMatchingIterEntries(tt)
 
-    def test_iter_entries_by_dir_specific_file_ids(self):
+    def test_iter_entries_by_dir_specific_files(self):
         tree = self.make_branch_and_tree('tree')
         tree.set_root_id(b'tree-root-id')
         self.build_tree(['tree/parent/', 'tree/parent/child'])
         tree.add(['parent', 'parent/child'], [b'parent-id', b'child-id'])
         tt = TreeTransform(tree)
-        self.assertMatchingIterEntries(tt, [b'tree-root-id', b'child-id'])
+        self.assertMatchingIterEntries(tt, ['', 'parent/child'])
 
     def test_symlink_content_summary(self):
         self.requireFeature(SymlinkFeature)
@@ -3455,9 +3456,9 @@
 
     def test_serialize_destruction(self):
         tt = self.make_destruction_preview()
-        foo_trans_id = tt.trans_id_tree_file_id(b'foo-id')
+        foo_trans_id = tt.trans_id_tree_path(u'foo\u1234')
         tt.unversion_file(foo_trans_id)
-        bar_trans_id = tt.trans_id_tree_file_id(b'bar-id')
+        bar_trans_id = tt.trans_id_tree_path('bar')
         tt.delete_contents(bar_trans_id)
         self.assertSerializesTo(self.destruction_records(), tt)
 
@@ -3612,7 +3613,7 @@
         self.assertRaises(NotImplementedError, tt.new_orphan, 'foo', 'bar')
 
     def _set_orphan_policy(self, wt, policy):
-        wt.branch.get_config_stack().set('bzr.transform.orphan_policy',
+        wt.branch.get_config_stack().set('transform.orphan_policy',
                                                policy)
 
     def _prepare_orphan(self, wt):
@@ -3741,7 +3742,7 @@
         self.parent_tree.lock_write()
         self.addCleanup(self.parent_tree.unlock)
         self.build_tree_contents([('parent/foo', b'bar')])
-        self.parent_tree.add('foo', b'foo-id')
+        self.parent_tree.add('foo')
         self.parent_tree.commit('added foo')
         child_controldir = self.parent_tree.controldir.sprout('child')
         self.child_tree = child_controldir.open_workingtree()
@@ -3761,7 +3762,7 @@
         """If the file to be linked has modified execute bit, don't link."""
         tt = TreeTransform(self.child_tree)
         try:
-            trans_id = tt.trans_id_tree_file_id(b'foo-id')
+            trans_id = tt.trans_id_tree_path('foo')
             tt.set_executability(True, trans_id)
             tt.apply()
         finally:

=== modified file 'breezy/tests/test_tree.py'
--- breezy/tests/test_tree.py	2018-02-18 15:21:06 +0000
+++ breezy/tests/test_tree.py	2018-03-25 00:39:16 +0000
@@ -28,6 +28,7 @@
 from breezy.tree import (
     FileTimestampUnavailable,
     InterTree,
+    find_previous_paths,
     )
 
 
@@ -430,3 +431,33 @@
         self.assertPathToKey(([u''], u'a'), u'a')
         self.assertPathToKey(([u'a'], u'b'), u'a/b')
         self.assertPathToKey(([u'a', u'b'], u'c'), u'a/b/c')
+
+
+class FindPreviousPathsTests(TestCaseWithTransport):
+
+    def test_new(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/b'])
+        tree.add(['b'])
+        revid1 = tree.commit('first')
+        tree1 = tree.branch.repository.revision_tree(revid1)
+
+        tree0 = tree.branch.repository.revision_tree(revision.NULL_REVISION)
+
+        self.assertEqual({'b': None}, find_previous_paths(tree1, tree0, ['b']))
+
+    def test_find_previous_paths(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/b'])
+        tree.add(['b'])
+        revid1 = tree.commit('first')
+        tree1 = tree.branch.repository.revision_tree(revid1)
+
+        tree.rename_one('b', 'c')
+        self.build_tree(['tree/b'])
+        tree.add(['b'])
+        revid2 = tree.commit('second')
+        tree2 = tree.branch.repository.revision_tree(revid2)
+
+        self.assertEqual({'c': 'b', 'b': None},
+                         find_previous_paths(tree2, tree1, ['b', 'c']))

=== modified file 'breezy/tests/test_ui.py'
--- breezy/tests/test_ui.py	2017-10-27 00:18:42 +0000
+++ breezy/tests/test_ui.py	2018-03-24 15:16:49 +0000
@@ -53,16 +53,17 @@
 
     def test_text_factory_confirm(self):
         # turns into reading a regular boolean
-        ui = ui_testing.TestUIFactory('n\n')
-        self.assertEqual(ui.confirm_action(u'Should %(thing)s pass?',
-            'breezy.tests.test_ui.confirmation',
-            {'thing': 'this'},),
-            False)
+        with ui_testing.TestUIFactory('n\n') as ui:
+            self.assertEqual(
+                False,
+                ui.confirm_action(
+                    u'Should %(thing)s pass?',
+                    'breezy.tests.test_ui.confirmation',
+                    {'thing': 'this'}))
 
     def test_text_factory_ascii_password(self):
         ui = ui_testing.TestUIFactory('secret\n')
-        pb = ui.nested_progress_bar()
-        try:
+        with ui.nested_progress_bar() as pb:
             self.assertEqual('secret',
                              self.apply_redirected(ui.stdin, ui.stdout,
                                                    ui.stderr,
@@ -72,8 +73,6 @@
             self.assertEqual('', ui.stdout.readline())
             # stdin should be empty
             self.assertEqual('', ui.stdin.readline())
-        finally:
-            pb.finished()
 
     def test_text_factory_unicode_password(self):
         """Test a unicode password."""
@@ -96,44 +95,51 @@
             "I'm sure!\nyes\n" # True
             "NO\n" # False
             "foo\n")
-        factory = ui_testing.TextUIFactory(stdin_text)
-        self.assertEqual(True, factory.get_boolean(u""))
-        self.assertEqual(False, factory.get_boolean(u""))
-        self.assertEqual(True, factory.get_boolean(u""))
-        self.assertEqual(False, factory.get_boolean(u""))
-        self.assertEqual(True, factory.get_boolean(u""))
-        self.assertEqual(False, factory.get_boolean(u""))
-        self.assertEqual(True, factory.get_boolean(u""))
-        self.assertEqual(False, factory.get_boolean(u""))
-        self.assertEqual("foo\n", factory.stdin.read())
-        # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
-        # return false on EOF
-        self.assertEqual(False, factory.get_boolean(u""))
+        with ui_testing.TextUIFactory(stdin_text) as factory:
+            self.assertEqual(True, factory.get_boolean(u""))
+            self.assertEqual(False, factory.get_boolean(u""))
+            self.assertEqual(True, factory.get_boolean(u""))
+            self.assertEqual(False, factory.get_boolean(u""))
+            self.assertEqual(True, factory.get_boolean(u""))
+            self.assertEqual(False, factory.get_boolean(u""))
+            self.assertEqual(True, factory.get_boolean(u""))
+            self.assertEqual(False, factory.get_boolean(u""))
+            self.assertEqual("foo\n", factory.stdin.read())
+            # stdin should be empty
+            self.assertEqual('', factory.stdin.readline())
+            # return false on EOF
+            self.assertEqual(False, factory.get_boolean(u""))
 
     def test_text_ui_choose_bad_parameters(self):
-        factory = ui_testing.TextUIFactory(u"")
-        # invalid default index
-        self.assertRaises(ValueError, factory.choose, u"", u"&Yes\n&No", 3)
-        # duplicated choice
-        self.assertRaises(ValueError, factory.choose, u"", u"&choice\n&ChOiCe")
-        # duplicated shortcut
-        self.assertRaises(ValueError, factory.choose, u"", u"&choice1\nchoi&ce2")
+        with ui_testing.TextUIFactory(u"") as factory:
+            # invalid default index
+            self.assertRaises(ValueError, factory.choose, u"", u"&Yes\n&No", 3)
+            # duplicated choice
+            self.assertRaises(
+                ValueError, factory.choose, u"", u"&choice\n&ChOiCe")
+            # duplicated shortcut
+            self.assertRaises(
+                ValueError, factory.choose, u"", u"&choice1\nchoi&ce2")
 
     def test_text_ui_choose_prompt_explicit(self):
         # choices with explicit shortcuts
-        factory = ui_testing.TextUIFactory(u"")
-        factory.choose(u"prompt", u"&yes\n&No\nmore &info")
-        self.assertEqual("prompt ([y]es, [N]o, more [i]nfo): \n", factory.stderr.getvalue())
+        with ui_testing.TextUIFactory(u"") as factory:
+            factory.choose(u"prompt", u"&yes\n&No\nmore &info")
+            self.assertEqual(
+                "prompt ([y]es, [N]o, more [i]nfo): \n",
+                factory.stderr.getvalue())
 
     def test_text_ui_choose_prompt_automatic(self):
         # automatic shortcuts
-        factory = ui_testing.TextUIFactory(u"")
-        factory.choose(u"prompt", u"yes\nNo\nmore info")
-        self.assertEqual("prompt ([y]es, [N]o, [m]ore info): \n", factory.stderr.getvalue())
+        with ui_testing.TextUIFactory(u"") as factory:
+            factory.choose(u"prompt", u"yes\nNo\nmore info")
+            self.assertEqual(
+                "prompt ([y]es, [N]o, [m]ore info): \n",
+                factory.stderr.getvalue())
 
     def test_text_ui_choose_return_values(self):
-        choose = lambda: factory.choose(u"", u"&Yes\n&No\nMaybe\nmore &info", 3)
+        def choose():
+            return factory.choose(u"", u"&Yes\n&No\nMaybe\nmore &info", 3)
         stdin_text = (
             "y\n" # 0
             "n\n" # 1
@@ -145,81 +151,79 @@
             "info\nmore info\n" # 3
             "Maybe\n" # 2
             "foo\n")
-        factory = ui_testing.TextUIFactory(stdin_text)
-        self.assertEqual(0, choose())
-        self.assertEqual(1, choose())
-        self.assertEqual(3, choose())
-        self.assertEqual(1, choose())
-        self.assertEqual(0, choose())
-        self.assertEqual(1, choose())
-        self.assertEqual(3, choose())
-        self.assertEqual(2, choose())
-        self.assertEqual("foo\n", factory.stdin.read())
-        # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
-        # return None on EOF
-        self.assertEqual(None, choose())
+        with ui_testing.TextUIFactory(stdin_text) as factory:
+            self.assertEqual(0, choose())
+            self.assertEqual(1, choose())
+            self.assertEqual(3, choose())
+            self.assertEqual(1, choose())
+            self.assertEqual(0, choose())
+            self.assertEqual(1, choose())
+            self.assertEqual(3, choose())
+            self.assertEqual(2, choose())
+            self.assertEqual("foo\n", factory.stdin.read())
+            # stdin should be empty
+            self.assertEqual('', factory.stdin.readline())
+            # return None on EOF
+            self.assertEqual(None, choose())
 
     def test_text_ui_choose_no_default(self):
         stdin_text = (
             " \n" # no default, invalid!
             " yes \n" # 0
             "foo\n")
-        factory = ui_testing.TextUIFactory(stdin_text)
-        self.assertEqual(0, factory.choose(u"", u"&Yes\n&No"))
-        self.assertEqual("foo\n", factory.stdin.read())
+        with ui_testing.TextUIFactory(stdin_text) as factory:
+            self.assertEqual(0, factory.choose(u"", u"&Yes\n&No"))
+            self.assertEqual("foo\n", factory.stdin.read())
 
     def test_text_ui_get_integer(self):
         stdin_text = (
             "1\n"
             "  -2  \n"
             "hmmm\nwhat else ?\nCome on\nok 42\n4.24\n42\n")
-        factory = ui_testing.TextUIFactory(stdin_text)
-        self.assertEqual(1, factory.get_integer(u""))
-        self.assertEqual(-2, factory.get_integer(u""))
-        self.assertEqual(42, factory.get_integer(u""))
+        with ui_testing.TextUIFactory(stdin_text) as factory:
+            self.assertEqual(1, factory.get_integer(u""))
+            self.assertEqual(-2, factory.get_integer(u""))
+            self.assertEqual(42, factory.get_integer(u""))
 
     def test_text_factory_prompt(self):
         # see <https://launchpad.net/bugs/365891>
-        factory = ui_testing.TextUIFactory()
-        factory.prompt(u'foo %2e')
-        self.assertEqual('', factory.stdout.getvalue())
-        self.assertEqual('foo %2e', factory.stderr.getvalue())
+        with ui_testing.TextUIFactory() as factory:
+            factory.prompt(u'foo %2e')
+            self.assertEqual('', factory.stdout.getvalue())
+            self.assertEqual('foo %2e', factory.stderr.getvalue())
 
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         out = ui_testing.StringIOAsTTY()
         self.overrideEnv('TERM', 'xterm')
         factory = ui_testing.TextUIFactory("yada\ny\n", stdout=out, stderr=out)
-        pb = factory.nested_progress_bar()
-        pb._avail_width = lambda: 79
-        pb.show_bar = False
-        pb.show_spinner = False
-        pb.show_count = False
-        pb.update("foo", 0, 1)
-        self.assertEqual(True,
-                         self.apply_redirected(None, factory.stdout,
-                                               factory.stdout,
-                                               factory.get_boolean,
-                                               u"what do you want"))
-        output = out.getvalue()
-        self.assertContainsRe(output,
-            "| foo *\r\r  *\r*")
-        self.assertContainsString(output,
-            r"what do you want? ([y]es, [n]o): what do you want? ([y]es, [n]o): ")
-        # stdin should have been totally consumed
-        self.assertEqual('', factory.stdin.readline())
+        with factory:
+            pb = factory.nested_progress_bar()
+            pb._avail_width = lambda: 79
+            pb.show_bar = False
+            pb.show_spinner = False
+            pb.show_count = False
+            pb.update("foo", 0, 1)
+            self.assertEqual(
+                True,
+                self.apply_redirected(
+                    None, factory.stdout, factory.stdout, factory.get_boolean,
+                    u"what do you want"))
+            output = out.getvalue()
+            self.assertContainsRe(output,
+                "| foo *\r\r  *\r*")
+            self.assertContainsString(output,
+                r"what do you want? ([y]es, [n]o): what do you want? ([y]es, [n]o): ")
+            # stdin should have been totally consumed
+            self.assertEqual('', factory.stdin.readline())
 
     def test_text_tick_after_update(self):
         ui_factory = ui_testing.TextUIFactory()
-        pb = ui_factory.nested_progress_bar()
-        try:
+        with ui_factory.nested_progress_bar() as pb:
             pb.update('task', 0, 3)
             # Reset the clock, so that it actually tries to repaint itself
             ui_factory._progress_view._last_repaint = time.time() - 1.0
             pb.tick()
-        finally:
-            pb.finished()
 
     def test_text_ui_getusername(self):
         ui = ui_testing.TextUIFactory('someuser\n\n')
@@ -242,11 +246,12 @@
         self.overrideEnv('BRZ_PROGRESS_BAR', 'text')
         ui_factory = ui_testing.TextUIFactory(
             stderr=ui_testing.StringIOAsTTY())
-        self.assertIsInstance(ui_factory._progress_view,
-            _mod_ui_text.TextProgressView)
-        ui_factory.be_quiet(True)
-        self.assertIsInstance(ui_factory._progress_view,
-            _mod_ui_text.NullProgressView)
+        with ui_factory:
+            self.assertIsInstance(ui_factory._progress_view,
+                _mod_ui_text.TextProgressView)
+            ui_factory.be_quiet(True)
+            self.assertIsInstance(ui_factory._progress_view,
+                _mod_ui_text.NullProgressView)
 
     def test_text_ui_show_user_warning(self):
         from ..bzr.groupcompress_repo import RepositoryFormat2a
@@ -373,10 +378,9 @@
         # there's no output; we just want to make sure this doesn't crash -
         # see https://bugs.launchpad.net/bzr/+bug/408201
         ui = ui_testing.TestUIFactory()
-        pb = ui.nested_progress_bar()
-        pb.update('hello')
-        pb.tick()
-        pb.finished()
+        with ui.nested_progress_bar() as pb:
+            pb.update('hello')
+            pb.tick()
 
 
 class CannedInputUIFactoryTests(tests.TestCase):

=== modified file 'breezy/tests/test_workingtree.py'
--- breezy/tests/test_workingtree.py	2018-02-18 15:49:32 +0000
+++ breezy/tests/test_workingtree.py	2018-03-25 00:39:16 +0000
@@ -257,7 +257,8 @@
         subtree = self.make_branch_and_tree('tree/a/b')
         self.assertEqual([('tree-reference', b'b-id')],
                          [(ie.kind, ie.file_id)
-                          for path, ie in tree.iter_entries_by_dir([b'b-id'])])
+                          for path, ie in tree.iter_entries_by_dir(
+                              specific_files=['a/b'])])
 
     def test_direct_subtree(self):
         tree = self.make_simple_tree()

=== modified file 'breezy/tests/testui.py'
--- breezy/tests/testui.py	2017-05-22 00:56:52 +0000
+++ breezy/tests/testui.py	2018-03-01 23:44:14 +0000
@@ -41,6 +41,13 @@
     def finished(self):
         self.depth -= 1
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.finished()
+        return False
+
     def update(self, message, count=None, total=None):
         if self.depth == 1:
             self._calls.append(("update", count, total, message))

=== modified file 'breezy/tests/ui_testing.py'
--- breezy/tests/ui_testing.py	2017-06-14 22:19:52 +0000
+++ breezy/tests/ui_testing.py	2018-03-24 15:16:49 +0000
@@ -62,6 +62,11 @@
             stderr = StringIOWithEncoding()
         super(TextUIFactory, self).__init__(stdin, stdout, stderr)
 
+    def _setup_streams(self):
+        self.raw_stdin = self.stdin
+        self.raw_stdout = self.stdout
+        self.raw_stderr = self.stderr
+
 
 class TestUIFactory(TextUIFactory):
     """A UI Factory for testing.

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2018-02-17 02:02:38 +0000
+++ breezy/transform.py	2018-03-24 23:49:53 +0000
@@ -69,6 +69,9 @@
     viewitems,
     viewvalues,
     )
+from .tree import (
+    find_previous_path,
+    )
 
 
 ROOT_PARENT = "root-parent"
@@ -90,8 +93,7 @@
 class TreeTransformBase(object):
     """The base class for TreeTransform and its kin."""
 
-    def __init__(self, tree, pb=None,
-                 case_sensitive=True):
+    def __init__(self, tree, pb=None, case_sensitive=True):
         """Constructor.
 
         :param tree: The tree that will be transformed, but not necessarily
@@ -132,7 +134,7 @@
         # The trans_id that will be used as the tree root
         root_id = tree.get_root_id()
         if root_id is not None:
-            self._new_root = self.trans_id_tree_file_id(root_id)
+            self._new_root = self.trans_id_tree_path('')
         else:
             self._new_root = None
         # Indicator of whether the transform has been applied
@@ -216,7 +218,7 @@
         # the physical root needs a new transaction id
         self._tree_path_ids.pop("")
         self._tree_id_paths.pop(old_root)
-        self._new_root = self.trans_id_tree_file_id(self._tree.get_root_id())
+        self._new_root = self.trans_id_tree_path('')
         if parent == old_root:
             parent = self._new_root
         self.adjust_path(name, parent, old_root)
@@ -285,17 +287,6 @@
         del self._new_parent[old_new_root]
         del self._new_name[old_new_root]
 
-    def trans_id_tree_file_id(self, inventory_id):
-        """Determine the transaction id of a working tree file.
-
-        This reflects only files that already exist, not ones that will be
-        added by transactions.
-        """
-        if inventory_id is None:
-            raise ValueError('None is not a valid file id')
-        path = self._tree.id2path(inventory_id)
-        return self.trans_id_tree_path(path)
-
     def trans_id_file_id(self, file_id):
         """Determine or set the transaction id associated with a file ID.
         A new id is only created for file_ids that were never present.  If
@@ -308,8 +299,8 @@
             return self._r_new_id[file_id]
         else:
             try:
-                next(self._tree.iter_entries_by_dir([file_id]))
-            except StopIteration:
+                path = self._tree.id2path(file_id)
+            except errors.NoSuchId:
                 if file_id in self._non_present_ids:
                     return self._non_present_ids[file_id]
                 else:
@@ -317,7 +308,7 @@
                     self._non_present_ids[file_id] = trans_id
                     return trans_id
             else:
-                return self.trans_id_tree_file_id(file_id)
+                return self.trans_id_tree_path(path)
 
     def trans_id_tree_path(self, path):
         """Determine (and maybe set) the transaction ID for a tree path."""
@@ -454,12 +445,14 @@
         else:
             return self.tree_kind(trans_id)
 
+    def tree_path(self, trans_id):
+        """Determine the tree path associated with the trans_id."""
+        return self._tree_id_paths.get(trans_id)
+
     def tree_file_id(self, trans_id):
         """Determine the file id associated with the trans_id in the tree"""
-        try:
-            path = self._tree_id_paths[trans_id]
-        except KeyError:
-            # the file is a new, unversioned file, or invalid trans_id
+        path = self.tree_path(trans_id)
+        if path is None:
             return None
         # the file is old; the old id is still valid
         if self._new_root == trans_id:
@@ -567,10 +560,9 @@
         parents.extend([t for t in self._removed_contents if
                         self.tree_kind(t) == 'directory'])
         for trans_id in self._removed_id:
-            file_id = self.tree_file_id(trans_id)
-            path = self._tree.id2path(file_id)
-            if file_id is not None:
-                if self._tree.stored_kind(path, file_id) == 'directory':
+            path = self.tree_path(trans_id)
+            if path is not None:
+                if self._tree.stored_kind(path) == 'directory':
                     parents.append(trans_id)
             elif self.tree_kind(trans_id) == 'directory':
                 parents.append(trans_id)
@@ -735,7 +727,8 @@
         active_tree_ids = all_ids.difference(removed_tree_ids)
         for trans_id, file_id in viewitems(self._new_id):
             if file_id in active_tree_ids:
-                old_trans_id = self.trans_id_tree_file_id(file_id)
+                path = self._tree.id2path(file_id)
+                old_trans_id = self.trans_id_tree_path(path)
                 conflicts.append(('duplicate id', old_trans_id, trans_id))
         return conflicts
 
@@ -901,7 +894,7 @@
                 to_trans_ids[to_file_id] = trans_id
         return from_trans_ids, to_trans_ids
 
-    def _from_file_data(self, from_trans_id, from_versioned, file_id):
+    def _from_file_data(self, from_trans_id, from_versioned, from_path):
         """Get data about a file in the from (tree) state
 
         Return a (name, parent, kind, executable) tuple
@@ -909,7 +902,8 @@
         from_path = self._tree_id_paths.get(from_trans_id)
         if from_versioned:
             # get data from working tree if versioned
-            from_entry = next(self._tree.iter_entries_by_dir([file_id]))[1]
+            from_entry = next(self._tree.iter_entries_by_dir(
+                    specific_files=[from_path]))[1]
             from_name = from_entry.name
             from_parent = from_entry.parent_id
         else:
@@ -978,12 +972,6 @@
             else:
                 to_versioned = True
 
-            from_name, from_parent, from_kind, from_executable = \
-                self._from_file_data(from_trans_id, from_versioned, file_id)
-
-            to_name, to_parent, to_kind, to_executable = \
-                self._to_file_data(to_trans_id, from_trans_id, from_executable)
-
             if not from_versioned:
                 from_path = None
             else:
@@ -992,6 +980,13 @@
                 to_path = None
             else:
                 to_path = final_paths.get_path(to_trans_id)
+
+            from_name, from_parent, from_kind, from_executable = \
+                self._from_file_data(from_trans_id, from_versioned, from_path)
+
+            to_name, to_parent, to_kind, to_executable = \
+                self._to_file_data(to_trans_id, from_trans_id, from_executable)
+
             if from_kind != to_kind:
                 modified = True
             elif to_kind in ('file', 'symlink') and (
@@ -1077,28 +1072,27 @@
         return revision_id
 
     def _text_parent(self, trans_id):
-        file_id = self.tree_file_id(trans_id)
+        path = self.tree_path(trans_id)
         try:
-            if (file_id is None or
-                    self._tree.kind(self._tree.id2path(file_id), file_id) != 'file'):
+            if path is None or self._tree.kind(path) != 'file':
                 return None
         except errors.NoSuchFile:
             return None
-        return file_id
+        return path
 
     def _get_parents_texts(self, trans_id):
         """Get texts for compression parents of this file."""
-        file_id = self._text_parent(trans_id)
-        if file_id is None:
+        path = self._text_parent(trans_id)
+        if path is None:
             return ()
-        return (self._tree.get_file_text(self._tree.id2path(file_id), file_id),)
+        return (self._tree.get_file_text(path),)
 
     def _get_parents_lines(self, trans_id):
         """Get lines for compression parents of this file."""
-        file_id = self._text_parent(trans_id)
-        if file_id is None:
+        path = self._text_parent(trans_id)
+        if path is None:
             return ()
-        return (self._tree.get_file_lines(self._tree.id2path(file_id), file_id),)
+        return (self._tree.get_file_lines(path),)
 
     def serialize(self, serializer):
         """Serialize this TreeTransform.
@@ -1407,7 +1401,7 @@
 
     def new_orphan(self, trans_id, parent_id):
         conf = self._tree.get_config_stack()
-        handle_orphan = conf.get('bzr.transform.orphan_policy')
+        handle_orphan = conf.get('transform.orphan_policy')
         handle_orphan(self, trans_id, parent_id)
 
 
@@ -1477,7 +1471,7 @@
 
 
 opt_transform_orphan = _mod_config.RegistryOption(
-    'bzr.transform.orphan_policy', orphaning_registry,
+    'transform.orphan_policy', orphaning_registry,
     help='Policy for orphaned files during transform operations.',
     invalid='warning')
 
@@ -1721,8 +1715,7 @@
             hook(self._tree, self)
         if not no_conflicts:
             self._check_malformed()
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             if precomputed_delta is None:
                 child_pb.update(gettext('Apply phase'), 0, 2)
                 inventory_delta = self._generate_inventory_delta()
@@ -1744,8 +1737,6 @@
                 raise
             else:
                 mover.apply_deletions()
-        finally:
-            child_pb.finished()
         if self.final_file_id(self.root) is None:
             inventory_delta = [e for e in inventory_delta if e[0] != '']
         self._tree.apply_inventory_delta(inventory_delta)
@@ -1757,10 +1748,9 @@
     def _generate_inventory_delta(self):
         """Generate an inventory delta for the current transform."""
         inventory_delta = []
-        child_pb = ui.ui_factory.nested_progress_bar()
         new_paths = self._inventory_altered()
         total_entries = len(new_paths) + len(self._removed_id)
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             for num, trans_id in enumerate(self._removed_id):
                 if (num % 10) == 0:
                     child_pb.update(gettext('removing file'), num, total_entries)
@@ -1775,9 +1765,6 @@
                 inventory_delta.append((path, None, file_id, None))
             new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
                                      new_paths)
-            entries = self._tree.iter_entries_by_dir(
-                viewvalues(new_path_file_ids))
-            old_paths = dict((e.file_id, p) for p, e in entries)
             final_kinds = {}
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
@@ -1805,14 +1792,15 @@
                     new_entry = inventory.make_entry(kind,
                         self.final_name(trans_id),
                         parent_file_id, file_id)
-                old_path = old_paths.get(new_entry.file_id)
+                try:
+                    old_path = self._tree.id2path(new_entry.file_id)
+                except errors.NoSuchId:
+                    old_path = None
                 new_executability = self._new_executability.get(trans_id)
                 if new_executability is not None:
                     new_entry.executable = new_executability
                 inventory_delta.append(
                     (old_path, path, new_entry.file_id, new_entry))
-        finally:
-            child_pb.finished()
         return inventory_delta
 
     def _apply_removals(self, mover):
@@ -1825,8 +1813,7 @@
         If inventory_delta is None, no inventory delta generation is performed.
         """
         tree_paths = sorted(viewitems(self._tree_path_ids), reverse=True)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             for num, (path, trans_id) in enumerate(tree_paths):
                 # do not attempt to move root into a subdirectory of itself.
                 if path == '':
@@ -1845,8 +1832,6 @@
                             raise
                     else:
                         self.rename_count += 1
-        finally:
-            child_pb.finished()
 
     def _apply_insertions(self, mover):
         """Perform tree operations that insert directory/inventory names.
@@ -1862,8 +1847,7 @@
         modified_paths = []
         new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
                                  new_paths)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
                     child_pb.update(gettext('adding file'), num, len(new_paths))
@@ -1890,8 +1874,6 @@
                     o_sha1, o_st_val = self._observed_sha1s[trans_id]
                     st = osutils.lstat(full_path)
                     self._observed_sha1s[trans_id] = (o_sha1, st)
-        finally:
-            child_pb.finished()
         for path, trans_id in new_paths:
             # new_paths includes stuff like workingtree conflicts. Only the
             # stuff in new_contents actually comes from limbo.
@@ -1964,10 +1946,8 @@
             path = self._tree_id_paths[parent_id]
         except KeyError:
             return
-        file_id = self.tree_file_id(parent_id)
-        if file_id is None:
-            return
-        entry = next(self._tree.iter_entries_by_dir([file_id]))[1]
+        entry = next(self._tree.iter_entries_by_dir(
+                specific_files=[path]))[1]
         children = getattr(entry, 'children', {})
         for child in children:
             childpath = joinpath(path, child)
@@ -2023,9 +2003,7 @@
             vf.fallback_versionedfiles.append(base_vf)
         return tree_revision
 
-    def _stat_limbo_file(self, file_id=None, trans_id=None):
-        if trans_id is None:
-            trans_id = self._transform.trans_id_file_id(file_id)
+    def _stat_limbo_file(self, trans_id):
         name = self._transform._limbo_name(trans_id)
         return os.lstat(name)
 
@@ -2135,7 +2113,7 @@
         self._all_children_cache[trans_id] = children
         return children
 
-    def iter_children(self, file_id):
+    def _iter_children(self, file_id):
         trans_id = self._transform.trans_id_file_id(file_id)
         for child_trans_id in self._all_children(trans_id):
             yield self._transform.final_file_id(child_trans_id)
@@ -2149,14 +2127,14 @@
             if self._transform.final_file_id(trans_id) is None:
                 yield self._final_paths._determine_path(trans_id)
 
-    def _make_inv_entries(self, ordered_entries, specific_file_ids=None,
+    def _make_inv_entries(self, ordered_entries, specific_files=None,
         yield_parents=False):
         for trans_id, parent_file_id in ordered_entries:
             file_id = self._transform.final_file_id(trans_id)
             if file_id is None:
                 continue
-            if (specific_file_ids is not None
-                and file_id not in specific_file_ids):
+            if (specific_files is not None and
+                unicode(self._final_paths.get_path(trans_id)) not in specific_files):
                 continue
             kind = self._transform.final_kind(trans_id)
             if kind is None:
@@ -2184,17 +2162,15 @@
         return ordered_ids
 
     def iter_child_entries(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        if file_id is None:
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
             raise errors.NoSuchFile(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
         todo = [(child_trans_id, trans_id) for child_trans_id in
                 self._all_children(trans_id)]
         for entry, trans_id in self._make_inv_entries(todo):
             yield entry
 
-    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+    def iter_entries_by_dir(self, specific_files=None, yield_parents=False):
         # This may not be a maximally efficient implementation, but it is
         # reasonably straightforward.  An implementation that grafts the
         # TreeTransform changes onto the tree's iter_entries_by_dir results
@@ -2202,16 +2178,16 @@
         # position.
         ordered_ids = self._list_files_by_dir()
         for entry, trans_id in self._make_inv_entries(ordered_ids,
-            specific_file_ids, yield_parents=yield_parents):
+            specific_files, yield_parents=yield_parents):
             yield unicode(self._final_paths.get_path(trans_id)), entry
 
     def _iter_entries_for_dir(self, dir_path):
         """Return path, entry for items in a directory without recursing down."""
-        dir_file_id = self.path2id(dir_path)
         ordered_ids = []
-        for file_id in self.iter_children(dir_file_id):
-            trans_id = self._transform.trans_id_file_id(file_id)
-            ordered_ids.append((trans_id, file_id))
+        dir_trans_id = self._path2trans_id(dir_path)
+        dir_id = self._transform.final_file_id(dir_trans_id)
+        for child_trans_id in self._all_children(dir_trans_id):
+            ordered_ids.append((child_trans_id, dir_id))
         for entry, trans_id in self._make_inv_entries(ordered_ids):
             yield unicode(self._final_paths.get_path(trans_id)), entry
 
@@ -2242,15 +2218,15 @@
                 yield path, 'V', entry.kind, entry.file_id, entry
 
     def kind(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
         return self._transform.final_kind(trans_id)
 
     def stored_kind(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
         try:
             return self._transform._new_contents[trans_id]
         except KeyError:
@@ -2265,31 +2241,28 @@
         if not self._content_change(file_id):
             return self._transform._tree.get_file_mtime(
                     self._transform._tree.id2path(file_id), file_id)
-        return self._stat_limbo_file(file_id).st_mtime
-
-    def _file_size(self, entry, stat_value):
-        path = self.id2path(entry.file_id)
-        return self.get_file_size(path, entry.file_id)
+        trans_id = self._path2trans_id(path)
+        return self._stat_limbo_file(trans_id).st_mtime
 
     def get_file_size(self, path, file_id=None):
         """See Tree.get_file_size"""
-        if file_id is None:
-            file_id = self.path2id(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
         kind = self._transform.final_kind(trans_id)
         if kind != 'file':
             return None
         if trans_id in self._transform._new_contents:
-            return self._stat_limbo_file(trans_id=trans_id).st_size
+            return self._stat_limbo_file(trans_id).st_size
         if self.kind(path, file_id) == 'file':
             return self._transform._tree.get_file_size(path, file_id)
         else:
             return None
 
     def get_file_verifier(self, path, file_id=None, stat_value=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
         kind = self._transform._new_contents.get(trans_id)
         if kind is None:
             return self._transform._tree.get_file_verifier(path, file_id)
@@ -2301,9 +2274,9 @@
                 fileobj.close()
 
     def get_file_sha1(self, path, file_id=None, stat_value=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
         kind = self._transform._new_contents.get(trans_id)
         if kind is None:
             return self._transform._tree.get_file_sha1(path, file_id)
@@ -2315,11 +2288,9 @@
                 fileobj.close()
 
     def is_executable(self, path, file_id=None):
-        if file_id is None:
-            file_id = self.path2id(path)
-        if file_id is None:
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
             return False
-        trans_id = self._transform.trans_id_file_id(file_id)
         try:
             return self._transform._new_executability[trans_id]
         except KeyError:
@@ -2398,7 +2369,7 @@
             file_id = self.path2id(path)
         if not self._content_change(file_id):
             return self._transform._tree.get_file(path, file_id)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
         name = self._transform._limbo_name(trans_id)
         return open(name, 'rb')
 
@@ -2444,7 +2415,7 @@
             file_id = self.path2id(path)
         if not self._content_change(file_id):
             return self._transform._tree.get_symlink_target(path)
-        trans_id = self._transform.trans_id_file_id(file_id)
+        trans_id = self._path2trans_id(path)
         name = self._transform._limbo_name(trans_id)
         return osutils.readlink(name)
 
@@ -2596,10 +2567,8 @@
     divert = set()
     try:
         pp.next_phase()
-        file_trans_id[wt.get_root_id()] = \
-            tt.trans_id_tree_file_id(wt.get_root_id())
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
+        file_trans_id[wt.get_root_id()] = tt.trans_id_tree_path('')
+        with ui.ui_factory.nested_progress_bar() as pb:
             deferred_contents = []
             num = 0
             total = len(tree.all_versioned_paths())
@@ -2636,7 +2605,7 @@
                         else:
                             divert.add(file_id)
                     if (file_id not in divert and
-                        _content_match(tree, entry, file_id, kind,
+                        _content_match(tree, entry, tree_path, file_id, kind,
                         target_path)):
                         tt.delete_contents(tt.trans_id_tree_path(tree_path))
                         if kind == 'directory':
@@ -2651,8 +2620,8 @@
                     executable = tree.is_executable(tree_path, file_id)
                     if executable:
                         tt.set_executability(executable, trans_id)
-                    trans_data = (trans_id, tree_path, entry.text_sha1)
-                    deferred_contents.append((file_id, trans_data))
+                    trans_data = (trans_id, file_id, tree_path, entry.text_sha1)
+                    deferred_contents.append((tree_path, trans_data))
                 else:
                     file_trans_id[file_id] = new_by_entry(
                             tree_path, tt, entry, parent_id, tree)
@@ -2663,8 +2632,6 @@
             offset = num + 1 - len(deferred_contents)
             _create_files(tt, tree, deferred_contents, pb, offset,
                           accelerator_tree, hardlink)
-        finally:
-            pb.finished()
         pp.next_phase()
         divert_trans = set(file_trans_id[f] for f in divert)
         resolver = lambda t, c: resolve_checkout(t, c, divert_trans)
@@ -2694,19 +2661,19 @@
         new_desired_files = desired_files
     else:
         iter = accelerator_tree.iter_changes(tree, include_unchanged=True)
-        unchanged = [(f, p[1]) for (f, p, c, v, d, n, k, e)
+        unchanged = [(p[0], p[1]) for (f, p, c, v, d, n, k, e)
                      in iter if not (c or e[0] != e[1])]
         if accelerator_tree.supports_content_filtering():
-            unchanged = [(f, p) for (f, p) in unchanged
-                         if not next(accelerator_tree.iter_search_rules([p]))]
+            unchanged = [(tp, ap) for (tp, ap) in unchanged
+                         if not next(accelerator_tree.iter_search_rules([ap]))]
         unchanged = dict(unchanged)
         new_desired_files = []
         count = 0
-        for file_id, (trans_id, tree_path, text_sha1) in desired_files:
-            accelerator_path = unchanged.get(file_id)
+        for unused_tree_path, (trans_id, file_id, tree_path, text_sha1) in desired_files:
+            accelerator_path = unchanged.get(tree_path)
             if accelerator_path is None:
-                new_desired_files.append((file_id,
-                    (trans_id, tree_path, text_sha1)))
+                new_desired_files.append((tree_path,
+                    (trans_id, file_id, tree_path, text_sha1)))
                 continue
             pb.update(gettext('Adding file contents'), count + offset, total)
             if hardlink:
@@ -2728,7 +2695,7 @@
                         pass
             count += 1
         offset += count
-    for count, ((trans_id, tree_path, text_sha1), contents) in enumerate(
+    for count, ((trans_id, file_id, tree_path, text_sha1), contents) in enumerate(
             tree.iter_files_bytes(new_desired_files)):
         if wt.supports_content_filtering():
             filters = wt._content_filter_stack(tree_path)
@@ -2750,21 +2717,20 @@
     return by_parent[old_parent]
 
 
-def _content_match(tree, entry, file_id, kind, target_path):
+def _content_match(tree, entry, tree_path, file_id, kind, target_path):
     if entry.kind != kind:
         return False
     if entry.kind == "directory":
         return True
-    path = tree.id2path(file_id)
     if entry.kind == "file":
         f = file(target_path, 'rb')
         try:
-            if tree.get_file_text(path, file_id) == f.read():
+            if tree.get_file_text(tree_path, file_id) == f.read():
                 return True
         finally:
             f.close()
     elif entry.kind == "symlink":
-        if tree.get_symlink_target(path, file_id) == os.readlink(target_path):
+        if tree.get_symlink_target(tree_path, file_id) == os.readlink(target_path):
             return True
     return False
 
@@ -2874,7 +2840,8 @@
             trace.warning(unicode(conflict))
         pp.next_phase()
         tt.apply()
-        working_tree.set_merge_modified(merge_modified)
+        if working_tree.supports_merge_modified():
+            working_tree.set_merge_modified(merge_modified)
     finally:
         target_tree.unlock()
         tt.finalize()
@@ -2885,21 +2852,15 @@
 def _prepare_revert_transform(working_tree, target_tree, tt, filenames,
                               backups, pp, basis_tree=None,
                               merge_modified=None):
-    child_pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as child_pb:
         if merge_modified is None:
             merge_modified = working_tree.merge_modified()
         merge_modified = _alter_files(working_tree, target_tree, tt,
                                       child_pb, filenames, backups,
                                       merge_modified, basis_tree)
-    finally:
-        child_pb.finished()
-    child_pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as child_pb:
         raw_conflicts = resolve_conflicts(tt, child_pb,
             lambda t, c: conflict_pass(t, c, target_tree))
-    finally:
-        child_pb.finished()
     conflicts = cook_conflicts(raw_conflicts, tt)
     return conflicts, merge_modified
 
@@ -2943,9 +2904,8 @@
                         if basis_tree is None:
                             basis_tree = working_tree.basis_tree()
                             basis_tree.lock_read()
-                        try:
-                            basis_path = basis_tree.id2path(file_id)
-                        except errors.NoSuchId:
+                        basis_path = find_previous_path(working_tree, basis_tree, wt_path)
+                        if basis_path is None:
                             if target_kind is None and not target_versioned:
                                 keep_content = True
                         else:
@@ -2977,15 +2937,12 @@
                     tt.create_symlink(target_tree.get_symlink_target(
                             target_path, file_id), trans_id)
                 elif target_kind == 'file':
-                    deferred_files.append((file_id, (trans_id, mode_id)))
+                    deferred_files.append((target_path, (trans_id, mode_id, file_id)))
                     if basis_tree is None:
                         basis_tree = working_tree.basis_tree()
                         basis_tree.lock_read()
                     new_sha1 = target_tree.get_file_sha1(target_path, file_id)
-                    try:
-                        basis_path = basis_tree.id2path(file_id)
-                    except errors.NoSuchId:
-                        basis_path = None
+                    basis_path = find_previous_path(target_tree, basis_tree, target_path)
                     if (basis_path is not None and
                         new_sha1 == basis_tree.get_file_sha1(basis_path, file_id)):
                         if file_id in merge_modified:
@@ -3015,9 +2972,8 @@
             if wt_executable != target_executable and target_kind == "file":
                 tt.set_executability(target_executable, trans_id)
         if working_tree.supports_content_filtering():
-            for index, ((trans_id, mode_id), bytes) in enumerate(
+            for (trans_id, mode_id, file_id), bytes in (
                 target_tree.iter_files_bytes(deferred_files)):
-                file_id = deferred_files[index][0]
                 # We're reverting a tree to the target tree so using the
                 # target tree to find the file path seems the best choice
                 # here IMO - Ian C 27/Oct/2009
@@ -3027,7 +2983,7 @@
                     ContentFilterContext(filter_tree_path, working_tree))
                 tt.create_file(bytes, trans_id, mode_id)
         else:
-            for (trans_id, mode_id), bytes in target_tree.iter_files_bytes(
+            for (trans_id, mode_id, file_id), bytes in target_tree.iter_files_bytes(
                 deferred_files):
                 tt.create_file(bytes, trans_id, mode_id)
         tt.fixup_new_roots()
@@ -3042,8 +2998,7 @@
     if pass_func is None:
         pass_func = conflict_pass
     new_conflicts = set()
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb:
         for n in range(10):
             pb.update(gettext('Resolution pass'), n+1, 10)
             conflicts = tt.find_conflicts()
@@ -3051,8 +3006,6 @@
                 return new_conflicts
             new_conflicts.update(pass_func(tt, conflicts))
         raise MalformedTransform(conflicts=conflicts)
-    finally:
-        pb.finished()
 
 
 def conflict_pass(tt, conflicts, path_tree=None):
@@ -3123,7 +3076,7 @@
                         if file_id is None:
                             file_id = tt.inactive_file_id(trans_id)
                         _, entry = next(path_tree.iter_entries_by_dir(
-                            [file_id]))
+                            specific_files=[path_tree.id2path(file_id)]))
                         # special-case the other tree root (move its
                         # children to current root)
                         if entry.parent_id is None:
@@ -3144,8 +3097,7 @@
         elif c_type == 'unversioned parent':
             file_id = tt.inactive_file_id(conflict[1])
             # special-case the other tree root (move its children instead)
-            if path_tree and path_tree.has_id(file_id):
-                if path_tree.path2id('') == file_id:
+            if path_tree and path_tree.path2id('') == file_id:
                     # This is the root entry, skip it
                     continue
             tt.version_file(file_id, conflict[1])
@@ -3259,9 +3211,9 @@
                 continue
             if executable[0] != executable[1]:
                 continue
-            trans_id = tt.trans_id_tree_file_id(file_id)
+            trans_id = tt.trans_id_tree_path(paths[1])
             tt.delete_contents(trans_id)
-            tt.create_hardlink(source_tree.id2abspath(file_id), trans_id)
+            tt.create_hardlink(source_tree.abspath(paths[0]), trans_id)
         tt.apply()
     finally:
         tt.finalize()

=== modified file 'breezy/transport/http/_urllib2_wrappers.py'
--- breezy/transport/http/_urllib2_wrappers.py	2018-01-23 01:12:07 +0000
+++ breezy/transport/http/_urllib2_wrappers.py	2018-03-08 23:16:56 +0000
@@ -669,7 +669,9 @@
     def http_request(self, request):
         """Common headers setting"""
 
-        request.headers.update(self._default_headers.copy())
+        for name, value in self._default_headers.items():
+            if name not in request.headers:
+                request.headers[name] = value
         # FIXME: We may have to add the Content-Length header if
         # we have data to send.
         return request

=== modified file 'breezy/transport/sftp.py'
--- breezy/transport/sftp.py	2018-02-18 19:23:48 +0000
+++ breezy/transport/sftp.py	2018-03-25 00:39:16 +0000
@@ -118,8 +118,8 @@
                 return asbytes()
         return s
 
-    _bad.func_code = _b_for_broken_paramiko.func_code
-    _bad_asbytes.func_code = _asbytes_for_broken_paramiko.func_code
+    _bad.__code__ = _b_for_broken_paramiko.__code__
+    _bad_asbytes.__code__ = _asbytes_for_broken_paramiko.__code__
 
 
 class SFTPLock(object):

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2018-02-17 02:57:14 +0000
+++ breezy/tree.py	2018-03-24 10:24:48 +0000
@@ -108,6 +108,14 @@
     trees or versioned trees.
     """
 
+    def supports_rename_tracking(self):
+        """Whether this tree supports rename tracking.
+
+        This defaults to True, but some implementations may want to override
+        it.
+        """
+        return True
+
     def has_versioned_directories(self):
         """Whether this tree can contain explicitly versioned directories.
 
@@ -205,7 +213,7 @@
         """
         raise NotImplementedError(self.id2path)
 
-    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+    def iter_entries_by_dir(self, specific_files=None, yield_parents=False):
         """Walk the tree in 'by_dir' order.
 
         This will yield each entry in the tree as a (path, entry) tuple.
@@ -231,8 +239,8 @@
           a, f, a/b, a/d, a/b/c, a/d/e, f/g
 
         :param yield_parents: If True, yield the parents from the root leading
-            down to specific_file_ids that have been requested. This has no
-            impact if specific_file_ids is None.
+            down to specific_files that have been requested. This has no
+            impact if specific_files is None.
         """
         raise NotImplementedError(self.iter_entries_by_dir)
 
@@ -309,9 +317,6 @@
         """
         raise NotImplementedError(self._comparison_data)
 
-    def _file_size(self, entry, stat_value):
-        raise NotImplementedError(self._file_size)
-
     def get_file(self, path, file_id=None):
         """Return a file object for the file file_id in the tree.
 
@@ -433,16 +438,14 @@
         this implementation, it is a tuple containing a single bytestring with
         the complete text of the file.
 
-        :param desired_files: a list of (file_id, identifier) pairs
+        :param desired_files: a list of (path, identifier) pairs
         """
-        for file_id, identifier in desired_files:
+        for path, identifier in desired_files:
             # We wrap the string in a tuple so that we can return an iterable
             # of bytestrings.  (Technically, a bytestring is also an iterable
             # of bytestrings, but iterating through each character is not
             # performant.)
-            # TODO(jelmer): Pass paths into iter_files_bytes
-            path = self.id2path(file_id)
-            cur_file = (self.get_file_text(path, file_id),)
+            cur_file = (self.get_file_text(path),)
             yield identifier, cur_file
 
     def get_symlink_target(self, path, file_id=None):
@@ -476,46 +479,6 @@
         """
         raise NotImplementedError(self.annotate_iter)
 
-    def _get_plan_merge_data(self, file_id, other, base):
-        from .bzr import versionedfile
-        vf = versionedfile._PlanMergeVersionedFile(file_id)
-        last_revision_a = self._get_file_revision(
-                self.id2path(file_id), file_id, vf, 'this:')
-        last_revision_b = other._get_file_revision(
-                other.id2path(file_id), file_id, vf, 'other:')
-        if base is None:
-            last_revision_base = None
-        else:
-            last_revision_base = base._get_file_revision(
-                    base.id2path(file_id), file_id, vf, 'base:')
-        return vf, last_revision_a, last_revision_b, last_revision_base
-
-    def plan_file_merge(self, file_id, other, base=None):
-        """Generate a merge plan based on annotations.
-
-        If the file contains uncommitted changes in this tree, they will be
-        attributed to the 'current:' pseudo-revision.  If the file contains
-        uncommitted changes in the other tree, they will be assigned to the
-        'other:' pseudo-revision.
-        """
-        data = self._get_plan_merge_data(file_id, other, base)
-        vf, last_revision_a, last_revision_b, last_revision_base = data
-        return vf.plan_merge(last_revision_a, last_revision_b,
-                             last_revision_base)
-
-    def plan_file_lca_merge(self, file_id, other, base=None):
-        """Generate a merge plan based lca-newness.
-
-        If the file contains uncommitted changes in this tree, they will be
-        attributed to the 'current:' pseudo-revision.  If the file contains
-        uncommitted changes in the other tree, they will be assigned to the
-        'other:' pseudo-revision.
-        """
-        data = self._get_plan_merge_data(file_id, other, base)
-        vf, last_revision_a, last_revision_b, last_revision_base = data
-        return vf.plan_lca_merge(last_revision_a, last_revision_b,
-                                 last_revision_base)
-
     def _iter_parent_trees(self):
         """Iterate through parent trees, defaulting to Tree.revision_tree."""
         for revision_id in self.get_parent_ids():
@@ -524,45 +487,6 @@
             except errors.NoSuchRevisionInTree:
                 yield self.repository.revision_tree(revision_id)
 
-    def _get_file_revision(self, path, file_id, vf, tree_revision):
-        """Ensure that file_id, tree_revision is in vf to plan the merge."""
-        if getattr(self, '_repository', None) is None:
-            last_revision = tree_revision
-            parent_keys = [(file_id, t.get_file_revision(path, file_id)) for t in
-                self._iter_parent_trees()]
-            vf.add_lines((file_id, last_revision), parent_keys,
-                         self.get_file_lines(path, file_id))
-            repo = self.branch.repository
-            base_vf = repo.texts
-        else:
-            last_revision = self.get_file_revision(path, file_id)
-            base_vf = self._repository.texts
-        if base_vf not in vf.fallback_versionedfiles:
-            vf.fallback_versionedfiles.append(base_vf)
-        return last_revision
-
-    def _check_retrieved(self, ie, f):
-        if not __debug__:
-            return
-        fp = osutils.fingerprint_file(f)
-        f.seek(0)
-
-        if ie.text_size is not None:
-            if ie.text_size != fp['size']:
-                raise errors.BzrError(
-                        "mismatched size for file %r in %r" %
-                        (ie.file_id, self._store),
-                        ["inventory expects %d bytes" % ie.text_size,
-                         "file is actually %d bytes" % fp['size'],
-                         "store is probably damaged/corrupt"])
-
-        if ie.text_sha1 != fp['sha1']:
-            raise errors.BzrError("wrong SHA-1 for file %r in %r" %
-                    (ie.file_id, self._store),
-                    ["inventory expects %s" % ie.text_sha1,
-                     "file is actually %s" % fp['sha1'],
-                     "store is probably damaged/corrupt"])
-
     def path2id(self, path):
         """Return the id for path in this tree."""
         raise NotImplementedError(self.path2id)
@@ -575,32 +499,23 @@
         """
         return self.path2id(path) is not None
 
-    def paths2ids(self, paths, trees=[], require_versioned=True):
-        """Return all the ids that can be reached by walking from paths.
-
-        Each path is looked up in this tree and any extras provided in
-        trees, and this is repeated recursively: the children in an extra tree
-        of a directory that has been renamed under a provided path in this tree
-        are all returned, even if none exist under a provided path in this
-        tree, and vice versa.
-
-        :param paths: An iterable of paths to start converting to ids from.
-            Alternatively, if paths is None, no ids should be calculated and None
-            will be returned. This is offered to make calling the api unconditional
-            for code that *might* take a list of files.
-        :param trees: Additional trees to consider.
-        :param require_versioned: If False, do not raise NotVersionedError if
-            an element of paths is not versioned in this tree and all of trees.
-        """
-        return find_ids_across_trees(paths, [self] + list(trees), require_versioned)
-
-    def iter_children(self, file_id):
-        """Iterate over the file ids of the children of an entry.
-
-        :param file_id: File id of the entry
-        :return: Iterator over child file ids.
-        """
-        raise NotImplementedError(self.iter_children)
+    def find_related_paths_across_trees(self, paths, trees=[],
+            require_versioned=True):
+        """Find related paths in tree corresponding to specified filenames in any
+        of `lookup_trees`.
+
+        All matches in all trees will be used, and all children of matched
+        directories will be used.
+
+        :param paths: The filenames to find related paths for (if None, returns
+            None)
+        :param trees: The trees to find file_ids within
+        :param require_versioned: if true, all specified filenames must occur in
+            at least one tree.
+        :return: a set of paths for the specified filenames and their children
+            in `tree`
+        """
+        raise NotImplementedError(self.find_related_paths_across_trees)
 
     def lock_read(self):
         """Lock this tree for multiple read only operations.
@@ -677,7 +592,7 @@
     def supports_content_filtering(self):
         return False
 
-    def _content_filter_stack(self, path=None, file_id=None):
+    def _content_filter_stack(self, path=None):
         """The stack of content filters for a path if filtering is supported.
 
         Readers will be applied in first-to-last order.
@@ -686,14 +601,11 @@
 
         :param path: path relative to the root of the tree
             or None if unknown
-        :param file_id: file_id or None if unknown
         :return: the list of filters - [] if there are none
         """
         filter_pref_names = filters._get_registered_names()
         if len(filter_pref_names) == 0:
             return []
-        if path is None:
-            path = self.id2path(file_id)
         prefs = next(self.iter_search_rules([path], filter_pref_names))
         stk = filters._get_filter_stack_for(prefs)
         if 'filters' in debug.debug_flags:
@@ -710,7 +622,7 @@
         """
         if self.supports_content_filtering():
             return lambda path, file_id: \
-                    self._content_filter_stack(path, file_id)
+                    self._content_filter_stack(path)
         else:
             return None
 
@@ -742,79 +654,6 @@
         return searcher
 
 
-def find_ids_across_trees(filenames, trees, require_versioned=True):
-    """Find the ids corresponding to specified filenames.
-
-    All matches in all trees will be used, and all children of matched
-    directories will be used.
-
-    :param filenames: The filenames to find file_ids for (if None, returns
-        None)
-    :param trees: The trees to find file_ids within
-    :param require_versioned: if true, all specified filenames must occur in
-        at least one tree.
-    :return: a set of file ids for the specified filenames and their children.
-    """
-    if not filenames:
-        return None
-    specified_path_ids = _find_ids_across_trees(filenames, trees,
-        require_versioned)
-    return _find_children_across_trees(specified_path_ids, trees)
-
-
-def _find_ids_across_trees(filenames, trees, require_versioned):
-    """Find the ids corresponding to specified filenames.
-
-    All matches in all trees will be used, but subdirectories are not scanned.
-
-    :param filenames: The filenames to find file_ids for
-    :param trees: The trees to find file_ids within
-    :param require_versioned: if true, all specified filenames must occur in
-        at least one tree.
-    :return: a set of file ids for the specified filenames
-    """
-    not_versioned = []
-    interesting_ids = set()
-    for tree_path in filenames:
-        not_found = True
-        for tree in trees:
-            file_id = tree.path2id(tree_path)
-            if file_id is not None:
-                interesting_ids.add(file_id)
-                not_found = False
-        if not_found:
-            not_versioned.append(tree_path)
-    if len(not_versioned) > 0 and require_versioned:
-        raise errors.PathsNotVersionedError(not_versioned)
-    return interesting_ids
-
-
-def _find_children_across_trees(specified_ids, trees):
-    """Return a set including specified ids and their children.
-
-    All matches in all trees will be used.
-
-    :param trees: The trees to find file_ids within
-    :return: a set containing all specified ids and their children
-    """
-    interesting_ids = set(specified_ids)
-    pending = interesting_ids
-    # now handle children of interesting ids
-    # we loop so that we handle all children of each id in both trees
-    while len(pending) > 0:
-        new_pending = set()
-        for file_id in pending:
-            for tree in trees:
-                if not tree.has_or_had_id(file_id):
-                    continue
-                for child_id in tree.iter_children(file_id):
-                    if child_id not in interesting_ids:
-                        new_pending.add(child_id)
-        interesting_ids.update(new_pending)
-        pending = new_pending
-    return interesting_ids
-
-
 class InterTree(InterObject):
     """This class represents operations taking place between two Trees.
 
@@ -840,16 +679,14 @@
         # it works for all trees.
         return True
 
-    def _changes_from_entries(self, source_entry, target_entry,
-        source_path=None, target_path=None):
+    def _changes_from_entries(self, source_entry, target_entry, source_path,
+                              target_path):
         """Generate a iter_changes tuple between source_entry and target_entry.
 
         :param source_entry: An inventory entry from self.source, or None.
         :param target_entry: An inventory entry from self.target, or None.
-        :param source_path: The path of source_entry, if known. If not known
-            it will be looked up.
-        :param target_path: The path of target_entry, if known. If not known
-            it will be looked up.
+        :param source_path: The path of source_entry.
+        :param target_path: The path of target_entry.
         :return: A tuple, item 0 of which is an iter_changes result tuple, and
             item 1 is True if there are any changes in the result tuple.
         """
@@ -863,8 +700,6 @@
             source_versioned = True
             source_name = source_entry.name
             source_parent = source_entry.parent_id
-            if source_path is None:
-                source_path = self.source.id2path(file_id)
             source_kind, source_executable, source_stat = \
                 self.source._comparison_data(source_entry, source_path)
         else:
@@ -877,8 +712,6 @@
             target_versioned = True
             target_name = target_entry.name
             target_parent = target_entry.parent_id
-            if target_path is None:
-                target_path = self.target.id2path(file_id)
             target_kind, target_executable, target_stat = \
                 self.target._comparison_data(target_entry, target_path)
         else:
@@ -893,8 +726,9 @@
         if source_kind != target_kind:
             changed_content = True
         elif source_kind == 'file':
-            if not self.file_content_matches(file_id, file_id, source_path,
-                    target_path, source_stat, target_stat):
+            if not self.file_content_matches(
+                    source_path, target_path,
+                    file_id, file_id, source_stat, target_stat):
                 changed_content = True
         elif source_kind == 'symlink':
             if (self.source.get_symlink_target(source_path, file_id) !=
@@ -939,18 +773,6 @@
         if extra_trees is not None:
             trees = trees + tuple(extra_trees)
         with self.lock_read():
-            # target is usually the newer tree:
-            specific_file_ids = self.target.paths2ids(specific_files, trees,
-                require_versioned=require_versioned)
-            if specific_files and not specific_file_ids:
-                # All files are unversioned, so just return an empty delta
-                # _compare_trees would think we want a complete delta
-                result = delta.TreeDelta()
-                fake_entry = inventory.InventoryFile('unused', 'unused', 'unused')
-                result.unversioned = [(path, None,
-                    self.target._comparison_data(fake_entry, path)[0]) for path in
-                    specific_files]
-                return result
             return delta._compare_trees(self.source, self.target, want_unchanged,
                 specific_files, include_root, extra_trees=extra_trees,
                 require_versioned=require_versioned,
@@ -992,17 +814,23 @@
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
         """
-        lookup_trees = [self.source]
-        if extra_trees:
-             lookup_trees.extend(extra_trees)
+        if not extra_trees:
+             extra_trees = []
+        else:
+             extra_trees = list(extra_trees)
         # The ids of items we need to examine to insure delta consistency.
         precise_file_ids = set()
         changed_file_ids = []
         if specific_files == []:
-            specific_file_ids = []
+            target_specific_files = []
+            source_specific_files = []
         else:
-            specific_file_ids = self.target.paths2ids(specific_files,
-                lookup_trees, require_versioned=require_versioned)
+            target_specific_files = self.target.find_related_paths_across_trees(
+                    specific_files, [self.source] + extra_trees,
+                    require_versioned=require_versioned)
+            source_specific_files = self.source.find_related_paths_across_trees(
+                    specific_files, [self.target] + extra_trees,
+                    require_versioned=require_versioned)
         if specific_files is not None:
             # reparented or added entries must have their parents included
             # so that valid deltas can be created. The seen_parents set
@@ -1022,10 +850,10 @@
             all_unversioned = collections.deque()
         to_paths = {}
         from_entries_by_dir = list(self.source.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids))
+            specific_files=source_specific_files))
         from_data = dict((e.file_id, (p, e)) for p, e in from_entries_by_dir)
         to_entries_by_dir = list(self.target.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids))
+            specific_files=target_specific_files))
         num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
         entry_count = 0
         # the unversioned path lookup only occurs on real trees - where there
@@ -1054,7 +882,7 @@
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)
             if changes or include_unchanged:
-                if specific_file_ids is not None:
+                if specific_files is not None:
                     new_parent_id = result[4][1]
                     precise_file_ids.add(new_parent_id)
                     changed_file_ids.append(result[0])
@@ -1085,12 +913,7 @@
             if file_id in to_paths:
                 # already returned
                 continue
-            if not self.target.has_id(file_id):
-                # common case - paths we have not emitted are not present in
-                # target.
-                to_path = None
-            else:
-                to_path = self.target.id2path(file_id)
+            to_path = find_previous_path(self.source, self.target, path)
             entry_count += 1
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)
@@ -1107,12 +930,12 @@
             yield(file_id, (path, to_path), changed_content, versioned, parent,
                   name, kind, executable)
         changed_file_ids = set(changed_file_ids)
-        if specific_file_ids is not None:
+        if specific_files is not None:
             for result in self._handle_precise_ids(precise_file_ids,
                 changed_file_ids):
                 yield result
 
-    def _get_entry(self, tree, file_id):
+    def _get_entry(self, tree, path):
         """Get an inventory entry from a tree, with missing entries as None.
 
         If the tree raises NotImplementedError on accessing .inventory, then
@@ -1122,20 +945,12 @@
         :param tree: The tree to lookup the entry in.
         :param file_id: The file_id to lookup.
         """
+        # No inventory available.
         try:
-            inventory = tree.root_inventory
-        except (AttributeError, NotImplementedError):
-            # No inventory available.
-            try:
-                iterator = tree.iter_entries_by_dir(specific_file_ids=[file_id])
-                return iterator.next()[1]
-            except StopIteration:
-                return None
-        else:
-            try:
-                return inventory[file_id]
-            except errors.NoSuchId:
-                return None
+            iterator = tree.iter_entries_by_dir(specific_files=[path])
+            return iterator.next()[1]
+        except StopIteration:
+            return None
 
     def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
         discarded_changes=None):
@@ -1183,14 +998,26 @@
                 # Examine file_id
                 if discarded_changes:
                     result = discarded_changes.get(file_id)
-                    old_entry = None
+                    source_entry = None
                 else:
                     result = None
                 if result is None:
-                    old_entry = self._get_entry(self.source, file_id)
-                    new_entry = self._get_entry(self.target, file_id)
+                    try:
+                        source_path = self.source.id2path(file_id)
+                    except errors.NoSuchId:
+                        source_path = None
+                        source_entry = None
+                    else:
+                        source_entry = self._get_entry(self.source, source_path)
+                    try:
+                        target_path = self.target.id2path(file_id)
+                    except errors.NoSuchId:
+                        target_path = None
+                        target_entry = None
+                    else:
+                        target_entry = self._get_entry(self.target, target_path)
                     result, changes = self._changes_from_entries(
-                        old_entry, new_entry)
+                        source_entry, target_entry, source_path, target_path)
                 else:
                     changes = True
                 # Get this parents parent to examine.
@@ -1201,35 +1028,33 @@
                             result[6][1] != 'directory'):
                         # This stopped being a directory, the old children have
                         # to be included.
-                        if old_entry is None:
+                        if source_entry is None:
                             # Reusing a discarded change.
-                            old_entry = self._get_entry(self.source, file_id)
+                            source_entry = self._get_entry(self.source, result[1][0])
                         precise_file_ids.update(
-                                self.source.iter_children(file_id))
+                                child.file_id
+                                for child in self.source.iter_child_entries(result[1][0]))
                     changed_file_ids.add(result[0])
                     yield result
 
     def file_content_matches(
-            self, source_file_id, target_file_id, source_path=None,
-            target_path=None, source_stat=None, target_stat=None):
+            self, source_path, target_path,
+            source_file_id=None, target_file_id=None,
+            source_stat=None, target_stat=None):
         """Check if two files are the same in the source and target trees.
 
         This only checks that the contents of the files are the same,
         it does not touch anything else.
 
-        :param source_file_id: File id of the file in the source tree
-        :param target_file_id: File id of the file in the target tree
         :param source_path: Path of the file in the source tree
         :param target_path: Path of the file in the target tree
+        :param source_file_id: Optional file id of the file in the source tree
+        :param target_file_id: Optional file id of the file in the target tree
         :param source_stat: Optional stat value of the file in the source tree
         :param target_stat: Optional stat value of the file in the target tree
         :return: Boolean indicating whether the files have the same contents
         """
         with self.lock_read():
-            if source_path is None:
-                source_path = self.source.id2path(source_file_id)
-            if target_path is None:
-                target_path = self.target.id2path(target_file_id)
             source_verifier_kind, source_verifier_data = (
                     self.source.get_file_verifier(
                         source_path, source_file_id, source_stat))
@@ -1485,3 +1310,36 @@
                     other_values.append(self._lookup_by_file_id(
                                             alt_extra, alt_tree, file_id))
                 yield other_path, file_id, None, other_values
+
+
+def find_previous_paths(from_tree, to_tree, paths):
+    """Find previous tree paths.
+
+    :param from_tree: From tree
+    :param to_tree: To tree
+    :param paths: Iterable over paths to search for
+    :return: Dictionary mapping from from_tree paths to paths in to_tree, or
+        None if there is no equivalent path.
+    """
+    ret = {}
+    for path in paths:
+        ret[path] = find_previous_path(from_tree, to_tree, path)
+    return ret
+
+
+def find_previous_path(from_tree, to_tree, path, file_id=None):
+    """Find previous tree path.
+
+    :param from_tree: From tree
+    :param to_tree: To tree
+    :param path: Path to search for
+    :return: path in to_tree, or None if there is no equivalent path.
+    """
+    if file_id is None:
+        file_id = from_tree.path2id(path)
+    if file_id is None:
+        raise errors.NoSuchFile(path)
+    try:
+        return to_tree.id2path(file_id)
+    except errors.NoSuchId:
+        return None

=== modified file 'breezy/ui/__init__.py'
--- breezy/ui/__init__.py	2017-05-22 00:56:52 +0000
+++ breezy/ui/__init__.py	2018-03-24 15:16:49 +0000
@@ -522,12 +522,7 @@
     """
     # this is now always TextUIFactory, which in turn decides whether it
     # should display progress bars etc
-    from .text import TextUIFactory, _wrap_in_stream, _wrap_out_stream
-    # GZ 2017-05-21: May want to rewrap streams on Python 3 if encoding config
-    if not PY3:
-        stdin = _wrap_in_stream(stdin)
-        stdout = _wrap_out_stream(stdout)
-        stderr = _wrap_out_stream(stderr)
+    from .text import TextUIFactory
     return TextUIFactory(stdin, stdout, stderr)
 
 

=== modified file 'breezy/ui/text.py'
--- breezy/ui/text.py	2017-06-14 22:19:52 +0000
+++ breezy/ui/text.py	2018-03-24 15:16:49 +0000
@@ -65,13 +65,14 @@
         variable is set to 'line-based', or if there is no controlling
         terminal.
         """
-        if os.environ.get('BRZ_TEXTUI_INPUT') != 'line-based' and \
-           self.ui.stdin == sys.stdin and self.ui.stdin.isatty():
+        is_tty = self.ui.raw_stdin.isatty()
+        if (os.environ.get('BRZ_TEXTUI_INPUT') != 'line-based' and
+                self.ui.raw_stdin == sys.stdin and is_tty):
             self.line_based = False
             self.echo_back = True
         else:
             self.line_based = True
-            self.echo_back = not self.ui.stdin.isatty()
+            self.echo_back = not is_tty
 
     def _build_alternatives(self, msg, choices, default):
         """Parse choices string.
@@ -126,7 +127,9 @@
             raise KeyboardInterrupt
         if char == chr(4): # EOF (^d, C-d)
             raise EOFError
-        return char.decode("ascii", "replace")
+        if isinstance(char, bytes):
+            return char.decode('ascii', 'replace')
+        return char
 
     def interact(self):
         """Keep asking the user until a valid choice is made.
@@ -173,8 +176,22 @@
         self.stdin = stdin
         self.stdout = stdout
         self.stderr = stderr
+        self._progress_view = NullProgressView()
+
+    def __enter__(self):
+        # Choose default encoding and handle py2/3 differences
+        self._setup_streams()
         # paints progress, network activity, etc
         self._progress_view = self.make_progress_view()
+        return self
+
+    def _setup_streams(self):
+        self.raw_stdin = _unwrap_stream(self.stdin)
+        self.stdin = _wrap_in_stream(self.raw_stdin)
+        self.raw_stdout = _unwrap_stream(self.stdout)
+        self.stdout = _wrap_out_stream(self.raw_stdout)
+        self.raw_stderr = _unwrap_stream(self.stderr)
+        self.stderr = _wrap_out_stream(self.raw_stderr)
 
     def choose(self, msg, choices, default=None):
         """Prompt the user for a list of alternatives.
@@ -642,7 +659,7 @@
 def _unwrap_stream(stream):
     inner = getattr(stream, "buffer", None)
     if inner is None:
-        inner = getattr(stream, "stream", None)
+        inner = getattr(stream, "stream", stream)
     return inner
 
 

=== modified file 'breezy/upgrade.py'
--- breezy/upgrade.py	2017-06-11 14:07:05 +0000
+++ breezy/upgrade.py	2018-02-24 15:50:23 +0000
@@ -116,12 +116,9 @@
         """
         transport = self.transport
         backup_relpath = transport.relpath(self.backup_newpath)
-        child_pb = ui.ui_factory.nested_progress_bar()
-        child_pb.update(gettext('Deleting backup.bzr'))
-        try:
+        with ui.ui_factory.nested_progress_bar() as child_pb:
+            child_pb.update(gettext('Deleting backup.bzr'))
             transport.delete_tree(backup_relpath)
-        finally:
-            child_pb.finished()
 
 
 def upgrade(url, format=None, clean_up=False, dry_run=False):
@@ -272,40 +269,38 @@
     """
     succeeded = []
     exceptions = []
-    child_pb = ui.ui_factory.nested_progress_bar()
-    child_pb.update(gettext('Upgrading bzrdirs'), 0, len(items))
-    for i, control_dir in enumerate(items):
-        # Do the conversion
-        location = control_dir.root_transport.base
-        bzr_object, bzr_label = _get_object_and_label(control_dir)
-        type_label = label or bzr_label
-        child_pb.update(gettext("Upgrading %s") % (type_label), i+1, len(items))
-        ui.ui_factory.note(gettext('Upgrading {0} {1} ...').format(type_label, 
-            urlutils.unescape_for_display(location, 'utf-8'),))
-        try:
-            if not dry_run:
-                cv = Convert(control_dir=control_dir, format=format)
-        except errors.UpToDateFormat as ex:
-            ui.ui_factory.note(str(ex))
-            succeeded.append(control_dir)
-            continue
-        except Exception as ex:
-            trace.warning('conversion error: %s' % ex)
-            exceptions.append(ex)
-            continue
-
-        # Do any required post processing
-        succeeded.append(control_dir)
-        if clean_up:
+    with ui.ui_factory.nested_progress_bar() as child_pb:
+        child_pb.update(gettext('Upgrading bzrdirs'), 0, len(items))
+        for i, control_dir in enumerate(items):
+            # Do the conversion
+            location = control_dir.root_transport.base
+            bzr_object, bzr_label = _get_object_and_label(control_dir)
+            type_label = label or bzr_label
+            child_pb.update(gettext("Upgrading %s") % (type_label), i+1, len(items))
+            ui.ui_factory.note(gettext('Upgrading {0} {1} ...').format(type_label, 
+                urlutils.unescape_for_display(location, 'utf-8'),))
             try:
-                ui.ui_factory.note(gettext('Removing backup ...'))
                 if not dry_run:
-                    cv.clean_up()
+                    cv = Convert(control_dir=control_dir, format=format)
+            except errors.UpToDateFormat as ex:
+                ui.ui_factory.note(str(ex))
+                succeeded.append(control_dir)
+                continue
             except Exception as ex:
-                trace.warning(gettext('failed to clean-up {0}: {1}') % (location, ex))
+                trace.warning('conversion error: %s' % ex)
                 exceptions.append(ex)
+                continue
 
-    child_pb.finished()
+            # Do any required post processing
+            succeeded.append(control_dir)
+            if clean_up:
+                try:
+                    ui.ui_factory.note(gettext('Removing backup ...'))
+                    if not dry_run:
+                        cv.clean_up()
+                except Exception as ex:
+                    trace.warning(gettext('failed to clean-up {0}: {1}') % (location, ex))
+                    exceptions.append(ex)
 
     # Return the result
     return succeeded, exceptions

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2018-02-18 19:18:40 +0000
+++ breezy/workingtree.py	2018-03-25 00:39:16 +0000
@@ -151,6 +151,11 @@
         """See `Tree.has_versioned_directories`."""
         return self._format.supports_versioned_directories
 
+    def supports_merge_modified(self):
+        """Indicate whether this workingtree supports storing merge_modified.
+        """
+        return self._format.supports_merge_modified
+
     def _supports_executable(self):
         if sys.platform == 'win32':
             return False
@@ -368,7 +373,12 @@
                            _fstat=osutils.fstat):
         """See Tree.get_file_with_stat."""
         abspath = self.abspath(path)
-        file_obj = open(abspath, 'rb')
+        try:
+            file_obj = open(abspath, 'rb')
+        except EnvironmentError as e:
+            if e.errno == errno.ENOENT:
+                raise errors.NoSuchFile(path)
+            raise
         stat_value = _fstat(file_obj.fileno())
         if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(path)
@@ -450,9 +460,6 @@
                     new_parents = [revision_id]
                 tree.set_parent_ids(new_parents)
 
-    def id2abspath(self, file_id):
-        return self.abspath(self.id2path(file_id))
-
     def get_file_size(self, path, file_id=None):
         """See Tree.get_file_size"""
         # XXX: this returns the on-disk size; it should probably return the
@@ -717,10 +724,7 @@
             return file_id
 
     def get_symlink_target(self, path, file_id=None):
-        if path is not None:
-            abspath = self.abspath(path)
-        else:
-            abspath = self.id2abspath(file_id)
+        abspath = self.abspath(path)
         target = osutils.readlink(abspath)
         return target
 
@@ -765,8 +769,6 @@
         raise NotImplementedError(self.flush)
 
     def kind(self, relpath, file_id=None):
-        if file_id is not None:
-            return osutils.file_kind(self.id2abspath(file_id))
         return osutils.file_kind(self.abspath(relpath))
 
     def list_files(self, include_root=False, from_dir=None, recursive=True):
@@ -904,8 +906,7 @@
 
     def put_file_bytes_non_atomic(self, path, bytes, file_id=None):
         """See MutableTree.put_file_bytes_non_atomic."""
-        with self.lock_write():
-            with file(self.abspath(path), 'wb') as stream:
+        with self.lock_write(), file(self.abspath(path), 'wb') as stream:
                 stream.write(bytes)
 
     def extras(self):
@@ -956,9 +957,6 @@
                 executable = bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
         return kind, executable, stat_value
 
-    def _file_size(self, entry, stat_value):
-        return stat_value.st_size
-
     def last_revision(self):
         """Return the last revision of the branch for this tree.
 
@@ -1289,11 +1287,12 @@
             resolved = _mod_conflicts.ConflictList()
             conflict_re = re.compile('^(<{7}|={7}|>{7})')
             for conflict in self.conflicts():
+                path = self.id2path(conflict.file_id)
                 if (conflict.typestring != 'text conflict' or
-                    self.kind(self.id2path(conflict.file_id), conflict.file_id) != 'file'):
+                    self.kind(path, conflict.file_id) != 'file'):
                     un_resolved.append(conflict)
                     continue
-                with open(self.id2abspath(conflict.file_id), 'rb') as my_file:
+                with open(self.abspath(path), 'rb') as my_file:
                     for line in my_file:
                         if conflict_re.search(line):
                             un_resolved.append(conflict)
@@ -1399,6 +1398,9 @@
 
     supports_versioned_directories = None
 
+    supports_merge_modified = True
+    """If this format supports storing merge modified hashes."""
+
     supports_setting_file_ids = True
     """If this format allows setting the file id."""
 
@@ -1407,6 +1409,11 @@
 
     supports_leftmost_parent_id_as_ghost = True
 
+    supports_righthand_parent_id_as_ghost = True
+
+    ignore_filename = None
+    """Name of file with ignore patterns, if any. """
+
     def initialize(self, controldir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """Initialize a new working tree in controldir.

=== modified file 'brz'
--- brz	2017-09-06 04:15:55 +0000
+++ brz	2018-03-24 15:16:49 +0000
@@ -89,14 +89,10 @@
 
 
 if __name__ == '__main__':
-    library_state = breezy.initialize()
-    library_state.__enter__()
-    try:
+    with breezy.initialize():
         exit_val = breezy.commands.main()
         if profiling:
             profile_imports.log_stack_info(sys.stderr)
-    finally:
-        library_state.__exit__(None, None, None)
 
     # By this point we really have completed everything we want to do, and
     # there's no point doing any additional cleanup.  Abruptly exiting here

=== modified file 'doc/en/release-notes/brz-3.0.txt'
--- doc/en/release-notes/brz-3.0.txt	2018-02-09 23:09:53 +0000
+++ doc/en/release-notes/brz-3.0.txt	2018-03-24 03:03:03 +0000
@@ -63,6 +63,10 @@
    out to the gnupg command. The ``gpg_signing_command`` option has been
    removed.  (Jelmer Vernooĳ, #847388)
 
+ * The ``bzr.transform.orphan_policy`` configuration option
+   has been renamed to ``transform.orphan_policy``.
+   (Jelmer Vernooĳ)
+
 New Features
 ************
 
@@ -260,6 +264,9 @@
 .. Major internal changes, unlikely to be visible to users or plugin 
    developers, but interesting for brz developers.
 
+* Allow overriding of default HTTP headers by caller.
+  (Jelmer Vernooĳ, #1753734)
+
 Testing
 *******
 

=== modified file 'setup.py'
--- setup.py	2018-02-18 19:18:40 +0000
+++ setup.py	2018-03-25 00:39:16 +0000
@@ -57,7 +57,10 @@
     'install_requires': [
         'six>=1.9.0',
         ],
-    }
+    'extras_require': {
+        'fastimport': ['fastimport'],
+        },
+}
 
 # The list of packages is automatically generated later. Add other things
 # that are part of BZRLIB here.

=== modified file 'tools/time_graph.py'
--- tools/time_graph.py	2017-08-26 13:58:53 +0000
+++ tools/time_graph.py	2018-02-24 15:50:23 +0000
@@ -39,14 +39,11 @@
 
 def all_heads_comp(g, combinations):
     h = []
-    pb = ui.ui_factory.nested_progress_bar()
-    try:
+    with ui.ui_factory.nested_progress_bar() as pb:
         for idx, combo in enumerate(combinations):
             if idx & 0x1f == 0:
                 pb.update('proc', idx, len(combinations))
             h.append(g.heads(combo))
-    finally:
-        pb.finished()
     return h
 
 combinations = []

